﻿#include "WetBulbGlobeTempCalc.h"

//WetBulbGlobeTemperature algorithm from Liljegren et al. (2008) code: https://github.com/mdljts/wbgt/blob/master/src/wbgt.c
//Reference:
//Liljegren, J. C., R. A. Carhart, P. Lawday, S. Tschopp, and R. Sharp. (2008). Modeling the Wet Bulb Globe Temperature Using Standard Meteorological Measurements. The Journal of Occupational and Environmental Hygiene, vol. 5:10, pp. 645-655. DOI: 10.1080/15459620802310770 

//WetBulbGlobeTempCalc::WetBulbGlobeTempCalc Constructor for use here with initialization list or pointer
//Note: Initialization list : or colon after function name initializes SolarCalculator and resistanceVar objects
WetBulbGlobeTempCalc::WetBulbGlobeTempCalc(Inputs* input, SolarCalculator* solarCalc)
	: solarCalculator(solarCalc)
{
	//cte 2024 Initialize class members explicitly in the constructor initializer list from WetBulbGlobeTempCalc.h, e.g., ...
	//... : Year_YYYY(0), Month_MM(0), Day_DD(0), Hour_HH(0), etc. 
	//Cp_Ratio_Air_to_Water =  HeatCapacity_Air_J_p_kg_p_K * Molecular_Weight_Air_kg_p_kmol / Molecular_Weight_Water_kg_p_kmol 
	//Note: Ratio relates the thermal properties of air to water vapor
	Cp_Ratio_Air_to_Water = HeatCapacity_Air_J_p_kg_p_K * Molecular_Weight_Air_kg_p_kmol / Molecular_Weight_Water_kg_p_kmol;
	//Ratio_GasConstant_to_MolecularWeightAir_m2_p_s2_K (m2/s2K) is R_Universal_Gas_Constant_J_p_kmol_K / Molecular_Weight_Air_kg_p_kmol; Units  m2/(s2 K) = J/(kg K) = [J/(kmol K)] / [kg/kmol]
	Ratio_GasConstant_to_MolecularWeightAir_m2_p_s2_K = R_Universal_Gas_Constant_J_p_kmol_K / Molecular_Weight_Air_kg_p_kmol;
	//Prandtl_Number_wind is modified Prandtl number relating momentum diffusivity or viscosity to thermal diffusivity
	Prandtl_Number_wind = HeatCapacity_Air_J_p_kg_p_K / (HeatCapacity_Air_J_p_kg_p_K + 1.25 * Ratio_GasConstant_to_MolecularWeightAir_m2_p_s2_K); 

	Wick_Emissivity_frac = 0.95;
	Wick_Albedo_frac = 0.4;
	Wick_Diameter_m = 0.007;
	Wick_Length_m = 0.0254;
	Globe_Emissivity_frac = 0.95;
	Globe_Albedo_frac = 0.05;
	Globe_Diameter_m = 0.0508;
	SurfaceGround_Emissivity_frac = 0.999;
	SurfaceGround_Albedo_frac = 0.45;
}

//WetBulbGlobeTemperature_Inputs function will manage call to calculate the wet bulb globe temperature
void WetBulbGlobeTempCalc::WetBulbGlobeTemperature_Inputs(Inputs* input, DataFolder* folder, DataFolder* reffolder, int MapPixel_ID, int timeStep, double Radiation_Shortwave_Direct_W_p_m2, double Radiation_Shortwave_Diffuse_W_p_m2, double Cos_ZenithAngle_Solar) {

	//cte 2024 Refactor to use pixel values of meteorological variables, such as ...
	// ... AmosphericPressure_hPa can be adjusted with Elevation_delta_km as computed in HeatFluxCal
	// ... Radiation_Shortwave_Wpm2 adjusted with Cos_IncidenceAngle_Solar as computed in HeatFluxCal and diminished by tree canopy cover

	//YYYYMMDD is input->SimulationDate_GD[timeStep]
	int Date_YYYYMMDD = input->SimulationDate_GD[timeStep];
	//HH_MM_SS is input->SimulationTime_HMS[timeStep]
	string HH_MM_SS = input->SimulationTime_HMS[timeStep];

	//Year_YYYY extracted from YYYYMMDD, first 4 spaces
	Year_YYYY = stoi(to_string(Date_YYYYMMDD).substr(0, 4));
	//Month_0_Julian_or_1_to_12_MM extracted from YYYYMMDD, next 2 spaces
	Month_MM = stoi(to_string(Date_YYYYMMDD).substr(4, 2));
	//Day_of_Month_or_Julian_if_Month_0_DD extracted from YYYYMMDD, next 2 spaces
	Day_DD = stoi(to_string(Date_YYYYMMDD).substr(6, 2));
	//Hour_HH extracted from HH_MM_SS; values must be 1 to 24
	Hour_HH = stoi(HH_MM_SS.substr(0, 2)) + 1;
	//Minute_Mn extracted from HH_MM_SS
	Minute_Mn = stoi(HH_MM_SS.substr(3, 2));
	//Tair_C = Tair_K - 273.15
	Tair_C = folder->VarDict["Tair_K"] - 273.15;
	//Tdew_C = Tdew_K - 273.15
	Tdew_C = folder->VarDict["Tdew_K"] - 273.15;

	//Time_averaging_MeteorologicalInputs_minutes (min) is set to thirty (30) min by Liljegren et al. (2008) code
	Time_averaging_MeteorologicalInputs_minutes = 30;
	//AtmopshericPressure_hPa = input->AtmPres_kPa[timeStep] * 10.0, convert kPa to hPa
	AtmopshericPressure_hPa = input->AtmPres_kPa[timeStep] * 10.0;

	//RelativeHumidity_percent computed
	//VaporPressure_Saturated_kPa defined from Eq 5.2 of Stull, Tetens' Formula
	double VaporPressure_Coefficient_kPa = 0.611;
	double VaporPressure_Coefficient_1 = 17.2694;
	double VaporPressure_Coefficient_2_K = 273.15;
	double VaporPressure_Coefficient_3_K = 35.86;
	double Tair_K = Tair_C + 273.15;
	double Tdew_K = Tdew_C + 273.15;
	//VarporPressure_Saturated_kPa computed using Eq 5.2 of Stull (2001), Teten's Formula
	double VaporPressure_Saturated_kPa = VaporPressure_Coefficient_kPa * exp((VaporPressure_Coefficient_1 * (Tair_K - VaporPressure_Coefficient_2_K)) / (Tair_K - VaporPressure_Coefficient_3_K));
	//VaporPressure_Actual_kPa computed using Eq 5.2 of Stull (2001), Teten's Formula
	double VaporPressure_Actual_kPa = VaporPressure_Coefficient_kPa * exp((VaporPressure_Coefficient_1 * (Tdew_K - VaporPressure_Coefficient_2_K)) / (Tdew_K - VaporPressure_Coefficient_3_K));
	//RelativeHumidity_percent computed as VaporPressure_Actual_kPa / VaporPressure_Saturated_kPa * 100
	RelativeHumidity_percent = VaporPressure_Actual_kPa / VaporPressure_Saturated_kPa * 100;

	//WindSpeed_mps
	WindSpeed_mps = input->WindSpd_mps[timeStep];
	//Height_WindSpeed_m (m) is RefParams["Height_Sensor_Wind_m"], reference height for wind speed measurements
	Height_WindSpeed_m = input->RefParams["Height_Sensor_Wind_m"];
	//Delta_Height_Tair_m (m) is set to zero (0) by Liljegren et al. (2008) code
	Delta_Height_Tair_m = 0;
	//Flag_urban is set to one (1) by Liljegren et al. (2008) code
	Flag_urban = 1;
	//Flag_urban is initialized to one (1) by Liljegren et al. (2008) code
	Flag_daytime = 1;

	//cte 2024 Refactor to warn if lat and long are not reasonable, or of input is not Albers
	//WetBulbGlobeTempCalc::WetBulbGlobeTemperature_Calculate called
	WetBulbGlobeTempCalc::WetBulbGlobeTemperature_Calculate(input, folder, reffolder, MapPixel_ID, timeStep, Radiation_Shortwave_Direct_W_p_m2, Radiation_Shortwave_Diffuse_W_p_m2, Cos_ZenithAngle_Solar);
}

//WetBulbGlobeTemperature_Calculate function will compute the wet bulb globe temperature
void WetBulbGlobeTempCalc::WetBulbGlobeTemperature_Calculate(Inputs* input, DataFolder* folder, DataFolder* reffolder, int MapPixel_ID, int timeStep, double Radiation_Shortwave_Direct_W_p_m2, double Radiation_Shortwave_Diffuse_W_p_m2, double Cos_ZenithAngle_Solar) {

	//Ratio_LAI_BAI_Tree_Actual_to_Max_frac (frac) is  ratio of LAI_BAI_Tree_m2_p_m2[timeStep] to LAI_BAI_map["LAI_BAI_Tree_max_m2pm2"]
	double Ratio_LAI_BAI_Tree_Actual_to_Max_frac = input->LAI_BAI_Tree_m2_p_m2[timeStep] / input->LAI_BAI_map["LAI_BAI_Tree_max_m2pm2"];

	//Radiation_Shortwave_Direct_W_p_m2 (W/m2) is Radiation_Shortwave_Direct_W_p_m2 * (1 - (TreeCover_frac * Ratio_LAI_BAI_Tree_Actual_to_Max_frac ))
	//Note: Radiation_Shortwave_Direct_W_p_m2 is reduced based on tree cover shade, with a maximum when Ratio_LAI_BAI_Tree_Actual_to_Max_frac = 1
	//Note: Consider Refactor to create shade based on NLCD Class 21-24 HeatFluxCall SkyViewFactor_Ground_Open_frac or ...
	//Note: ... Area_Roof_Urban21_frac, or ratio Height_Avg_Urban22_m to Width_Road_Urban22_m; but Radiation_Shortwave_Direct_W_p_m2 may include this influence
	Radiation_Shortwave_Direct_W_p_m2 = Radiation_Shortwave_Direct_W_p_m2 * (1 - (folder->ParamDict["TreeCover_frac"] * Ratio_LAI_BAI_Tree_Actual_to_Max_frac));
	//Radiation_Shortwave_Wpm2 is Radiation_Shortwave_Direct_W_p_m2 + Radiation_Shortwave_Diffuse_W_p_m2, combining direct and diffuse
	Radiation_Shortwave_Wpm2 = Radiation_Shortwave_Direct_W_p_m2 + Radiation_Shortwave_Diffuse_W_p_m2;

	//If Radiation_Shortwave_Wpm2 > 0 then 
	if (Radiation_Shortwave_Wpm2 > 0) {
		//Radiation_Shortwave_direct_frac is Radiation_Shortwave_Direct_W_p_m2 / Radiation_Shortwave_Wpm2
		//Note: Liljegren et al. (2008) code called calc_solar_parameters and solarposition functions to obtain radiation parameters
		//Note : Liljegren et al. (2008) explain Eq 13 is empirically derived from 1 year of data from Oklahoma, shown in Figure 
		Radiation_Shortwave_direct_frac = Radiation_Shortwave_Direct_W_p_m2 / Radiation_Shortwave_Wpm2;
	}
	//Else Radiation_Shortwave_direct_frac is 0
	else {
		Radiation_Shortwave_direct_frac = 0;
	}

	//If Height_WindSpeed_m from HydroPlusConfig.xml not equal to Height_WindSpeed_Reference_m for Twbg calculations, then enter
	//Note: Height_WindSpeed_m = 10 m for most NOAA weather stations, while Height_WindSpeed_Reference_m = 2 m, so lower wind speed
	if (Height_WindSpeed_m != Height_WindSpeed_Reference_m) {
		//Flag_daytime set to True if Cos_ZenithAngle_Solar > 0, or False otherwise
		Flag_daytime = (Cos_ZenithAngle_Solar > 0) ? 1 : 0;
		//StabilityClass_Atmosphere_int is determined from function StabilityClass_Estimator
		StabilityClass_Atmosphere_int = WetBulbGlobeTempCalc::StabilityClass_Estimator(Flag_daytime, WindSpeed_mps, Radiation_Shortwave_Wpm2, Delta_Height_Tair_m);
		//WindSpeed_estimated_mps (m/s) computed in est_wind_speed to adjust from measured height to Twbg height
		//Note: Height_WindSpeed_m = 10 m for most NOAA weather stations, while Height_WindSpeed_Reference_m = 2 m, so lower wind speed
		WindSpeed_estimated_mps = WetBulbGlobeTempCalc::est_wind_speed(WindSpeed_mps, Height_WindSpeed_m, StabilityClass_Atmosphere_int, Flag_urban);
		WindSpeed_mps = WindSpeed_estimated_mps;
	}

	//Tair_K converted from Tair_C + 273.15
	double Tair_K = Tair_C + 273.15;
	//RelativeHumidity_frac(frac) converted from RelativeHumidity_percent * 0.01
	double RelativeHumidity_frac = RelativeHumidity_percent * 0.01;

	// Calculate globe and natural wet bulb temperatures
	//cte 2024 Refactor to pass corrected values for Radiation_Shortwave_Wpm2, Radiation_Shortwave_direct_frac
	//Tglobe_C computed in Tglobe_Calc
	Tglobe_C = WetBulbGlobeTempCalc::Tglobe_Calc(Tair_K, RelativeHumidity_frac, AtmopshericPressure_hPa, WindSpeed_mps, Radiation_Shortwave_Wpm2, Radiation_Shortwave_direct_frac, Cos_ZenithAngle_Solar);
	//Twetbulbnatural_C computed in Twetbulb_Calc with last term sent = 1
	Twetbulbnatural_C = WetBulbGlobeTempCalc::Twetbulb_Calc(Tair_K, RelativeHumidity_frac, AtmopshericPressure_hPa, WindSpeed_mps, Radiation_Shortwave_Wpm2, Radiation_Shortwave_direct_frac, Cos_ZenithAngle_Solar, 1);
	//Twetbulbpsychrometric_C computed in Twetbulb_Calc with last term sent = 0
	Twetbulbpsychrometric_C = WetBulbGlobeTempCalc::Twetbulb_Calc(Tair_K, RelativeHumidity_frac, AtmopshericPressure_hPa, WindSpeed_mps, Radiation_Shortwave_Wpm2, Radiation_Shortwave_direct_frac, Cos_ZenithAngle_Solar, 0);

	//Twetbulbglobe_C = 0.7 * Twetbulbnatural_C + 0.2 * Tglobe_C + 0.1 * Tair_C
	Twetbulbglobe_C = 0.7 * Twetbulbnatural_C + 0.2 * Tglobe_C + 0.1 * Tair_C;

	//Twetbulbglobe_K is Twetbulbglobe_C + 273.15
	folder->VarDict["Twetbulbglobe_K"] = Twetbulbglobe_C + 273.15;
}

// Tglobe_Calc function
double WetBulbGlobeTempCalc::Tglobe_Calc(double Tair_K, double RelativeHumidity_frac, double AtmopshericPressure_hPa, double WindSpeed_mps,
	double Radiation_Shortwave_Wpm2, double Radiation_Shortwave_direct_frac, double Cos_ZenithAngle_Solar) {
	
	double Tsfc_K = Tair_K;
	double Tglobe_prev_K = Tair_K;
	bool converged = false;
	int iter = 0;

	// Iterative loop
	while (!converged && iter < Iterations_Maximum) {
		iter++;
		double Tref_K = 0.5 * (Tglobe_prev_K + Tair_K);

		// Coefficient_ConvectiveHeatTransfer computed from function h_sphere_in_air
		// Using Eq 10 and following paragraph in Liljegren et al. (2008)
		double Coefficient_ConvectiveHeatTransfer = WetBulbGlobeTempCalc::h_sphere_in_air(Globe_Diameter_m, Tref_K, AtmopshericPressure_hPa, WindSpeed_mps);

		// Tglobe_new_K from Eq 17 in Liljegren et al. (2008)
		Tglobe_new_K = pow((0.5 * (WetBulbGlobeTempCalc::emis_atm(Tair_K, RelativeHumidity_frac) * pow(Tair_K, 4) + SurfaceGround_Emissivity_frac * pow(Tsfc_K, 4)) - Coefficient_ConvectiveHeatTransfer / (Sigma_Stefan_Boltzmann * Globe_Emissivity_frac) * (Tglobe_prev_K - Tair_K) + Radiation_Shortwave_Wpm2 / (2 * Sigma_Stefan_Boltzmann * Globe_Emissivity_frac) * (1 - Globe_Albedo_frac) * (Radiation_Shortwave_direct_frac * (1 / (2 * Cos_ZenithAngle_Solar) - 1) + 1 + SurfaceGround_Albedo_frac)), 0.25);

		// Check for convergence
		if (abs(Tglobe_new_K - Tglobe_prev_K) < Tolerance_Convergence) {
			converged = true;
		}

		// Update Tglobe_prev_K with relaxation weights (0.9 and 0.1)
		// Note: Liljegren et al. (2008) describe this as an iterative, relaxation solution
		Tglobe_prev_K = 0.9 * Tglobe_prev_K + 0.1 * Tglobe_new_K;
	}

	// Return final Tglobe value in Celsius if converged, otherwise return error value
	return converged ? Tglobe_new_K - 273.15 : -9999;
}

// Twetbulb_Calc function
double WetBulbGlobeTempCalc::Twetbulb_Calc(double Tair_K, double RelativeHumidity_frac, double AtmopshericPressure_hPa, double WindSpeed_mps,
	double Radiation_Shortwave_Wpm2, double Radiation_Shortwave_direct_frac, double Cos_ZenithAngle_Solar, int rad_flag) {
	double Tsfc_K = Tair_K;
	double ZenithAngle_rad = acos(Cos_ZenithAngle_Solar);
	double VaporPres_actual_mb = RelativeHumidity_frac * WetBulbGlobeTempCalc::esat(Tair_K, 0);
	double Tdew_K = WetBulbGlobeTempCalc::dew_point(VaporPres_actual_mb, 0);
	double Twetbulb_prev_K = Tdew_K;
	bool converged = false;
	int iter = 0;

	// Iterative loop to find convergence
	while (!converged && iter < Iterations_Maximum) {
		iter++;
		double Tref_K = 0.5 * (Twetbulb_prev_K + Tair_K);

		// DeltaFnet_HeatFluxToWick_K_p_m2 from Eq 12 in Liljegren et al. (2008)
		double DeltaFnet_HeatFluxToWick_K_p_m2 = (Sigma_Stefan_Boltzmann * Wick_Emissivity_frac *
			(0.5 * (WetBulbGlobeTempCalc::emis_atm(Tair_K, RelativeHumidity_frac) * pow(Tair_K, 4) +
				SurfaceGround_Emissivity_frac * pow(Tsfc_K, 4)) -
				pow(Twetbulb_prev_K, 4)) +
			(1 - Wick_Albedo_frac) * Radiation_Shortwave_Wpm2 *
			((1 - Radiation_Shortwave_direct_frac) *
				(1 + 0.25 * Wick_Diameter_m / Wick_Length_m) +
				Radiation_Shortwave_direct_frac *
				((tan(ZenithAngle_rad) / M_PI) +
					0.25 * Wick_Diameter_m / Wick_Length_m) +
				SurfaceGround_Albedo_frac));

		double VaporPressure_wick_hPa = WetBulbGlobeTempCalc::esat(Twetbulb_prev_K, 0);
		double Air_Density_kg_p_m3 = AtmopshericPressure_hPa * 100 / (287.05 * Tref_K); // Ideal gas law
		double Schmidt_Number_mass = WetBulbGlobeTempCalc::viscosity(Tref_K) / (Air_Density_kg_p_m3 * WetBulbGlobeTempCalc::diffusivity(Tref_K, AtmopshericPressure_hPa));
		double Coefficient_ConvectiveHeatTransfer = WetBulbGlobeTempCalc::h_cylinder_in_air(Wick_Diameter_m, Wick_Length_m, Tref_K, AtmopshericPressure_hPa, WindSpeed_mps);

		// Twetbulb_new_K from Eq 9 in Liljegren et al. (2008)
		Twetbulb_new_K = Tair_K -
			WetBulbGlobeTempCalc::evap(Tref_K) / Cp_Ratio_Air_to_Water *
			(VaporPressure_wick_hPa - VaporPres_actual_mb) /
			(AtmopshericPressure_hPa - VaporPressure_wick_hPa) *
			pow(Prandtl_Number_wind / Schmidt_Number_mass, 0.56) +
			DeltaFnet_HeatFluxToWick_K_p_m2 / Coefficient_ConvectiveHeatTransfer * rad_flag;

		// Check for convergence
		if (abs(Twetbulb_new_K - Twetbulb_prev_K) < Tolerance_Convergence) {
			converged = true;
		}

		// Update Twetbulb_prev_K with relaxation weights
		Twetbulb_prev_K = 0.9 * Twetbulb_prev_K + 0.1 * Twetbulb_new_K;
	}

	// Return Twetbulb_new_C in degrees Celsius if converged, or -9999 otherwise
	Twetbulb_new_C = Twetbulb_new_K - 273.15;
	return converged ? Twetbulb_new_C : -9999;
}

//cte 2024 Refactor to put into Solar_and_Meteorological_Calc
//StabilityClass_Estimator to estimate the stability class based on Flag_daytime, wind WindSpeed_mps, Radiation_Shortwave_Wpm2 irradiance, and temperature difference
//Note : Reference: EPA - 454 / 5 - 99 - 005, 2000, section 6.2.5
//Reference : U.S.Environmental Protection Agency(EPA) : Meteorological data processing. In Meteorological Monitoring Guidance for Regulatory Modeling Applications (EPA - 454 / R - 99 - 005).Washington, D.C. : EPA, 2000
int WetBulbGlobeTempCalc::StabilityClass_Estimator(bool Flag_daytime, double WindSpeed_mps, double Radiation_Shortwave_Wpm2, double Delta_Height_Tair_m) {
	//Stability class lookup table
	//The Pasquill - Gifford(P - G) stability classes range A - F or G, with 6 or 7 items, based on solar radiation, cloud cover, and wind speed
	//P - G stability classes are a set of categories used in meteorology to describe atmospheric stability, which affects pollutant dispersion.
	//A indicating very unstable conditions and F or G representing very stable conditions.
	//0 indicating very unstable conditions and 5 or 6 representing very stable conditions.

	//Note: Column j = 4 (5th from left) and j = 7 (last)are not used in calculations
	const vector<vector<int>> Pasquill_Gifford_StabilityClasses_array = {
		{1, 1, 2, 4, 0, 5, 6, 0},
		{1, 2, 3, 4, 0, 5, 6, 0},
		{2, 2, 3, 4, 0, 4, 4, 0},
		{3, 3, 4, 4, 0, 0, 0, 0},
		{3, 4, 4, 4, 0, 0, 0, 0},
		{0, 0, 0, 0, 0, 0, 0, 0}
	};

	int j = 0, i = 0;

	//Determine the column index j based on Flag_daytime conditions and Radiation_Shortwave_Wpm2 irradiance
	if (Flag_daytime) {
		if (Radiation_Shortwave_Wpm2 >= 925.0) {
			//j is column index
			j = 0;
		}
		else if (Radiation_Shortwave_Wpm2 >= 675.0) {
			j = 1;
		}
		else if (Radiation_Shortwave_Wpm2 >= 175.0) {
			j = 2;
		}
		else {
			j = 3;
		}
	}
	else {
		j = (Delta_Height_Tair_m >= 0.0) ? 6 : 5;
	}

	// Determine the row index `i` based on wind speed
	if (Flag_daytime) {
		if (WindSpeed_mps >= 6.0) {
			//i is row index 
			i = 4;
		}
		else if (WindSpeed_mps >= 5.0) {
			i = 3;
		}
		else if (WindSpeed_mps >= 3.0) {
			i = 2;
		}
		else if (WindSpeed_mps >= 2.0) {
			i = 1;
		}
		else {
			i = 0;
		}
	}
	else {
		if (WindSpeed_mps >= 2.5) {
			i = 2;
		}
		else if (WindSpeed_mps >= 2.0) {
			i = 1;
		}
		else {
			i = 0;
		}
	}

	//Return the stability class from the lookup table Pasquill_Gifford_StabilityClasses_array
	return Pasquill_Gifford_StabilityClasses_array[i][j];
}

//est_wind_speed to calculate the wind WindSpeed_mps at a height for Twbg calculations using stability class and Flag_urban/rural factors.
//Note: Reference: EPA-454/5-99-005, 2000, section 6.2.5
//Reference: U.S. Environmental Protection Agency (EPA): Meteorological data processing. In Meteorological Monitoring Guidance for Regulatory Modeling Applications (EPA-454/R-99-005). Washington, D.C.: EPA, 2000
double WetBulbGlobeTempCalc::est_wind_speed(double WindSpeed_mps, double Height_WindSpeed_m, int StabilityClass_Atmosphere_int, bool Flag_urban) {

	// Define the Exponent_WindSpeed_AdjustForHeight_Eqn arrays for Flag_urban and rural areas
	const vector<double> urban_exp = { 0.15, 0.15, 0.20, 0.25, 0.30, 0.30 };
	const vector<double> rural_exp = { 0.07, 0.07, 0.10, 0.15, 0.35, 0.55 };

	// Choose the appropriate Exponent_WindSpeed_AdjustForHeight_Eqn based on Flag_urban or rural setting
	double Exponent_WindSpeed_AdjustForHeight_Eqn = Flag_urban
		? urban_exp[StabilityClass_Atmosphere_int - 1]
		: rural_exp[StabilityClass_Atmosphere_int - 1];

	// Estimate the wind WindSpeed_mps adjusted for the reference height
	//WindSpeed_estimated_mps (m/s) computed in est_wind_speed to adjust from measured height to Twbg height
	//Note: Height_WindSpeed_m = 10 m for most NOAA weather stations, while Height_WindSpeed_Reference_m = 2 m, so lower wind speed
	double WindSpeed_estimated_mps = WindSpeed_mps * pow(Height_WindSpeed_Reference_m / Height_WindSpeed_m, Exponent_WindSpeed_AdjustForHeight_Eqn);

	// Ensure WindSpeed_mps does not fall below the minimum WindSpeed_mps
	WindSpeed_estimated_mps = max(WindSpeed_estimated_mps, WindSpeed_Minimum_mps);

	return WindSpeed_estimated_mps;
}

// h_sphere_in_air to calculate the convective heat transfer coefficient for flow around a sphere
double WetBulbGlobeTempCalc::h_sphere_in_air(double Sphere_Diameter_m, double Tair_K, double AtmopshericPressure_hPa, double WindSpeed_mps) {
	// Calculate air Air_Density_kg_p_m3 using the ideal gas law
	// Note: *100 converts hPa to Pa, Ratio_GasConstant_to_MolecularWeightAir_m2_p_s2_K * Tair_K ensures correct units
	double Air_Density_kg_p_m3 = AtmopshericPressure_hPa * 100 / (Ratio_GasConstant_to_MolecularWeightAir_m2_p_s2_K * Tair_K);

	// Calculate Reynolds number, ensuring speed is above the minimum using viscosity function
	double Reynolds_Number_wind = MAX(WindSpeed_mps, WindSpeed_Minimum_mps) * Air_Density_kg_p_m3 * Sphere_Diameter_m / WetBulbGlobeTempCalc::viscosity(Tair_K);

	// Calculate Nusselt number with Reynolds number and Prandtl number
	// Eq 16 from Liljegren et al (2008)
	double Nusselt_Number_wind = 2.0 + 0.6 * sqrt(Reynolds_Number_wind) * pow(Prandtl_Number_wind, 0.3333);

	// Calculate convective heat transfer coefficient in units of W/(m2·K)
	return Nusselt_Number_wind * WetBulbGlobeTempCalc::thermal_cond(Tair_K) / Sphere_Diameter_m;
}

// h_cylinder_in_air: convective heat transfer coefficient calculation
// Reference: Bird, Stewart, and Lightfoot (BSL), page 409.
// Reference: Bird, R.B., W.E. Stewart, and E.N. Lightfoot: Transport Phenomena. New York: John Wiley & Sons, 1960. pp. 649–652.
// Coefficient_ConvectiveHeatTransfer from Eq 10 and following paragraph in Liljegren et al. (2008)
double WetBulbGlobeTempCalc::h_cylinder_in_air(double Diameter_cylinder_m, double Length_cylinder_m, double Tair_K, double AtmopshericPressure_hPa, double WindSpeed_mps) {

	// Air_Density_kg_p_m3 = AtmopshericPressure_hPa * 100 / (Ratio_GasConstant_to_MolecularWeightAir_m2_p_s2_K * Tair_K)
	double Air_Density_kg_p_m3 = AtmopshericPressure_hPa * 100 / (Ratio_GasConstant_to_MolecularWeightAir_m2_p_s2_K * Tair_K);

	// Reynolds number estimator
	double Reynolds_Number_wind = MAX(WindSpeed_mps, WindSpeed_Minimum_mps) * Air_Density_kg_p_m3 * Diameter_cylinder_m / WetBulbGlobeTempCalc::viscosity(Tair_K);

	// Nusselt number estimator from Eq 10 and following paragraph in Liljegren et al. (2008)
	// Note: Reynolds_Number_wind^0.6 from Nusselt_Number_wind * Reynolds_Number_wind^(-1) * Prandtl_Number_wind^(a-1) = b * Reynolds_Number_wind^(-c),
	// where a = 0.56, b = 0.281, c = 0.4, hence combine Reynolds_Number_wind terms on RHS, = Reynolds_Number_wind^(1-c) = Reynolds_Number_wind^0.6
	// Note: Prandtl_Number_wind^0.4 from Nusselt_Number_wind * Reynolds_Number_wind^(-1) * Prandtl_Number_wind^(a-1) = b * Reynolds_Number_wind^(-c),
	// where a = 0.56, b = 0.281, c = 0.4, hence move Prandtl_Number_wind term to RHS, = Prandtl_Number_wind^(1-a) = Prandtl_Number_wind^0.44
	// Note: Consider refactor to change Prandtl_Number_wind^0.4 to Prandtl_Number_wind^0.44
	double Nusselt_Number_wind = 0.281 * pow(Reynolds_Number_wind, 0.6) * pow(Prandtl_Number_wind, 0.4);

	// Return Nusselt_Number_wind * thermal_cond(Tair_K) / Diameter_cylinder_m
	return Nusselt_Number_wind * WetBulbGlobeTempCalc::thermal_cond(Tair_K) / Diameter_cylinder_m;
}

// emis_atm to calculate the atmospheric emissivity
// Reference: Oke (2nd edition), page 373.
// Reference: Oke, T.R.: Boundary Layer Climates, 2nd Ed. New York: Methuen & Co., 1978. pp. 373.
double WetBulbGlobeTempCalc::emis_atm(double Tair_K, double RelativeHumidity_frac) {
	// Calculate vapor pressure (hPa) using the relative humidity and saturation vapor pressure
	double VaporPres_hPa = RelativeHumidity_frac * WetBulbGlobeTempCalc::esat(Tair_K, 0);

	// Calculate atmospheric emissivity (fraction) using the formula from Oke (1978)
	double Emissivity_Atmosphere_frac = 0.575 * pow(VaporPres_hPa, 0.143);

	// Return the atmospheric emissivity
	return Emissivity_Atmosphere_frac;
}

// esat to calculate saturation vapor pressure over water (phase=0) or ice (phase=1).
// Buck's (1981) approximation (eqn 3) of Wexler's (1976) formulae
// Buck, A.L.: New equations for computing vapor pressure and enhancement factor. J. Appl. Meteorol. 20:1527–1532 (1981).
double WetBulbGlobeTempCalc::esat(double Tair_K, int phase) {
	double VaporPres_saturated_mb;

	if (phase == 0) {
		// Saturation vapor pressure over water
		VaporPres_saturated_mb = 6.1121 * exp(17.502 * (Tair_K - 273.15) / (Tair_K - 32.18));
		// Correction for moist air, if pressure is available (turned off by Liljegren)
		// VaporPres_saturated_mb = (1.0007 + (3.46E-6 * pres)) * VaporPres_saturated_mb;

	}
	else {
		// Saturation vapor pressure over ice
		VaporPres_saturated_mb = 6.1115 * exp(22.452 * (Tair_K - 273.15) / (Tair_K - 32.18));
		// Correction for moist air, if pressure is available (turned off by Liljegren)
		// VaporPres_saturated_mb = (1.0003 + (4.18E-6 * pres)) * VaporPres_saturated_mb;
	}

	// Correction for moist air if pressure unavailable for pressure > 800 mb, used by Liljegren
	VaporPres_saturated_mb = 1.004 * VaporPres_saturated_mb;

	// Correction for moist air if pressure unavailable for pressure < 200 mb (turned off by Liljegren)
	// VaporPres_saturated_mb = 1.0034 * VaporPres_saturated_mb;

	return VaporPres_saturated_mb;
}


// thermal_cond to calculate the thermal conductivity of air, W/(m K)
// Reference: Bird, Stewart, and Lightfoot (BSL), page 257.
// Reference: Bird, R.B., W.E. Stewart, and E.N. Lightfoot: Transport Phenomena. New York: John Wiley & Sons, 1960. pp. 649–652.
double WetBulbGlobeTempCalc::thermal_cond(double Tair_K) {
	// Thermal conductivity of air (W/(m·K))
	double Thermal_Conductivity_Air_W_p_m_K = (HeatCapacity_Air_J_p_kg_p_K +
		1.25 * Ratio_GasConstant_to_MolecularWeightAir_m2_p_s2_K) * WetBulbGlobeTempCalc::viscosity(Tair_K);
	return Thermal_Conductivity_Air_W_p_m_K;
}

// evap to calculate the heat of evaporation, J/(kg K), for temperature in the range 283-313 K.
// Reference: Van Wylen and Sonntag, Table A.1.1
// Reference: Van Wylen and Richard E. Sonntag, Fundamentals of Classical Thermodynamics, English/SI Version, 3rd ed. (New York: John Wiley & Sons, 1986).
double WetBulbGlobeTempCalc::evap(double Tair_K) {
	// Calculate latent heat of evaporation (J/(kg K))
	double LatentHeat_Evaporation_J_p_kg_K = (313.15 - Tair_K) / 30.0 * -71100.0 + 2.4073E6;
	return LatentHeat_Evaporation_J_p_kg_K;
}

// viscosity to calculate viscosity
// Reference: Bird, Stewart, and Lightfoot (BSL), page 23
// Reference: Bird, R.B., W.E. Stewart, and E.N. Lightfoot: Transport Phenomena. New York: John Wiley & Sons, 1960. pp. 649–652.
double WetBulbGlobeTempCalc::viscosity(double Tair_K) {
	double Viscosity_Air_Pa_s = 2.6693E-6 * sqrt(Molecular_Weight_Air_kg_p_kmol * Tair_K) /
		(3.617 * 3.617 * (((Tair_K / 97.0 - 2.9) / 0.4) * -0.034 + 1.048));
	return Viscosity_Air_Pa_s;
}

// diffusivity to calculate the diffusivity of water vapor in air, m²/s
// Reference: Bird, Stewart, and Lightfoot (BSL), page 505.
// Reference: Bird, R.B., W.E. Stewart, and E.N. Lightfoot: Transport Phenomena. New York: John Wiley & Sons, 1960. pp. 649–652.
double WetBulbGlobeTempCalc::diffusivity(double Tair_K, double AtmopshericPressure_hPa) {
	// Critical properties of air and water
	const double Pcrit_air = 36.4, Pcrit_h2o = 218.0; // Critical pressures (bar)
	const double Tcrit_air = 132.0, Tcrit_h2o = 647.3; // Critical temperatures (K)
	const double Coeff_a = 3.640E-4, Power_b = 2.334; // Coefficients for diffusivity calculation

	// Intermediate calculations
	double Pcrit_air_water_1p3 = pow(Pcrit_air * Pcrit_h2o, 1.0 / 3.0);
	double Tcrit_air_water_5p12 = pow(Tcrit_air * Tcrit_h2o, 5.0 / 12.0);
	double Tcrit_air_water_1p2 = sqrt(Tcrit_air * Tcrit_h2o);
	double MolecularWeightMix_air_water_1p2 = sqrt(1.0 / Molecular_Weight_Air_kg_p_kmol +
		1.0 / Molecular_Weight_Water_kg_p_kmol);
	double Ratio_AtmPressure_Actual_to_SeaLevelConstant = AtmopshericPressure_hPa / 1013.25;

	// Diffusivity of water vapor in air (m²/s)
	double Diffusivity_WaterVapor_m2_p_s = (Coeff_a *
		pow(Tair_K / Tcrit_air_water_1p2, Power_b) *
		Pcrit_air_water_1p3 *
		Tcrit_air_water_5p12 *
		MolecularWeightMix_air_water_1p2 /
		Ratio_AtmPressure_Actual_to_SeaLevelConstant * 1E-4);
	return Diffusivity_WaterVapor_m2_p_s;
}


//cte 2024 Potentiall replace with input->Tdew
// dew_point to calculate the dew point temperature over water (phase=0) or ice (phase=1).
// Note: derived from Buck's (1981) approximation (eqn 3) of Wexler's (1976) formulae, includes 1.004 factor for moist air
double WetBulbGlobeTempCalc::dew_point(double VaporPres_hPa, int phase) {
	double Tdew_K;
	if (phase == 0) {
		// Dew point temperature over water
		Tdew_K = 273.15 + 240.97 * log(VaporPres_hPa / (6.1121 * 1.004)) /
			(17.502 - log(VaporPres_hPa / (6.1121 * 1.004)));
	}
	else {
		// Dew point temperature over ice
		Tdew_K = 273.15 + 272.55 * log(VaporPres_hPa / (6.1121 * 1.004)) /
			(22.452 - log(VaporPres_hPa / (6.1121 * 1.004)));
	}
	return Tdew_K;
}

