﻿#ifndef WetBulbGlobeTempCalc_H
#define WetBulbGlobeTempCalc_H
/*
* HydroPlus is source code and models developed and managed by Theodore Endreny and his students at SUNY ESF, te@esf.edu
* The US Forest Service and Davey Tree have provided strategic software development support through funding and personnel
* Attribution for use of software and code is requested. Please see Executive.cpp for documentation.
*/
#include <iostream>
#include <vector>
#include <string>
#include <iostream>
//_USE_MATH_DEFINES gives access to Pi as M_PI, and must precede #include <cmath> or <math.h>
#define _USE_MATH_DEFINES
#include <cmath>
#include <math.h>
#include "..\Inputs\Inputs.h"
#include "..\DataFolder.h"
#include "..\Inputs\SolarCalculator.h"

//Ratio_Radian_to_Degree is 0.017453292519943295
#define Ratio_Radian_to_Degree (M_PI / 180.0) 
//Radiation_Shortwave_SolarConstant_Wpm2 is 1367 W/m2
#define Radiation_Shortwave_SolarConstant_Wpm2 1367.0
//Stefan-Boltzmann constant is 5.670374419E-8 W/(m^2*K^4)
#define Sigma_Stefan_Boltzmann 5.670374419E-8
//Specific heat capacity of air at constant pressure (J/kg/K) 1006.0 at ~290 to 315 K, 
//Note: 1003.5 used in Liljegren et al. (2008), but this does not seem to make physical sense
#define HeatCapacity_Air_J_p_kg_p_K 1006.0
//Molecular_Weight_Air_kg_p_kmol is 28.97 kg/kilomole or g/mole of air
#define Molecular_Weight_Air_kg_p_kmol 28.97 
//Molecular_Weight_Water_kg_p_kmol is 18.015 kg/kilomole or g/mole of water
#define Molecular_Weight_Water_kg_p_kmol 18.015 
//R_Universal_Gas_Constant_J_p_kmol_K is universal gas constant 8314.34 J/(kilomole K) 
#define R_Universal_Gas_Constant_J_p_kmol_K 8314.34 
//Cos_ZenithAngle_Minimum minimum cos zenith angle, cos(89.5 deg) = cos(1.562 rad) Eq 13 and Eq 14 Liljegren et al. (2008)
#define Cos_ZenithAngle_Minimum 0.00873 
//Ratio_RadiationShortwaveActual_to_RadiationShortwaveMax_Minimum_frac is 0.85 from Liljegren et al. (2008) github code
#define Ratio_RadiationShortwaveActual_to_RadiationShortwaveMax_Minimum_frac 0.85
//Height_WindSpeed_Reference_m (m) set to 2.0 from Liljegren et al. (2008) github code; height experienced for Twbg
#define Height_WindSpeed_Reference_m 2.0
//WindSpeed_Minimum_mps (m/s) is 0.13 set by Liljegren et al. (2008) github code
#define WindSpeed_Minimum_mps 0.13
//Tolerance_Convergence is 0.02 set by Liljegren et al. (2008) github code
#define Tolerance_Convergence 0.02
//Iterations_Maximum is 500 set by Liljegren et al. (2008) github code
#define Iterations_Maximum 500

//MAX function for finding maximum of x_var and y_var using conditional ternary operator (?:)
//Note: Ternary operator if x_var>=y_var is true, then x_var is taken, otherwise y_var taken.
#define MAX(x_var,y_var) (((x_var)>=(y_var)) ? (x_var) : (y_var))        
#define MIN(x_var,y_var) (((x_var)<=(y_var)) ? (x_var) : (y_var))        

//using namespace std allows for use of standard namespace without the preface std::
using namespace std;       

//class WetBulbGlobeTempCalc
class WetBulbGlobeTempCalc
{
private:

	//SolarCalculator class pointer solarCalculator accesses function
	//Note: solarCalculator is owned by SimulationCoordinator so is not deleted with Destructor
	SolarCalculator* solarCalculator;

	//vector<vector<pair<double, double>>> Latitude_Longitude_vectorPair_dd vector created to store latitude and longitude as decimal degree
	vector<vector<pair<double, double>>> Latitude_Longitude_vectorPair_dd;
	//Flag_LatitudeLongitude_Computed flag initialized to false, to track if Latitude_Longitude_vectorPair_dd computed
	bool Flag_LatitudeLongitude_Computed = false; 
	double Cp_Ratio_Air_to_Water;
	double Ratio_GasConstant_to_MolecularWeightAir_m2_p_s2_K;
	double Prandtl_Number_wind;
	double Wick_Emissivity_frac;
	double Wick_Albedo_frac;
	double Wick_Diameter_m;
	double Wick_Length_m;
	double Globe_Emissivity_frac;
	double Globe_Albedo_frac;
	double Globe_Diameter_m;
	double SurfaceGround_Emissivity_frac;
	double SurfaceGround_Albedo_frac;

	int Year_YYYY;
	int Month_MM;
	int Day_DD;
	int Hour_HH;
	int Minute_Mn;
	int GMT_Offset_int;
	int Time_averaging_MeteorologicalInputs_minutes;
	double Latitude_NH_pos_dd;
	double Longitude_WH_pos_dd;
	double AtmopshericPressure_hPa;
	double Tair_C;
	double Tdew_C;
	double RelativeHumidity_percent;
	double WindSpeed_mps;
	double Height_WindSpeed_m;
	double Delta_Height_Tair_m;
	int Flag_urban;
	int Flag_daytime;

	double Radiation_Shortwave_Wpm2;
	double Radiation_Shortwave_TopOfAtm_Wpm2;
	double Radiation_Shortwave_adjusted_Wpm2;
	double Ratio_RadiationShortwaveActual_to_RadiationShortwaveMax_frac;
	double Radiation_Shortwave_direct_frac;

	int Hour_GreenwichMeanTime_int;
	double Day_and_Hour_decimal;
	double Cos_ZenithAngle_Solar;
	double days_1900;
	double Solar_RightAscension_Apparent_hr;
	double Solar_Declination_Apparent_rad; 
	double Solar_AltitudeAngle_rad; 
	double Solar_AltitudeAngle_deg;
	double Solar_AzimuthAngle_rad;
	double Distance_Earth_to_Sun_astrologicalunits_frac;
	double Solar_AltitudeAngle_RefractionCorrection_rad;
	double Solar_AltitudeAngle_RefractionCorrection_deg;
	int StabilityClass_Atmosphere_int;
	double WindSpeed_estimated_mps;

	double Solar_HourAngle_Local_rad;
	double Tangent_Solar_AltitudeAngle;

	double Cos_Solar_AltitudeAngle;
	double Cos_Solar_AzimuthAngle;
	double Sine_Solar_AzimuthAngle;

	double Tglobe_new_K;
	double Twetbulb_new_K;
	double Twetbulb_new_C;

	double Twetbulbglobe_C;
	double Tglobe_C;
	double Twetbulbnatural_C; 
	double Twetbulbpsychrometric_C;


public:
	//WetBulbGlobeTempCalc(Inputs* input, SolarCalculator* solarCalc) : solarCalculator(solarCalc) {} constructor must match .h class name
	//Note: Variables defined in the constructor are shared across all void functions, otherwise limited to void function
	//cte 2024 WetBulbGlobeTempCalc(SolarCalculator* solarCalc) : solarCalculator(solarCalc) {}
	WetBulbGlobeTempCalc(Inputs* input, SolarCalculator* solarCalc);
	//~WetBulbGlobeTempCalc() (with {}) destructor that ensures resources used by the object can be released 
	~WetBulbGlobeTempCalc() {};

	//WetBulbGlobeTemperature_Inputs function will manage call to calculate the wet bulb globe temperature
	void WetBulbGlobeTemperature_Inputs(Inputs* input, DataFolder* folder, DataFolder* reffolder, int MapPixel_ID, int timeStep, double Radiation_Shortwave_Direct_W_p_m2, double Radiation_Shortwave_Diffuse_W_p_m2, double Cos_ZenithAngle_Solar);
	//WetBulbGlobeTemperature_Calculate function will compute the wet bulb globe temperature
	void WetBulbGlobeTemperature_Calculate(Inputs* input, DataFolder* folder, DataFolder* reffolder, int MapPixel_ID, int timeStep, double Radiation_Shortwave_Direct_W_p_m2, double Radiation_Shortwave_Diffuse_W_p_m2, double Cos_ZenithAngle_Solar);
	double Twetbulb_Calc(double Tair_K, double RelativeHumidity_frac, double AtmopshericPressure_hPa, double WindSpeed_mps,
		double Radiation_Shortwave_Wpm2, double Radiation_Shortwave_direct_frac, double Cos_ZenithAngle, int rad_flag);
	//calc_solar_parameters
	void calc_solar_parameters(Inputs* input, int timeStep, double Radiation_Shortwave_Direct_W_p_m2, double Radiation_Shortwave_Diffuse_W_p_m2);
	void solarposition(Inputs* input, int timeStep,int days_1900, double Latitude_NH_pos_dd, double Longitude_WH_pos_dd);
	double Tglobe_Calc(double Tair_K, double RelativeHumidity_frac, double AtmopshericPressure_hPa, double WindSpeed_mps,
		double Radiation_Shortwave_Wpm2, double Radiation_Shortwave_direct_frac, double Cos_ZenithAngle);
	double h_sphere_in_air(double Sphere_Diameter_m, double Tair_K, double AtmopshericPressure_hPa, double WindSpeed_mps);
	double h_cylinder_in_air(double Diameter_cylinder_m, double Length_cylinder_m, double Tair_K, double AtmopshericPressure_hPa, double WindSpeed_mps);
	int StabilityClass_Estimator(bool Flag_daytime, double WindSpeed_mps, double Radiation_Shortwave_Wpm2, double Delta_Height_Tair_m);
	double est_wind_speed(double WindSpeed_mps, double Height_WindSpeed_m, int StabilityClass_Atmosphere_int, bool Flag_urban);
	double diffusivity(double Tair_K, double AtmopshericPressure_hPa);
	double dew_point(double VaporPres_hPa, int phase);
	double thermal_cond(double Tair_K);
	double esat(double Tair_K, int phase);
	double viscosity(double Tair_K);
	double emis_atm(double Tair_K, double RelativeHumidity_frac);
	double evap(double Tair_K);
};

#endif