#include "ShortVegEvaporationCalc.h"

void ShortVegEvaporationCalc::calculateShortVegEvaporationTemperature(Inputs* input, DataFolder* folder, int timeStep)
{
	//If timeStep equals 1, then call statistical calculation based on weather input data, given the spatial calculation has not yet been called
	if (timeStep == 0) {
		//Call ShortVegEvaporationCalc::calculateShortVegEvaporationStatistical to estimate evaporation
		ShortVegEvaporationCalc::calculateShortVegEvaporationStatistical(input, folder, timeStep);
		//return is C++ function to leave the .cpp
		return;
	}

	//Initialize LAI_BAI_SVeg_m2_p_m2 
	double LAI_BAI_SVeg_m2_p_m2 = input->LAI_BAI_SVeg_m2_p_m2[timeStep];
	//Note: If LAI_BAI_SVeg_m2_p_m2  < 1, then set to 1 so that its position as denominator does not enlarge SVegLEE_W_p_m2 when / LAI_BAI_SVeg_m2_p_m2 
	if (LAI_BAI_SVeg_m2_p_m2 < 1) { LAI_BAI_SVeg_m2_p_m2 = 1; }

	//Temperature_Air_C (C) is converted from folder Tair_K (K) 
	double Temperature_Air_C = folder->VarDict["Tair_K"] - 273.15;

	//DensityWater_kg_p_m3 is density of water (kg/m^3) from Equation in Figure 11.1.1 McCutcheon (1993)
	double DensityWater_kg_p_m3 = 1000 * (1 - ((Temperature_Air_C + 288.9414) / (508929.2 * (Temperature_Air_C + 68.12963))) *
		pow((Temperature_Air_C - 3.9863), 2));

	//LatentHeatOfVaporizaton_MJ_p_kg (MJ/kg) from Eq 13.36 Chin (2021) is latent heat of vaporization
	double LatentHeatOfVaporizaton_MJ_p_kg = 2.501 - 0.002361 * Temperature_Air_C;
	//LatentHeatOfVaporizaton_J_p_kg (J/kg) is converted from MJ to K
	double LatentHeatOfVaporizaton_J_p_kg = LatentHeatOfVaporizaton_MJ_p_kg * Ratio_J_to_MJ;

	//EvaporationSublimation_SVegCanopy_m_p_s (m/s) initialized
	double EvaporationSublimation_SVegCanopy_m_p_s = 0;

	//If Storage_PerviousPondedWater_m > Constant_1E_negative6 then presume SVegLEE_W_p_m2 needs to be reduced due to some used to evaporate depression storage
	if (folder->VarDict["Storage_PerviousPondedWater_m"] > Constant_1E_negative6) {
		//EvaporationSublimation_SVegCanopy_m_p_s (m/s) derived from SVegLEE_W_p_m2 (W/m2) divided by (LatentHeatOfVaporizaton_J_p_kg * DensityWater_kg_p_m3) ...
		//Note: If depression storage received SVegLEE_W_p_m2 / LAI_BAI_SVeg_m2_p_m2, then SVegLEE_W_p_m2 - (SVegLEE_W_p_m2 / LAI_BAI_SVeg_m2_p_m2) remains
		//Note: RHS numerator is W/m2 = J/s/m2, and RHS denominator is (J/kg) * (kg/m3) = J/m3, therefore RHS quotient is J/s/m2 / J/m3 = m/s.
		EvaporationSublimation_SVegCanopy_m_p_s = (folder->VarDict["SVegLEE_W_p_m2"] - (folder->VarDict["SVegLEE_W_p_m2"] / LAI_BAI_SVeg_m2_p_m2)) / (LatentHeatOfVaporizaton_J_p_kg * DensityWater_kg_p_m3);
	}
	//Else Storage_PerviousPondedWater_m <= Constant_1E_negative6 and use full amount of SVegLEE_W_p_m2 
	else {
		//EvaporationSublimation_SVegCanopy_m_p_s (m/s) derived from SVegLEE_W_p_m2 (W/m2) divided by (LatentHeatOfVaporizaton_J_p_kg * DensityWater_kg_p_m3) ...
		//Note: RHS numerator is W/m2 = J/s/m2, and RHS denominator is (J/kg) * (kg/m3) = J/m3, therefore RHS quotient is J/s/m2 / J/m3 = m/s.
		EvaporationSublimation_SVegCanopy_m_p_s = folder->VarDict["SVegLEE_W_p_m2"] / (LatentHeatOfVaporizaton_J_p_kg * DensityWater_kg_p_m3);
	}
	//Control for negative values
	if (EvaporationSublimation_SVegCanopy_m_p_s <= 0) { EvaporationSublimation_SVegCanopy_m_p_s = 0; }

	//EvaporationSublimation_SVegCanopy_m (m) is product of EvaporationSublimation_SVegCanopy_m_p_s (m/s) and SimulationTimeStep_sec[timeStep]
	folder->VarDict["EvaporationSublimation_SVegCanopy_m"] = EvaporationSublimation_SVegCanopy_m_p_s * input->SimulationTimeStep_sec[timeStep];

	//If EvaporationSublimation_SVegCanopy_m (m) is greater than Storage_Rain_SVegCanopy_m, then set to Storage_Rain_SVegCanopy_m
	if (folder->VarDict["EvaporationSublimation_SVegCanopy_m"] > folder->VarDict["Storage_Rain_SVegCanopy_m"]) {
		folder->VarDict["EvaporationSublimation_SVegCanopy_m"] = folder->VarDict["Storage_Rain_SVegCanopy_m"];
	}

	//Storage_Rain_SVegCanopy_m (m) is updated, defined as Storage_Rain_SVegCanopy_m - EvaporationSublimation_SVegCanopy_m
	folder->VarDict["Storage_Rain_SVegCanopy_m"] = folder->VarDict["Storage_Rain_SVegCanopy_m"] - folder->VarDict["EvaporationSublimation_SVegCanopy_m"];
}

void ShortVegEvaporationCalc::calculateShortVegEvaporationStatistical(Inputs* input, DataFolder* folder, int timeStep)
{
	double SublimationSnow_SVegCanopy_m = 0.0;        // Initialize SublimationSnow_SVegCanopy_m (amount of evaporated snow) = 0 
	double EvaporationRain_SVegCanopy_m = 0.0;           // Initialize EvaporationRain_SVegCanopy_m (amount of evaporated water) = 0
	double EvaporationSublimation_SVegCanopy_m = 0.0;                // Initialize ShortVegEP (potential evaporation) = 0

	// SublimationSnow_SVegCanopy_m (the amount of evaporated snow) = PEShortVegSnow (the potential evaporation of snow) 
	SublimationSnow_SVegCanopy_m = input->PotentialSublimation_SnowOnTree_m[timeStep];

	//routine to evaporate liquid water after evaporating any snow water equivalent
	//enter conditional if potential evaporation for snow water exceeds stored snow water
	if (SublimationSnow_SVegCanopy_m > folder->VarDict["Storage_Snow_SVegCanopy_m"]) {

		//calcluate fraction of potential evaporation utilzed by snow water evaporation
		double Sublimation_to_EvaporationSublimation_frac = folder->VarDict["Storage_Snow_SVegCanopy_m"] / SublimationSnow_SVegCanopy_m;

		//evaporate all of the snow water, which may be 0 when ShortVegStoredSWE is 0
		SublimationSnow_SVegCanopy_m = folder->VarDict["Storage_Snow_SVegCanopy_m"];

		//convert Storage_TreeCanopy_Max_m to value greater than 0 to avoid division by zero
		if (folder->VarDict["Storage_SVegCanopy_Max_m"] <= 0) {
			folder->VarDict["Storage_SVegCanopy_Max_m"] = 1.0e-10;
		}

		//evaporate fraction of water stored in shortVeg
		//Note: Regulation of evaporation by 2/3 function is used in Liang et al. (1994) VIC paper
		EvaporationRain_SVegCanopy_m = (1 - Sublimation_to_EvaporationSublimation_frac) * input->PotentialEvaporation_WaterOnTree_m[timeStep] *
			pow(folder->VarDict["Storage_Rain_SVegCanopy_m"] / folder->VarDict["Storage_SVegCanopy_Max_m"], (2.0 / 3.0));

		if (EvaporationRain_SVegCanopy_m > folder->VarDict["Storage_Rain_SVegCanopy_m"]) {
			EvaporationRain_SVegCanopy_m = folder->VarDict["Storage_Rain_SVegCanopy_m"];
		}
		//reduce the shortVeg stored water by the tree evaporation of water
		folder->VarDict["Storage_Rain_SVegCanopy_m"] = folder->VarDict["Storage_Rain_SVegCanopy_m"] - EvaporationRain_SVegCanopy_m;
	}
	//reduce the shortVeg stored snow by the shortVeg evaporation of snow
	//if Storage_Snow_SVegCanopy_m was 0, then SublimationSnow_SVegCanopy_m was set to 0 in the above condition
	folder->VarDict["Storage_Snow_SVegCanopy_m"] = folder->VarDict["Storage_Snow_SVegCanopy_m"] - SublimationSnow_SVegCanopy_m;

	EvaporationSublimation_SVegCanopy_m = SublimationSnow_SVegCanopy_m + EvaporationRain_SVegCanopy_m;

	folder->VarDict["EvaporationSublimation_SVegCanopy_m"] = EvaporationSublimation_SVegCanopy_m;
	folder->VarDict["EvaporationRain_SVegCanopy_m"] = EvaporationRain_SVegCanopy_m;
	folder->VarDict["SublimationSnow_SVegCanopy_m"] = SublimationSnow_SVegCanopy_m;

}




