﻿#ifndef Inputs_H
#define Inputs_H
/*
* HydroPlus is source code and models developed and managed by Theodore Endreny and his students at SUNY ESF, te@esf.edu
* The US Forest Service and Davey Tree have provided strategic software development support through funding and personnel
* Attribution for use of software and code is requested. Please see Executive.cpp for documentation.
*/
#include "Inputs.h"
#include <string>
#include <map>
//_USE_MATH_DEFINES gives access to Pi as M_PI, and must precede #include <cmath> or <math.h>
#define _USE_MATH_DEFINES
#include <cmath>
#include <math.h>
#include <fstream>
#include <sstream>
#include <unordered_set>
#include <numeric>
#include <iomanip>
#include <unordered_map>
#include <functional>
#include <stdexcept>
#include <utility>
#include "..\XMLReader\XML_Document.h"
#include "WeatherPreprocessor.h"

//Forward declaration of class TerrainProcessor
class TerrainProcessor;

//Earth_Radius_m 6378137.0 m
#define Earth_Radius_m 6378137.0
//Earth_Eccentricity is 0.081819191; related to Flattening_WGS84_Ellipsoid = 1 / 298.257223563
#define Earth_Eccentricity 0.081819191

//ratio of degrees to radians
#define Ratio_Degree_to_Radian 180 / M_PI
//ratio of radians to degrees
#define Ratio_Radian_to_Degree M_PI / 180
//ratio of hour to seconds
#define Ratio_Hour_to_Second 1.0/3600.0
//ratio of seconds to hour
#define Ratio_Second_to_Hour 3600
//ratio of hour to day
#define Ratio_Hour_to_Day 24
//ratio of metric tonne to kg 
#define Ratio_Tonne_to_kg 1000
//Conversion between fraction and percent
#define Ratio_Decimal_to_Percent 0.01
//Define constant effective zero value for sums of water
#define Constant_1E_negative10 1.E-10     
//Define constant effective zero value for fractions of area
#define Constant_1E_negative5 1.E-5     
//ratio of kg to mg
#define Ratio_kg_to_mg 1000000
//ratio of mm*ha to L
#define Ratio_mmHa_to_L 10000

//MapMetadata Struct initialized
struct MapMetadata {
	//cte 2024 Albers
	double lowerLeftX_m;
	double lowerLeftY_m;
	double cellSize_m;
	int ncols;
	int nrows;
};

// ProjectionParams_Albers struct contains WKID 102008 North America Albers Equal Area Conic projection parameters
// Note: Source: Snyder, J.P., (1987). Map projections: A working manual. USGS Professional Paper 1395, https://pubs.usgs.gov/publication/pp1395
// Note: WKID 102008 is North America Albers Equal Area Conic projection, with predefined parameters:
// Note: ... Latitude of Origin (rad): 23∘ N = 23×(π/180) = 0.4014 radians
// Note: ... Central Meridian (rad): −96∘ W = −96×(π/180) = −1.6755 radians
// Note: ... Standard Parallel 1 (rad​): 29.5∘ N = 29.5×(π/180) = 0.5149 radians
// Note: ... Standard Parallel 2 (rad​): 45.5∘ N = 45.5×(π/180) = 0.7941 radians
struct ProjectionParams_Albers {
	// phi0 = Latitude of origin (radians)
	double phi0 = 23.0 * M_PI / 180.0;
	// lambda0 = Central meridian (radians) 
	double lambda0 = -96.0 * M_PI / 180.0;
	// phi1 = Standard parallel 1 (radians)
	double phi1 = 29.5 * M_PI / 180.0;
	// phi2 = Standard parallel 2 (radians)
	double phi2 = 45.5 * M_PI / 180.0;
};

// ProjectionParams_Lambert struct contains WKID 3034 European Lambert Conformal Conic projection parameters
// Source: Snyder, J.P., (1987). Map projections: A working manual. USGS Professional Paper 1395, https://pubs.usgs.gov/publication/pp1395
// Note: WKID 3034 is a Lambert Conformal Conic projection, commonly used for pan-European mapping, with predefined parameters:
// Note: ... Latitude of Origin (rad): 52∘ N = 52×(π/180) = 0.9076 radians
// Note: ... Central Meridian (rad): 10∘ E = 10×(π/180) = 0.1745 radians
// Note: ... Standard Parallel 1 (rad): 35∘ N = 35×(π/180) = 0.6109 radians
// Note: ... Standard Parallel 2 (rad): 65∘ N = 65×(π/180) = 1.1345 radians
// Note: ... False Easting (meters): 4000000.0
// Note: ... False Northing (meters): 2800000.0
struct ProjectionParams_Lambert {
	// phi0 = Latitude of origin (radians)
	double phi0 = 52.0 * M_PI / 180.0;
	// lambda0 = Central meridian (radians)
	double lambda0 = 10.0 * M_PI / 180.0;
	// phi1 = Standard parallel 1 (radians)
	double phi1 = 35.0 * M_PI / 180.0;
	// phi2 = Standard parallel 2 (radians)
	double phi2 = 65.0 * M_PI / 180.0;
	// False easting and northing (meters)
	double falseEasting = 4000000.0;
	double falseNorthing = 2800000.0;
};

// ProjectionParams_UTM struct contains WKID 32600 and 32700 UTM northern and southern hemisphere projection parameters
// Note: Source: Snyder, J.P., (1987). Map projections: A working manual. USGS Professional Paper 1395, https://pubs.usgs.gov/publication/pp1395
// Note: WKID 102008 is North America Albers Equal Area Conic projection, with predefined parameters:
struct ProjectionParams_UTM {
	// UTM zone number (1–60)
	int zone;
	// Central meridian of the UTM zone (in radians)
	double centralMeridian;
	// False easting (usually 500,000 meters)
	double falseEasting;
	// False northing (0 meters for north, 10,000,000 meters for south)
	double falseNorthing;
	// Scale factor at the central meridian (0.9996)
	double scaleFactor;
};

//EnsureTrailingBackslash function to ensure it has no parenthesis and ends with backslash
string EnsureTrailingBackslash(const string& directoryPath);
bool EnsureFileExists(const string& filePath);

class Inputs
{
public:

	Inputs(string &Directory_Input_CLArg);
	void ProcessTopographyData();
	static void InitializeLAIVariables();
	static void CalcTreeLAI(int JulianDay);
	static void CalcShortVegLAI(int JulianDay);
	//Date_YYYYMMDD_to_JulianDay function calculates the Julian day based on inputs of year, month, and days
	int Date_YYYYMMDD_to_JulianDay(int Date_YYYYMMDD);
	//Date_YYYYMMDD_to_JulianDate_to_JulianDay function calculates the Julian day based on subtracting two Julian dates
	int Date_YYYYMMDD_to_JulianDate_to_JulianDay(int Date_YYYYMMDD);
	//ExtractGregorianDateParts function extracts YYYY MM DD and if available HH into an array
	static void ExtractGregorianDateParts(int Date_YYYYMMDDHH, int DateParts_Gregorian[]);
	//get reversed flow direction 
	static double D8_flowdir_reverse(double flowdir);
	//get manning n and Hydraulic radius
	static double getManningn(double nlcd_categories);
	static double getHydraulicradius(double nlcd_categories);
	static double get_P_release_mean(double nlcd_categories);
	//convert the code of fdr from esri to whitebox which is using in HydroPlus
	static double flowdircode_esri_to_whitebox(double flowdir);
	static double flowdircode_whitebox_to_esri(double flowdir);
	//calculates the mean value of a vector while excluding a specified NODATA value
	static double calculateMean(const vector<double>& data);
	//Inputs::ProcessTemperatureExecutionParams function reads HydroPlusConfig.xml inputs for Spatial Buffer with GI
	void ProcessSpatialBufferGIExecutionParams();
	//update bulk area for each GI device
	double BufferGIContributingAreaCalc(int gi_index);
	double BufferGIAreaCalc(int gi_index);
	bool MapAllignment_check(const vector<double>& map1, const vector<double>& map2, const string& map1Name, const string& map2Name);
	//temp input/output processing
	void ProcessTemperatureExecutionParams();
	int get_Q_count() { return (totalDischargeCount - 1); }
	int getcols() { return nDemDataCols; }
	int getrows() { return nDemDataRows; }
	double getNODATA() { return NODATA; }
	double getxllcorner() { return nDemDataxllcorner; }
	double getyllcorner() { return nDemDatayllcorner; }

	void setcols(int x) { nDemDataCols = x; }
	void setrows(int x) { nDemDataRows = x; }
	void setxllcorner(double x) { nDemDataxllcorner = x; }
	void setyllcorner(double x) { nDemDatayllcorner = x; }

	int getCount() { return (totalCount); }

	//vector<vector<pair<double, double>>> Latitude_Longitude_vectorPair_dd vector created to store latitude and longitude as decimal degree
	vector<vector<pair<double, double>>> Latitude_Longitude_vectorPair_dd;
	//Flag_LatitudeLongitude_Computed flag initialized to false, to track if Latitude_Longitude_vectorPair_dd computed
	bool Flag_LatitudeLongitude_Computed = false;


	//HydroPlusConfig.xml parameters
	static unordered_map<string, string> SimulationStringParams;
	unordered_map<string, double> SimulationNumericalParams;
	unordered_map<string, double> LandCoverParams;
	//map<string, double> VegetationParams;
	//map<string, double> SoilParams;
	static unordered_map<string, double> LocationParams;

	static unordered_map<string, double> RefParams;
	unordered_map<string, string> RefStringParams;

	vector<vector<unordered_map<string, double>>> DataDrawers;
	vector<vector<unordered_map<string, string>>> StringDataDrawers;

	unordered_map<string, double> TemperatureCalculationParams;
	unordered_map<string, string> TemperatureExecutionParams;

	unordered_map<string, double> CoolRiverVariables;
	unordered_map<string, string> CoolBuildingStringParams;
	unordered_map<string, double> CoolBuildingNumericalParams;
	unordered_map<string, double> BufferTemporalLoadingNumericalParams;
	unordered_map<string, double> BufferSpatialWeightingNumericalParams;
	static unordered_map<string, string> BufferSpatialWeightingStringParams;
	unordered_map<string, string>BufferNutrientsStringParams;

	//Weather.csv parameters
	vector<string> SimulationTime_HMS;
	vector<int> SimulationDate_GD;
	vector<int> SimulationDate_GDH;
	vector<int> SimulationDate_HH;
	vector<int> SimulationTimeStep_sec;
	int SimulationTimePeriod_seconds; 
	int SimulationTimePeriod_timeSteps;
	int timeSteps_in_Day;

	//vector<int> UserOutput_SimulationStartHour_HH, UserOutput_SimulationEndHour_HH;
	//AtmPres_kPa (kPa)
	vector<double> AtmPres_kPa;
	vector<double> WindSpd_mps;
	//WindDir_deg (degrees,N=0=360); used in EnergyLoad model
	vector<double> WindDir_deg;
	//Precip_mpdt (meters, per time step, dt; typically m/h)
	vector<double> Precip_mpdt;
	vector<double> RadiationNet_Wpm2;
	vector<double> Tair_F;
	vector<double> Tdew_F;
	vector<double> Rain_m;
	vector<double> Snow_m;
	double TotalRain_m;
	double TotalPrecip_m;
	double TotalSnow_m;
	//Defining vectors for meteorological inputs
	double Precipitation_Sum_m, Rain_Sum_m, Snow_Sum_m;
	vector<double> TemperatureAir_TS_F, TemperatureDewPoint_TS_F, NetRadiation_TS_Wpm2, WindSpeed_TS_mps, WindDirection_TS_deg, PressureAtm_TS_kPa, 
		Precipitation_TS_mpdt, Rain_TS_mpdt, Snow_TS_mpdt, SnowDepth_TS_m;
	vector<double> Runoff_Observed_m;
	//MonthlyAvgTair_C computed in CreateFile_EPW.cpp and used in CreateFile_IDF 
	vector<float> MonthlyAvgTair_C;

	//Flag_BlockGroupsProcessed to control use of Block Group data
	bool Flag_BlockGroupsProcessed;

	//Evaporation.csv parameters
	bool Flag_ComputePET;
	string evapXMLVal;
	vector<double> PotentialEvaporation_WaterOnGround_m;
	vector<double> PotentialEvaporation_WaterOnTree_m;
	vector<double> PotentialEvapotranspiration_TreeCover_m;
	vector<double> PotentialSublimation_SnowOnGround_m;
	vector<double> PotentialSublimation_SnowOnTree_m;
	vector<double> PotentialEvaporation_Ground_m;
	vector<double> PotentialEvaporation_Tree_m;
	vector<double> PotentialEvapotranspiration_m;
	vector<double> PotentialSublimation_Ground_m;
	vector<double> PotentialSublimation_Tree_m;

	//Radiation.csv parameters
	vector<double> Radiation_Shortwave_Direct_Wpm2;  
	vector<double> Radiation_Shortwave_Diffuse_Wpm2; 
	vector<double> Radiation_Longwave_Downwelling_Wpm2;
	vector<double> Radiation_Longwave_Upwelling_Wpm2;
	vector<double> Latitude_rad;
	vector<double> HourAngle_Solar_rad;
	vector<double> DeclinationAngle_Solar_rad;
	vector<double> ZenithAngle_Solar_rad;
	vector<double> CosAzimuth;
	vector<double> SinAzimuth;
	vector<double> AzimuthAngle_Solar_N_0_Reda_rad;

	//Distributed cover maps: landcover.asc, imperviouscover.asc, treecover.asc, dem.asc, blockgroup.asc, ah_max_w_p_m2.asc
	//Note: File extension .asc is recognized by ArcGIS and QGIS as a standard input type when using the 6 header rows
	vector<double> NLCDLandCoverData;
	static vector<double> ImperCoverData, PerCoverdata;
	vector<double> TreeCoverData;
	vector<double> TotalCover;
	vector<double> SoilCover;
	vector<double> WaterCover;
	vector<double> ImperviousNoTree;
	vector<double> ShortVegCover;
	vector<double> ConnectedImpervious;
	vector<double> FracEgreenTree;
	vector<double> FracEgreenShortVeg;
	vector<double> TreeOnPervious;
	vector<double> TreeOnImpervious;
	//cte 202402 Refactoring GI
	vector<double> PermeablePavementCover;
	static vector<double> Waters01, Waters0;
	static vector<double> NHDfdr, NHDfdr_reverse, DEMfdr, negDEMfdr;
	static vector<double> s_trans;
	static vector<double>  s_Ksat;
	static double s_trans_avg, s_depth_avg;
	static vector<double> s_depth;
	static vector<double> s_wtable;
	static vector<double> gi_area, gi_c_area;

	//distributed cover: anthropogenic heat maps from AH4GUC at maximum hourly value
	vector<double> AnthropogenicHeat_Max_W_p_m2;
	//AH4GUC_HourlyValue_to_MaxValue_frac_line is line of input
	string AH4GUC_HourlyValue_to_MaxValue_frac_line;
	//AH4GUC_HourlyValue_to_MaxValue_frac_str defined as vector
	vector<string> AH4GUC_HourlyValue_to_MaxValue_frac_str;
	//AH4GUC_HourlyValue_to_MaxValue_frac_str defined as vector
	vector<double> AH4GUC_HourlyValue_to_MaxValue_frac;

	unordered_set<int> LocationIdsToPrintTempOutput;
	vector<int> StartDatesForRowColTempOutput;
	vector<int> EndDatesForRowColTempOutput;
	vector<int> StartDatesForTimeBasedTempOutput;
	vector<int> EndDatesForTimeBasedTempOutput;
	vector<int> StartingBlockIdsToPrintTempOutput;
	vector<int> EndingBlockIdsToPrintTempOutput;
	vector<int> BlockGoupData;

	string demXMLVal;
	double AveSMD;
	double Size_TI_Value_Vector;
	double Size_TI_Area_Vector;
	vector<double> TI_Area_frac;
	double TI_Value_Average;

	//Define TI_Value vector double
	vector<double> TI_Value;
	//Define DEM vector double
	vector<double> DEM;
	vector<double> Slope;
	vector<double> FlowDir;
	static vector<double> FlowAccum;
	//Define SlopeGround_rad vector double
	vector<double> SlopeGround_rad;
	//Define AspectGround_N_0_rad vector double
	vector<double> AspectGround_N_0_rad;

	//Define RepoDict map double
	unordered_map<string, double> RepoDict;
	//Define RepoVecDict map vector
	unordered_map<string, vector<double>> RepoVecDict;

	//LAI calculations
	static unordered_map<string, double> LAI_BAI_map;
	static vector<double> LAI_BAI_Tree_m2_p_m2;
	static vector<double> LAI_BAI_SVeg_m2_p_m2;
	static vector<double> LAI_Tree_m2_p_m2;
	static vector<double> LAI_SVeg_m2_p_m2;

	double MinAllowableWindSpeed;
	//Define non-routed runoff (m) vectors, pervious, directly connected impervious, sub-surface, and total
	//PAQi_m (m) is pervious area runoff
	vector<double> PAQi_m;
	//DCIAQi_m (m) is directly connected impervious runoff, the impervious runoff not sent to pervious area
	vector<double> DCIAQi_m;
	//WAQi_m (m) is water area runoff
	vector<double> WAQi_m;
	//SurQi_m (m) is suurface area runoff
	vector<double> SurfQi_m;
	//SubQi_m (m) is subsurface pervious area runoff
	vector<double> SubQi_m;
	//Qi_m (m) is total runoff
	vector<double> Qi_m;

	//Define routed runoff (m) vectors, directly connected impervious, pervious, water, sub-surface, and total
	vector<double> WAQi_Routed_m;
	vector<double> PAQi_Routed_m;
	vector<double> DCIAQi_Routed_m;
	vector<double> SurfQi_Routed_m;
	vector<double> SubQi_Routed_m;
	vector<double> Qi_Routed_m;

	// For CoolRiver RiverTimeSeries parameters
	vector<int> DatesRiverTime;
	vector<string> RiverTimeSeries;

	vector<double> Sediment_T_C;
	vector<double> Cloudiness_F;
	vector<double> Humidity_Per;

	vector<vector<double>> SW_Q_cms;
	vector<vector<double>> SW_T_C;

	map<int, vector<double>> SW_Q_cms1;
	map<int, vector<double>> SW_T_C1;

	map<int, vector<double>> GW_Q_cms1;
	map<int, vector<double>> GW_T_C1;

	vector<vector<double>> GW_Q_cms;
	vector<vector<double>> GW_T_C;
	vector<int> number;
	vector<double> dischargeDist;
	vector<double> dischargeValue1;

	// For CoolRiver RiverStationSeries parameters
	vector<int> StationNumber;
	vector<int> StationDistanceBedData;
	vector<double> DepthMesurments;

	vector<string> TypeOfRiver;
	vector<double> HorizontalBedConnectivity;

	vector<double> BedParticleSize;
	vector<double> Embeddedness;

	vector<int> StationDistanceMorphology;
	vector<double> AreaR_m2;
	vector<double> WidthR_m;

	vector<double> DepthR_m;
	vector<double> SlopeR_f;

	vector<int> RowRiver;
	vector<int> ColRiver;
	vector<double> LongitudeR_frac;
	vector<double> LatitudeR_frac;
	vector<double> ZRiver_m;

	vector<int> StationDistanceShading;
	vector<double> EastBankHR_m;
	vector<double> EastTreeHR_m;
	vector<double> EastBuildingHR_m;
	vector<double> EastBankDistR_m;
	vector<double> EastCanDistR_m;
	vector<double> EastBuildingDistR_m;
	vector<double> EastBufferWR_m;

	vector<double> WestBBankHR_m;
	vector<double> WestBankHR_m;
	vector<double> WestTreeHR_m;
	vector<double> WestBuildingHR_m;
	vector<double> WestBankDistR_m;
	vector<double> WestCanDistR_m;
	vector<double> WestBuildingDistR_m;
	vector<double> WestBufferWR_m;

	vector<int> StationDistanceShadingPercent;
	vector<double> ElevationR_m;
	vector<double> StreamAzimuthR_deg;
	vector<double> ShadeFactorR_f;
	vector<double> ViewToSkyR_f;
	//HecRas.dat file
	vector<double> downStreamDist;
	vector<double> totalQ;
	vector<double> chElev;
	vector<double> waterSurElev;
	vector<double> velChnl;
	vector<double> xArea;
	vector<double> topWidth;
	vector<double> waterSurSlope;
	vector<double> wettedPerimeter;

	//CoolRiver variable
	int totalCount;
	//ECdynamic cell number
	double cell_count = 0.0, cell_count_Nowater = 0.0;
	vector<double> Cell_number;
	map <string, vector<double>> EC_vec_map;
	map <string, vector<double>> EMC_vec_map;
	map <string, vector<double>> water_volume_vec_map;
	vector<double> DischargeRecord;
	vector<int> month, day, year;
	static double cellsize;
	double foldersize;
	//GI Water Quality
	//Pollutants.csv pollutant types TSS, BOD, .., Zn are read into vector Pollutant_Name[], used to ID HydroPlusConfig.xml removal efficiencies
	//Note: The HydroPlusConfig.xml elements TSS, BOD, .., Zn are removal efficiencies, units percent
	vector<string> Pollutant_Name;
	vector<double> Pollutant_ConcentrationMean_mg_p_L;
	vector<double> Pollutant_ConcentrationMedian_mg_p_L;

	//Initialize NLCD land cover types to zero for summing in Inputs::readNLCDLandCoverFile function
	int LC21 = 0, LC22 = 0, LC23 = 0, LC24 = 0, LC41 = 0, LC42 = 0, LC43 = 0, LC71 = 0, LC81 = 0, LC82 = 0;
	int LC31 = 0, LC52 = 0;
	//water area
	int LC11 = 0, LC90 = 0, LC95 = 0;

	int run_ECDynamicModel = 1;
	//Flag_CoolAir_ImperviousCoverAdjust initialized to 0 to result in no change to land cover
	int Flag_CoolAir_ImperviousCoverAdjust = 0;
	//Scenario_CoolAir_Base_TCI_to_TC_frac initialized to -1 to result in no change to land cover
	double Scenario_CoolAir_Base_TCI_to_TC_frac = -1;
	//Scenario_CoolAir_Alternative_TC_frac initialized to 0.0 to result in no change to land cover
	double Scenario_CoolAir_Alternative_TC_frac = 0.0;
	//Scenario_CoolAir_Alternative_IC_frac initialized to 1.0 to result in no change to land cover
	double Scenario_CoolAir_Alternative_IC_frac = 1.0;

	static double xllcorner;
	static double yllcorner;
	int row_ReferenceStation;
	int col_ReferenceStation;
	int MapPixel_ReferenceStation_ID;
	static int ncols;
	static int nrows;
	static double NODATA;
	static double Length_Pixel_Side_m;


	vector<string> Station_OBJECTID_vec, Station_USAF_vec, Station_WBAN_vec, Station_Path_folder_vec, Station_TI_option_vec, Station_NLCD_Class_vec;
	vector<long double>  Station_Elev_m_vec, Station_Slope_deg_vec, Station_Aspect_deg_vec, Station_TC_per_vec, Station_IC_per_vec, Station_SVeg_per_vec, Station_Soil_per_vec, Station_Water_per_vec;
	vector<long double>  Station_Easting_m_vec, Station_Northing_m_vec;

	double conductivity_min, soildepth_min;
	vector<string> GI_list;
	vector<double> GI_ContributingArea_list;
	vector<int> GI_ulcorner_rows, GI_ulcorner_cols, GI_lrcorner_rows, GI_lrcorner_cols;

private:
	string Directory_Input_CLArg;
	int nDemDataCols;
	int nDemDataRows;
	double nDemDataxllcorner;
	double nDemDatayllcorner;
	int totalDischargeCount;

	//readRunoff_ObservedFile function reads Runoff_Observed_for_Calibration.csv file
	bool readRunoff_ObservedFile();

	//RataDieNumber function converts Gregorian date YYYYMMDD into an integer number based on Rata Die System 
	int RataDieNumber(int y, int m, int d);

	//GregorianToJulianDate function converts Gregorian date YYYYMMDD into an integer number based on Julian Date System
	int GregorianToJulianDate(int DateParts_Gregorian[]);
	
	//HydroPlusConfigFile.xml
	void readHydroPlusConfigFile();

	//Distributed cover maps: landcover.asc, imperviouscover.asc, treecover.asc, dem.asc, blockgroup.asc, ah_max_w_p_m2.asc
	//Note: File extension .asc is recognized by ArcGIS and QGIS as a standard input type when using the 6 header rows
	bool readNLCDLandCoverFile(string &Directory_Input_CLArg, double NODATA_code);
	bool readTreeCoverFile(string &Directory_Input_CLArg, double NODATA_code);
	//bool readImperviousFile(string &Directory_Input_CLArg, double NODATA_code);
	bool readImperviousFile(string& Directory_Input_CLArg, double NODATA_code);
	bool readBlockGroupOutputFileMap(string& Directory_Input_CLArg, double NODATA_code);
	//readAnthropogenicHeatMap bool function to read anthropogenic heat maps from AH4GUC
	bool readAnthropogenicHeatMap(string& Directory_Input_CLArg, double NODATA_code);
	void calcNLCDLandCoverData(int NumberOfLocations, double NODATA_code);

	bool readRiverTimeSeriesFile(double startDate, double endDate);
	bool readRiverGWTimeSeriesFile(double startDate, double endDate);
	bool readRiverSWTimeSeriesFile(double startDate, double endDate);
	bool readRiverStationSeriesFile(string &Directory_Input_CLArg, double NODATA_code);
	//cte bool readHecRasFile(double startDate, double endDate);
	bool readHecRasFile();
	
	//02012020 ECDynamic
	bool readDischargeData(string& Directory_Input_CLArg);
	bool readPollutionData(string& Directory_Input_CLArg);
	bool readExportCoefficientTable();
	bool readMultipleWeatherStations_List_CSV();
	void readSoilDepthMap_cm();
	void readHydraulicConductivityMap_umps();
	void calculateSoilTransmissivityChangeUnits();
	bool readWatertableMap_cm();
	//readWaterFeaturesMap function for SpatialBuffer inputs
	void readWaterFeaturesMap();
	void readNHDFlowDirectionMap();

	//writeAsciiMap takes a variableName and MapData_vec to create ArcGIS ready cover maps
	void writeAsciiMap(const string& variableName, const vector<double>& MapData_vec);
	//generate warning of printing out incomplete EC data
	void printIncompleteECDataWarning(const string& landCoverType, const string& landCoverRange, const string& huc);
	//LatitudeLongitude_Projection_Manager will create latitude longitude pair by calling Projection_Albers_to_LatitudeLongitude function
	void LatitudeLongitude_Projection_Manager(vector<vector<pair<double, double>>>& Latitude_Longitude_vectorPair_dd);
};

#endif