#include "WeatherPreprocessor.h"
#include "Inputs.h"



//Note: Flag_HottestDaySimulated = 1 searches all weather files for hottest hour, allowing differences between stations across years, months, and days
//Note: Model output file z_StationInfoMap.csv contains variable SimulationDate_Hottest_GD that reports the hottest date used for each station
//Note: Flag_MultipleStation = 1 computes distance between stations and map pixels that use the same projection, e.g., Albers_Conical_Equal_Area, Lambert_Azimuthal_Equal_Area, or UTM

//Variables defined in WeatherPreprocessor.h need to be declared in WeatherPreprocessor.cpp
vector<long double>  WeatherPreprocessor::Distance_Pixel_to_Station_vec_km;
vector<string>  WeatherPreprocessor::OBJECTID_shortest, WeatherPreprocessor::USAF_shortest, WeatherPreprocessor::WBAN_shortest,
WeatherPreprocessor::Path_folder_shortest, WeatherPreprocessor::TI_option_shortest, WeatherPreprocessor::NLCD_Class_shortest;
vector<long double> WeatherPreprocessor::Easting_shortest, WeatherPreprocessor::Northing_shortest, WeatherPreprocessor::Elev_m_shortest, WeatherPreprocessor::Distance_shortest,
WeatherPreprocessor::Aspect_deg_shortest, WeatherPreprocessor::Slope_deg_shortest, WeatherPreprocessor::TC_per_shortest, WeatherPreprocessor::IC_per_shortest,
WeatherPreprocessor::SVeg_per_shortest, WeatherPreprocessor::Soil_per_shortest, WeatherPreprocessor::Water_per_shortest;
int WeatherPreprocessor::SimulationDate_Hottest_GD;
string WeatherPreprocessor::SimulationTime_Hottest_HMS;
map<string, vector<double>> WeatherPreprocessor::WeatherMap;
vector<int> WeatherPreprocessor::SimulationDate_GD_AllRecord, WeatherPreprocessor::SimulationDate_GDH_AllRecord, WeatherPreprocessor::SimulationDate_HH_AllRecord;
vector<string>  WeatherPreprocessor::SimulationTime_HMS_AllRecord;
vector<double> WeatherPreprocessor::RadiationNet_Wpm2_AllRecord, WeatherPreprocessor::Tair_F_AllRecord, WeatherPreprocessor::Tdew_F_AllRecord, WeatherPreprocessor::Rain_m_AllRecord, WeatherPreprocessor::Snow_m_AllRecord, WeatherPreprocessor::WindSpd_mps_AllRecord, WeatherPreprocessor::WindDir_deg_AllRecord, WeatherPreprocessor::AtmPres_kPa_AllRecord, WeatherPreprocessor::Precip_mpdt_AllRecord;
int WeatherPreprocessor::Actual_hottest_GDH_index;
int WeatherPreprocessor::SimulationDateStart_GDH, WeatherPreprocessor::SimulationDateStop_GDH;
int WeatherPreprocessor::Date_YYYYMMDD, WeatherPreprocessor::Time_HHMMSS, WeatherPreprocessor::timeStepMetData, WeatherPreprocessor::previous_YYYYMMDD, WeatherPreprocessor::previous_HMS;
string WeatherPreprocessor::Time_HHMMSS_str;

//Distance_Calc_EastingNorthing function computes distance between stations and map pixels in 2 steps, 1st distance between each point, 2nd the square root of those distances squared and summed
//Note: Flag_MultipleStation = 1 computes distance between stations and map pixels that use the same projection, e.g., Albers_Conical_Equal_Area, Lambert_Azimuthal_Equal_Area, or UTM
//Note: The Station_easting_m and Station_northing_m are read from the WeatherStationAttributeFile, and PixelCenter_easting_m and PixelCenter_northing_m are read from the input maps
long double WeatherPreprocessor::Distance_Calc_EastingNorthing(long double PixelCenter_easting_m, long double PixelCenter_northing_m, long double Station_easting_m, long double Station_northing_m)
{
	//Distance_easting_m is distance along easting between station and pixel center
	long double Distance_easting_m = Station_easting_m - PixelCenter_easting_m;
	//Distance_northing_m is distance along northing between station and pixel center
	long double Distance_northing_m = Station_northing_m - PixelCenter_northing_m;

	//Distance_Pixel_to_Station_m (m) uses standard distance formula, taking the square root of the sum of squared distances in the northing and easting directions 
	double Distance_Pixel_to_Station_m = sqrt(Distance_easting_m * Distance_easting_m + Distance_northing_m * Distance_northing_m);
	return Distance_Pixel_to_Station_m * Ratio_km_to_m; //convert to km
}

//cte 2024 Li Zhang: Please explain this section with comments or to me in person
//sorting and keeping track of indexes
template <typename T>
vector<size_t> sort_indexes(const vector<T>& Vector) {

	// initialize original index locations
	vector<size_t> idx(Vector.size());
	iota(idx.begin(), idx.end(), 0);

	// sort indexes based on comparing values in DistanceVector
	// using stable_sort instead of sort to avoid unnecessary index re-orderings when v contains elements of equal values 
	stable_sort(idx.begin(), idx.end(), [&Vector](size_t i1, size_t i2) {return Vector[i1] < Vector[i2]; });

	return idx;
}

//WeatherStationDistance_sort function will find nearest weather stations
//Note: The Station_Easting_m and Station_Northing_m are read from the WeatherStationAttributeFile, and PixelCenter_easting_m and PixelCenter_northing_m are read from the input maps
//Note: The function will work when stations and map pixels use the same projection, e.g., Albers_Conical_Equal_Area, Lambert_Azimuthal_Equal_Area, or UTM
void WeatherPreprocessor::WeatherStationDistance_sort(Inputs* input, long double PixelCenter_easting_m, long double PixelCenter_northing_m) {
	//WeatherStationNum defined as input->SimulationNumericalParams["WeatherStationNum"], number of stations used in spatial weighting
	int WeatherStationNum = int(input->SimulationNumericalParams["WeatherStationNum"]);

	//For Loop through all members in LatDD_vec.size()
	//Note: LatDD_vec is same size as LonDD_vec, and these contain the data from the reference weather stations
	for (int i = 0; i < input->Station_Easting_m_vec.size(); i++) {

		//Station_Easting_x_m (m) defined by input-> Station_Easting_m_vec (m) vector, obtained from WeatherStationAttributeFile in coordinates identical to those of map
		long double Station_Easting_x_m = input->Station_Easting_m_vec[i];
		//Station_Northing_y_m (m) defined by input-> Station_Northing_m_vec (m) vector, obtained from WeatherStationAttributeFile in coordinates identical to those of map
		long double Station_Northing_y_m = input->Station_Northing_m_vec[i];
		//Call Distance_Calc_EastingNorthing function, which requires the four variables passed to contain the same map projection (e.g., Albers, Lampert, UTM, etc.)
		//Distance_Pixel_to_Station_km (km) is computed with Distance_Calc_LatLong function, which sends back distance in km, taking inputs in distance m
		long double Distance_Pixel_to_Station_km = Distance_Calc_EastingNorthing(PixelCenter_easting_m, PixelCenter_northing_m, Station_Easting_x_m, Station_Northing_y_m);
		//Distance_Pixel_to_Station_vec_km has Distance_Pixel_to_Station_km added
		Distance_Pixel_to_Station_vec_km.push_back(Distance_Pixel_to_Station_km);
	}

	//For Loop through all WeatherStationNum, which constains the number of nearest stations to include in weighting
	for (int i = 0; i < WeatherStationNum; i++) {
		//StationID contains the weather station index of the nearest station, e.g. 12
		//Note: sort_indexes returns the location in the vector of the ith smallest distance; e.g., index locaiton of first, second, and third nearest
		int StationID = int(sort_indexes(Distance_Pixel_to_Station_vec_km)[i]);
		//Distance_WeatherStation_km contains weather station ditance to the nearest station, e.g. 16093
		long double Distance_WeatherStation_km = Distance_Pixel_to_Station_vec_km[StationID];

		//Variable vectors have members added for each of the weather stations identified to influence the pixel
		Distance_shortest.push_back(Distance_WeatherStation_km);
		OBJECTID_shortest.push_back(input->Station_OBJECTID_vec[StationID]);
		USAF_shortest.push_back(input->Station_USAF_vec[StationID]);
		WBAN_shortest.push_back(input->Station_WBAN_vec[StationID]);
		Easting_shortest.push_back(input->Station_Easting_m_vec[StationID]);
		Northing_shortest.push_back(input->Station_Northing_m_vec[StationID]);
		Elev_m_shortest.push_back(input->Station_Elev_m_vec[StationID]);
		Slope_deg_shortest.push_back(input->Station_Slope_deg_vec[StationID]);
		Aspect_deg_shortest.push_back(input->Station_Aspect_deg_vec[StationID]);
		TI_option_shortest.push_back(input->Station_TI_option_vec[StationID]);
		NLCD_Class_shortest.push_back(input->Station_NLCD_Class_vec[StationID]);
		TC_per_shortest.push_back(input->Station_TC_per_vec[StationID]);
		IC_per_shortest.push_back(input->Station_IC_per_vec[StationID]);
		SVeg_per_shortest.push_back(input->Station_SVeg_per_vec[StationID]);
		Soil_per_shortest.push_back(input->Station_Soil_per_vec[StationID]);
		Water_per_shortest.push_back(input->Station_Water_per_vec[StationID]);
		Path_folder_shortest.push_back(input->Station_Path_folder_vec[StationID]);
		//albersEasting_shortest.push_back(input->Station_Easting_m_vec[StationID]);
		//albersNorthing_shortest.push_back(input->Station_Northing_m_vec[StationID]);
	}
	//Distance_Pixel_to_Station_vec_km vector is cleared
	Distance_Pixel_to_Station_vec_km.clear();
}

//convertHrMinSecToInt function converts string HrMinSec (HH:MM:SS) to an integer (HHMMSS)
//Note: Function used by readWeatherFile
int convertHrMinSecToInt(const string& timeStr) {
	//timeHHMMSS_str makes a copy of the timeStr 
	string timeHHMMSS_str = timeStr;
	//timeHHMMSS_str is parsed to remove colons
	timeHHMMSS_str.erase(remove(timeHHMMSS_str.begin(), timeHHMMSS_str.end(), ':'), timeHHMMSS_str.end());
	//timeHHMMSS_str is converted to integer
	int timeHHMMSS_int = stoi(timeHHMMSS_str);
	return timeHHMMSS_int;
}

//VectorsOfWeatherData function will assign Weather.csv lines of data to model vectors for single and multiple stations
void WeatherPreprocessor::VectorsOfWeatherData(Inputs* input, istream& lineWeatherFile, bool Flag_SingleStation, bool Flag_ProcessTimeStep)
{
	string temp;
	//initialize date as -1, other variables to -99999
	double tempTair_F = -99999;
	double tempTdew_F = -99999;
	double tempNetRadiation_Wpm2 = -99999;
	double tempWindSpd_mps = -99999;
	//tempWindDir_deg for EnergyLoad model
	double tempWindDir_deg = -99999;
	double tempAtmPres_kPa = -99999;
	double tempPrecip_mph = -99999;
	double tempPrecip_m = -99999;
	double tempSnowDepth_m = -99999;

	int date_InputDataRow_HH;
	int date_InputDataRow_GDH;
	int Date_JulianDay;
	int Counter_inputRow = 0;
	//timeStep_InputDataRow_sec initialized to HydroPlusConfig.xml TimeStep_sec
	int timeStep_InputDataRow_sec = input->SimulationNumericalParams["TimeStep_sec"];
	int SimulationTimeStep_sec;

	//calculateTimeStep lambda function to calculate time difference in seconds
	auto calculateTimeStep = [&](int current_YYYYMMDD, int current_HMS) -> int {
		//If timeStepMetData not 0 (e.g., 1st timeStepMetData w 0 indexing) then reading 2nd row of data and enter with two pairs of time
		if (timeStepMetData != 0) {
			//struct tm has instance previous_tm and current_tm initialized to 0
			struct tm previous_tm = { 0 }, current_tm = { 0 };

			//previous_tm struct items are populated by parsing previous timestep variables
			previous_tm.tm_year = (previous_YYYYMMDD / 10000) - 1900;
			previous_tm.tm_mon = ((previous_YYYYMMDD / 100) % 100) - 1;
			previous_tm.tm_mday = previous_YYYYMMDD % 100;
			previous_tm.tm_hour = previous_HMS / 10000;
			previous_tm.tm_min = (previous_HMS / 100) % 100;
			previous_tm.tm_sec = previous_HMS % 100;

			//current_tm struct items are populated by parsing current timestep variables
			current_tm.tm_year = (current_YYYYMMDD / 10000) - 1900;
			current_tm.tm_mon = ((current_YYYYMMDD / 100) % 100) - 1;
			current_tm.tm_mday = current_YYYYMMDD % 100;
			current_tm.tm_hour = current_HMS / 10000;
			current_tm.tm_min = (current_HMS / 100) % 100;
			current_tm.tm_sec = current_HMS % 100;

			//previous_time is a time_t object derived from mktime function being sent &previous_tm
			time_t previous_time = mktime(&previous_tm);
			//current_time is a time_t object derived from mktime function being sent &current_tm
			time_t current_time = mktime(&current_tm);
			//SimulationTimeStep_sec (seconds) is derived from difftime function sent current_time abd previous_time
			SimulationTimeStep_sec = static_cast<int>(difftime(current_time, previous_time));
		}
		//Else If timeStepMetData is 0 (e.g., 0 indexing) then without two pairs of time and need HydroPlusConfig.xml variable
		else {
			//SimulationTimeStep_sec (seconds) assigned as SimulationNumericalParams["TimeStep_sec"] for 1st timestep
			//Note: input->SimulationTimeStep_sec vector replaces input->SimulationNumericalParams["TimeStep_sec"]
			SimulationTimeStep_sec = input->SimulationNumericalParams["TimeStep_sec"];
		}

		//return SimulationTimeStep_sec, which is derived from data or default time step for the first timestep
		return SimulationTimeStep_sec;
		};

	//If Flag_ProcessTimeStep is true then enter to record time step information
	//Note: For multiple station runs, this is not activated for each line of data read to vectors
	if (Flag_ProcessTimeStep == true) {
		//Multiple Station implmentation; note vectors are cleared prior to each station, so only last station is kept
		//Time_HHMMSS returned from convertHrMinSecToInt(Time_HHMMSS_str) int function
		Time_HHMMSS = convertHrMinSecToInt(Time_HHMMSS_str);
		//timeStep_InputDataRow_sec returned from calculateTimeStep(Date_YYYYMMDD, Time_HHMMSS) function
		timeStep_InputDataRow_sec = calculateTimeStep(Date_YYYYMMDD, Time_HHMMSS);
		//If timeStep_InputDataRow_sec <= 0 then throw warning
		if (timeStep_InputDataRow_sec <= 0) {
			cout << "Warning: The Weather.csv has problematic date or time chronology at row: " << Counter_inputRow + 1 << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: The Weather.csv likely has 2 or more sections with the same date: " << Date_YYYYMMDD << endl;
			cout << "Correction: Search and remove from Weather.csv any instances of problematic chronological order." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}
		//SimulationTimeStep_sec vector populated with timeStep_InputDataRow_sec 
		input->SimulationTimeStep_sec.push_back(timeStep_InputDataRow_sec);
		//timeStepMetData advanced when within simulation date range for use with input->SimulationTimeStep_sec[timeStep] 
		timeStepMetData = timeStepMetData + 1;
		//final area within while loop reading Weather.csv
		//previous_GDH udpated to date_InputDataRow_GDH
		previous_YYYYMMDD = Date_YYYYMMDD;
		//previous_HMS udpated to Time_HHMMSS
		previous_HMS = Time_HHMMSS;
	}
	//Date_JulianDay from Date_YYYYMMDD_to_JulianDay function sending Date_YYYYMMDD as YYYYMMDD in first 8 digits
	//Note: Date_YYYYMMDD read from Weather.csv in upper part of this WeatherPreprocessor::readWeatherFile function
	//Note: If Flag_HottestDaySimulated = 1 then regardless of actual hottest day, output is given date from HydroPlusConfig.xml
	Date_JulianDay = input->Date_YYYYMMDD_to_JulianDay(Date_YYYYMMDD);
	//CalcTreeLAI function obtains LAI for Date_JulianDay
	Inputs::CalcTreeLAI(Date_JulianDay);
	//CalcShortVegLAI function obtains LAI for Date_JulianDay
	Inputs::CalcShortVegLAI(Date_JulianDay);

	//Reading air temperature input (F)
	std::getline(lineWeatherFile, temp, ',');
	istringstream ssTair(temp);
	ssTair >> tempTair_F;
	//If Flag_SingleStation is true then save a single station to input->variable
	if (Flag_SingleStation == true) {
		input->Tair_F.push_back(tempTair_F);
	}
	//Else save as multiple station to variable with _AllRecord 
	else {
		Tair_F_AllRecord.push_back(tempTair_F);
	}

	//Reading dew point temperature input (F)
	std::getline(lineWeatherFile, temp, ',');
	istringstream ssDew(temp);
	ssDew >> tempTdew_F;
	//If Flag_SingleStation is true then save a single station to input->variable
	if (Flag_SingleStation == true) {
		input->Tdew_F.push_back(tempTdew_F);
	}
	//Else save as multiple station to variable with _AllRecord 
	else {
		Tdew_F_AllRecord.push_back(tempTdew_F);
	}

	//Reading net radiation input (W/m^2)
	std::getline(lineWeatherFile, temp, ',');
	istringstream ssRn(temp);
	ssRn >> tempNetRadiation_Wpm2;
	//If Flag_SingleStation is true then save a single station to input->variable
	if (Flag_SingleStation == true) {
		input->RadiationNet_Wpm2.push_back(tempNetRadiation_Wpm2);
	}
	//Else save as multiple station to variable with _AllRecord 
	else {
		RadiationNet_Wpm2_AllRecord.push_back(tempNetRadiation_Wpm2);
	}

	//Reading wind speed input (m/s)
	std::getline(lineWeatherFile, temp, ',');
	istringstream ssWind(temp);
	ssWind >> tempWindSpd_mps;
	//If Flag_SingleStation is true then save a single station to input->variable
	if (Flag_SingleStation == true) {
		input->WindSpd_mps.push_back(tempWindSpd_mps);
	}
	//Else save as multiple station to variable with _AllRecord 
	else {
		WindSpd_mps_AllRecord.push_back(tempWindSpd_mps);
	}

	//If Model_Selection is CoolBuilding then read WindDir_deg
	if (input->SimulationStringParams["Model_Selection"] == "CoolBuilding") {
		//Reading atmospheric pressure input (kPa)
		std::getline(lineWeatherFile, temp, ',');
		istringstream ssWindDir(temp);
		ssWindDir >> tempWindDir_deg;
		//If Flag_SingleStation is true then save a single station to input->variable
		if (Flag_SingleStation == true) {
			input->WindDir_deg.push_back(tempWindDir_deg);
		}
		//Else save as multiple station to variable with _AllRecord 
		else {
			WindDir_deg_AllRecord.push_back(tempWindDir_deg);
		}
	}

	//Reading atmospheric pressure input (kPa)
	std::getline(lineWeatherFile, temp, ',');
	istringstream ssAtmPres(temp);
	ssAtmPres >> tempAtmPres_kPa;
	//If Flag_SingleStation is true then save a single station to input->variable
	if (Flag_SingleStation == true) {
		input->AtmPres_kPa.push_back(tempAtmPres_kPa);
	}
	//Else save as multiple station to variable with _AllRecord 
	else {
		AtmPres_kPa_AllRecord.push_back(tempAtmPres_kPa);
	}

	//Reading precipitation input (m/hr)
	std::getline(lineWeatherFile, temp, ',');
	istringstream ssPrecip(temp);
	ssPrecip >> tempPrecip_mph;
	//tempPrecip_m (m) = tempPrecip_mph (m/h) * Ratio_Hour_to_Second * timeStep_InputDataRow_sec
	tempPrecip_m = tempPrecip_mph * Ratio_Hour_to_Second * timeStep_InputDataRow_sec;
	//If Flag_SingleStation is true then save a single station to input->variable
	if (Flag_SingleStation == true) {
		//Note: For multiple stations, these vectors are updated later
		input->Precip_mpdt.push_back(tempPrecip_m);
		input->TotalPrecip_m += tempPrecip_m;
	}
	//Else save as multiple station to variable with _AllRecord 
	else {
		Precip_mpdt_AllRecord.push_back(tempPrecip_m);
	}

	//Threshold_Precipitation_m (m) catches faulty data read for precipitation, triggered perhaps by wrong ordered columns 
	//Note: Reading CoolBuilding Weather.csv can cause an error, mistaking atmospheric pressure for precipition
	int Threshold_Precipitation_m = 50;
	//If tempPrecip_m > Threshold_Precipitation_m, then abort program
	if (tempPrecip_m > Threshold_Precipitation_m) {
		cout << "Warning: The Weather.csv seems to ahve Precipitation_Rate_mph values > " << Threshold_Precipitation_m << "." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HydroPlus does not simulate the hydrology of such large precipitation rates." << endl;
		cout << "Explanation: This error may arise if Weather.csv erroneously organizes data columns." << endl;
		cout << "Explanation: The 8th data column of the Weather.csv file should contain precipitation data." << endl;
		cout << "Explanation: Weather.csv data columns for i-Tree have the following sequence:" << endl << endl;
		cout << "YYYYMMDD,HH:MM:SS,Temperature_Air_F(F),Temperature_DewPoint_F(F),Radiation_Net_Wpm2(W/m^2),Wind_Speed_mps(m/s),Pressure_Atmosphere_kPa(kPa),Precipitation_Rate_mph(m/h),Snow_DepthOnGround_m(m)" << endl << endl;
		cout << "Correction: Generate new Weather.csv with WeatherPrepConfig.xml file, setting parameter model to Hydro." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//If tempTair_F > 32 F then store tempPrecip_m to Rain_m vector, and increment TotalRain_m
	if (tempTair_F > 32.0) {
		//If Flag_SingleStation is true then save a single station to input->variable
		if (Flag_SingleStation == true) {
			input->Rain_m.push_back(tempPrecip_m);
			input->Snow_m.push_back(0);
			input->TotalRain_m += tempPrecip_m;
		}
		//Else save as multiple station to variable with _AllRecord 
		else {
			Rain_m_AllRecord.push_back(tempPrecip_m);
			Snow_m_AllRecord.push_back(0);
		}
	}
	//Else If tempTair_F <= 32 F then store tempPrecip_m to Snow_m vector, and increment TotalSnow_m
	else {
		//If Flag_SingleStation is true then save a single station to input->variable
		if (Flag_SingleStation == true) {
			input->Rain_m.push_back(0);
			input->Snow_m.push_back(tempPrecip_m);
			input->TotalSnow_m += tempPrecip_m;
		}
		//Else save as multiple station to variable with _AllRecord 
		else {
			Rain_m_AllRecord.push_back(0);
			Snow_m_AllRecord.push_back(tempPrecip_m);
		}
	}

	//Reading Snow_DepthOnGround_m or snow depth on ground (m) but not using variable in model
	//Note: Linebreak \n is read to signify end of row
	//Note: if Tair_F <= 32 then tempPrecip_m converted to Snow_m as liquid equivalent
	std::getline(lineWeatherFile, temp, '\n');
	istringstream ssSnowDepth(temp);
	ssSnowDepth >> tempSnowDepth_m;
}

//readWeatherFile function reads in Weather.csv time series data for HydroPlus models for all models and conditions 
//Note: Header 9 variables of Weather.csv: YYYYMMDD,HH:MM:SS,Temperature_Air_F(F),Temperature_DewPoint_F(F),Radiation_Net_Wpm2(W/m^2),Wind_Speed_mps(m/s),Pressure_Atmosphere_kPa(kPa),Precipitation_Rate_mph(m/h),Snow_DepthOnGround_m(m)
//Note: i-Tree Energy Weather.csv header row has additional field after Wind_Speed_mps(m/s), Wind_Direction_deg(deg)
bool WeatherPreprocessor::readWeatherFile(Inputs* input) {
	
	//if ReadStation_path.empty() then assign to InputDirectory for calls from Input
	if (ReadStation_path.empty()) {
		ReadStation_path = input->SimulationStringParams["InputDirectory"];
	}
	//readWeather created as part of fstream, defined as the ReadStation_path and file name for Weather.csv
	fstream readWeather(ReadStation_path + "Weather.csv");

	//if readWeather does not exist, alert user
	if (!readWeather) {
		cout << "Warning: Weather.csv could not be opened." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the Weather.csv file is missing there is no meteorological data to drive the simulation." << endl;
		cout << "Correction: Create a Weather.csv file, either manually or perhaps using the i-Tree WeatherPrep.exe utility." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	//Initialize variables
	input->TotalRain_m = 0.0;
	input->TotalPrecip_m = 0.0;
	input->TotalSnow_m = 0.0;
	string lineFromCSV;
	string tempStr = "";
	string temp;
	string Date_YYYYMMDD_str;
	int date_InputDataRow_HH;
	int date_InputDataRow_GDH;
	int Counter_inputRow = 0;
	//timeStep_InputDataRow_sec initialized to HydroPlusConfig.xml TimeStep_sec
	bool Flag_SingleStation = true;
	bool Flag_ProcessTimeStep = false;

	Inputs::InitializeLAIVariables();

	//getline reads in header from readWeather, stores in tempStr
	//Note: getline function of fstream library takes characters from 1st item & stores them in 2nd item, until delimitation character found
	getline(readWeather, tempStr);

	//numColumnsInFile within tempStr using .begin and .end, adding 1 to move from base 0 to 1 for count
	int numColumnsInFile = count(tempStr.begin(), tempStr.end(), ',') + 1;
	//Weather_Columns_count = 9 for all Model_Selection types except EnergyLoad
	int Weather_Columns_count = 9;
	string Weather_Column_names = "YYYYMMDD,HH:MM:SS,Temperature_Air_F(F),Temperature_DewPoint_F(F),Radiation_Net_Wpm2(W/m^2),Wind_Speed_mps(m/s),Pressure_Atmosphere_kPa(kPa),Precipitation_Rate_mph(m/h),Snow_DepthOnGround_m(m)";
	//If Model_Selection is CoolBuilding set to Weather_Columns_count = 10
	if (input->SimulationStringParams["Model_Selection"] == "CoolBuilding") {
		Weather_Columns_count = 10;
		Weather_Column_names = "YYYYMMDD,HH:MM:SS,Temperature_Air_F(F),Temperature_DewPoint_F(F),Radiation_Net_Wpm2(W/m^2),Wind_Speed_mps(m/s),WindDirection_deg(deg),Pressure_Atmosphere_kPa(kPa),Precipitation_Rate_mph(m/h),Snow_DepthOnGround_m(m)";
	}
	//If numColumnsInFile is not equal to Weather_Columns_count then warn and abort
	if (numColumnsInFile != Weather_Columns_count) {
		cout << endl;
		cout << "Error: Weather.csv has an incorrect number of data columns." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HydroPlus model needs" << Weather_Columns_count << "columns of data to properly assign Weather.csv variables." << endl;
		cout << "Correction: Update the Weather.csv file to contain the following comma separated variables:" << endl;
		cout << Weather_Column_names << endl;
		//abort simulation due to incorrect CSV format
		abort();
	}


	//while readWeather uses fstream good function to return true unless error flag is thrown
	// Read and process each row
	while (getline(readWeather, lineFromCSV)) {

		//If lineFromCSV (string) is empty then continue, which reads next line of file; this skips empty lines
		if (lineFromCSV.empty()) {
			continue;
		}
		//istringstream lineWeatherFile(lineFromCSV) creates object lineWeatherFile from string lineFromCSV
		istringstream lineWeatherFile(lineFromCSV);

		//Date_YYYYMMDD_str is obtained as 1st string before comma, using getline lineWeatherFile
		getline(lineWeatherFile, Date_YYYYMMDD_str, ',');
		//Time_HHMMSS_str is obtained as 2nd string before comma, using getline lineWeatherFile
		getline(lineWeatherFile, Time_HHMMSS_str, ',');

		//Counter_inputRow advanced
		Counter_inputRow = Counter_inputRow + 1;
		
		//Date_YYYYMMDD is obtained using stoi function with Date_YYYYMMDD_str
		Date_YYYYMMDD = stoi(Date_YYYYMMDD_str);
		//Time_HHMMSS extracted with stoi function with Time_HHMMSS_str and .substr functions, 1st number is position, 2nd is length
		Time_HHMMSS = stoi(Time_HHMMSS_str.substr(0, 2) + Time_HHMMSS_str.substr(3, 2) + Time_HHMMSS_str.substr(6, 2));
		//date_InputDataRow_GDH is created by Date_YYYYMMDD * 100 creating space for adding first two digits of Time_HHMMSS
		date_InputDataRow_GDH = (Date_YYYYMMDD * 100) + (Time_HHMMSS / 10000);
		//date_InputDataRow_HH integer from string with atoi function, extracting 2 digits of temp w substr function; c_str() points to temp
		date_InputDataRow_HH = atoi(Time_HHMMSS_str.substr(0, 2).c_str());

		//If Counter_inputRow equals 1 then check if Weather.csv first date is not greater than HydroPlusConfig.xml StartDate_YYYYMMDD
		if (Counter_inputRow == 1) {
			//timeStepMetData set to 0
			timeStepMetData = 0;
			//Check if year matches HydroPlusConfig.xml StartDate_YYYYMMDD Year (and below if Weather.csv contains full YYYYMMDD called for in HydroPlusConfig.xml)
			//HydroPlusConfig_StartDay_str_YYYYMMDD is string form of HydroPlusConfig.xml input StartDate_YYYYMMDD
			string HydroPlusConfig_StartDay_str_YYYYMMDD = to_string(input->SimulationNumericalParams["StartDate_YYYYMMDD"]);
			//WeatherInput_StartDay_YYYY is string form of Weather.csv input date column
			string WeatherInput_StartDay_str_YYYYMMDD = to_string(Date_YYYYMMDD);
			//WeatherInput_StartDay_YYYYMMDD defined as integer of Date_YYYYMMDD input
			int WeatherInput_StartDay_YYYYMMDD = Date_YYYYMMDD;
			//HydroPlusConfig_StartDay_YYYYMMDD is integer form of substring from HydroPlusConfig_StartDay_str_YYYYMMDD, taking first 8 elements
			int HydroPlusConfig_StartDay_YYYYMMDD = atoi(HydroPlusConfig_StartDay_str_YYYYMMDD.substr(0, 8).c_str());

			//if HydroPlusConfig_StartDay_YYYYMMDD > HydroPlusConfig_StartDay_YYYYMMDD then error with inputs or expectations
			if (WeatherInput_StartDay_YYYYMMDD > HydroPlusConfig_StartDay_YYYYMMDD) {
				cout << "Warning: The Weather.csv 1st column for date, YYYYMMDD, has a starting date of " << WeatherInput_StartDay_YYYYMMDD << " ..." << endl;
				cout << "... which is later than the HydroPlusConfig.xml parameter StartDate_YYYYMMDD, set to " << HydroPlusConfig_StartDay_YYYYMMDD << "." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: The HydroPlus model needs the Weather.csv to contain the date given in HydroPlusConfig.xml." << endl;
				cout << "Correction: Either generate new Weather.csv or change the StartDate_YYYYMMDD in the HydroPlusConfig.xml file." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}
		}

		//If Flag_HottestDaySimulated equals 1 then 
		//Note: Flag_HottestDaySimulated = 1 searches all weather files for hottest hour, allowing differences between stations across years, months, and days
		//Note: Model output file z_StationInfoMap.csv contains variable SimulationDate_Hottest_GD that reports the hottest date used for each station
		if (input->SimulationStringParams["Flag_HottestDaySimulated"] == "1") {
			//SimulationDate_GD vector contains Date_YYYYMMDD, list of Gregorian dates (YYYYMMDD) from input file
			SimulationDate_GD_AllRecord.push_back(static_cast<int>(Date_YYYYMMDD));
			//SimulationDate_GDH vector contains date_InputDataRow_GDH, list of Gregorian dates and hour (YYYYMMDDHH) from input file
			SimulationDate_GDH_AllRecord.push_back(static_cast<int>(date_InputDataRow_GDH));
			//SimulationDate_HH_AllRecord vector contains date_InputDataRow_HH, hour data from input file
			SimulationDate_HH_AllRecord.push_back(static_cast<int>(date_InputDataRow_HH));
			//SimulationTime_HMS_AllRecord vector contains Time_HHMMSS_str, time as HH:MM:SS string from input file
			SimulationTime_HMS_AllRecord.push_back(Time_HHMMSS_str);
			//Flag_SingleStation set to false
			Flag_SingleStation = false;

			//If date_InputDataRow_GDH <= SimulationDateStop_GDH && date_InputDataRow_GDH >= SimulationDateStart_GDH then
			if (date_InputDataRow_GDH <= input->SimulationNumericalParams["SimulationDateStop_GDH"] &&
				date_InputDataRow_GDH >= input->SimulationNumericalParams["SimulationDateStart_GDH"]) {
				//SimulationTime_HH vector will contain the HH:MM:SS values
				input->SimulationDate_HH.push_back(static_cast<int>(date_InputDataRow_HH));
				//SimulationTime_HMS vector will contain the HH:MM:SS values
				input->SimulationTime_HMS.push_back(Time_HHMMSS_str);
				//Flag_ProcessTimeStep set to true
				Flag_ProcessTimeStep = true;

			}
			//VectorsOfWeatherData function called for multiple station
			//Note: Each line of Weather.csv is read for multiple station settings, not only those within the start and stop dates
			WeatherPreprocessor::VectorsOfWeatherData(input, lineWeatherFile, Flag_SingleStation, Flag_ProcessTimeStep);
			//Flag_ProcessTimeStep reset to false
			Flag_ProcessTimeStep = false;
		}

		//If Flag_HottestDaySimulated != 1 or Model_Selection != SpatialTemperatureHydro And ...
		//... If date_InputDataRow_GDH is between SimulationDateStop_GDH and SimulationDateStart_GDH then proceed to read input
		if ((input->SimulationStringParams["Flag_HottestDaySimulated"] != "1" || input->SimulationStringParams["Model_Selection"] != "SpatialTemperatureHydro") && (date_InputDataRow_GDH <= input->SimulationNumericalParams["SimulationDateStop_GDH"] &&
				date_InputDataRow_GDH >= input->SimulationNumericalParams["SimulationDateStart_GDH"])) {
			
			//SimulationDate_GD push_back Date_YYYYMMDD
			input->SimulationDate_GD.push_back(static_cast<int>(Date_YYYYMMDD));
			//SimulationDate_GDH push_back date_InputDataRow_GDH
			input->SimulationDate_GDH.push_back(static_cast<int>(date_InputDataRow_GDH));
			//SimulationTime_HMS push_back Time_HHMMSS_str
			input->SimulationTime_HMS.push_back(Time_HHMMSS_str);
			//Flag_SingleStation set to true
			Flag_SingleStation = true;
			//Flag_ProcessTimeStep reset to true
			Flag_ProcessTimeStep = true;
			//VectorsOfWeatherData function called for single station
			WeatherPreprocessor::VectorsOfWeatherData(input, lineWeatherFile, Flag_SingleStation, Flag_ProcessTimeStep);
		}
	}

	//If SimulationDate_GD.size() <= 0 and input->SimulationStringParams["Flag_MultipleStation"] != "1" then error with inputs or expectations about simulation dates
	if (input->SimulationDate_GD.size() <= 0 && input->SimulationStringParams["Flag_MultipleStation"] != "1") {
		cout << "Warning: The HydroPlusConfig.xml StartDate_YYYYMMDD and StopDate_YYYYMMDD are not within Weather.csv dates." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HydroPlus model needs the Weather.csv to contain the dates given in HydroPlusConfig.xml." << endl;
		cout << "Correction: Change the dates in Weather.csv or in HydroPlusConfig.xml StartDate_YYYYMMDD and StopDate_YYYYMMDD." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	readWeather.close();

	//SimulationTimePeriod_seconds (sec) is total simulated time
	//Note: Uses function accumulate(input->SimulationTimeStep_sec.begin(), input->SimulationTimeStep_sec.end(),0)
	input->SimulationTimePeriod_seconds = accumulate(input->SimulationTimeStep_sec.begin(), input->SimulationTimeStep_sec.end(),0);
	//SimulationTimePeriod_timeSteps (counter of time steps) is SimulationTimeStep_sec.size()
	input->SimulationTimePeriod_timeSteps = input->SimulationTimeStep_sec.size();

	//If Flag_HottestDaySimulated equals 1, search for the hottest date
	//Note: Flag_HottestDaySimulated = 1 searches all weather files for hottest hour, allowing differences between stations across years, months, and days
	//Note: Model output file z_StationInfoMap.csv contains variable SimulationDate_Hottest_GD that reports the hottest date used for each station
	if (input -> SimulationStringParams["Flag_HottestDaySimulated"] == "1" ) {
		//Tair_Max_index contains index of the hottest hour
		int Tair_Max_index = int(sort_indexes(Tair_F_AllRecord).back());
		//Tair_Max_F contains the hottest temp
		long double Tair_Max_F = Tair_F_AllRecord[Tair_Max_index];
		//SimulationDate_Hottest_GD (GD) is date of hottest hour
		SimulationDate_Hottest_GD = SimulationDate_GD_AllRecord[Tair_Max_index];
		//SimulationTime_Hottest_HMS (HMS) is time of hottest hour (real hottest hour and time)
		SimulationTime_Hottest_HMS = SimulationTime_HMS_AllRecord[Tair_Max_index];
		
		Actual_hottest_GDH_index = Tair_Max_index;
		//steps_move = User_given_hottest_GDH_index - Actual_hottest_GDH_index;
		//update SimulationTime_Hottest_HMS and SimulationDate_Hottest_GD to move the data record

		// reporting the hottest hour synchronously on the hour given by the HydroPlusConfig.xml StopDate_YYYYMMDD variable
		int total_timestep = input->SimulationTimePeriod_timeSteps;
		for (int timeStep = 0; timeStep < total_timestep; ++timeStep) {
			//(total_timestep - 1 - timeStep) is the steps track back forward
			//since we put actual hottest data to StopDate_YYYYMMDD, we only need to go back forward.
			input->SimulationDate_GD.push_back(SimulationDate_Hottest_GD);
			//assembling GDH for output writer 
			input->SimulationDate_GDH.push_back(SimulationDate_Hottest_GD * 100 + input->SimulationDate_HH[timeStep]);

			input->Tair_F.push_back(Tair_F_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)]);
			input->Tdew_F.push_back(Tdew_F_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)]);
			input->RadiationNet_Wpm2.push_back(RadiationNet_Wpm2_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)]);
			input->WindSpd_mps.push_back(WindSpd_mps_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)]);
			//If Model_Selection is CoolBuilding then add WindDir_deg to list
			if (input->SimulationStringParams["Model_Selection"] == "CoolBuilding") {
				input->WindDir_deg.push_back(WindDir_deg_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)]);
			}
			input->AtmPres_kPa.push_back(AtmPres_kPa_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)]);
			input->Precip_mpdt.push_back(Precip_mpdt_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)]);
			input->Rain_m.push_back(Rain_m_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)]);
			input->Snow_m.push_back(Snow_m_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)]);
			//total items need to accumulate
			input->TotalPrecip_m += Precip_mpdt_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)];
			input->TotalRain_m += Rain_m_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)];
			input->TotalSnow_m += Snow_m_AllRecord[Actual_hottest_GDH_index - (total_timestep - 1 - timeStep)];
		}
	}
	
	//MeteorologicalVariableName_string string contains variable names listed in header column of Weather.csv 
	vector<string> MeteorologicalVariableName_string = { "RadiationNet_Wpm2", "Tair_F", "Tdew_F", "WindSpd_mps", "AtmPres_kPa", "Precip_mpdt", "Rain_m", "Snow_m","TotalPrecip_m", "TotalRain_m", "TotalSnow_m"};

	//If Model_Selection is CoolBuilding then add WindDir_deg to list
	if (input->SimulationStringParams["Model_Selection"] == "CoolBuilding") {
		//MeteorologicalVariableName_string push_back WindDir_deg
		MeteorologicalVariableName_string.push_back("WindDir_deg");
	}
	//StationID_string string contains OBJECTID_shortest[i] item, station ID of nearest weather station
	string StationID_string = ReadStationID;
	string StationID_VariableName;
			
	//actual read date and time
	if (input->SimulationStringParams["Flag_HottestDaySimulated"] == "1") {
		SimulationDateStart_GDH = SimulationDate_GDH_AllRecord[Actual_hottest_GDH_index - (input->SimulationTimePeriod_timeSteps - 1)];
		SimulationDateStop_GDH = SimulationDate_GDH_AllRecord[Actual_hottest_GDH_index];
	}
	
	//For Loop with string name through MeteorologicalVariableName_Weather_string vector list
	for (string Station_Variable_name : MeteorologicalVariableName_string) {
		//StationID_VariableName is string appended by StationID_string and Station_Variable_name, e.g., 05_RadiationNet_Wpm2
		StationID_VariableName = StationID_string + "_" + Station_Variable_name;
		
		if (Station_Variable_name == "RadiationNet_Wpm2") {
			WeatherMap[StationID_VariableName] = input->RadiationNet_Wpm2;
		}
		else if (Station_Variable_name == "Tair_F") {
			WeatherMap[StationID_VariableName] = input->Tair_F;
		}
		else if (Station_Variable_name == "Tdew_F") {
			WeatherMap[StationID_VariableName] = input->Tdew_F;
		}
		else if (Station_Variable_name == "WindSpd_mps") {
			WeatherMap[StationID_VariableName] = input->WindSpd_mps;
		}
		//If Model_Selection is CoolBuilding
		else if (Station_Variable_name == "WindDir_deg") {
			WeatherMap[StationID_VariableName] = input->WindDir_deg;
		}
		else if (Station_Variable_name == "AtmPres_kPa") {
			WeatherMap[StationID_VariableName] = input->AtmPres_kPa;
		}
		else if (Station_Variable_name == "Precip_mpdt") {
			WeatherMap[StationID_VariableName] = input->Precip_mpdt;
		}
		else if (Station_Variable_name == "Rain_m") {
			WeatherMap[StationID_VariableName] = input->Rain_m;
		}
		else if (Station_Variable_name == "Snow_m") {
			WeatherMap[StationID_VariableName] = input->Snow_m;
		}
		else if (Station_Variable_name == "TotalPrecip_m") {
			WeatherMap[StationID_VariableName] = { input->TotalPrecip_m };
		}
		else if (Station_Variable_name == "TotalRain_m") {
			WeatherMap[StationID_VariableName] = { input->TotalRain_m };
		}
		else if (Station_Variable_name == "TotalSnow_m") {
			WeatherMap[StationID_VariableName] = { input->TotalSnow_m };
		}
	}
	
	SimulationDate_GD_AllRecord.clear();
	SimulationDate_GDH_AllRecord.clear();
	SimulationTime_HMS_AllRecord.clear();
	SimulationDate_HH_AllRecord.clear();
	RadiationNet_Wpm2_AllRecord.clear();
	Tair_F_AllRecord.clear();
	Tdew_F_AllRecord.clear();
	Rain_m_AllRecord.clear();
	Snow_m_AllRecord.clear();
	WindSpd_mps_AllRecord.clear();
	WindDir_deg_AllRecord.clear();
	AtmPres_kPa_AllRecord.clear();
	Precip_mpdt_AllRecord.clear();
	
	return true;
}

//Variables defined in WeatherPreprocessor.h need to be declared in WeatherPreprocessor.cpp; relates to static vs non-static function
map<string, vector<double>> WeatherPreprocessor::EvaporationMap;

//readEvaporationFile function will read the input files containing meteorological data
//Note: Header 7 variables of Evaporation.csv: YYYYMMDD,HH:MM:SS,PotentialEvapotranspiration_TreeCover_mph(m/h),PotentialEvaporation_WaterOnGround_mph(m/h),PotentialEvaporation_WaterOnTree_mph(m/h),PotentialSublimation_SnowOnGround_mph(m/h),PotentialSublimation_SnowOnTree_mph(m/h)
//Note: readEvaporationFile is called only when Flag_ReadEvaporationData = 1 and Model = StatisticalHydro
bool WeatherPreprocessor::readEvaporationFile(Inputs* input)
{
	//readEvaporation created as part of fstream, defined as the ReadStation_path and file name for Evaporation.csv
	fstream readEvaporation(ReadStation_path + "Evaporation.csv");

	//If meteorlogical input data file does not exist, then alert user and abort
	if (!readEvaporation) {
		cout << "Warning: Evaporation.csv could not be opened." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the Evaporation.csv file is missing there is no evapotranspiration or sublimation data to drive the simulation." << endl;
		cout << "Correction: Create an Evaporation.csv file, either manually or perhaps using the i-Tree WeatherPrep.exe utility." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	string tempStr = "";
	double tempDbl = -1.0;
	string temp;
	double Date_YYYYMMDD = -1.0;
	double tempPotentialEvapotranspiration_TreeCover_m = -1.0;
	double tempPotentialEvaporation_WaterOnGround_m = -1.0;
	double tempPotentialEvaporation_WaterOnTree_m = -1.0;
	double tempPotentialSublimation_SnowOnGround_m = -1.0;
	double tempPotentialSublimation_SnowOnTree_m = -1.0;
	double date_InputDataRow_HH;
	double date_InputDataRow_GDH;
	string HrMinSec;
	int Counter_inputRow = 0;
	int timeStep = 0;

	getline(readEvaporation, tempStr);

	//numColumnsInFile within tempStr using .begin and .end, adding 1 to move from base 0 to 1 for count
	int numColumnsInFile = count(tempStr.begin(), tempStr.end(), ',') + 1;
	//Weather_Columns_count = 7 for all Model_Selection types
	int Evaporation_Columns_count = 7;
	string Evaporation_Column_names = "YYYYMMDD,HH:MM:SS,PotentialEvapotranspiration_TreeCover_mph(m/h),PotentialEvaporation_WaterOnGround_mph(m/h),PotentialEvaporation_WaterOnTree_mph(m/h),PotentialSublimation_SnowOnGround_mph(m/h),PotentialSublimation_SnowOnTree_mph(m/h)";
	//If numColumnsInFile is not equal to Evaporation_Columns_count then warn and abort
	if (numColumnsInFile != Evaporation_Columns_count) {
		cout << endl;
		cout << "Error: Evaporation.csv has an incorrect number of data columns." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HydroPlus model needs" << Evaporation_Columns_count << "columns of data to properly assign Evaporation.csv variables." << endl;
		cout << "Correction: Update the Evaporation.csv file to contain the following comma separated variables:" << endl;
		cout << Evaporation_Column_names << endl;
		//abort simulation due to incorrect CSV format
		abort();
	}

	while (readEvaporation.good())
	{
		getline(readEvaporation, temp, ',');

		//temp.erase will trim leading and then trailing whitespace (optional but recommended)
		temp.erase(0, temp.find_first_not_of(" \t\r\n"));
		temp.erase(temp.find_last_not_of(" \t\r\n") + 1);

		//if (temp.empty()) then continue, skip the line if it's empty or contains only whitespace
		if (temp.empty()) {
			continue;
		}

		//Counter_inputRow advances
		Counter_inputRow = Counter_inputRow + 1;
		//istringstream stores an instance of the string temp in the new ss string
		istringstream ss(temp);
		//ss string is passed into Date_YYYYMMDD as double
		ss >> Date_YYYYMMDD;

		getline(readEvaporation, temp, ',');
		istringstream ssTime(temp);
		ssTime >> HrMinSec;

		//If Counter_inputRow equals 1 then at start of file then check if Weather.csv first date is not greater than HydroPlusConfig.xml StartDate_YYYYMMDD
		if (Counter_inputRow == 1) {
			//Check if year matches HydroPlusConfig.xml StartDate_YYYYMMDD Year (and below if Weather.csv contains full YYYYMMDD called for in HydroPlusConfig.xml)
			//HydroPlusConfig_StartDay_str_YYYYMMDD is string form of HydroPlusConfig.xml input StartDate_YYYYMMDD
			string HydroPlusConfig_StartDay_str_YYYYMMDD = to_string(input->SimulationNumericalParams["StartDate_YYYYMMDD"]);
			//WeatherInput_StartDay_YYYY is string form of Weather.csv input date column
			string WeatherInput_StartDay_str_YYYYMMDD = to_string(Date_YYYYMMDD);
			//WeatherInput_StartDay_YYYYMMDD defined as integer of Date_YYYYMMDD input
			int WeatherInput_StartDay_YYYYMMDD = Date_YYYYMMDD;
			//HydroPlusConfig_StartDay_YYYYMMDD is integer form of substring from HydroPlusConfig_StartDay_str_YYYYMMDD, taking first 8 elements
			int HydroPlusConfig_StartDay_YYYYMMDD = atoi(HydroPlusConfig_StartDay_str_YYYYMMDD.substr(0, 8).c_str());

			//if HydroPlusConfig_StartDay_YYYYMMDD > HydroPlusConfig_StartDay_YYYYMMDD then error with inputs or expectations
			if (WeatherInput_StartDay_YYYYMMDD > HydroPlusConfig_StartDay_YYYYMMDD) {
				cout << "Warning: The Evaporation.csv 1st column for date, YYYYMMDD, has a starting date of " << WeatherInput_StartDay_YYYYMMDD << " ..." << endl;
				cout << "... which is later than the HydroPlusConfig.xml parameter StartDate_YYYYMMDD, set to " << HydroPlusConfig_StartDay_YYYYMMDD << "." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: The HydroPlus model needs the Evaporation.csv to contain the date given in HydroPlusConfig.xml." << endl;
				cout << "Correction: Either generate new Evaporation.csv or change the StartDate_YYYYMMDD in the HydroPlusConfig.xml file." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}
		}

		//Algorithm to extract hour integer from section of temp string, used to refine location where input rows are read
		//date_InputDataRow_HH integer from string with atoi function, extracting 2 digits of temp w substr function; c_str() points to temp
		date_InputDataRow_HH = atoi(temp.substr(0, 2).c_str());

		//date_InputDataRow_GDH contains Gregorian Date and Hour, extending 8 to 10 digits by multiplication with 100, adding hour
		date_InputDataRow_GDH = (Date_YYYYMMDD * 100) + date_InputDataRow_HH;

		//If (Flag_HottestDaySimulated equals 1 & SimulationDateStop_GDH and SimulationDateStart_GDH are for hottest day) OR ..
		//... If (Flag_HottestDaySimulated equals 0 & input->["SimulationDateStop_GDH"] .. are for HydroPlusConfig input)
		//Note: If Flag_HottestDaySimulated = 1 then Weather.csv for hottest hour, with YYYYMMDD differences between stations
		//Note: Output file z_StationInfoMap.csv contains SimulationDate_Hottest_GD reporting hottest date used for each station
		if (((input->SimulationStringParams["Flag_HottestDaySimulated"] == "1") && (date_InputDataRow_GDH <= SimulationDateStop_GDH && date_InputDataRow_GDH >= SimulationDateStart_GDH)) || ((input->SimulationStringParams["Flag_HottestDaySimulated"] != "1") && (date_InputDataRow_GDH <= input->SimulationNumericalParams["SimulationDateStop_GDH"] && date_InputDataRow_GDH >= input->SimulationNumericalParams["SimulationDateStart_GDH"]))) {

			getline(readEvaporation, temp, ',');
			istringstream ssPet(temp);
			ssPet >> tempPotentialEvapotranspiration_TreeCover_m;
			//tempPotentialEvapotranspiration_TreeCover_m (m) adjusted by TimeStep_sec and Ratio_Hour_to_Second
			tempPotentialEvapotranspiration_TreeCover_m = (tempPotentialEvapotranspiration_TreeCover_m * Ratio_Hour_to_Second * input->SimulationTimeStep_sec[timeStep]);
			input->PotentialEvapotranspiration_TreeCover_m.push_back(tempPotentialEvapotranspiration_TreeCover_m);

			getline(readEvaporation, temp, ',');
			istringstream ssPe(temp);
			ssPe >> tempPotentialEvaporation_WaterOnGround_m;
			//tempPotentialEvaporation_WaterOnGround_m (m) adjusted by TimeStep_sec and Ratio_Hour_to_Second
			tempPotentialEvaporation_WaterOnGround_m = (tempPotentialEvaporation_WaterOnGround_m * Ratio_Hour_to_Second * input->SimulationTimeStep_sec[timeStep]);
			input->PotentialEvaporation_WaterOnGround_m.push_back(tempPotentialEvaporation_WaterOnGround_m);

			getline(readEvaporation, temp, ',');
			istringstream ssPetree(temp);
			ssPetree >> tempPotentialEvaporation_WaterOnTree_m;
			//tempPotentialEvaporation_WaterOnTree_m (m) adjusted by TimeStep_sec and Ratio_Hour_to_Second
			tempPotentialEvaporation_WaterOnTree_m = (tempPotentialEvaporation_WaterOnTree_m * Ratio_Hour_to_Second * input->SimulationTimeStep_sec[timeStep]);
			input->PotentialEvaporation_WaterOnTree_m.push_back(tempPotentialEvaporation_WaterOnTree_m);

			getline(readEvaporation, temp, ',');
			istringstream ssPesnow(temp);
			ssPesnow >> tempPotentialSublimation_SnowOnGround_m;
			//tempPotentialSublimation_SnowOnGround_m (m) adjusted by TimeStep_sec and Ratio_Hour_to_Second
			tempPotentialSublimation_SnowOnGround_m = (tempPotentialSublimation_SnowOnGround_m * Ratio_Hour_to_Second * input->SimulationTimeStep_sec[timeStep]);
			input->PotentialSublimation_SnowOnGround_m.push_back(tempPotentialSublimation_SnowOnGround_m);

			//Linebreak \n is read to signify end of row
			getline(readEvaporation, temp, '\n');
			istringstream ssPotentialSublimation_SnowOnTree_m(temp);
			ssPotentialSublimation_SnowOnTree_m >> tempPotentialSublimation_SnowOnTree_m;
			//tempPotentialSublimation_SnowOnTree_m (m) adjusted by TimeStep_sec and Ratio_Hour_to_Second
			tempPotentialSublimation_SnowOnTree_m = (tempPotentialSublimation_SnowOnTree_m * Ratio_Hour_to_Second * input->SimulationTimeStep_sec[timeStep]);
			input->PotentialSublimation_SnowOnTree_m.push_back(tempPotentialSublimation_SnowOnTree_m);
			//timeStep advanced when within simulation date range for use with input->SimulationTimeStep_sec[timeStep] 
			timeStep = timeStep + 1;
		}
		//If Flag_HottestDaySimulated != 1 or Model_Selection != SpatialTemperatureHydro && date_InputDataRow_GDH is between SimulationDateStop_GDH and SimulationDateStart_GDH then proceed to read input
		else if ((input->SimulationStringParams["Flag_HottestDaySimulated"] != "1" || input->SimulationStringParams["Model_Selection"] != "SpatialTemperatureHydro") && (date_InputDataRow_GDH <= input->SimulationNumericalParams["SimulationDateStop_GDH"] &&
			date_InputDataRow_GDH >= input->SimulationNumericalParams["SimulationDateStart_GDH"])) {
			
			getline(readEvaporation, temp, ',');
			istringstream ssPet(temp);
			ssPet >> tempPotentialEvapotranspiration_TreeCover_m;
			//Multiplying tempPotentialEvapotranspiration_TreeCover_m (m) by TimeStep_sec and Ratio_Hour_to_Second
			tempPotentialEvapotranspiration_TreeCover_m = (tempPotentialEvapotranspiration_TreeCover_m * Ratio_Hour_to_Second * input->SimulationTimeStep_sec[timeStep]);
			input->PotentialEvapotranspiration_TreeCover_m.push_back(tempPotentialEvapotranspiration_TreeCover_m);

			getline(readEvaporation, temp, ',');
			istringstream ssPe(temp);
			ssPe >> tempPotentialEvaporation_WaterOnGround_m;
			//Multiplying tempPotentialEvaporation_WaterOnGround_m (m) by TimeStep_sec and Ratio_Hour_to_Second
			tempPotentialEvaporation_WaterOnGround_m = (tempPotentialEvaporation_WaterOnGround_m * Ratio_Hour_to_Second * input->SimulationTimeStep_sec[timeStep]);
			input->PotentialEvaporation_WaterOnGround_m.push_back(tempPotentialEvaporation_WaterOnGround_m);

			getline(readEvaporation, temp, ',');
			istringstream ssPetree(temp);
			ssPetree >> tempPotentialEvaporation_WaterOnTree_m;
			//Multiplying tempPotentialEvaporation_WaterOnTree_m (m) by TimeStep_sec and Ratio_Hour_to_Second
			tempPotentialEvaporation_WaterOnTree_m = (tempPotentialEvaporation_WaterOnTree_m * Ratio_Hour_to_Second * input->SimulationTimeStep_sec[timeStep]);
			input->PotentialEvaporation_WaterOnTree_m.push_back(tempPotentialEvaporation_WaterOnTree_m);

			getline(readEvaporation, temp, ',');
			istringstream ssPesnow(temp);
			ssPesnow >> tempPotentialSublimation_SnowOnGround_m;
			//Multiplying tempPotentialSublimation_SnowOnGround_m (m) by TimeStep_sec and Ratio_Hour_to_Second
			tempPotentialSublimation_SnowOnGround_m = (tempPotentialSublimation_SnowOnGround_m * Ratio_Hour_to_Second * input->SimulationTimeStep_sec[timeStep]);
			input->PotentialSublimation_SnowOnGround_m.push_back(tempPotentialSublimation_SnowOnGround_m);

			//Linebreak \n is read to signify end of row
			getline(readEvaporation, temp, '\n');
			istringstream ssPetreesnow(temp);
			ssPetreesnow >> tempPotentialSublimation_SnowOnTree_m;
			//Multiplying tempPotentialSublimation_SnowOnTree_m (m) by TimeStep_sec and Ratio_Hour_to_Second
			tempPotentialSublimation_SnowOnTree_m = (tempPotentialSublimation_SnowOnTree_m * Ratio_Hour_to_Second * input->SimulationTimeStep_sec[timeStep]);
			input->PotentialSublimation_SnowOnTree_m.push_back(tempPotentialSublimation_SnowOnTree_m);
			//timeStep advanced when within simulation date range for use with input->SimulationTimeStep_sec[timeStep] 
			timeStep = timeStep + 1;

		}
		//Else If date_InputDataRow_GDH is not between SimulationDateStop_GDH and SimulationDateStart_GDH then advance to next date
		else {
			getline(readEvaporation, temp);
		}
	}	

	//If PotentialEvapotranspiration_TreeCover_m.size() <= 0 and input->SimulationStringParams["Flag_MultipleStation"] != "1" then error with inputs or expectations about simulation dates
	if (input->PotentialEvapotranspiration_TreeCover_m.size() <= 0 && input->SimulationStringParams["Flag_MultipleStation"] != "1") {
		cout << "Warning: The HydroPlusConfig.xml StartDate_YYYYMMDD and StopDate_YYYYMMDD are not within Evaporation.csv dates." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HydroPlus model needs the Evaporation.csv to contain the dates given in HydroPlusConfig.xml." << endl;
		cout << "Correction: Change the dates in Evaporation.csv or in HydroPlusConfig.xml StartDate_YYYYMMDD and StopDate_YYYYMMDD." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	readEvaporation.close();

	//saving all data to map
	vector<string> MeteorologicalVariableName_string = { "PotentialEvapotranspiration_TreeCover_m", "PotentialEvaporation_WaterOnGround_m", "PotentialEvaporation_WaterOnTree_m", "PotentialSublimation_SnowOnGround_m", "PotentialSublimation_SnowOnTree_m" };
	string 	StationID_string = ReadStationID;
	string StationID_VariableName;

	for (string name : MeteorologicalVariableName_string) {
		//key is like 05_RadiationNet_Wpm2
		StationID_VariableName = StationID_string + "_" + name;

		if (name == "PotentialEvapotranspiration_TreeCover_m") {
			EvaporationMap[StationID_VariableName]= input->PotentialEvapotranspiration_TreeCover_m;
		}
		else if (name == "PotentialEvaporation_WaterOnGround_m") {
			EvaporationMap[StationID_VariableName]= input->PotentialEvaporation_WaterOnGround_m;
		}
		else if (name == "PotentialEvaporation_WaterOnTree_m") {
			EvaporationMap[StationID_VariableName]= input->PotentialEvaporation_WaterOnTree_m;
		}
		else if (name == "PotentialSublimation_SnowOnGround_m") {
			EvaporationMap[StationID_VariableName]= input->PotentialSublimation_SnowOnGround_m;
		}
		else if (name == "PotentialSublimation_SnowOnTree_m") {
			EvaporationMap[StationID_VariableName]= input->PotentialSublimation_SnowOnTree_m;
		}
	}
	return true;
}

//Variables defined in WeatherPreprocessor.h need to be declared in WeatherPreprocessor.cpp; relates to static vs non-static function
map<string, vector<double>> WeatherPreprocessor::RadiationMap;

//readRadiationFile function reads in time series data for HydroPlus models
//Note: Header of Radiation.csv: YYYYMMDD,HH:MM:SS,Radiation_Shortwave_Direct_Wpm2(W/m^2),Radiation_Shortwave_Diffuse_Wpm2(W/m^2),Radiation_Longwave_Downwelling_Wpm2(W/m^2),Radiation_Longwave_Upwelling_Wpm2(W/m^2),Latitude_rad(Radian),HourAngle_rad(Radian),DeclinationAngle_rad(Radian)
bool WeatherPreprocessor::readRadiationFile(Inputs* input) {

	//Initialize variables
	int Date_YYYYMMDD = 0;
	double tempDirSW = 0;
	double tempDiffSW = 0;
	double tempDownLW = 0;
	double tempUpLW = 0;
	double tempLatitude = 0;
	double ntemphourAngle = 0.0;
	double ntempDecline = 0.0;
	double date_InputDataRow_HH;
	double date_InputDataRow_GDH;
	int Counter_inputRow = 0;

	string HrMinSec;
	string temphourAngle;
	string tempDecline;
	string tempStr = "";
	string temp;

	//readRadiation created as part of fstream, defined as the ReadStation_path and file name for Radiation.csv
	ifstream readRadiation(ReadStation_path + "Radiation.csv");

	//If meteorlogical input data file does not exist, then alert user and abort
	if (!readRadiation) {
		cout << "Warning: Radiation.csv could not be opened." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the Radiation.csv file is missing there is no short or longwave radiation data to drive the simulation." << endl;
		cout << "Correction: Create a Radiation.csv file, either manually or perhaps using the i-Tree WeatherPrep.exe utility." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	getline(readRadiation, tempStr);

	//numColumnsInFile within tempStr using .begin and .end, adding 1 to move from base 0 to 1 for count
	int numColumnsInFile = count(tempStr.begin(), tempStr.end(), ',') + 1;
	//Radiation_Columns_count = 9 for all Model_Selection types
	int Radiation_Columns_count = 9;
	string Radiation_Column_names = "YYYYMMDD,HH:MM:SS,Radiation_Shortwave_Direct_Wpm2(W/m^2),Radiation_Shortwave_Diffuse_Wpm2(W/m^2),Radiation_Longwave_Downwelling_Wpm2(W/m^2),Radiation_Longwave_Upwelling_Wpm2(W/m^2),Latitude_rad(Radian),HourAngle_rad(Radian),DeclinationAngle_rad(Radian)";
	//If numColumnsInFile is not equal to Evaporation_Columns_count then warn and abort
	if (numColumnsInFile != Radiation_Columns_count) {
		cout << endl;
		cout << "Error: Radiation.csv has an incorrect number of data columns." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HydroPlus model needs" << Radiation_Columns_count << "columns of data to properly assign Radiation.csv variables." << endl;
		cout << "Correction: Update the Radiation.csv file to contain the following comma separated variables:" << endl;
		cout << Radiation_Column_names << endl;
		//abort simulation due to incorrect CSV format
		abort();
	}

	while (readRadiation.good()) {
		getline(readRadiation, temp, ',');

		//temp.erase will trim leading and then trailing whitespace (optional but recommended)
		temp.erase(0, temp.find_first_not_of(" \t\r\n"));
		temp.erase(temp.find_last_not_of(" \t\r\n") + 1);

		//if (temp.empty()) then continue, skip the line if it's empty or contains only whitespace
		if (temp.empty()) {
			continue;
		}

		//Counter_inputRow advances
		Counter_inputRow = Counter_inputRow + 1;
		//istringstream stores an instance of the string temp in the new ss string
		istringstream ss(temp);
		//ss string is passed into Date_YYYYMMDD as double
		ss >> Date_YYYYMMDD;

		getline(readRadiation, temp, ',');
		istringstream time(temp);
		time >> HrMinSec;

		//If Counter_inputRow equals 1 then at start of file then check if Weather.csv first date is not greater than HydroPlusConfig.xml StartDate_YYYYMMDD
		if (Counter_inputRow == 1) {
			//Check if year matches HydroPlusConfig.xml StartDate_YYYYMMDD Year (and below if Weather.csv contains full YYYYMMDD called for in HydroPlusConfig.xml)
			//HydroPlusConfig_StartDay_str_YYYYMMDD is string form of HydroPlusConfig.xml input StartDate_YYYYMMDD
			string HydroPlusConfig_StartDay_str_YYYYMMDD = to_string(input->SimulationNumericalParams["StartDate_YYYYMMDD"]);
			//WeatherInput_StartDay_YYYY is string form of Weather.csv input date column
			string WeatherInput_StartDay_str_YYYYMMDD = to_string(Date_YYYYMMDD);
			//WeatherInput_StartDay_YYYYMMDD defined as integer of Date_YYYYMMDD input
			int WeatherInput_StartDay_YYYYMMDD = Date_YYYYMMDD;
			//HydroPlusConfig_StartDay_YYYYMMDD is integer form of substring from HydroPlusConfig_StartDay_str_YYYYMMDD, taking first 8 elements
			int HydroPlusConfig_StartDay_YYYYMMDD = atoi(HydroPlusConfig_StartDay_str_YYYYMMDD.substr(0, 8).c_str());

			//if HydroPlusConfig_StartDay_YYYYMMDD > HydroPlusConfig_StartDay_YYYYMMDD then error with inputs or expectations
			if (WeatherInput_StartDay_YYYYMMDD > HydroPlusConfig_StartDay_YYYYMMDD) {
				cout << "Warning: The Radiation.csv 1st column for date, YYYYMMDD, has a starting date of " << WeatherInput_StartDay_YYYYMMDD << " ..." << endl;
				cout << "... which is later than the HydroPlusConfig.xml parameter StartDate_YYYYMMDD, set to " << HydroPlusConfig_StartDay_YYYYMMDD << "." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: The HydroPlus model needs the Radiation.csv to contain the date given in HydroPlusConfig.xml." << endl;
				cout << "Correction: Either generate new Radiation.csv or change the StartDate_YYYYMMDD in the HydroPlusConfig.xml file." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}
		}

		//Algorithm to extract hour integer from section of temp string, used to refine location where input rows are read
		//date_InputDataRow_HH integer from string with atoi function, extracting 2 digits of temp w substr function; c_str() points to temp
		date_InputDataRow_HH = atoi(temp.substr(0, 2).c_str());
		//date_InputDataRow_GDH contains Gregorian Date and Hour, extending 8 to 10 digits by multiplication with 100, adding hour
		date_InputDataRow_GDH = (Date_YYYYMMDD * 100) + date_InputDataRow_HH;


		//If (Flag_HottestDaySimulated equals 1 & SimulationDateStop_GDH and SimulationDateStart_GDH are for hottest day) OR ..
		//... If (Flag_HottestDaySimulated equals 0 & input->["SimulationDateStop_GDH"] .. are for HydroPlusConfig input)
		//Note: If Flag_HottestDaySimulated = 1 then Weather.csv for hottest hour, with YYYYMMDD differences between stations
		//Note: Output file z_StationInfoMap.csv contains SimulationDate_Hottest_GD reporting hottest date used for each station
		if (((input->SimulationStringParams["Flag_HottestDaySimulated"] == "1") && (date_InputDataRow_GDH <= SimulationDateStop_GDH && date_InputDataRow_GDH >= SimulationDateStart_GDH)) || ((input->SimulationStringParams["Flag_HottestDaySimulated"] != "1") && (date_InputDataRow_GDH <= input->SimulationNumericalParams["SimulationDateStop_GDH"] && date_InputDataRow_GDH >= input->SimulationNumericalParams["SimulationDateStart_GDH"]))) {

			getline(readRadiation, temp, ',');
			istringstream ssDirsw(temp);
			ssDirsw >> tempDirSW;
			input->Radiation_Shortwave_Direct_Wpm2.push_back(tempDirSW);

			getline(readRadiation, temp, ',');
			istringstream ssDiffsw(temp);
			ssDiffsw >> tempDiffSW;
			input->Radiation_Shortwave_Diffuse_Wpm2.push_back(tempDiffSW);

			getline(readRadiation, temp, ',');
			istringstream ssDownlw(temp);
			ssDownlw >> tempDownLW;
			//Note: Radiation is positive downward, w/ Radiation_Longwave_Downwelling_Wpm2 (W/m2) added within net radiation balances
			input->Radiation_Longwave_Downwelling_Wpm2.push_back(tempDownLW);

			getline(readRadiation, temp, ',');
			istringstream ssUplw(temp);
			ssUplw >> tempUpLW;
			//Note: Radiation is positive downward, w/ Radiation_Longwave_Upwelling_Wpm2 (W/m2) subtracted within net radiation balances
			input->Radiation_Longwave_Upwelling_Wpm2.push_back(tempUpLW);

			//cte 2024 Refactor to take only one value, as it is constant
			//cte 2024 Use this value as backup if Albers map is not available to compute Latitude
			getline(readRadiation, temp, ',');
			istringstream latitude(temp);
			latitude >> tempLatitude;
			input->Latitude_rad.push_back(tempLatitude);

			getline(readRadiation, temp, ',');
			istringstream hourangle(temp);
			hourangle >> ntemphourAngle;
			input->HourAngle_Solar_rad.push_back(ntemphourAngle);

			//Linebreak \n is read to signify end of row
			getline(readRadiation, temp, '\n');
			istringstream tempdecline(temp);
			tempdecline >> ntempDecline;
			input->DeclinationAngle_Solar_rad.push_back(ntempDecline);
		}
		//If Flag_HottestDaySimulated != 1 or Model_Selection != SpatialTemperatureHydro && date_InputDataRow_GDH is between SimulationDateStop_GDH and SimulationDateStart_GDH then proceed to read input
		else if ((input->SimulationStringParams["Flag_HottestDaySimulated"] != "1" || input->SimulationStringParams["Model_Selection"] != "SpatialTemperatureHydro") && (date_InputDataRow_GDH <= input->SimulationNumericalParams["SimulationDateStop_GDH"] &&
			date_InputDataRow_GDH >= input->SimulationNumericalParams["SimulationDateStart_GDH"])) {

			getline(readRadiation, temp, ',');
			istringstream ssDirsw(temp);
			ssDirsw >> tempDirSW;
			input->Radiation_Shortwave_Direct_Wpm2.push_back(tempDirSW);

			getline(readRadiation, temp, ',');
			istringstream ssDiffsw(temp);
			ssDiffsw >> tempDiffSW;
			input->Radiation_Shortwave_Diffuse_Wpm2.push_back(tempDiffSW);

			getline(readRadiation, temp, ',');
			istringstream ssDownlw(temp);
			ssDownlw >> tempDownLW;
			//Note: Radiation is positive downward, w/ Radiation_Longwave_Downwelling_Wpm2 (W/m2) added within net radiation balances
			input->Radiation_Longwave_Downwelling_Wpm2.push_back(tempDownLW);

			getline(readRadiation, temp, ',');
			istringstream ssUplw(temp);
			ssUplw >> tempUpLW;
			//Note: Radiation is positive downward, w/ Radiation_Longwave_Upwelling_Wpm2 (W/m2) subtracted within net radiation balances
			input->Radiation_Longwave_Upwelling_Wpm2.push_back(tempUpLW);

			//cte 2024 Refactor to take only one value, as it is constant
			//cte 2024 Use this value as backup if Albers map is not available to compute Latitude
			getline(readRadiation, temp, ',');
			istringstream latitude(temp);
			latitude >> tempLatitude;
			input->Latitude_rad.push_back(tempLatitude);

			getline(readRadiation, temp, ',');
			istringstream hourangle(temp);
			hourangle >> ntemphourAngle;
			input->HourAngle_Solar_rad.push_back(ntemphourAngle);

			//Linebreak \n is read to signify end of row
			getline(readRadiation, temp, '\n');
			istringstream tempdecline(temp);
			tempdecline >> ntempDecline;
			input->DeclinationAngle_Solar_rad.push_back(ntempDecline);

		}
		//Else If date_InputDataRow_GDH is not between SimulationDateStop_GDH and SimulationDateStart_GDH then advance to next date
		else {
			getline(readRadiation, temp, '\n');
		}
	}

	//If Radiation_Shortwave_Direct_Wpm2.size() <= 0 and input->SimulationStringParams["Flag_MultipleStation"] != "1" then error with inputs or expectations about simulation dates
	if (input->Radiation_Shortwave_Direct_Wpm2.size() <= 0 && input->SimulationStringParams["Flag_MultipleStation"] != "1") {
		cout << "Warning: The HydroPlusConfig.xml StartDate_YYYYMMDD and StopDate_YYYYMMDD are not within Radiation.csv dates." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HydroPlus model needs the Radiation.csv to contain the dates given in HydroPlusConfig.xml." << endl;
		cout << "Correction: Change the dates in Radiation.csv or in HydroPlusConfig.xml StartDate_YYYYMMDD and StopDate_YYYYMMDD." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	readRadiation.close();

	//MeteorologicalVariableName_string string vector defined with variable names in header row of Radiation.csv file
	vector<string> MeteorologicalVariableName_string = { "Radiation_Shortwave_Direct_Wpm2", "Radiation_Shortwave_Diffuse_Wpm2", "Radiation_Longwave_Downwelling_Wpm2", "Radiation_Longwave_Upwelling_Wpm2", 
		"Latitude_rad", "HourAngle_Solar_rad", "DeclinationAngle_Solar_rad"};
	//StationID_string string defined with OBJECTID_shortest[i], StationID for nearest station
	string StationID_string = ReadStationID;
	string StationID_VariableName;

	//For Loop through MeteorologicalVariableName_string
	for (string Variable_name : MeteorologicalVariableName_string) {
		//StationID_VariableName is appended StationID_string and Variable_name, e.g., 05_DirectShortWaveRadiation
		StationID_VariableName = StationID_string + "_" + Variable_name;

		//If Variable_name equals Radiation_Shortwave_Direct_Wpm2 then
		if (Variable_name == "Radiation_Shortwave_Direct_Wpm2") {
			//RadiationMap[StationID_VariableName] equals Radiation_Shortwave_Direct_Wpm2
			RadiationMap[StationID_VariableName]= input->Radiation_Shortwave_Direct_Wpm2;
		}
		else if (Variable_name == "Radiation_Shortwave_Diffuse_Wpm2") {
			RadiationMap[StationID_VariableName]= input->Radiation_Shortwave_Diffuse_Wpm2;
		}
		else if (Variable_name == "Radiation_Longwave_Downwelling_Wpm2") {
			RadiationMap[StationID_VariableName]= input->Radiation_Longwave_Downwelling_Wpm2;
		}
		else if (Variable_name == "Radiation_Longwave_Upwelling_Wpm2") {
			RadiationMap[StationID_VariableName]= input->Radiation_Longwave_Upwelling_Wpm2;
		}
		else if (Variable_name == "Latitude_rad") {
			RadiationMap[StationID_VariableName]= input->Latitude_rad;
		}
		else if (Variable_name == "HourAngle_Solar_rad") {
			RadiationMap[StationID_VariableName]= input->HourAngle_Solar_rad;
		}
		else if (Variable_name == "DeclinationAngle_Solar_rad") {
			RadiationMap[StationID_VariableName]= input->DeclinationAngle_Solar_rad;
		}
	}
	
return true;
}

//resizeWeatherVectors function will size the vectors to the TotalTimeSteps, filling them with NODATA value of -9999
void WeatherPreprocessor::resizeWeatherVectors(Inputs* input) {
	//Variable vectors are resized based on SimulationNumericalParams["TotalTimeSteps"], containing value -9999 which is NODATA
	//Note: .resize function only needed when vectors are filled without .pushback function
	input->RadiationNet_Wpm2.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->Tair_F.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->Tdew_F.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->WindSpd_mps.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->WindDir_deg.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->AtmPres_kPa.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->Precip_mpdt.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->Rain_m.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->Snow_m.resize(input->SimulationTimePeriod_timeSteps, -9999);

	input->Radiation_Shortwave_Direct_Wpm2.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->Radiation_Shortwave_Diffuse_Wpm2.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->Radiation_Longwave_Downwelling_Wpm2.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->Radiation_Longwave_Upwelling_Wpm2.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->Latitude_rad.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->HourAngle_Solar_rad.resize(input->SimulationTimePeriod_timeSteps, -9999);
	input->DeclinationAngle_Solar_rad.resize(input->SimulationTimePeriod_timeSteps, -9999);

	//If Flag_ReadEvaporationData = 1 and SimulationStringParams["Model_Selection"] != SpatialTemperatureHydro then proceed to call readEvaporationFile function
	//Note: Vectors should be zero size in order to compute evaporation data in PotentialEvaporationCalc::Method_PenmanMonteith function
	if (input->SimulationStringParams["Flag_ReadEvaporationData"] == "1" && input->SimulationStringParams["Model_Selection"] != "SpatialTemperatureHydro") {
		input->PotentialEvapotranspiration_TreeCover_m.resize(input->SimulationTimePeriod_timeSteps, -9999);
		input->PotentialEvaporation_WaterOnGround_m.resize(input->SimulationTimePeriod_timeSteps, -9999);
		input->PotentialEvaporation_WaterOnTree_m.resize(input->SimulationTimePeriod_timeSteps, -9999);
		input->PotentialSublimation_SnowOnGround_m.resize(input->SimulationTimePeriod_timeSteps, -9999);
		input->PotentialSublimation_SnowOnTree_m.resize(input->SimulationTimePeriod_timeSteps, -9999);
	}
}

//clearWeatherVectors function will clear the vectors between station calls
void WeatherPreprocessor::clearWeatherVectors(Inputs* input) {
	//Variable vectors are resized based on SimulationNumericalParams["TotalTimeSteps"], containing value -9999 which is NODATA
	//Note: .resize function only needed when vectors are filled without .pushback function

	//Time vectors should likely be cleared before filled by any weather station in multiple station calls
	input->SimulationTime_HMS.clear();
	input->SimulationDate_GD.clear();
	input->SimulationDate_GDH.clear();
	input->SimulationDate_HH.clear();
	input->SimulationTimeStep_sec.clear();

	//Meterological vectors need to be cleared before filled by any weather station in multiple station calls
	input->RadiationNet_Wpm2.clear();
	input->Tair_F.clear();
	input->Tdew_F.clear();
	input->WindSpd_mps.clear();
	input->WindDir_deg.clear();
	input->AtmPres_kPa.clear();
	input->Precip_mpdt.clear();
	input->Rain_m.clear();
	input->Snow_m.clear();

	input->Radiation_Shortwave_Direct_Wpm2.clear();
	input->Radiation_Shortwave_Diffuse_Wpm2.clear();
	input->Radiation_Longwave_Downwelling_Wpm2.clear();
	input->Radiation_Longwave_Upwelling_Wpm2.clear();
	input->Latitude_rad.clear();
	input->HourAngle_Solar_rad.clear();
	input->DeclinationAngle_Solar_rad.clear();

	input->PotentialEvapotranspiration_TreeCover_m.clear();
	input->PotentialEvaporation_WaterOnGround_m.clear();
	input->PotentialEvaporation_WaterOnTree_m.clear();
	input->PotentialSublimation_SnowOnGround_m.clear();
	input->PotentialSublimation_SnowOnTree_m.clear();
}

//Variables defined in WeatherPreprocessor.h need to be declared in WeatherPreprocessor.cpp; relates to static vs non-static function
vector<long double>  WeatherPreprocessor::Weight_StationInverseDistance_vec_frac, WeatherPreprocessor::Distance_InverseSquared_vec;
map<string, vector<string> > WeatherPreprocessor::StationInfoMap;
map<int, map<int, vector<string> > >  WeatherPreprocessor::PixelNearestStation;
//int WeatherPreprocessor::Date_Hottest_YYYYMMDD;
string WeatherPreprocessor::ReadStation_path, WeatherPreprocessor::ReadStationID;
//StationMapID vector will contain string integers as ID to each weather station that has influence across maps used in simulation
//vector<string> WeatherPreprocessor::StationMapID;
long double  AlbersEasting,AlbersNorthing;

//CalcWeightsandRead function is called to compute the distance based weight between pixel and weather station
//Note: Flag_MultipleStation = 1 computes distance between stations and map pixels that use the same projection, e.g., Albers_Conical_Equal_Area, Lambert_Azimuthal_Equal_Area, or UTM
void WeatherPreprocessor::CalcWeightsandRead(long double PixelCenter_easting_m, long double PixelCenter_northing_m, Inputs* input, int MapPixel_ID) {

	//WeatherStationDistance_sort function is called get the station distance vector for the pixel center (units convert between decimal degree and radians)
	WeatherPreprocessor::WeatherStationDistance_sort(input, PixelCenter_easting_m, PixelCenter_northing_m);
	//Power_InverseDistance variable defined with SimulationNumericalParams["Power_InverseDistance"]
	long double Power_InverseDistance = input->SimulationNumericalParams["Power_InverseDistance"];
	long double Weight_StationInverseDistance_frac, Distance_InverseSquared;
	//For Loop through SimulationNumericalParams["WeatherStationNum"]
	for (int i = 0; i < input->SimulationNumericalParams["WeatherStationNum"]; i++) {
		//Distance_InverseSquared computed as 1 / (Distance_shortest)^2
		Distance_InverseSquared = 1.0 / pow(Distance_shortest[i], Power_InverseDistance);
		//Distance_InverseSquared_vec vector receives Distance_InverseSquared 
		Distance_InverseSquared_vec.push_back (Distance_InverseSquared);
	}
	//Distance_InverseSquared_sum is summation of all Distance_InverseSquared values in vector
	long double Distance_InverseSquared_sum = accumulate(Distance_InverseSquared_vec.begin(), Distance_InverseSquared_vec.end(), 0.0);

	//For Loop through SimulationNumericalParams["WeatherStationNum"]
	for (int i = 0; i < input->SimulationNumericalParams["WeatherStationNum"]; i++) {
		//Weight_StationInverseDistance_frac (fraction) is the ratio of the Distance_InverseSquared_vec value over the Distance_InverseSquared_sum
		Weight_StationInverseDistance_frac = Distance_InverseSquared_vec[i] / Distance_InverseSquared_sum;
		//Weight_StationInverseDistance_vec_frac vector receives Weight_StationInverseDistance_frac
		Weight_StationInverseDistance_vec_frac.push_back(Weight_StationInverseDistance_frac);

		//If conditional met (StationInfoMap.find(OBJECTID_shortest[i]) == StationInfoMap.end()) then OBJECTID_shortest[i] not in StationInfoMap, and new item is added
		if (StationInfoMap.find(OBJECTID_shortest[i]) == StationInfoMap.end()) {
			//ReadStation_path is set to Path_folder_shortest
			ReadStation_path = Path_folder_shortest[i];
			//ReadStationID is set to OBJECTID_shortest[i]
			ReadStationID = OBJECTID_shortest[i];

			//clearWeatherVectors called between stations to clear vectors of prior data
			clearWeatherVectors(input);

			//Call functions to read meteorological data readWeatherFile, readRadiationFile, and possibly readEvaporationFile
			readWeatherFile(input);
			readRadiationFile(input);

			//cte 2024 Debug to determine why this does not work for CoolAir_MultiQ01 TestCase
			//HydroPlusConfig_StopDay_str_YYYYMMDD is string form of HydroPlusConfig.xml input StopDate_YYYYMMDD
			/*string HydroPlusConfig_StopDay_str_YYYYMMDD = to_string(input->SimulationNumericalParams["StopDate_YYYYMMDD"]);
			//HydroPlusConfig_StopDay_YYYYMMDD is integer form of substring from HydroPlusConfig_StopDay_str_YYYYMMDD, taking first 8 elements
			int HydroPlusConfig_StopDay_YYYYMMDD = atoi(HydroPlusConfig_StopDay_str_YYYYMMDD.substr(0, 8).c_str());
			//WeatherInput_StopDay_YYYYMMDD is integer form of Weather.csv input vector last item
			int WeatherInput_StopDay_YYYYMMDD = SimulationDate_GD.back();

			//If WeatherInput_StopDay_YYYYMMDD < HydroPlusConfig_StopDay_YYYYMMDD then alert user and abort
			if (WeatherInput_StopDay_YYYYMMDD < HydroPlusConfig_StopDay_YYYYMMDD) {
				cout << "Warning: The Weather.csv 1st column for date, YYYYMMDD, has a ending date of " << WeatherInput_StopDay_YYYYMMDD << " ..." << endl;
				cout << "... which is earlier than the HydroPlusConfig.xml parameter StopDate_YYYYMMDD, set to " << HydroPlusConfig_StopDay_YYYYMMDD << "." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: The HydroPlus model needs Weather.csv to contain all dates up to the StopDate_YYYYMMDD." << endl;
				cout << "Correction: Either generate new Weather.csv or change the StopDate_YYYYMMDD in the HydroPlusConfig.xml file." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}*/

			//Check if Radiation and Weather files have same length
			//Length_RadiationInputs is integer length of 1st variable from Radiation.csv
			int Length_RadiationInputs = input->Radiation_Shortwave_Direct_Wpm2.size();
			//Length_WeatherInputs is integer length of 1st variable from Weather.csv
			int Length_WeatherInputs = input->Tair_F.size();
			//If Length_WeatherInputs not equal to Length_RadiationInputs then alert user and abort
			if (Length_WeatherInputs != Length_RadiationInputs) {
				cout << "Warning: The Weather.csv has " << Length_WeatherInputs << " items, but Radiation.csv has " << Length_RadiationInputs << " items." << endl;
				cout << "This program cannot verify if they are correct datasets properly overlapping in time." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: The HydroPlus model needs Weather.csv and Radiation.csv to contain identical rows of data." << endl;
				cout << "Correction: Generate new Weather.csv or Radiation.csv data, or make those datasets have the same row count and dates." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}

			//If Flag_ReadEvaporationData = 1 and SimulationStringParams["Model_Selection"] != SpatialTemperatureHydro then proceed to call readEvaporationFile function
			if (input->SimulationStringParams["Flag_ReadEvaporationData"] == "1" && input->SimulationStringParams["Model_Selection"] != "SpatialTemperatureHydro") {
				readEvaporationFile(input);

				//cte 2024 Debug to determine why this does not work for CoolRiver_Unsteady TestCase
				//Check if Evaporation and Weather files have same length
				//Length_EvaporationInputs is integer length of 1st variable from Evaporation.csv
				/*int Length_EvaporationInputs = PotentialEvapotranspiration_TreeCover_m.size();
				//Length_WeatherInputs is integer length of 1st variable from Weather.csv
				int Length_WeatherInputs = Tair_F.size();

				//If Length_WeatherInputs not equal to Length_EvaporationInputs then alert user and abort
				if (Length_WeatherInputs != Length_EvaporationInputs) {
					cout << "Warning: The Weather.csv has " << Length_WeatherInputs << " items, but Evaporation.csv has " << Length_EvaporationInputs << " items." << endl;
					cout << "This program cannot verify if they are correct datasets properly overlapping in time." << endl;
					cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
					cout << "Explanation: The HydroPlus model needs Weather.csv and Evaporation.csv to contain identical rows of data." << endl;
					cout << "Correction: Generate new Weather.csv or Evaporation.csv data, or make those datasets have the same row count and dates." << endl;
					//Call abort function, which ends the HydroPlus.exe simulation
					abort();
				}
				*/
			}
			
			//If SimulationStringParams["Flag_HottestDaySimulated"] equals 1 then create vector with instructions to simulate hottest day
			if (input->SimulationStringParams["Flag_HottestDaySimulated"] == "1") {
				//StationInfoMap[OBJECTID_shortest[i]] is created, contianing data to map once the station is read
				//Note: Vector contains 2 additional variables, SimulationDate_Hottest_GD[i]), SimulationTime_Hottest_HMS[i] 
				StationInfoMap[OBJECTID_shortest[i]] = { USAF_shortest[i],WBAN_shortest[i],to_string(Easting_shortest[i]),to_string(Northing_shortest[i]),
					to_string(Elev_m_shortest[i]), to_string(Slope_deg_shortest[i]), to_string(Aspect_deg_shortest[i]),TI_option_shortest[i],
					NLCD_Class_shortest[i], to_string(TC_per_shortest[i]), to_string(IC_per_shortest[i]),to_string(SVeg_per_shortest[i]),
					to_string(Soil_per_shortest[i]), to_string(Water_per_shortest[i]), to_string(SimulationDate_Hottest_GD), SimulationTime_Hottest_HMS};
			}
			//Else SimulationStringParams["Flag_HottestDaySimulated"] equals 0 then create vector without instructions to simulate hottest day
			else {
				//StationInfoMap[OBJECTID_shortest[i]] is created, contianing data to map once the station is read
				//Note: Vector does not contain 2 variables, SimulationDate_Hottest_GD[i]), SimulationTime_Hottest_HMS[i] 
				StationInfoMap[OBJECTID_shortest[i]] = { USAF_shortest[i],WBAN_shortest[i],to_string(Easting_shortest[i]),to_string(Northing_shortest[i]),
					to_string(Elev_m_shortest[i]), to_string(Slope_deg_shortest[i]), to_string(Aspect_deg_shortest[i]),TI_option_shortest[i],
					NLCD_Class_shortest[i], to_string(TC_per_shortest[i]), to_string(IC_per_shortest[i]),to_string(SVeg_per_shortest[i]),
					to_string(Soil_per_shortest[i]), to_string(Water_per_shortest[i]), "NA(Flag_HottestDaySimulated turned off)", "NA(Flag_HottestDaySimulated turned off)"};
			}
		}
	}	

	//temp_subPixelNearestStationMap vector created as temporary storage for weather station data
	//Note: temp_subPixelNearestStationMap[a][b], where [a] contains 1 of 3 stations, and [b] contains station ID, distance, and weight
	map<int, vector<string> > temp_subPixelNearestStationMap;
	//For Loop through WeatherStationNum to populate temp_subPixelNearestStationMap with ...
	for (int i = 0; i < input->SimulationNumericalParams["WeatherStationNum"]; i++) {
		//temp_subPixelNearestStationMap receives string for OBJECTID_shortest, containing weather station ID
		temp_subPixelNearestStationMap[i].push_back(OBJECTID_shortest[i]);
		//temp_subPixelNearestStationMap receives string for Distance_shortest, containing weather station distance
		temp_subPixelNearestStationMap[i].push_back(to_string(Distance_shortest[i]));
		//temp_subPixelNearestStationMap receives string for Weight_StationInverseDistance_vec_frac, containing weather station weight
		temp_subPixelNearestStationMap[i].push_back(to_string(Weight_StationInverseDistance_vec_frac[i]));
	}
	//PixelNearestStation[MapPixel_ID][0,1,2] receives data from vector temp_subPixelNearestStationMap
	//Note: PixelNearestStation[MapPixel_ID][a][b], where [a] contains 1 of 3 stations, and [b] contains station ID, distance, and weight
	PixelNearestStation[MapPixel_ID] = temp_subPixelNearestStationMap;

	//For Loop through WeatherStationNum to clear the temporary vector temp_subPixelNearestStationMap
	for (int i = 0; i < input->SimulationNumericalParams["WeatherStationNum"]; i++) {
		//temp_subPixelNearestStationMap emptied with C++ clear command
		temp_subPixelNearestStationMap[i].clear();
	}

	//Vectors will contain each map pixel, and contain data for stations with the shortest distance and best weights
	//Vectors are cleared at start
	Distance_shortest.clear();
	OBJECTID_shortest.clear();
	USAF_shortest.clear();
	WBAN_shortest.clear();
	Easting_shortest.clear();
	Northing_shortest.clear();
	Elev_m_shortest.clear();
	Slope_deg_shortest.clear();
	Aspect_deg_shortest.clear();
	TI_option_shortest.clear();
	NLCD_Class_shortest.clear();
	TC_per_shortest.clear();
	IC_per_shortest.clear();
	SVeg_per_shortest.clear();
	Soil_per_shortest.clear();
	Water_per_shortest.clear();
	Path_folder_shortest.clear();
	Distance_InverseSquared_vec.clear();
	Weight_StationInverseDistance_vec_frac.clear();
}

//Variables defined in WeatherPreprocessor.h need to be declared in WeatherPreprocessor.cpp; relates to static vs non-static function
long double WeatherPreprocessor::PixelCenter_X_MapLowerLeft_m, WeatherPreprocessor::PixelCenter_Y_MapLowerLeft_m;
vector<long double> WeatherPreprocessor::Map_Easting_X_m_vec, WeatherPreprocessor::Map_Northing_Y_m_vec;

//get the (x,y) for each pixel center, then calculate weight and read weather data
void WeatherPreprocessor::MapPixelCalcWeightsandRead(Inputs* input) {
	
	//1. To get corner location, left lower pixel
	//PixelCenter_X_MapLowerLeft_m is defined as pixel center, function of map input xllcorner + 0.5 * foldersize, where foldersize is pixel size
	PixelCenter_X_MapLowerLeft_m = input->xllcorner + 0.5 * input->foldersize;
	PixelCenter_Y_MapLowerLeft_m = input->yllcorner + 0.5 * input->foldersize;

	//2. For each pixel, get Latitude_DecimalDegree, Longitude_DecimalDegree location
	for (int MapPixel_ID = 0; MapPixel_ID < input->LocationParams["nCols"] * input->LocationParams["nRows"]; ++MapPixel_ID) {
		
		//If input vector NLCDLandCoverData at MapPixel_ID is not equal to NODATA, then enter for analysis
		//Note: Conditional required to avoid entering for SpatialTemperatureHydro for NODATA, which has no DataFolder 
		if (input->NLCDLandCoverData[MapPixel_ID] != input->LocationParams["NODATA"]) {
			//Note: Conversion between MapPixel_ID and row & column pair, with MapPixel_ID starting at 0, row & col starting at 1
			//Note: Conversion Eq: row=(int)(MapPixel_ID/input->ncols+1); col=(MapPixel_ID%(int)(input->ncols)+1); MapPixel_ID = (row * nCols) + col
			int row = (int)(MapPixel_ID / (int)(input->LocationParams["nCols"]) + 1);
			int col = (MapPixel_ID % (int)(input->LocationParams["nCols"]) + 1);

			//another option, calc x, y first, then call UTMXYToLatLon just for the 1st pixel
			//Longitude_DecimalDegree = lllon + (col - 1) * input->foldersize / meterofdgree;
			//Latitude_DecimalDegree = lllat + (row - 1) * input->foldersize / meterofdgree;

			//Map_Northing_Y_m (m) is defined as PixelCenter_Y_MapLowerLeft_m + incremental row progression, where foldersize is pixel cellsize
			long double Map_Northing_Y_m = PixelCenter_Y_MapLowerLeft_m + ((int)(input->LocationParams["nRows"]) - row) * input->foldersize;
			//Map_Easting_X_m (m) is defined as PixelCenter_X_MapLowerLeft_m + incremental col progression, where foldersize is pixel cellsize
			long double Map_Easting_X_m = PixelCenter_X_MapLowerLeft_m + (col - 1) * input->foldersize;

			//CalcWeightsandRead function called with decimal degree data to compute station spatial weights
			//CalcWeightsandRead(Latitude_DecimalDegree, Longitude_DecimalDegree, input, MapPixel_ID);
			CalcWeightsandRead(Map_Easting_X_m, Map_Northing_Y_m, input, MapPixel_ID);
			//Map_Easting_X_m_vec (m) contains vector of Map_Easting_X_m (m)
			Map_Easting_X_m_vec.push_back(Map_Easting_X_m);
			//Map_Northing_Y_m_vec (m) contains vector of Map_Northing_Y_m (m)
			Map_Northing_Y_m_vec.push_back(Map_Northing_Y_m);
		}
	}
}

int WeatherPreprocessor::centerMapPixel_ID;

//get the (x,y) for map center pixel, then calculate weight and read weather data
void WeatherPreprocessor::MapCenterCalcWeightsandRead(Inputs* input) {
	//center_East_m (m) equals xllcorner + 0.5 * nCols * cellsize
	long double center_East_m = input->LocationParams["xllcorner"] + 0.5 * input->LocationParams["nCols"] * input->LocationParams["cellsize"];
	//center_North_m (m) equals yllcorner + 0.5 * nRows * cellsize
	long double center_North_m = input->LocationParams["yllcorner"] + 0.5 * input->LocationParams["nRows"] * input->LocationParams["cellsize"];
	//calculatethe center pixel ID
	centerMapPixel_ID = (input->LocationParams["nRows"] / 2) * input->LocationParams["nCols"] + (input->LocationParams["nCols"] / 2);
	//CalcWeightsandRead function called with decimal degree data to compute station spatial weights
	CalcWeightsandRead(center_East_m, center_North_m, input, centerMapPixel_ID);
	// Store the center coordinates in the vectors
	Map_Easting_X_m_vec.push_back(center_East_m);
	Map_Northing_Y_m_vec.push_back(center_North_m);
}


//WeatherDataReadandProcessing function will determine pixel centers and distance to weather stations
//Note: Flag_MultipleStation = 1 computes distance between stations and map pixels that use the same projection, e.g., Albers_Conical_Equal_Area, Lambert_Azimuthal_Equal_Area, or UTM
//Note: This is run before simulating time steps and pixel loops
void WeatherPreprocessor::WeatherDataReadandProcessing(Inputs* input) {
	
	//If Model_Selection equals SpatialTemperatureHydro then 
	if (input->SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro") {
		//For each pixel, get Latitude_DecimalDegree, Longitude_DecimalDegree location, get the distance and weight, then read weather file
		MapPixelCalcWeightsandRead(input);
	}
	//Else If Model_Selection equals StatisticalHydro then 
	else if (input->SimulationStringParams["Model_Selection"] == "StatisticalHydro") {
		//For DEM.asc map, centroid from header data xllcorner + 0.5 * ncols * cellsize, and yllcorner + 0.5 * nrows * cellsize, then read weather file
		MapCenterCalcWeightsandRead(input);
	}
	//MapPixelNearestWeatherStationWriter function called to Write output of weather preprocessor
	MapPixelNearestWeatherStationWriter(input);
	//WeatherStationInformationWriter function called to Write output of weather preprocessor
	WeatherStationInformationWriter(input);
	//resizeWeatherVectors function called to resize weather vectors to the size of total time step
	resizeWeatherVectors(input);
}

//WeightedInputMap defined within WeatherPreprocessor::WeightedInputMap structure, after declaration in WeatherPreprocessor.h
map<int, map<string, vector<double> > > WeatherPreprocessor::WeightedInputMap;
//ApplyingWeights_to_pixel_at_timestep function computes spatially weighted values for each MapPixel_ID
void WeatherPreprocessor::ApplyingWeights_to_pixel_at_timestep(Inputs* input, int MapPixel_ID, int timeStep) {
	//MeteorologicalVariableName_Weather_string string contains variable names listed in header column of Weather.csv 
	vector<string> MeteorologicalVariableName_Weather_string = { "Tair_F", "Tdew_F", "RadiationNet_Wpm2", "WindSpd_mps", "AtmPres_kPa", "Precip_mpdt","Rain_m", "Snow_m","TotalPrecip_m", "TotalRain_m", "TotalSnow_m" };
	//MeteorologicalVariableName_Evaporation_string string contains variable names listed in header column of Evaporation.csv 
	vector<string> MeteorologicalVariableName_Evaporation_string = { "PotentialEvapotranspiration_TreeCover_m", "PotentialEvaporation_WaterOnGround_m", 
		"PotentialEvaporation_WaterOnTree_m", "PotentialSublimation_SnowOnGround_m", "PotentialSublimation_SnowOnTree_m" };
	//MeteorologicalVariableName_Radiation_string string contains variable names listed in header column of Radiation.csv 
	vector<string> MeteorologicalVariableName_Radiation_string = { "Radiation_Shortwave_Direct_Wpm2", "Radiation_Shortwave_Diffuse_Wpm2", 
		"Radiation_Longwave_Downwelling_Wpm2", "Radiation_Longwave_Upwelling_Wpm2", "Latitude_rad", "HourAngle_Solar_rad", "DeclinationAngle_Solar_rad" };
	
	//If Model_Selection is CoolBuilding then add WindDir_deg to list
	if (input->SimulationStringParams["Model_Selection"] == "CoolBuilding") {
		//MeteorologicalVariableName_Weather_string push_back WindDir_deg
		MeteorologicalVariableName_Weather_string.push_back("WindDir_deg");
	}

	/*cte 2025
	input->Tair_F.clear();
	input->Tdew_F.clear();
	input->RadiationNet_Wpm2.clear();
	input->WindSpd_mps.clear();
	input->WindDir_deg.clear();
	input->AtmPres_kPa.clear();
	input->Precip_mpdt.clear();
	input->Rain_m.clear();
	input->Snow_m.clear();
	*/

	//1. applying weights to weather data and saving to map WeightedInputMap
	//For Loop with string name through MeteorologicalVariableName_Weather_string vector list
	for (string Station_Variable_name : MeteorologicalVariableName_Weather_string) {
		//MeteorologicalVariable_weightedSum initialized to 0
		double MeteorologicalVariable_weightedSum = 0;
		double MeteorologicalVariable_Total_weightedSum = 0;
		//For Loop with integer i through WeatherStationNum
		for (int i = 0; i < input->SimulationNumericalParams["WeatherStationNum"]; i++) {
			//StationID_string defined as PixelNearestStation[MapPixel_ID][i][0], which is station i's StationID
			string StationID_string = PixelNearestStation[MapPixel_ID][i][0];
			//StationID_VariableName defined as combination of keynamepart1_name and Station_Variable_name
			string StationID_VariableName = StationID_string + "_" + Station_Variable_name;

			//STID_TotalPrecip_m just have one value in this vector
			if (Station_Variable_name == "TotalPrecip_m" || Station_Variable_name == "TotalRain_m" || Station_Variable_name == "TotalSnow_m") {
				MeteorologicalVariable_weightedSum = MeteorologicalVariable_weightedSum + WeatherMap[StationID_VariableName][0] * stod(PixelNearestStation[MapPixel_ID][i][2]);
			}
			else {
				//MeteorologicalVariable_weightedSum is added to MeteorologicalVariable_weightedSum and product of WeatherMap[StationID_VariableName] Value at timeStep and Station Weight, contained in PixelNearestStation[MapPixel_ID][i][2]
				MeteorologicalVariable_weightedSum = MeteorologicalVariable_weightedSum + WeatherMap[StationID_VariableName][timeStep] * stod(PixelNearestStation[MapPixel_ID][i][2]);
			}
		}
		//cte 202407 Consider refactor for MultipleWeatherStation=1 to replace input->Vars with WeightedInputMap[MapPixel_ID][Station_Variable_name] ...
		//cte ... ShortVegInterceptionCalc.cpp, ..., PotentialEvaporationCalc.cpp, ..., etc., 
	
		//If Station_Variable_name equals Tair_F then
		if (Station_Variable_name == "Tair_F") {
			//Tair_F[timeStep] equals MeteorologicalVariable_weightedSum
			input->Tair_F[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "Tdew_F") {
			input->Tdew_F[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if(Station_Variable_name == "RadiationNet_Wpm2") {
			input->RadiationNet_Wpm2[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "WindSpd_mps") {
			input->WindSpd_mps[timeStep] = MeteorologicalVariable_weightedSum;
		}
		//If Model_Selection is CoolBuilding
		else if (Station_Variable_name == "WindDir_deg") {
			input->WindDir_deg[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "AtmPres_kPa") {
			input->AtmPres_kPa[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "Precip_mpdt") {
			input->Precip_mpdt[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "Rain_m") {
			input->Rain_m[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "Snow_m") {
			input->Snow_m[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "TotalPrecip_m") {
			input->TotalPrecip_m = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "TotalRain_m") {
			input->TotalRain_m = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "TotalSnow_m") {
			input->TotalSnow_m = MeteorologicalVariable_weightedSum;
		}
		WeightedInputMap[MapPixel_ID][Station_Variable_name].push_back(MeteorologicalVariable_weightedSum);
	}
	//If Flag_ReadEvaporationData = 1 and SimulationStringParams["Model_Selection"] != SpatialTemperatureHydro then proceed to call readEvaporationFile function
	if (input->SimulationStringParams["Flag_ReadEvaporationData"] == "1" && input->SimulationStringParams["Model_Selection"] != "SpatialTemperatureHydro") {
		//2. applying weights to evaporation data and saving to map WeightedInputMap
		//For Loop with string name through MeteorologicalVariableName_Evaporation_string vector list
		for (string Station_Variable_name : MeteorologicalVariableName_Evaporation_string) {
			//MeteorologicalVariable_weightedSum initialized to 0
			double MeteorologicalVariable_weightedSum = 0;
			//For Loop with integer i through WeatherStationNum
			for (int i = 0; i < input->SimulationNumericalParams["WeatherStationNum"]; i++) {
				//StationID_string defined as PixelNearestStation[MapPixel_ID][i][0], which is station i's StationID
				string StationID_string = PixelNearestStation[MapPixel_ID][i][0];
				//StationID_VariableName defined as combination of keynamepart1_name and Station_Variable_name
				string StationID_VariableName = StationID_string + "_" + Station_Variable_name;
				//MeteorologicalVariable_weightedSum is added to MeteorologicalVariable_weightedSum and product of EvaporationMap[StationID_VariableName] Value at timeStep and Station Weight, contained in PixelNearestStation[MapPixel_ID][i][2]
				MeteorologicalVariable_weightedSum = MeteorologicalVariable_weightedSum + EvaporationMap[StationID_VariableName][timeStep] * stod(PixelNearestStation[MapPixel_ID][i][2]);
			}

			//If Station_Variable_name equals PotentialEvapotranspiration_TreeCover_m then
			if (Station_Variable_name == "PotentialEvapotranspiration_TreeCover_m") {
				//PotentialEvapotranspiration_TreeCover_m[timeStep] equals MeteorologicalVariable_weightedSum
				input->PotentialEvapotranspiration_TreeCover_m[timeStep] = MeteorologicalVariable_weightedSum;
			}
			else if (Station_Variable_name == "PotentialEvaporation_WaterOnGround_m") {
				input->PotentialEvaporation_WaterOnGround_m[timeStep] = MeteorologicalVariable_weightedSum;
			}
			else if (Station_Variable_name == "PotentialEvaporation_WaterOnTree_m") {
				input->PotentialEvaporation_WaterOnTree_m[timeStep] = MeteorologicalVariable_weightedSum;
			}
			else if (Station_Variable_name == "PotentialSublimation_SnowOnGround_m") {
				input->PotentialSublimation_SnowOnGround_m[timeStep] = MeteorologicalVariable_weightedSum;
			}
			else if (Station_Variable_name == "PotentialSublimation_SnowOnTree_m") {
				input->PotentialSublimation_SnowOnTree_m[timeStep] = MeteorologicalVariable_weightedSum;
			}
			WeightedInputMap[MapPixel_ID][Station_Variable_name].push_back(MeteorologicalVariable_weightedSum);
		}
	}
	//3. applying weights to radiation data and saving to map WeightedInputMap
	//For Loop with string name through MeteorologicalVariableName_Radiation_string vector list
	for (string Station_Variable_name : MeteorologicalVariableName_Radiation_string) {
		//MeteorologicalVariable_weightedSum initialized to 0
		double MeteorologicalVariable_weightedSum = 0;
		//For Loop with integer i through WeatherStationNum
		for (int i = 0; i < input->SimulationNumericalParams["WeatherStationNum"]; i++) {
			//StationID_string defined as PixelNearestStation[MapPixel_ID][i][0], which is station i's StationID
			string StationID_string = PixelNearestStation[MapPixel_ID][i][0];
			//StationID_VariableName defined as combination of keynamepart1_name and Station_Variable_name
			string StationID_VariableName = StationID_string + "_" + Station_Variable_name;
			//MeteorologicalVariable_weightedSum is added to MeteorologicalVariable_weightedSum and product of RadiationMap[StationID_VariableName] Value at timeStep and Station Weight, contained in PixelNearestStation[MapPixel_ID][i][2]
			MeteorologicalVariable_weightedSum = MeteorologicalVariable_weightedSum + RadiationMap[StationID_VariableName][timeStep] * stod(PixelNearestStation[MapPixel_ID][i][2]);
		}
		//If Station_Variable_name equals Radiation_Shortwave_Direct_Wpm2 then
		if (Station_Variable_name == "Radiation_Shortwave_Direct_Wpm2") {
			//Radiation_Shortwave_Direct_Wpm2[timeStep] equals MeteorologicalVariable_weightedSum
			input->Radiation_Shortwave_Direct_Wpm2[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "Radiation_Shortwave_Diffuse_Wpm2") {
			input->Radiation_Shortwave_Diffuse_Wpm2[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "Radiation_Longwave_Downwelling_Wpm2") {
			input->Radiation_Longwave_Downwelling_Wpm2[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "Radiation_Longwave_Upwelling_Wpm2") {
			input->Radiation_Longwave_Upwelling_Wpm2[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "Latitude_rad") {
			input->Latitude_rad[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "HourAngle_Solar_rad") {
			input->HourAngle_Solar_rad[timeStep] = MeteorologicalVariable_weightedSum;
		}
		else if (Station_Variable_name == "DeclinationAngle_Solar_rad") {
			input->DeclinationAngle_Solar_rad[timeStep] = MeteorologicalVariable_weightedSum;
		}
		//WeightedInputMap[MapPixel_ID][Station_Variable_name] contains weighted average value of variable using distance to all weather stations
		//Note: WeightedInputMap[MapPixel_ID]["Radiation_Shortwave_Direct_Wpm2"][timeStep] is how it is written when accessed.
		WeightedInputMap[MapPixel_ID][Station_Variable_name].push_back(MeteorologicalVariable_weightedSum);
	}

	//4. applying weights to elevation
	//MeteorologicalVariable_weightedSum initialized to zero
	double MeteorologicalVariable_weightedSum = 0;
	//For Loop through WeatherStationNum
	for (int i = 0; i < input->SimulationNumericalParams["WeatherStationNum"]; i++) {
		//StationID_int defined as PixelNearestStation[MapPixel_ID][i][0], which is station i's StationID
		string StationID_int = PixelNearestStation[MapPixel_ID][i][0];
		//StationWeight_frac (fraction) is influence of the station on the local pixel, computed as the inverse distance squared
		double StationWeight_frac = stod(PixelNearestStation[MapPixel_ID][i][2]);
		//StationElevation_m (m) is elevation of the station 
		double StationElevation_m = stod(StationInfoMap[StationID_int][4]);

		//MeteorologicalVariable_weightedSum is added to MeteorologicalVariable_weightedSum and product of StationInfoMap [4=Elevation] and Station Weight, contained in PixelNearestStation[MapPixel_ID][StationID_string][2]
		MeteorologicalVariable_weightedSum = MeteorologicalVariable_weightedSum + StationElevation_m * StationWeight_frac;
	}
	//WeightedInputMap[MapPixel_ID]["Elevation_WeatherStations_weighted_m"] receives value, at each time step which is not needed
	WeightedInputMap[MapPixel_ID]["Elevation_WeatherStations_weighted_m"].push_back(MeteorologicalVariable_weightedSum);
}

//MapPixelNearestWeatherStationWriter function writes optional output with row for each map pixel, its nearest weather stations distances and weights
//Note: Header is: Map_pixelID, Ranking,WeatherStationID,Distance_km,Weight_frac
void WeatherPreprocessor::MapPixelNearestWeatherStationWriter( Inputs* input ) {

	//If Flag_ExtendedOutputs equals 1 then write z_WeightedInputMap.csv output, which for each pixel has time series of each weather variable
	if (input->SimulationStringParams["Flag_ExtendedOutputs"] == "1") {

		//Note: File z_NearestStationWeight.csv will contain weighted weather station data values, computed as spatially weighted averages of observed data, not computed by HeatFluxCal.cpp
		string File_NearestStationWeight_csv = input->SimulationStringParams["OutputFolder_Path"] + "z_NearestStationWeight.csv";
		ofstream outfileNearestStationWeight(File_NearestStationWeight_csv);
		if (!outfileNearestStationWeight.good()) {
			cout << "Warning: The file " << File_NearestStationWeight_csv << " could not be accessed for writing output." << endl;
			return;
		}

		//outfileNearestStationWeight file has header written
		outfileNearestStationWeight << "Map_pixelID" << "," <<" Albers_x" << ", " << "Albers_y" << ", " << " Ranking" << "," << "WeatherStationID" << "," << "Distance_km" << "," << "Weight_frac" << endl;

		//For loop with auto it from PixelNearestStation.begin() to not PixelNearestStation.end()
		auto it_x = Map_Easting_X_m_vec.begin();
		auto it_y = Map_Northing_Y_m_vec.begin();


		//For loop with it through contents of map container with key and value pairs
		for (auto it = PixelNearestStation.begin(); it != PixelNearestStation.end(); it++) {

			//Note: Variables .first and .second access key-value pairs in map or unordered_map containers
			//Note: (*it) and (*it2) are iterators pointing to elements of a map, and ...
			//Note: ... .first and .second are used to access the key and value of those elements.
			//Note: (*it).first: Accesses the key of the map element pointed to by the iterator it.
			//Note: (*it).second: Accesses the value associated with the key of the map element pointed to by the iterator it.
			//Note: (*it2).first: Accesses the key of the map element (likely an inner map) pointed to by the iterator it2.
			//Note: (*it2).second: Accesses the value associated with the key of the map element (likely an inner map) pointed to by the iterator it2.

			//For loop with auto it2 from ((*it).second).begin() to not ((*it).second).end()
			for (auto it2 = ((*it).second).begin(); it2 != ((*it).second).end(); it2++) {
				//outfileNearestStationWeight file has the following lines written ...
				outfileNearestStationWeight
					//*it.first is the map pixel ID, the vector form of map row and col numbers
					<< (*it).first << ","
					//write out Albers xy location
					<< to_string(*it_x) << ","
					<< to_string(*it_y) << ","
					//*it2.first is the station rank, with 1 being nearest
					<< (*it2).first << ","
					//*it2.second[0] is the station ID, first column in input file WeatherStationAttributeTable.csv
					<< (*it2).second[0] << ","
					//*it2.second[1] is the station distance (km)
					<< (*it2).second[1] << "," 
					//*it2.second[2] is the station weight (fraction)
					<< (*it2).second[2] << endl;
			}
			it_x++;
			it_y++;
		}
		//outfileNearestStationWeight is closed at the last pixel
		outfileNearestStationWeight.close();
	}
}

//WeatherStationInformationWriter function writes optional output with ID, location, and characteristics of weather stations
//Note: Header is: OBJECTID,USAF,WBAN,Station_Easting_m, Station_Northing_m, Station_Elev_m, Station_Slope_deg,Station_Aspect_deg,Station_TI_option,NLCD_Class,Station_TC_per,Station_IC_per,Station_SVeg_per,Station_Soil_per,Station_Water_per,SimulationDate_Hottest_GD,SimulationTime_Hottest_HMS
void WeatherPreprocessor::WeatherStationInformationWriter(Inputs* input) {

	string File_StationInfoMap_csv = input->SimulationStringParams["OutputFolder_Path"] + "z_StationInfoMap.csv";
	ofstream outfileWeatherStationInfo(File_StationInfoMap_csv);
	if (!outfileWeatherStationInfo.good()) {
		cout << "Warning: The file " << File_StationInfoMap_csv << " could not be accessed for writing output." << endl;
		return;
	}

	//outfileWeatherStationInfo file has header written
	outfileWeatherStationInfo << "OBJECTID" << "," << "USAF" << "," << "WBAN" << "," << "Easting" << ", " << "Northing" << ", " << "Elev_m" << ", "
		<< "Slope_deg" << "," << "Aspect_deg" << "," << "Station_TI_option" << "," << "NLCD_Class" << "," << "Station_TC_per" << "," << "Station_IC_per" << "," 
		<< "Station_SVeg_per" << "," << "Station_Soil_per" << "," << "Station_Water_per" << ","
		<< "SimulationDate_Hottest_GD" << "," << "SimulationTime_Hottest_HMS"  << endl;
	
	//For loop with auto it from StationInfoMap.begin() to not StationInfoMap.end(); 
	//Note: StationInfoMap contains all weather stations for simulation, listed in file identified by HydroPlusConfig.xml element WeatherStationAttributeFile 
	//Note: StationInfoMap is a map that has key value pairs, accessed in .first or .second position
	for (auto it = StationInfoMap.begin(); it != StationInfoMap.end(); it++) {
		//outfileWeatherStationInfo file has the following line written ...
		outfileWeatherStationInfo
			//(*it).first is weather station OBJECTID
			<< (*it).first;
			//For loop with auto it2 from (*it).second.begin() to not (*it).second.end()
			for (auto it2 = (*it).second.begin(); it2 != (*it).second.end(); it2++) {
				//outfileWeatherStationInfo file has all variables listed in header written ...
				outfileWeatherStationInfo
					<< "," << *it2;
			}
			outfileWeatherStationInfo << endl;
	}
	//outfileWeatherStationInfo file is closed 
	outfileWeatherStationInfo.close();
}

//MapPixelWeightedWeatherInputsWriter function writes optional output with row for each map pixel and its weighted meteorological inputs 
//Note: Each pixel has as many rows as there are meteorological variables, and each variable is written for each time step. 
//Note: Header is: Map_pixelID, Variable, TimeStep, etc. 
void WeatherPreprocessor::MapPixelWeightedWeatherInputsWriter(Inputs* input) {

	//If Flag_ExtendedOutputs equals 1 then write z_WeightedInputMap.csv output, which for each pixel has time series of each weather variable
	if (input->SimulationStringParams["Flag_ExtendedOutputs"] == "1") {

		//File_WeightedInputMap_csv defined as OutputDirectory and contains the file z_WeightedInputMap.csv
		string File_WeightedInputMap_csv = input->SimulationStringParams["OutputFolder_Path"] + "z_WeightedInputMap.csv";
		ofstream outfileInputMap(File_WeightedInputMap_csv);
		if (!outfileInputMap.good()) {
			cout << "Warning: The file " << File_WeightedInputMap_csv << " could not be accessed for writing output." << endl;
			return;
		}
		//outfileInputMap receiving header row with Map_pixelID and Variable and ...
		outfileInputMap << "Map_pixelID" << "," << " Variable" << ",";
		//For loop used to write to header TimeStep value for TotalTimeSteps
		for (int i = 0; i < input->SimulationTimePeriod_timeSteps; i++) {
			//outfileInputMap receives TimeStep header
			//Note: TimeStep is appended to i, counter of TotalTimeSteps by converting integer to string with to_string function
			outfileInputMap << "TimeStep" + to_string(i) << ",";
		}
		//outfileInputMap receives endl to close line
		outfileInputMap << endl;

		//For loop through all map pixels with auto it variable from WeightedInputMap.begin to WeightedInputMap.end
		for (auto it = WeightedInputMap.begin(); it != WeightedInputMap.end(); it++) {
			//Note: Variables .first and .second access key-value pairs in map or unordered_map containers
			//Note: (*it) and (*it2) are iterators pointing to elements of a map, and ...
			//Note: ... .first and .second are used to access the key and value of those elements.
			//Note: (*it).first: Accesses the key of the map element pointed to by the iterator it.
			//Note: (*it).second: Accesses the value associated with the key of the map element pointed to by the iterator it.
			//Note: (*it2).first: Accesses the key of the map element (likely an inner map) pointed to by the iterator it2.
			//Note: (*it2).second: Accesses the value associated with the key of the map element (likely an inner map) pointed to by the iterator it2.

			//For loop through all variables with auto it2 variable using pointer to *it.second to define begin and end of variable list
			for (auto it2 = ((*it).second).begin(); it2 != ((*it).second).end(); it2++) {
				//outfileInputMap receives content of 
				outfileInputMap
					//(*it).first is pointer to pixel ID
					<< (*it).first << ","
					//(*it2).first is pointer to variable name
					<< (*it2).first << ",";
				//For loop through all time steps to append the variable values
				for (int i = 0; i < input->SimulationTimePeriod_timeSteps; i++) {
					//outfileInputMap receives (*it2).second as pointer to variable value
					outfileInputMap << (*it2).second[i] << ",";
				}
				//outfileInputMap receives endl to close line
				outfileInputMap << endl;
			}
		}
		//outfileInputMap is closed at the last pixel
		outfileInputMap.close();
	}
}

//Tair_Mesoscale_byStationID_K and AbsHumidity_Mesoscale_byStationID_kg_p_m3 need declaration and definition
map<int, vector<double>> WeatherPreprocessor::Tair_Mesoscale_byStationID_K, WeatherPreprocessor::AbsHumidity_Mesoscale_byStationID_kg_p_m3;
//Tair_weighted_K, Tdew_weighted_K need declaration and definition
double WeatherPreprocessor::Tair_weighted_K, WeatherPreprocessor::Tdew_weighted_K, WeatherPreprocessor::Tair_mesoScale_weighted_K, 
WeatherPreprocessor::AbsHumidity_mesoScale_weighted_kg_p_m3;

//WeatherPreprocessor::ComputeSpatiallyWeightedVariables function updates Tair variables by weight and elevation
//Note: For multiple stations, WeatherProcessor::ComputeSpatiallyWeightedVariables uses ELR to update Tair_weighted_K, which updates Tair_meso_K
void WeatherPreprocessor::ComputeSpatiallyWeightedVariables(Inputs* input, int MapPixel_ID, int timeStep) {
	//Tair_mesoScale_weighted_K will contain a weighted value from multiple stations 
	Tair_mesoScale_weighted_K = 0;
	//AbsHumidity_mesoScale_weighted_kg_p_m3 will contain a weighted value from multiple stations 
	AbsHumidity_mesoScale_weighted_kg_p_m3 = 0;
	//Tair_weighted_K will contain a weighted value from multiple stations, presuming no variation in land cover between stations
	Tair_weighted_K = 0;
	//Tdew_weighted_K will contain a weighted value from multiple stations, presuming no variation in land cover between stations 
	Tdew_weighted_K = 0;
	//keyname_Tair_F and keyname_Tdew_F initialized
	string keyname_Tair_F, keyname_Tdew_F;

	//For loop through WeatherStationNum to compute the weighted mesoscale temperature and humidity
	for (int i = 0; i < input->SimulationNumericalParams["WeatherStationNum"]; i++) {
		//StationID_int contains the unique ID of the weather station used in the map area, converted from string to integer with stoi function
		int StationID_int = stoi(PixelNearestStation[MapPixel_ID][i][0]);
		//StationMapID_string contains the unique ID of the weather station used in the map area
		string StationMapID_string = PixelNearestStation[MapPixel_ID][i][0];
		//StationWeight_frac (fraction) is influence of the station on the local pixel, computed as the inverse distance squared
		double StationWeight_frac = stod(PixelNearestStation[MapPixel_ID][i][2]);

		//Consider refactor to adjust temperature to common elevation
		//Tair_mesoScale_weighted_K (K) is incrementally updated to contain the inverse distance weighted value of station Tair_mesoScale_K values
		//Note: stod(Weatherpre->PixelNearestStation[MapPixel_ID][i][2] provides the station weight [2] for the MapPixel_ID pixel, stod is string to double
		Tair_mesoScale_weighted_K = Tair_mesoScale_weighted_K + Tair_Mesoscale_byStationID_K[StationID_int][timeStep] *	StationWeight_frac;
		//AbsHumidity_mesoScale_weighted_kg_p_m3 (kg/m3) is incrementally updated to contain the inverse distance weighted value of station AbsHumidity_mesoScale_kg_p_m3 values
		//Note: stod(Weatherpre->PixelNearestStation[MapPixel_ID][i][2] provides the station weight [2] for the MapPixel_ID pixel, stod is string to double
		AbsHumidity_mesoScale_weighted_kg_p_m3 = AbsHumidity_mesoScale_weighted_kg_p_m3 + AbsHumidity_Mesoscale_byStationID_kg_p_m3[StationID_int][timeStep] * StationWeight_frac;

		//keyname_Tair_F defined for specific weather station ID with variable ID_Tair_F 
		keyname_Tair_F = StationMapID_string + "_" + "Tair_F";
		//keyname_Tdew_F defined for specific weather station ID with variable ID_Tdew_F
		keyname_Tdew_F = StationMapID_string + "_" + "Tdew_F";
		//Tair_station_K (K) defined as WeatherMap[keyname_Tair_F (F) converted to K
		double Tair_station_K = (WeatherMap[keyname_Tair_F][timeStep] - 32) * 5.0 / 9.0 + 273.15;
		//Tdew_station_K (K) defined as WeatherMap[keyname_Tdew_F (F) converted to K
		double Tdew_station_K = (WeatherMap[keyname_Tdew_F][timeStep] - 32) * 5.0 / 9.0 + 273.15;
		//Tair_weighted_K (K) is incrementally updated to contain the inverse distance weighted value of station Tair_station_K
		//Note: stod(Weatherpre->PixelNearestStation[MapPixel_ID][i][2] provides the station weight [2] for the MapPixel_ID pixel, stod is string to double
		Tair_weighted_K = Tair_weighted_K + Tair_station_K * stod(PixelNearestStation[MapPixel_ID][i][2]);
		//Tdew_weighted_K (K) is incrementally updated to contain the inverse distance weighted value of station Tdew_station_K
		//Note: stod(Weatherpre->PixelNearestStation[MapPixel_ID][i][2] provides the station weight [2] for the MapPixel_ID pixel, stod is string to double
		Tdew_weighted_K = Tdew_weighted_K + Tdew_station_K * stod(PixelNearestStation[MapPixel_ID][i][2]);
	}
	//Taken from HeatFluxCal::AdiabaticLapseRates function:
	//Note: Even if Flag_MultipleStation=1 and WeatherPreprocessor lapse rate adjusted Tair_weighted_K, HeatFluxCal lapse rate adjustment is needed 
	//Note: Function calculates the effect of elevation difference with the reference station on local cell air temperature above ground
	//Note: Function modifies Stull (2000) theory of cooling by dry adiabatic lapse rate (ALR) with environmental lapse rate (ELR) of ~5.5 C/km, ...
	//Note: Assumes climbing 1 km up mountain air on mountain cools at ELR, while air lifting 1 km above ground cools at ALR.
	//Note: Stull (2000) gives dry adiabatic lapse rate Gamma_dry_C_p_km = 9.8 C/km; dew point lapse rate Gamma_dew_C_p_km = 1.8 C/km derived Eq 5.8
	//Note: Elevation_lcl_km = 1/(9.8 - 1.8) * (Tair_K - Tdew_K), Elevation_lcl_km = lifting condensation level based on Eq 5.8 Stull (2000)
	//Note: Elevation_lcl_km = 1/(5.5 - 1.8) * (Tair_K - Tdew_K), where Gamma_EnvironmentalLapseRate_C_p_km = 5.5 C/km, based on modification
	//Note: Elevation_delta_km = (Elevation_pixel - Elevation_referenceStation) and sets Tair_K = Tdew_K at Elevation_lcl_km
	//Note: If Elevation_delta_km <= Elevation_lcl_km Then Tair_update = Tair_K - Gamma_dry_C_p_km * Elevation_delta_km, based on Stull (2000)
	//Note: Else Tair_update = Tair_K - (Gamma_dry_C_p_km * Elevation_lcl_km) - [Gamma_dew_C_p_km * (Elevation_delta_km - Elevation_lcl_km)]; Stull (2000)
	//Note: If Elevation_delta_km <= Elevation_lcl_km Then Tair_update = Tair_K - Gamma_EnvironmentalLapseRate_C_p_km * Elevation_delta_km
	//Note: Else Tair_update = Tair_K - (Gamma_EnvironmentalLapseRate_C_p_km * Elevation_lcl_km) - [Gamma_dew_C_p_km * (Elevation_delta_km - Elevation_lcl_km)] based modification
	//Note: Tdew_update = Tdew_K - Gamma_dew_C_p_km * Elevation_delta_km based on Stull (2000) and modification
	//Note: For differencing temperatures, Kelvin can be used in place of Celsius, as they are linearly related
	//Stull, R. (2000). Meteorology for Scientists and Engineers, 2nd Edition. Brooks Cole. ISBN 0-534-37214-7

	//Elevation_Station_weighted_m (m) is spatially weighted average of station elevations, 
	//Note: WeightedInputMap[MapPixel_ID]["Elevation_WeatherStations_weighted_m"] is computed for each map pixel, only at timeStep = 0
	double Elevation_Station_weighted_m = WeightedInputMap[MapPixel_ID]["Elevation_WeatherStations_weighted_m"][0];
	//Elevation_delta_km (km) is height of pixel above station, equals (DEM(MapPixel_ID) - Elevation_Station_weighted_m), convert from m to km
	double Elevation_delta_km = (input->DEM[MapPixel_ID] - Elevation_Station_weighted_m) * Ratio_km_to_m;
	
	//Elevation_lcl_km (km) equals 1 / (Gamma_EnvironmentalLapseRate_C_p_km - Gamma_dew_C_p_km) * (Tair_weighted_K - Tdew_weighted_K)
	//Note: Elevation_lcl_km (km) is lifting condensation level, modified from from Eq 5.8 Stull (2000) use the environmental lapse rate
	double Elevation_lcl_km = 1 / (Gamma_EnvironmentalLapseRate_C_p_km - Gamma_dew_C_p_km) * (Tair_weighted_K - Tdew_weighted_K);

	//If Elevation_delta_km is below Elevation_lcl_km, then the lifting condensation level was not reached and air remains unsaturated
	if (Elevation_delta_km <= Elevation_lcl_km) {
		//Tair_weighted_K (K) equals Tair_weighted_K - Gamma_EnvironmentalLapseRate_C_p_km * Elevation_delta_km
		//Note: Tair_weighted_K (K) modified from Eq 5.12 Stull (2000) as explained above to use the environmental lapse rate
		//Note: Even if Flag_MultipleStation=1 and WeatherPreprocessor lapse rate adjusted Tair_weighted_K, HeatFluxCal lapse rate adjustment is needed 
		Tair_weighted_K = Tair_weighted_K - Gamma_EnvironmentalLapseRate_C_p_km * Elevation_delta_km;
	}
	//Else, Tair_K cools at adiabatic lapse rate dry Gamma_dry_C_p_km until Elevation_lcl_km, then becomes saturated and cools at Gamma_dew_C_p_km
	else {
		//Tair_weighted_K (K) equals Tair_weighted_K - Gamma_EnvironmentalLapseRate_C_p_km * Elevation_lcl_km;
		//Note: Tair_weighted_K (K) is initially updated using environmental lapse rate until Elevation_lcl_km, from Eq 5.12 Stull (2000)
		Tair_weighted_K = Tair_weighted_K - Gamma_EnvironmentalLapseRate_C_p_km * Elevation_lcl_km;

		//Tair_localCell_K (K) is subsequently udpated using adiabatic lapse rate for dew point air, modified Eq 5.13 Stull (2000)
		//Note: Tair_weighted_K (K) modified from Eq 5.12 Stull (2000) as explained above to use the environmental lapse rate
		//Note: Even if Flag_MultipleStation=1 and WeatherPreprocessor lapse rate adjusted Tair_weighted_K, HeatFluxCal lapse rate adjustment is needed 
		Tair_weighted_K = Tair_weighted_K - Gamma_dew_C_p_km * (Elevation_delta_km - Elevation_lcl_km);
	}
	//Tdew_weighted_K (K) is updated based on the adiabatic lapse rate for saturated dew point air, modified Eq 5.13 Stull (2000)
	//Note: Even if Flag_MultipleStation=1 and WeatherPreprocessor lapse rate adjusted Tdew_weighted_K, HeatFluxCal lapse rate adjustment is needed 
	Tdew_weighted_K = Tdew_weighted_K - Gamma_dew_C_p_km * Elevation_delta_km;
}

//Distance_Calc_LatLong function computes distance between pixel center and weather station using latitude and longitude, not currently used by HydroPlus
//Note: Distance_Calc_LatLong function is not currently called by HydroPlus, and remains in code in case there is a need for this calculation
//Note: Theory uses Haversine Formula https://en.wikipedia.org/wiki/Haversine_formula, computing distance between points on a sphere using lat and long 
long double WeatherPreprocessor::Distance_Calc_LatLong(long double PixelCenter_Latitude_DD, long double PixelCenter_Longitude_DD, long double Station_Latitude_DD, long double Station_Longitude_DD)
{
	//PixelCenter_Latitude_rad (rad) is converted from decimal degree to radians
	long double PixelCenter_Latitude_rad = toRadians(PixelCenter_Latitude_DD);
	//PixelCenter_Longitude_rad (rad) is converted from decimal degree to radians
	long double PixelCenter_Longitude_rad = toRadians(PixelCenter_Longitude_DD);
	//station_lat_rad (rad) is converted from decimal degree to radians
	long double station_lat_rad = toRadians(Station_Latitude_DD);
	//station_long_rad (rad) is converted from decimal degree to radians
	long double station_long_rad = toRadians(Station_Longitude_DD);
	//Distance_Longitude_rad (radians) is distance along longitude between station and pixel center
	long double Distance_Longitude_rad = station_long_rad - PixelCenter_Longitude_rad;
	//Distance_Latitude_rad (radians) is distance along latitude between station and pixel center
	long double Distance_Latitude_rad = station_lat_rad - PixelCenter_Latitude_rad;

	//Distance_Pixel_to_Station_rad (rad) is partial result of latitude longitude points distance formula
	//Note: Theory based on Haversine Formula
	long double Distance_Pixel_to_Station_rad = pow(sin(Distance_Latitude_rad / 2), 2) + cos(PixelCenter_Latitude_rad) * cos(station_lat_rad) * pow(sin(Distance_Longitude_rad / 2), 2);
	//Distance_Pixel_to_Station_rad is completed result of distance formula
	Distance_Pixel_to_Station_rad = 2 * asin(sqrt(Distance_Pixel_to_Station_rad));

	//RadiusEarth_km (km) is average radius of Earth elipsoid, given as 6371 km
	long double RadiusEarth_km = 6371;
	//Distance_Pixel_to_Station_rad (km) is scaled from radians to km using radius of Earth
	double Distance_Pixel_to_Station_km = Distance_Pixel_to_Station_rad * RadiusEarth_km;
	return Distance_Pixel_to_Station_km;
}

//WeatherPreprocessor::toRadians function converts degrees to radians
long double WeatherPreprocessor::toRadians(const long double degree) {
	// M_PI as the value of pi accurate to 1e-30
	long double one_deg = (M_PI) / 180;
	return (one_deg * degree);
}

/* Helpful debugging code for print WeatherMap from Li Zhang
	for (auto it = WeatherMap.begin(); it != WeatherMap.end(); it++) {
		cout << it->first << ": ";
		for (auto it2 = (it->second).begin(); it2 != (it->second).end(); it2++) {
			cout << *it2 << ", ";
		}
		cout << endl;
	}
*/
