#include "SimulationCoordinator.h"

#include "Interception/TreeInterceptionCalc.h"
#include "Interception/ShortVegInterceptionCalc.h"
#include "Evaporation/TreeEvaporationCalc.h"
#include "Evaporation/ShortVegEvaporationCalc.h"
#include "Evaporation/OpenWaterEvaporationCalc.h"
#include "Evaporation/PotentialEvaporationCalc.h"
#include "SnowOnGround/SnowAblationOpenAreaCalc.h"
#include "SnowOnGround/SnowAblationUnderShortVegCalc.h"
#include "SnowOnGround/SnowAblationUnderTreeCalc.h"
#include "StorageSurface/FluxToImpervious.h"
#include "StorageSurface/FluxToPervious.h"
#include "StorageSurface/Flux_to_Water.h"
#include "StorageSurface/DepressionImpervious.h"
#include "StorageSurface/DepressionPervious.h"
#include "StorageSurface/DepressionWater.h"
#include "Infiltration/InfiltrationExponentialDecay.h"
#include "Infiltration/InfiltrationPowerDecay.h"
#include "LateralDistribution/Flux_LateralGroundWater.h"
#include "StorageSubsurface/StorageDeficit_VadoseZone.h"
#include "Runoff/RunoffSubsurface.h"
#include "Runoff/RunoffSummation.h"
#include "Aggregation/AggregateOutput.h"
#include "Runoff/RoutingDiffusion.h"
#include "WaterQuality/Loading_WaterQuality_EMC.h"
#include "WaterQuality/Removal_StormwaterPollutants.h"
#include "CoolAir/HeatFluxCal.h"
#include "CoolAir/BlockGroupHandler.h"
#include "CoolAir/WetBulbGlobeTempCalc.h"
#include "GreenInfrastructure/Inflow_StormwaterDevice.h"
#include "CoolRiver/SolarCalculation.h"
#include "CoolRiver/StreamTemperature.h"
#include "Buffer/DynamicFunctions.h"
#include "Buffer/Buffer.h"
#include "CoolBuilding/EnergyLoad_Calc.h"
#include "Outputs/RoutedRunoff_TimeSeries.h"
#include "Outputs/TemperatureOutputWriter.h"
#include "Outputs/CoolRiverOutputWriter.h"
#include "Outputs/WriteOutputs.h"
#include "Inputs/WeatherPreprocessor.h"
#include "Buffer/Buffer.h"
#include <algorithm>
#include <limits>
#include <cstddef>
#include <string>
#include <initializer_list>

void SimulationCoordinator::SimulationCoordinator_Launch(DataOrganizer *organizer, Inputs *input, string inputDirectory, WeatherPreprocessor* Weatherpre)
{
	//Print to Command Prompt Xterm the HydroPlusConfig.xml Model type 
	cout << "Model: " << input->SimulationStringParams["Model_Selection"] << endl;
	//If Model is SpatialTemperatureHydro or StatisticalHydro then Print to Command Prompt Algorithm_SoilKsatDecay and Algorithm_FlowDirection
	if (input->SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro" || input->SimulationStringParams["Model_Selection"] == "StatisticalHydro"|| input->SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
		//Print to Command Prompt Xterm the HydroPlusConfig.xml Infiltration type 
		cout << "Infiltration: " << input->SimulationStringParams["Algorithm_SoilKsatDecay"] << endl;
		//Print to Command Prompt Xterm the HydroPlusConfig.xml FlowPathAlgorithm type 
		cout << "Flow Path Algorithm: " << input->SimulationStringParams["Algorithm_FlowDirection"] << endl;
	}
	//If Model is SpatialTemperatureHydro then Print to Command Prompt Flag_MultipleStation and Flag_HottestDaySimulated status
	if (input->SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro") {
		//If SimulationStringParams Flag_MultipleStation == 1 then 
		if (input->SimulationStringParams["Flag_MultipleStation"] == "1") {
			//Print to Command Prompt Xterm the HydroPlusConfig.xml Flag_MultipleStation setting = Multiple
			cout << "Weather Stations: Multiple" << endl;
		}
		//Else if single weather station is active
		else {
			//Print to Command Prompt Xterm the HydroPlusConfig.xml Flag_MultipleStation setting = Single
			cout << "Weather Stations: Single" << endl;
		}
		//If SimulationStringParams Flag_HottestDaySimulated == 1 then 
		if (input->SimulationStringParams["Flag_HottestDaySimulated"] == "1") {
			//Print to Command Prompt Xterm the HydroPlusConfig.xml Flag_HottestDaySimulated setting = True
			cout << "Hottest Day Simulated: True" << endl;
		}
	}
	//Print to Command Prompt Xterm the computer is running the simulation 
	cout << "Running Simulation ..." << endl;

	//If Model_Selection is StatisticalHydro then call RunStatisticalModel
	if (input->SimulationStringParams["Model_Selection"] == "StatisticalHydro") {
		RunStatisticalModel(organizer, input, inputDirectory);
	}
	//If Model_Selection is SpatialTemperatureHydro then call RunTemperatureSpatialModel
	else if (input->SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro") {
		RunTemperatureSpatialModel(organizer, input, inputDirectory, Weatherpre);
	}
	//If Model_Selection is CoolRiver then call RunCoolRiverModel
	else if (input->SimulationStringParams["Model_Selection"] == "CoolRiver")	{
		RunCoolRiverModel(organizer, input, inputDirectory);
	}
	//If Model_Selection is ECDynamic then call DynamicFunctions::YearAverage
	else if (input->SimulationStringParams["Model_Selection"] == "ECDynamic") {
		//cte 2024: Ask Li Zhang why a second conditional is used
		if (input->run_ECDynamicModel == 1) {
			DynamicFunctions::YearAverage(input);
		}
	}
	//If Model_Selection is SpatialBuffer then call BufferModel_HotSpotMaps_DiffusePollution_main
	else if (input->SimulationStringParams["Model_Selection"] == "SpatialBuffer") {
		//Buffer:BufferModel_HotSpotMaps_DiffusePollution_main model called when Model_Selection = SpatialBuffer
		Buffer::BufferModel_HotSpotMaps_DiffusePollution_main(input);
	}
	//If Model_Selection is SpatialBufferwGI then call multiple models, 
	else if (input->SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
		//RunStatisticalModel model called when Model_Selection = SpatialBufferwGI to determine runoff generation
		RunStatisticalModel(organizer, input, inputDirectory); //get GI inflow Q
		//Buffer:BufferModel_HotSpotMaps_DiffusePollution_main model called when Model_Selection = SpatialBufferwGI
		//Note: Get pollutant C (EC*area*time/Q)
		Buffer::BufferModel_HotSpotMaps_DiffusePollution_main(input); 
		//RunStatisticalModel model called when Model_Selection = SpatialBufferwGI to determine runoff trapping in GI
		RunStatisticalModel(organizer, input, inputDirectory);//run GI
	}				
	//If Model_Selection is CoolBuilding then call EnergyLoad
	else if (input->SimulationStringParams["Model_Selection"] == "CoolBuilding") {
		//cte 2024 Refactor to call the model twice, with and without trees and then read E+ output to determine load 
		EnergyLoad_Calc::EnergyLoad(input);
	}
}

void SimulationCoordinator::RunTemperatureSpatialModel(DataOrganizer *organizer, Inputs *input, string inputDirectory, WeatherPreprocessor* Weatherpre)
{
	//SolarCalculator solarCalculator(input) creates single instance of SolarCalculator which is accessed by pointer elsewhere
	SolarCalculator solarCalculator(input);

	//HeatFluxCal input structure from HeatFluxCal.h is referenced as heatFluxCalc object
	//Note: HeatFluxCal object heatFluxCalc created using a constructor that initializes the solarCalculator pointer
	//Note: Optionally, create a pointer without &solarCalculator, supported by the similar constructor
	HeatFluxCal heatFluxCalc(input, &solarCalculator);
	//BlockGroupHandler structure from BlockGroupHandler.h is referenced as handleBlockGroups object
	BlockGroupHandler handleBlockGroups(input);

	//TotalTimeSteps (hr) is defined in HydroPlusConfig.xml key or redefined in Inputs::readHydroPlusConfigFile() if key = 0 or empty
	//Li 01222025 made change to write daily block group summaries for times less than 24 hours. 
	bool needDailyBGFiles = 1;
	bool needHourlyBGFiles = (input->TemperatureExecutionParams["PrintBlockGroupDailyAndHourly"] == "true" || input->TemperatureExecutionParams["PrintBlockGroupDailyAndHourly"] == "True");
	
	//If Algorithm_FlowDirection equals D8, then write out ContributingAreaPixels, list of contributing cell ID list for each pixel
	if (input->SimulationStringParams["Algorithm_FlowDirection"] == "D8") {
		Buffer::BufferModel_IntermediateOutputs_writer("ContributingAreaPixels");
	}
	//TemperatureOutputWriter functions called before entering timeStep loop to prepare files for output at each time step
	//If Flag_BlockGroupsProcessed equals 1 then process Block Group data
	if (input->Flag_BlockGroupsProcessed == 1) {
		//Call TemperatureOutputWriter::generate_BlockGroup_Files function to prepare Blockgroup files
		TemperatureOutputWriter::generate_BlockGroup_Files(handleBlockGroups.BlockGroupIDToItsFolder, input->SimulationStringParams["OutputFolder_Path"], needHourlyBGFiles, needDailyBGFiles);
	}

	//Call TemperatureOutputWriter::generateRowColFiles function to prepare RowColHeat and RowColHydro files
	TemperatureOutputWriter::generateRowColFiles(organizer, input);

	//If HydroPlusConfig Model is SpatialTemperatureHydro, and MapPixel_ReferenceStation_ID is larger than organizer->DataDrawers.size then it does not exist, and it must abort program
	//Note: MapPixel_ReferenceStation_ID is defined by HydroPlusConfig.xml element RefWeatherLocationRowCol by converting row,col to MapPixel_ID in Inputs::ProcessTemperatureExecutionParams
	if ((input->SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro") && (input->MapPixel_ReferenceStation_ID > organizer->DataDrawers.size())) {
		cout << "Warning: The HydroPlusConfig.xml has the RefWeatherLocationRowCol element set to a value that does not exist." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The RefWeatherLocationRowCol row and col does not exist in the input maps (e.g., dem.asc)." << endl;
		cout << "Correction: In the HydroPlusConfig.xml file, set the RefWeatherLocationRowCol to values within the input map domain." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	//ReferenceFolder pointer is created from DataDrawers[input->MapPixel_ReferenceStation_ID][0]
	DataFolder *ReferenceFolder = organizer->DataDrawers[input->MapPixel_ReferenceStation_ID][0];

	//Iteration_Maximum initialized to IterationCount from HydroPlusConfig.xml file
	int Iteration_Maximum = input->TemperatureCalculationParams["IterationCount"];
	//Flag_AdiabaticLapseRate = 1 to simulate effect of elevation on temperature; = 0 to ignore effect of elevation
	bool Flag_AdiabaticLapseRate = input->TemperatureCalculationParams["Flag_AdiabaticLapseRate"];

	//If Flag_MultipleStation equals 1 then launch algorithm to compute weights for weather stations
	if (input->SimulationStringParams["Flag_MultipleStation"] == "1") {
		//Print to Command Prompt Xterm the computer is obtaining multiple station records
		cout << "Processing meteorological data for multiple stations. This may take some time ..." << endl;
		//WeatherPreprocessor::WeatherDataReadandProcessing function will read in map for all pixels, including NODATA
		WeatherPreprocessor::WeatherDataReadandProcessing(input);
	}

	//Loop through all time steps in simulation, where TotalTimeSteps is defined in HydroPlusConfig.xml or readHydroPlusConfigFile() if element = 0 or empty
	for (int timeStep = 0; timeStep < input->SimulationTimePeriod_timeSteps; ++timeStep) {

		//If TotalTimeSteps > 3 And modulus timeStep and 20% of TotalTimeSteps equals zero, then write output to Xterm window to update user
		//Note: Modulus % function fails when TotalTimeSteps < 4 
		if (input->SimulationTimePeriod_timeSteps > 3 && timeStep % int(0.25 * input->SimulationTimePeriod_timeSteps) == 0) {
			cout << "Simulating time step " << timeStep + 1 << " of " << input->SimulationTimePeriod_timeSteps << endl;
		}
		//If TotalTimeSteps < 4 or timeStep equals TotalTimeSteps, using modulus operator, then write output to Xterm window to update user
		else if (timeStep + 1 == input->SimulationTimePeriod_timeSteps) {
			cout << "Simulating time step " << timeStep + 1 << " of " << input->SimulationTimePeriod_timeSteps << endl;
		}

		//StorageDeficit_VadoseZone_TS_Prior_m is constant during time step, used by all folders while loop through DataDrawers iteratively updates AveSMD in StorageDeficit_VadoseZone
		//Note:	StorageDeficit_VadoseZone_TS_Prior_m (m) avoids use of iteratively changing input->RepoDict["StorageDeficit_VadoseZone_m"], updated in StorageDeficit_VadoseZone.cpp for each DataDrawer
		input->RefParams["StorageDeficit_VadoseZone_TS_Prior_m"] = input->RepoDict["StorageDeficit_VadoseZone_m"];

		//Flag_simulateReferenceStation initialized to one at start of each timeStep
		bool Flag_simulateReferenceStation = 1;
		//Counter_SimulatedUnit is used to identify start of simulated folders, initialized to zero
		input->RepoDict["Counter_SimulatedUnit"] = 0;

		//Loop through the DataDrawers, where the upper bound is based on the general DataDrawer attribute .size()
		//Note: For model = SpatialTemperatureHydro, MapPixel_ID has maximum = maxRows * maxCols, and DataFolder_ID typically has maximum = 1
		//Note: DataDrawers.size() contains all map pixels = maxRows * maxCols, of ASCII map
		//Note: Conversion between MapPixel_ID and row & column pair, with MapPixel_ID starting at 0, row & col starting at 1
		//Note: Conversion Eq: row=(int)(MapPixel_ID/input->ncols+1); col=(MapPixel_ID%(int)(input->ncols)+1); MapPixel_ID = (row * nCols) + col
		for (int MapPixel_ID = 0; MapPixel_ID < organizer->DataDrawers.size(); ++MapPixel_ID) {
			//Loop through the DataFolders in each DataDrawer[MapPixel_ID], upper bound indicated by .size()
			//Note: Each DataDrawer contains only 1 BulkArea DataFolder for in HydroPlusConfig.xml for Model = SpatialTemperatureHydro 
			for (int DataFolder_ID = 0; DataFolder_ID < organizer->DataDrawers[MapPixel_ID].size(); ++DataFolder_ID) {

				//If organizer->DataDrawers[MapPixel_ID][DataFolder_ID]->ParamStringDict["Type"] is not "NODATA" then proceed
				//Note: folder->ParamStringDict["Type"] defined as string "NODATA" in BuildDataOrganizer::BuildSpatialDataOrganizer
				if (organizer->DataDrawers[MapPixel_ID][DataFolder_ID]->ParamStringDict["Type"] != "NODATA") {

					//Counter_SimulatedUnit is used to identify start of simulated folders, increased by one
					input->RepoDict["Counter_SimulatedUnit"] = input->RepoDict["Counter_SimulatedUnit"] + 1;

					//cte 2025 Refactor to perhaps not call this if we don't have latitude and longitude
					//solarCalculator.SolarCalculator_Inputs called sending input, MapPixel_ID, timeStep
					//solarCalculator.SolarCalculator_Inputs(input, MapPixel_ID, timeStep);

					//if Flag_simulateReferenceStation equals 1, then calculate the reference station energy balance to prepare reference values
					//Note: This conditional will generate Tair_Mesoscale_K and AbsHumidity_Mesoscale_K values needed for local energy balance
					//Note: Flag_simulateReferenceStation is set to 1 each timeStep, then set to 0 at end of this If statement
					if (Flag_simulateReferenceStation == 1) {

						//If Flag_MultipleStation equals one then compute spatially weighted meteorological variables
						if (input->SimulationStringParams["Flag_MultipleStation"] == "1") {
							//ApplyingWeights_to_pixel_at_timestep function computes weighted meteorological end elevation data for this pixel of this time step
							WeatherPreprocessor::ApplyingWeights_to_pixel_at_timestep(input, input->MapPixel_ReferenceStation_ID, timeStep);
						}
						//runCalculations function called, sending input pointer, DataDrawer ID, DataFolder ID, and timeStep
						//Note: input->MapPixel_ReferenceStation_ID must be sent to runCalculations function when in the Flag_simulateReferenceStation == 1 condition
						runCalculations(input, organizer->DataDrawers[input->MapPixel_ReferenceStation_ID], organizer->DataDrawers[input->MapPixel_ReferenceStation_ID][DataFolder_ID], timeStep);

						//If Flag_MultipleStation is true (1) then for each weather station treat it as a reference station and find its MesoScale air temperature at start of each timestep
						//Note: This conditional will generate Tair_Mesoscale_byStationID_K (K) and AbsHumidity_Mesoscale_byStationID_kg_p_m3 (kg/m3) vectors of values 
						if (input->SimulationStringParams["Flag_MultipleStation"] == "1") {
							//it defined as Weatherpre->StationInfoMap.begin(), and advanced below
							auto it = Weatherpre->StationInfoMap.begin();
							//For loop with i from 0 to < Weatherpre->StationInfoMap.size, increment by 1 through all weather stations in simulation
							//Note: StationInfoMap contains all weather stations for simulation, listed in file identified by HydroPlusConfig.xml element WeatherStationAttributeFile 
							for (int i = 0; i < Weatherpre->StationInfoMap.size(); i++) {
								//StationID_string is string of it->first, taken from Weatherpre->StationMapID[i], unique identifier of Weather Stations
								string StationID_string = it->first;
								//StationID_string is integer of Weatherpre->StationMapID[i], unique identifier of Weather Stations, using C++ stoi command
								int StationID_int = stoi(it->first);
								//int StationID_int = stoi(StationID[i]);
								//HeatFluxCal.cpp energy balance functions called for ReferenceFolder before entering for loops of all map pixels
								//Note: This requires initializing some water content variables for EnergyFlux_WaterFlux_DefineTerms					
								//EnergyFlux_WaterFlux_DefineTerms function calculates energy balance once for each folder, before water balance 
								//Note: ReferenceFolder passed twice to handle ReferenceFolder and Folder as two distinct or identical folders
								heatFluxCalc.EnergyFlux_WaterFlux_DefineTerms(input, ReferenceFolder, ReferenceFolder, input->MapPixel_ReferenceStation_ID, Flag_simulateReferenceStation, timeStep, Weatherpre, StationID_string);
								//Tair_AbsHumidity_LayerBalance function calls AerodynamicResistance_CanopyToMesoLayer and Tair_AbsHumidity_LayerBalance 
								//Note: ReferenceFolder is sent twice, representing since it is the reference station
								heatFluxCalc.Tair_AbsHumidity_LayerBalance(input, ReferenceFolder, ReferenceFolder,timeStep);

								//Tair_mesoScale_K (K) saved to Weatherpre Tair_Mesoscale_byStationID_K 2D vector with StationID_int
								Weatherpre->Tair_Mesoscale_byStationID_K[StationID_int].push_back(heatFluxCalc.Tair_mesoScale_K);
								//AbsHumidity_mesoScale_kg_p_m3 (K) saved to Weatherpre AbsHumidity_Mesoscale_byStationID_kg_p_m3 2D vector with StationID_int
								Weatherpre->AbsHumidity_Mesoscale_byStationID_kg_p_m3[StationID_int].push_back(heatFluxCalc.AbsHumidity_mesoScale_kg_p_m3);
								//it index is advanced by one
								it++;
							}
							//ComputeSpatiallyWeightedVariables function will compute spatially weighted meteorological variables
							WeatherPreprocessor::ComputeSpatiallyWeightedVariables(input, input->MapPixel_ReferenceStation_ID, timeStep);
							//RescaleVariables_CoolAir function will rescale latent and sensible energy fluxes, compute corresponding mesoScale values for Tair and AbsoluteHumidity
							heatFluxCalc.RescaleVariables_CoolAir(input, ReferenceFolder, ReferenceFolder, timeStep);
							//CollectVariables_CoolAir function will store all values to memory
							heatFluxCalc.CollectVariables_CoolAir(input, ReferenceFolder, ReferenceFolder, &solarCalculator, MapPixel_ID, timeStep);
						}
						//Else If Flag_MultipleStation is false (0) then only for a single reference station find its MesoScale air temperature at start of each timestep
						else {
							//HeatFluxCal.cpp energy balance functions called for ReferenceFolder before entering for loops of all map pixels
							//Note: This requires initializing some water content variables for EnergyFlux_WaterFlux_DefineTerms		
							//EnergyFlux_WaterFlux_DefineTerms function calculates energy balance once for each folder, before water balance 
							//Note: ReferenceFolder passed twice to handle ReferenceFolder and Folder as two distinct or identical folders
							heatFluxCalc.EnergyFlux_WaterFlux_DefineTerms(input, ReferenceFolder, ReferenceFolder, input->MapPixel_ReferenceStation_ID, Flag_simulateReferenceStation, timeStep, Weatherpre, "NA");
							//Tair_AbsHumidity_LayerBalance function calls AerodynamicResistance_CanopyToMesoLayer and Tair_AbsHumidity_LayerBalance 
							//Note: ReferenceFolder is sent twice, representing since it is the reference station
							heatFluxCalc.Tair_AbsHumidity_LayerBalance(input, ReferenceFolder, ReferenceFolder, timeStep);

							//RescaleVariables_CoolAir function will rescale latent and sensible energy fluxes, compute corresponding mesoScale values for Tair and AbsoluteHumidity
							heatFluxCalc.RescaleVariables_CoolAir(input, ReferenceFolder, ReferenceFolder, timeStep);
							//CollectVariables_CoolAir function will store all values to memory
							heatFluxCalc.CollectVariables_CoolAir(input, ReferenceFolder, ReferenceFolder, &solarCalculator, MapPixel_ID, timeStep);
						}

						//Flag_simulateReferenceStation set to zero until next timeStep
						Flag_simulateReferenceStation = 0;
					}

					//if MapPixel_ID is not input->MapPixel_ReferenceStation_ID of reference station, then complete energy balance functions for pixel
					if (MapPixel_ID != input->MapPixel_ReferenceStation_ID) {

						//If Flag_MultipleStation equals one then compute spatially weighted meteorological variables
						if (input->SimulationStringParams["Flag_MultipleStation"] == "1") {
							//ApplyingWeights_to_pixel_at_timestep function computes weighted meteorological & elevation data for pixel at time step
							//Note: Weighting is needed before calling runCalculations, which will rely on weighted terms for Flag_MultipleStation=1
							WeatherPreprocessor::ApplyingWeights_to_pixel_at_timestep(input, MapPixel_ID, timeStep);
						}
						//runCalculations function called, sending input pointer, DataDrawer ID, DataFolder ID, and timeStep
						//Note: MapPixel_ID must be sent to runCalculations function when in the Flag_simulateReferenceStation != 1 condition
						runCalculations(input, organizer->DataDrawers[MapPixel_ID], organizer->DataDrawers[MapPixel_ID][DataFolder_ID], timeStep);

						//If Flag_MultipleStation equals one then compute spatially weighted meteorological variables
						if (input->SimulationStringParams["Flag_MultipleStation"] == "1") {
							//ComputeSpatiallyWeightedVariables function will compute spatially weighted meteorological variables
							WeatherPreprocessor::ComputeSpatiallyWeightedVariables(input, MapPixel_ID, timeStep);
						}
						//HeatFluxCal.cpp energy balance functions called for all map pixels after earlier call by ReferenceFolder 
						//EnergyFlux_WaterFlux_DefineTerms function calculates energy balance once for each folder, before water balance 
						//Note: ReferenceFolder passed once, to handle ReferenceFolder and Folder as two distinct
						heatFluxCalc.EnergyFlux_WaterFlux_DefineTerms(input, ReferenceFolder, organizer->DataDrawers[MapPixel_ID][DataFolder_ID], MapPixel_ID, Flag_simulateReferenceStation, timeStep, Weatherpre, "NA");
						//Tair_AbsHumidity_LayerBalance_NumericalMethod function calculates temperature and humidity based on iterative numerical method
						heatFluxCalc.Tair_AbsHumidity_LayerBalance_NumericalMethod(input, organizer->DataDrawers[MapPixel_ID][DataFolder_ID], ReferenceFolder, Weatherpre, MapPixel_ID, timeStep, Iteration_Maximum);
						//If Flag_AdiabaticLapseRate equals 1 then call AdiabaticLapseRates to simulate lapse rate effect on air temperature
						//Note: Even if Flag_MultipleStation=1 and WeatherPreprocessor lapse rate adjusted Tair_weighted_K, this adjustment is needed 
						if (Flag_AdiabaticLapseRate == 1) {
							//AdiabaticLapseRates function adjusts air and dew point temperatures based on elevation at environmental lapse rate
							heatFluxCalc.AdiabaticLapseRates(input, organizer->DataDrawers[MapPixel_ID][DataFolder_ID], ReferenceFolder, timeStep, MapPixel_ID, Weatherpre);
						}
						//RescaleVariables_CoolAir function rescales latent and sensible energy fluxes, computes related mesoScale varaiables
						heatFluxCalc.RescaleVariables_CoolAir(input, organizer->DataDrawers[MapPixel_ID][DataFolder_ID], ReferenceFolder, timeStep);
						//CollectVariables_CoolAir function stores all values to memory, screens for an extreme value Tair_K error
						heatFluxCalc.CollectVariables_CoolAir(input, organizer->DataDrawers[MapPixel_ID][DataFolder_ID], ReferenceFolder, &solarCalculator, MapPixel_ID, timeStep);
					}

					//RunoffSummation::SumAcrossCoverTypes function called to sum folders of non-routed baseflow, pervious flow, impervious flow
					RunoffSummation::SumAcrossCoverTypes(input, organizer->DataDrawers[MapPixel_ID][DataFolder_ID], timeStep);		
					//AggregateOutput::SumCatchmentTotals function called to sum catchment variables into a total for each time step
					AggregateOutput::SumCatchmentTotals(input, organizer->DataDrawers,
						organizer->DataDrawers[MapPixel_ID][DataFolder_ID], MapPixel_ID, DataFolder_ID, timeStep);
					//TemperatureOutputWriter rowColFileBatchProcessor function called to process row and column output 
					TemperatureOutputWriter::rowColFileBatchProcessor(timeStep, MapPixel_ID, 
						organizer->DataDrawers[MapPixel_ID][DataFolder_ID], input->SimulationStringParams["OutputFolder_Path"], input);

					//If Flag_BlockGroupsProcessed equals 1 then process Block Group data
					if (input->Flag_BlockGroupsProcessed == 1) {
						//handleBlockGroups AddToBlockGroup function called to create block group output
						handleBlockGroups.AddToBlockGroup(MapPixel_ID, input, organizer->DataDrawers[MapPixel_ID][DataFolder_ID]);
					}					
				}
			}
		}
		//If Flag_BlockGroupsProcessed equals 1 then process Block Group data
		if (input->Flag_BlockGroupsProcessed == 1) {
			//handleBlockGroups TimeStepProcessing function called to process block group output, sending input and timeStep
			handleBlockGroups.TimeStepProcessing(input, timeStep);
		}
		//TemperatureOutputWriter called to write output with timeStepResultsFileProcessor function
		TemperatureOutputWriter::timeStepResultsFileProcessor(timeStep, organizer, input->SimulationStringParams["OutputFolder_Path"], input, Weatherpre);
		//AggregateOutput::SumTimeStepTotals function called to sum catchment variables into a total for each time step
		AggregateOutput::SumTimeStepTotals(input, timeStep);
		//RoutingDiffusion::RoutingRunoff will route summed flows from BulkArea for each time step
		RoutingDiffusion::RoutingRunoff(input, timeStep);
		//WriteOutputs::TimeSeriesOutput function writes time series output
		WriteOutputs::TimeSeriesOutput(input, organizer, timeStep);
	}
	//Write weighted inputs
	if (input->SimulationStringParams["Flag_MultipleStation"] == "1") {
		WeatherPreprocessor::MapPixelWeightedWeatherInputsWriter(input);
	}
	//RoutedRunoff_TimeSeries::ScalingRunoffDepth_WritingOutput function scales routed flows to ensure their volume balances
	RoutedRunoff_TimeSeries::ScalingRunoffDepth_WritingOutput(input);

	//WriteOutputs::SummaryOutput function writes summary files 
	WriteOutputs::SummaryOutput(organizer, input);

	cout << "Writing output to: " << input->SimulationStringParams["OutputFolder_Path"] << endl;
	cout << "===" << endl;
}


void SimulationCoordinator::RunStatisticalModel(DataOrganizer *organizer, Inputs *input, string inputDirectory)
{
	//If Flag_MultipleStation equals 1 then launch algorithm to compute weights for weather stations
	if (input->SimulationStringParams["Flag_MultipleStation"] == "1") {
		//Print to Command Prompt Xterm the computer is obtaining multiple station records
		cout << "Processing meteorological data for multiple stations. This may take some time ..." << endl;
		//WeatherPreprocessor::WeatherDataReadandProcessing function will read in map for all pixels, including NODATA
		WeatherPreprocessor::WeatherDataReadandProcessing(input);
	}
	//Loop through all time steps in simulation, where TotalTimeSteps is defined in HydroPlusConfig.xml or readHydroPlusConfigFile() if element = 0 or empty
	for (int timeStep = 0; timeStep < input->SimulationTimePeriod_timeSteps; ++timeStep) {

		//If Flag_MultipleStation equals one then compute spatially weighted meteorological variables
		//only for one center pixel, doesn't need to be included into the pixel loop
		if (input->SimulationStringParams["Flag_MultipleStation"] == "1") {
			//ApplyingWeights_to_pixel_at_timestep function computes weighted meteorological end elevation data for this pixel of this time step
			WeatherPreprocessor::ApplyingWeights_to_pixel_at_timestep(input, WeatherPreprocessor::centerMapPixel_ID, timeStep);
		}
		//Note: This Xterm output code for regular use, and should not be removed
		//If modulus timeStep and 20% of TotalTimeSteps equals zero, then write output to Xterm window to update user
		if (timeStep % int(0.25 * input->SimulationTimePeriod_timeSteps) == 0) {
			cout << "Simulating time step " << timeStep + 1 << " of " << input->SimulationTimePeriod_timeSteps << endl;
		}
		//If timeStep equals TotalTimeSteps, using modulus operator, then write output to Xterm window to update user
		else if (timeStep + 1 == input->SimulationTimePeriod_timeSteps) {
			cout << "Simulating time step " << timeStep + 1 << " of " << input->SimulationTimePeriod_timeSteps << endl;
		}

		//StorageDeficit_VadoseZone_TS_Prior_m is constant during time step, used by all folders while loop through DataDrawers iteratively updates AveSMD in StorageDeficit_VadoseZone
		//Note:	StorageDeficit_VadoseZone_TS_Prior_m (m) avoids use of iteratively changing input->RepoDict["StorageDeficit_VadoseZone_m"], updated in StorageDeficit_VadoseZone.cpp for each DataDrawer
		input->RefParams["StorageDeficit_VadoseZone_TS_Prior_m"] = input->RepoDict["StorageDeficit_VadoseZone_m"];

		//Counter_SimulatedUnit is used to identify start of simulated folders, initialized to zero
		input->RepoDict["Counter_SimulatedUnit"] = 0;

		//Loop through the DataDrawers, where the upper bound is based on the general DataDrawer attribute .size()
		for (int MapPixel_ID = 0; MapPixel_ID < organizer->DataDrawers.size(); ++MapPixel_ID) {

			//Loop through the DataFolders in each DataDrawer[MapPixel_ID], upper bound indicated by .size()
			for (int DataFolder_ID = 0; DataFolder_ID < organizer->DataDrawers[MapPixel_ID].size(); ++DataFolder_ID) {

				//Counter_SimulatedUnit is used to identify start of simulated folders, increased by one
				input->RepoDict["Counter_SimulatedUnit"] = input->RepoDict["Counter_SimulatedUnit"] + 1;

				//runCalculations function called, sending input pointer, DataDrawer ID, DataFolder ID, and timeStep
				runCalculations(input, organizer->DataDrawers[MapPixel_ID], organizer->DataDrawers[MapPixel_ID][DataFolder_ID], timeStep);

				//RunoffSummation::SumAcrossCoverTypes function called to sum folders of non-routed baseflow, pervious flow, impervious flow, water flow for BulkArea
				RunoffSummation::SumAcrossCoverTypes(input, organizer->DataDrawers[MapPixel_ID][DataFolder_ID], timeStep);

				//Loading_WaterQuality_EMC::Pollutant_Load_via_EMC function computes loading via event mean concentration theory
				Loading_WaterQuality_EMC::Pollutant_Load_via_EMC(input, organizer->DataDrawers[MapPixel_ID][DataFolder_ID], timeStep);
				
				//Removal_StormwaterPollutants::Pollutant_Removal_via_GI function computes loading via event mean concentration theory
				Removal_StormwaterPollutants::Pollutant_Removal_via_GI(input, organizer->DataDrawers[MapPixel_ID][DataFolder_ID], timeStep);

				//AggregateOutput::SumCatchmentTotals function called to sum catchment variables into a total for each time step
				AggregateOutput::SumCatchmentTotals(input, organizer->DataDrawers,
					organizer->DataDrawers[MapPixel_ID][DataFolder_ID], MapPixel_ID, DataFolder_ID, timeStep);
				
				//only need to run for the last timestep
				if (timeStep == (input->SimulationTimePeriod_timeSteps - 1)) {
					//if it's running SpatialBuffer with GI model and the flag_reduce_EC_EMC_after_GI tuened on and if HydroPlusConfig.xml Type equals GI
					if (input->SimulationStringParams["Model_Selection"] == "SpatialBufferwGI" && organizer->DataDrawers[MapPixel_ID][DataFolder_ID]->ParamStringDict["Type"] != "BulkArea" && Buffer::flag_reduce_EC_EMC_after_GI) {
						//start the process of reducing EC ans EMC value for contributing area
						Removal_StormwaterPollutants::reduce_EC_EMC_after_GI(input, organizer->DataDrawers[MapPixel_ID][DataFolder_ID]);
						//recreate the contributing area weighted EC and EMC map
						Buffer::BufferModel_IntermediateOutputs_writer("total_ec_tn_wtd_reduced");
						Buffer::BufferModel_IntermediateOutputs_writer("total_ec_tp_wtd_reduced");
						Buffer::BufferModel_IntermediateOutputs_writer("total_ec_tss_wtd_reduced");
						Buffer::BufferModel_IntermediateOutputs_writer("total_emc_tn_wtd_reduced");
						Buffer::BufferModel_IntermediateOutputs_writer("total_emc_tp_wtd_reduced");
						Buffer::BufferModel_IntermediateOutputs_writer("total_emc_tss_wtd_reduced");
					}
				}
			}
		}
		
		//AggregateOutput::SumTimeStepTotals function called to sum catchment variables into a total for each time step
		AggregateOutput::SumTimeStepTotals(input, timeStep);
		//WriteOutputs::TimeSeriesOutput function writes time series output
		WriteOutputs::TimeSeriesOutput(input, organizer, timeStep);
		//RoutingDiffusion::RoutingRunoff will route summed flows from BulkArea for each time step
		RoutingDiffusion::RoutingRunoff(input, timeStep);
	}
	//if Flag_MultipleStation == "1", write out the weighted input
	if (input->SimulationStringParams["Flag_MultipleStation"] == "1") {
		WeatherPreprocessor::MapPixelWeightedWeatherInputsWriter(input);
	}
	//RoutedRunoff_TimeSeries::ScalingRunoffDepth_WritingOutput function scales routed flows to ensure their volume balances
	RoutedRunoff_TimeSeries::ScalingRunoffDepth_WritingOutput(input);
	//WriteOutputs::SummaryOutput function writes summary files 
	WriteOutputs::SummaryOutput(organizer, input);

	cout << "Writing output to: " << input->SimulationStringParams["OutputFolder_Path"] << endl;
}

//SimulationCoordinator::runCalculations function defines functors based on model and then sequentially goes through water balance
void SimulationCoordinator::runCalculations(Inputs* input, vector<DataFolder *> &drawer, DataFolder* folder, int timeStep)
{
	//Note: Functors are defined below with functors.emplace("NameOfFunctor", FunctionCpp::FunctionName)
	//Note: Funtors are called by passing path, e.g., FunctionName(Inputs* input, DataFolder* folder, map<string, void(*)(Inputs* input, DataFolder* folder, int timeStep) >& functors, int timeStep)
	map<string, void (*)(Inputs * input, DataFolder * folder, int timeStep)> functors;
	if (input->SimulationStringParams["Model_Selection"] == "StatisticalHydro"|| input->SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
		functors.emplace("calculateTreeEvaporation", TreeEvaporationCalc::calculateTreeEvaporationStatistical);
		functors.emplace("calculateShortVegEvaporation", ShortVegEvaporationCalc::calculateShortVegEvaporationStatistical);
		//Defining functor for StatisticalHydro for pervious ponded water called calculatePerDepEvap in Evaporation folder 
		functors.emplace("calculatePerDepEvap", OpenWaterEvaporationCalc::calculatePerDepEvapStatistical);
		//Defining functor for StatisticalHydro for impervious ponded water called calculateImpDepEvapTemperature in Evaporation folder 
		functors.emplace("calculateImpDepEvap", OpenWaterEvaporationCalc::calculateImpDepEvapStatistical);
		//Defining functor for StatisticalHydro for water cover ponded water called calculateWaterDepEvapTemperature in Evaporation folder 
		functors.emplace("calculateWaterDepEvap", OpenWaterEvaporationCalc::calculateWaterDepEvapStatistical);
		//Defining functor for StatisticalHydro for snow on ground sublimation called calculateGroundSublimationStatistical in Evaporation folder 
		functors.emplace("calculateGroundSublimation", OpenWaterEvaporationCalc::calculateGroundSublimationStatistical);
	}

	if (input->SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro") {
		functors.emplace("calculateTreeEvaporation", TreeEvaporationCalc::calculateTreeEvaporationTemperature);
		functors.emplace("calculateShortVegEvaporation", ShortVegEvaporationCalc::calculateShortVegEvaporationTemperature);
		//Defining functor for SpatialTemperatureHydro for pervious ponded water called calculatePerDepEvap in Evaporation folder 
		functors.emplace("calculatePerDepEvap", OpenWaterEvaporationCalc::calculatePerDepEvapTemperature);
		//Defining functor for SpatialTemperatureHydro for impervious ponded water called calculateImpDepEvapTemperature in Evaporation folder 
		functors.emplace("calculateImpDepEvap", OpenWaterEvaporationCalc::calculateImpDepEvapTemperature);
		//Defining functor for SpatialTemperatureHydro for water cover ponded water called calculateWaterDepEvapTemperature in Evaporation folder 
		functors.emplace("calculateWaterDepEvap", OpenWaterEvaporationCalc::calculateWaterDepEvapTemperature);
		//Defining functor for SpatialTemperatureHydro for snow on ground sublimation called calculateGroundSublimationTemperature in Evaporation folder 
		functors.emplace("calculateGroundSublimation", OpenWaterEvaporationCalc::calculateGroundSublimationTemperature);
	}

	if (input->SimulationStringParams["Algorithm_SoilKsatDecay"] == "PowerDecay") {
		functors.emplace("calculateInfiltration", InfiltrationPowerDecay::calculate);
	}
	if (input->SimulationStringParams["Algorithm_SoilKsatDecay"] == "ExponentialDecay") {
		functors.emplace("calculateInfiltration", InfiltrationExponentialDecay::calculate);
	}

	//BulkArea DataFolder processes
	//If ParamStringDict["Type"] not equal to BulkArea then allow BulkArea DataFolder processes
	if (folder->ParamStringDict["Type"] == "BulkArea") {
		//If SimulationStringParams["Flag_ReadEvaporationData"] not (!=) 1 then call PotentialEvaporationCalc::Method_PenmanMonteith
		if (input->SimulationStringParams["Flag_ReadEvaporationData"] != "1") {
			PotentialEvaporationCalc::Method_PenmanMonteith(input, folder, timeStep);
		}
		TreeInterceptionCalc::calculate(input, folder, timeStep);
		ShortVegInterceptionCalc::calculate(input, folder, timeStep);
		functors["calculateTreeEvaporation"](input, folder, timeStep);
		functors["calculateShortVegEvaporation"](input, folder, timeStep);
		functors["calculateGroundSublimation"](input, folder, timeStep);
		SnowAblationOpenAreaCalc::SnowAblationOnGround(input, folder, timeStep);
		SnowAblationUnderTreeCalc::calculate(input, folder, timeStep);
		SnowAblationUnderShortVegCalc::calculate(input, folder, timeStep);
		FluxToImpervious::calculate(input, folder, timeStep);
		DepressionImpervious::calculate(input, folder, timeStep, functors);
		FluxToPervious::calculate(input, folder, timeStep, drawer);
		functors["calculateInfiltration"](input, folder, timeStep);
		DepressionPervious::calculate(input, folder, timeStep, functors);
		Flux_to_Water::calculate(input, folder, timeStep);
		DepressionWater::calculate(input, folder, timeStep, functors);
		//Flux_LateralGroundWater::calculate is only called for BulkArea, not for Green Infrastructure (GI)
		//Note: Bulk Area represents uses topographic index (TI) bins with unique soil moisture deficits to represent variable source area theory
		//Note: Green Infrastructure uses average TI and soil moisture deficit to determine average Depth_to_Groundwater_Table_m, and ...
		//Note: ... If Depth_to_Groundwater_Table_m is below the GI layer (e.g., soil, vault) then that layer can drain to vadose zone
		Flux_LateralGroundWater::calculate(input, folder, timeStep);
		RunoffSubsurface::calculate(input, folder, timeStep);
		StorageDeficit_VadoseZone::calculate(input, folder, timeStep);

		//If GI is operational in HydroPlusConfig.xml then create RepoDict variables to share in GI
		if (input->RepoDict["Flag_GI_Simulated"] == 1) {
			input->RepoDict["Area_BulkArea_m2"] = folder->ParamDict["Area_m2"];
			input->RepoDict["Area_BulkArea_Pervious_m2"] = folder->ParamDict["Area_m2"] * folder->ParamDict["PerviousCover_frac"];
			input->RepoDict["Area_BulkArea_Impervious_m2"] = folder->ParamDict["Area_m2"] * folder->ParamDict["ImperviousCover_frac"];
		}
	}

	//GreenInfrastructure DataFolder processes
	//If ParamStringDict["Type"] not equal to BulkArea then allow GreenInfrastructure DataFolder processes
	//Note: GI Types are Bioretention, InfilTrench, PermeablePavement, RainGarden, Swale, GreenRoof, RainBarrel, RoofDisconnect
	if (folder->ParamStringDict["Type"] != "BulkArea") {
		//If SimulationStringParams["Flag_ReadEvaporationData"] not (!=) 1 then call PotentialEvaporationCalc::Method_PenmanMonteith
		if (input->SimulationStringParams["Flag_ReadEvaporationData"] != "1") {
			PotentialEvaporationCalc::Method_PenmanMonteith(input, folder, timeStep);
		}
		TreeInterceptionCalc::calculate(input, folder, timeStep);
		ShortVegInterceptionCalc::calculate(input, folder, timeStep);
		functors["calculateTreeEvaporation"](input, folder, timeStep);
		functors["calculateShortVegEvaporation"](input, folder, timeStep);
		functors["calculateGroundSublimation"](input, folder, timeStep);
		SnowAblationOpenAreaCalc::SnowAblationOnGround(input, folder, timeStep);
		SnowAblationUnderTreeCalc::calculate(input, folder, timeStep);
		SnowAblationUnderShortVegCalc::calculate(input, folder, timeStep);
		FluxToImpervious::calculate(input, folder, timeStep);
		FluxToPervious::calculate(input, folder, timeStep, drawer);
		Flux_to_Water::calculate(input, folder, timeStep);

		//GI_InitializeAndLaunchWaterBalance function initializes variables and launches the GI water balance, calling Flux_Manager_StormwaterDevice
		//Note: BulkArea transfer of water to GI is through Flux_BulkArea_to_GI_Runon_m3 (m3), managed in Inflow_StormwaterDevice.cpp
		//Note: Flux_BulkArea_to_GI_Runon_m3 includes Runoff_Impervious_BAtoGI_m3, Runoff_SatExcess_BAtoGI_m3, Runoff_InfilExcess_BAtoGI_m3.
		//Note: GI transfer of water to groundwater is through GI_Drainage_VadoseZone_m3, managed in StorageDeficit_VadoseZone::calculate function ...
		//Note: ... GI uses average TI and soil moisture deficit to determine average Depth_to_Groundwater_Table_m, and ...
		//Note: ... If Depth_to_Groundwater_Table_m is below the GI layer (e.g., soil, vault) then that layer can drain to vadose zone
		//Note: GI transfer of water to runoff is through Runoff_Pervious_m3, Runoff_Impervious_m3, Runoff_Water_m3, and Runoff_Vault_m3, managed in RunoffSummation::calculate function 
		Inflow_StormwaterDevice::GI_InitializeAndLaunchWaterBalance(input, folder, timeStep, functors);

		//If Flag_Exfiltration_to_Catchment equals one, then allow for subsurface runoff from saturated zone, and vadose zone drainage to saturated zone
		//Note: When Flag_Exfiltration_to_Catchment != 1, mounding does not occur below GI
		if (folder->ParamDict["Flag_Exfiltration_to_Catchment"] == 1) {
			RunoffSubsurface::calculate(input, folder, timeStep);
			StorageDeficit_VadoseZone::calculate(input, folder, timeStep);
		}
	}
}

void SimulationCoordinator::RunCoolRiverModel(DataOrganizer* organizer, Inputs* input, string inputDirectory)
{
	vector<double> solarAzimuth;
	vector<double> solarAltitude;

	//SolarCalculation pointer to instance sc constructed from SolarCalculation class
	SolarCalculation* sc = new SolarCalculation(input);

	//TotalTimeSteps defined by input->SimulationTimePeriod_timeSteps 
	int TotalTimeSteps = input->SimulationTimePeriod_timeSteps;

	//For Loop timeStep through all TotalTimeSteps based on HydroPlusConfig StartDate_YYYYMMDD and StopDate_YYYYMMDD
	for (int timeStep = 0; timeStep < TotalTimeSteps; ++timeStep) {
		//Call SolarCalculation::SolarHour function sending timeStep variable
		sc->SolarHour(input, timeStep);
		//Call SolarCalculation::getSolarAltitude function to populate solarAltitude vector
		solarAltitude.push_back(sc->getSolarAltitude());
		//Call SolarCalculation::getSolarAzimuth function to populate solarAltitude vector
		solarAzimuth.push_back(sc->getSolarAzimuth());
	}
	//StreamTemperature pointer to instance st constructed from StreamTemperature class
	StreamTemperature* st = new StreamTemperature(input, sc);

	//Call StreamTemperature::avgTemperature function
	//Note: avgTemperature function manages most of the CoolRiver functions, including interpolation of spatial and temporal variables
	//cte 2024 Refactor avgTemperature to more cleanly divide total distance into river stations that include start (0m) and finish (1980m)
	//Consider refactor avgTemperature to write river station 2D files with actual river station distance; check why it is ordered DS to US
	//Consider refactor avgTemperature to only clarify difference between TotalTimeMod_CN, TotalTimeUns, TotalTime_HR 
	st->avgTemperature();

	//TotalTimeSteps_cnt (hr) is defined in HydroPlusConfig.xml key or redefined in Inputs::readHydroPlusConfigFile() if key = 0 or empty
	//TotalTimeSteps_cnt (hr) is total time steps for model simulation in time step of hour, later converted to minute; 
	//	Same time step for input RiverTimeSeries.csv, RiverSWTimeSeries.csv and RiverGWTimeSeries.csv
	int TotalTimeSteps_cnt = input->SimulationTimePeriod_timeSteps;
	int LengthSimulated_River_m = input->CoolRiverVariables["CR_totDist"];
	//dx_Interval_m (m) is computational interval in streamwise distance x
	int dx_Interval_m = input->CoolRiverVariables["CR_interval_dx"];
	int calcMethod = input->CoolRiverVariables["CR_calcMethod"];

	//TimeSimulated_hr (hr) presumes time steps are in hour; Consider Refactor to allow for other assumptions
	int TimeSimulated_hr = TotalTimeSteps_cnt;
	//CrossSectionsSimulated_cnt represents the LengthSimulated_River_m (m) / dx_Interval_m (m), as computational nodes
	int CrossSectionsSimulated_cnt = (LengthSimulated_River_m / dx_Interval_m);

	// Crank-Nicolson method
	int TotalTimeMod_CN = 1;
	int TotalTimeUns = 1;
	int TotalTime_HR = 1;

	if (calcMethod == 1) {
		TotalTimeMod_CN = TotalTimeSteps_cnt;
	}

	//Call Calculations for calculationMethod1 
	CoolRiverOutputWriter::calculationMethod1(input, inputDirectory, st, calcMethod, LengthSimulated_River_m, TotalTimeMod_CN);

	if (calcMethod == 2) {
		//TotalTimeUns (min) converts TotalTimeSteps_cnt from hours to minutes
		TotalTimeUns = TotalTimeSteps_cnt * 60;
	}

	//Call Calculations for calculationMethod2 
	CoolRiverOutputWriter::calculationMethod2(input, inputDirectory, st, calcMethod, LengthSimulated_River_m, TotalTimeUns, CrossSectionsSimulated_cnt, TotalTimeSteps_cnt);

	if (calcMethod == 3) {
		//TotalTimeUns (min) converts TimeSimulated_hr from hours to minutes
		TotalTime_HR = (TimeSimulated_hr - 1) * 60;
	}

	//Call Calculations for calculationMethod3 
	CoolRiverOutputWriter::calculationMethod3(input, inputDirectory, st, calcMethod, LengthSimulated_River_m, TotalTime_HR, CrossSectionsSimulated_cnt);

	//switch function checks calcMethod value
	switch (calcMethod) {
	//case 1 when HydroPlusConfig.xml calcMethod = 1, steady
	case 1:

		if (input->CoolRiverVariables["CR_Minute_output"] == 0) {
			//When CR_Minute_output is 0, then we are setting values in output for calcMethod1
			CoolRiverOutputWriter::swtichCalMethod1True(input, st, LengthSimulated_River_m, TotalTimeSteps_cnt);
		}
		else {
			//When CR_Minute_output is other than 0, then we are setting values in output for calcMethod1
			CoolRiverOutputWriter::swtichCalMethod1False(input, st, CrossSectionsSimulated_cnt, TotalTimeMod_CN);
		}
		break;

	//case 3 when HydroPlusConfig.xml calcMethod = 3, HEC-RAS steady
	case 3:
		if (input->CoolRiverVariables["CR_Minute_output"] == 0) {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
				//When CR_Minute_output is 0 and CR_Extended is 1, then we are setting values in output for calcMethod3
				CoolRiverOutputWriter::swtichCalMethod3ExtTrue(input, st, CrossSectionsSimulated_cnt, TimeSimulated_hr);

			}
			//When CR_Minute_output is 0, then we are setting values in output for calcMethod3
			CoolRiverOutputWriter::swtichCalMethod3True(input, st, CrossSectionsSimulated_cnt, TimeSimulated_hr);
		}
		else {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
				//When CR_Minute_output is other than 0 and CR_Extended is 1, then we are setting values in output for calcMethod3
				CoolRiverOutputWriter::swtichCalMethod3FalseExtTrue(input, st, CrossSectionsSimulated_cnt, TotalTime_HR);

			}
			//When CR_Minute_output is other than 0, then we are setting values in output for calcMethod3
			CoolRiverOutputWriter::swtichCalMethod3False(input, st, CrossSectionsSimulated_cnt, TotalTime_HR);
		}
		break;
	//case 2 when HydroPlusConfig.xml calcMethod = 2, unsteady
	case 2:
		if (input->CoolRiverVariables["CR_Minute_output"] == 0) {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
				//When CR_Minute_output is 0 and CR_Extended is 1, then we are setting values in output for calcMethod2
				CoolRiverOutputWriter::swtichCalMethod2ExtTrue(input, st, CrossSectionsSimulated_cnt, TotalTimeSteps_cnt);
			}
			//When CR_Minute_output is other than 0 , then we are setting values in output for calcMethod2
			CoolRiverOutputWriter::swtichCalMethod2True(input, st, CrossSectionsSimulated_cnt, TotalTimeSteps_cnt);
		}
		else {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
				//When CR_Minute_output other than 0 and CR_Extended is 1, then we are setting values in output for calcMethod2
				CoolRiverOutputWriter::swtichCalMethod2FalseExtTrue(input, st, CrossSectionsSimulated_cnt, TotalTimeSteps_cnt);

			}
			//When CR_Minute_output is other than 0, then we are setting values in output for calcMethod2
			CoolRiverOutputWriter::swtichCalMethod2False(input, st, CrossSectionsSimulated_cnt, TotalTimeSteps_cnt);
		}
		break;

	//default is when switch does not find case = 1, 2, or 3
	default:
		cout << "Warning: HydroPlusConfig.xml element CR_calcMethod not set to allowable options 1, 2, or 3." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HydroPlus model uses CR_calcMethod to select appropriate algorithms." << endl;
		cout << "Correction: Set CR_calcMethod to 1, 2, or 3 in the HydroPlusConfig.xml CoolRiver_Variables section." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	delete sc;

	cout << "Writing output to: " << input->SimulationStringParams["OutputFolder_Path"] << endl;
	cout << "===" << endl;
}
