#include "SnowAblationUnderShortVegCalc.h"

//References:
//Lundquist, J. D., Dickerson-Lange, S., Gutmann, E., Jonas, T., Lumbrazo, C., & Reynolds, D. (2021). Snow interception modelling: Isolated observations have led to many land surface models lacking appropriate temperature sensitivities. Hydrological Processes, 35(7), e14274. doi:https://doi.org/10.1002/hyp.14274
//Note: Lundquist et al. (2021) updates snow unloading and melt routines. 
//Sanow, J. E., Fassnacht, S. R., Kamin, D. J., Sexstone, G. A., Bauerle, W. L., & Oprea, I. (2018). Geometric Versus Anemometric Surface Roughness for a Shallow Accumulating Snowpack. Geosciences, 8(12), 463. 
//Note: Sanow et al. (2018) demonstrate methods to adjust snow roughness length based on snow depth; median value z0_snow = 0.024 used by UCAR Community Land Model
//Sexstone, G. A., Clow, D. W., Fassnacht, S. R., Liston, G. E., Hiemstra, C. A., Knowles, J. F., & Penn, C. A. (2018). Snow Sublimation in Mountain Environments and Its Sensitivity to Forest Disturbance and Climate Warming. Water Resources Research, 54(2), 1191-1211. doi:https://doi.org/10.1002/2017WR021172
//Note: Sexstone et al. (2018) use SnowModel and demonstrate relative rates of sublimation, melt, and wind on snow ablation for ground and canopy. Canopy loss not always > Ground loss.

//Consider refactor if snow intercepting, storing, unloading and below canopy snow is simulated differently for grass or other short vegetation
//SnowAblationUnderShortVegCalc::calculate function accounts for snow melt and snow sublimation, which combine to form ablation
//Note: Variables with snow depth are liquid and snow water equivalent depths, unless variable name contains SnowCrystal and is a frozen crystalline depth. 
void SnowAblationUnderShortVegCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	SnowAblationUnderShortVeg(input, folder, timeStep);
}

//SnowAblationUnderShortVegCalc::SnowAblationUnderShortVeg function accounts for snow melt and snow sublimation, which combine to form ablation
void SnowAblationUnderShortVegCalc::SnowAblationUnderShortVeg(Inputs *input, DataFolder *folder, int timeStep)
{

	double Storage_Snow_underSVegCanopy_m = folder->VarDict["Storage_Snow_underSVegCanopy_m"] + folder->VarDict["ThroughFall_Snow_SVegCanopy_m"];
	folder->VarDict["Storage_Snow_underSVegCanopy_m"] = Storage_Snow_underSVegCanopy_m;


	double ThroughFall_Rain_SVegCanopy_m = folder->VarDict["ThroughFall_Rain_SVegCanopy_m"];                                 
	double Tair_F = input->Tair_F[timeStep];  
	double RadiationNet_Wpm2 = input->RadiationNet_Wpm2[timeStep];								
	double Tdew_F = input->Tdew_F[timeStep];                             
	double UV = input->WindSpd_mps[timeStep];                  
	double Kp = 0.8;
	double Za = 2.0;
	double Zb = 2.0;



	double Mp = 0.0129 * ThroughFall_Rain_SVegCanopy_m * (Tair_F - 32);
	double Ms = 0.0000317;
	double Ml = 0.0000307 * (Tair_F - 32);
	double Mh = 0.0000044 * Kp* pow(Za*Zb, -(1.0 / 6.0)) * (Tair_F - 32) * UV;
	double Me = 0.0000124 * pow(Za*Zb, -(1.0 / 6.0)) * (Tdew_F - 32) * UV;
	double Mg = 0.000021;

	folder->VarDict["Melt_Snow_underSVegCanopy_m"] = Ms + Ml + Mp + Mg + 0.3 * (Mh + Me);   //0.3 can be changed to other value in range 0.3 to 1.0

	if (folder->VarDict["Melt_Snow_underSVegCanopy_m"] < 0.0)
		folder->VarDict["Melt_Snow_underSVegCanopy_m"] = 0.0;

	if (folder->VarDict["Melt_Snow_underSVegCanopy_m"] > folder->VarDict["Storage_Snow_underSVegCanopy_m"])
		folder->VarDict["Melt_Snow_underSVegCanopy_m"] = folder->VarDict["Storage_Snow_underSVegCanopy_m"];

	folder->VarDict["Storage_Snow_underSVegCanopy_m"] = folder->VarDict["Storage_Snow_underSVegCanopy_m"] - 
		folder->VarDict["Melt_Snow_underSVegCanopy_m"];

	//SublimationSnow_Potential_m (m) defined as VarDict SublimationSnow_Potential_m, which is generated by functor calculateGroundSublimation in OpenWaterEvaporationCalc::calculateGroundSublimationStatistical or calculateGroundSublimationTemperature
	//Note: From SimulationCoordinator::runCalculations calculateGroundSublimation functor was called, which computes SublimationSnow_Potential_m (m) of ground sublimation 
	//Note: To call functor from this function, function must contain (.., int timeStep, std::map<std::string, void(*)(Inputs* input, DataFolder* folder, int timeStep) >& functors)
	double SublimationSnow_Potential_m = folder->VarDict["SublimationSnow_Potential_m"];

	if (SublimationSnow_Potential_m > folder->VarDict["Storage_Snow_underSVegCanopy_m"])
		SublimationSnow_Potential_m = folder->VarDict["Storage_Snow_underSVegCanopy_m"];

	folder->VarDict["Storage_Snow_underSVegCanopy_m"] = folder->VarDict["Storage_Snow_underSVegCanopy_m"] - SublimationSnow_Potential_m;

	folder->VarDict["Precipitation_RainSnowMelt_underSVegCanopy_m"] = folder->VarDict["Melt_Snow_underSVegCanopy_m"] + ThroughFall_Rain_SVegCanopy_m;

	//SublimationSnow_underSVegCanopy_m (m) updated with SublimationSnow_Potential_m
	folder->VarDict["SublimationSnow_underSVegCanopy_m"] = SublimationSnow_Potential_m;
	//Control for negative values
	if (folder->VarDict["SublimationSnow_underSVegCanopy_m"] < 0) { folder->VarDict["SublimationSnow_underSVegCanopy_m"] = 0; }

	folder->VarDict["Storage_RainSnow_SVegCanopy_m"] = folder->VarDict["Storage_Rain_SVegCanopy_m"] + folder->VarDict["Storage_Snow_SVegCanopy_m"];
}
