#include "SnowAblationUnderTreeCalc.h"

//References:
//Lundquist, J. D., Dickerson-Lange, S., Gutmann, E., Jonas, T., Lumbrazo, C., & Reynolds, D. (2021). Snow interception modelling: Isolated observations have led to many land surface models lacking appropriate temperature sensitivities. Hydrological Processes, 35(7), e14274. doi:https://doi.org/10.1002/hyp.14274
//Note: Lundquist et al. (2021) updates snow unloading and melt routines. 
//Sanow, J. E., Fassnacht, S. R., Kamin, D. J., Sexstone, G. A., Bauerle, W. L., & Oprea, I. (2018). Geometric Versus Anemometric Surface Roughness for a Shallow Accumulating Snowpack. Geosciences, 8(12), 463. 
//Note: Sanow et al. (2018) demonstrate methods to adjust snow roughness length based on snow depth; median value z0_snow = 0.024 used by UCAR Community Land Model
//Sexstone, G. A., Clow, D. W., Fassnacht, S. R., Liston, G. E., Hiemstra, C. A., Knowles, J. F., & Penn, C. A. (2018). Snow Sublimation in Mountain Environments and Its Sensitivity to Forest Disturbance and Climate Warming. Water Resources Research, 54(2), 1191-1211. doi:https://doi.org/10.1002/2017WR021172
//Note: Sexstone et al. (2018) use SnowModel and demonstrate relative rates of sublimation, melt, and wind on snow ablation for ground and canopy. Canopy loss not always > Ground loss.

//Note: Variables with snow depth are liquid and snow water equivalent depths, unless variable name contains SnowCrystal and is a frozen crystalline depth. 
//SnowAblationUnderTreeCalc::calculate function accounts for snow melt and snow sublimation, which combine to form ablation
void SnowAblationUnderTreeCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	SnowAblationUnderTree(input, folder, timeStep);
}

//SnowAblationUnderTreeCalc::SnowAblationUnderShortVeg function accounts for snow melt and snow sublimation, which combine to form ablation
void SnowAblationUnderTreeCalc::SnowAblationUnderTree(Inputs *input, DataFolder *folder, int timeStep)
{

	// Update the Storage_Snow_underTreeCanopy_m to include the SWE throughfall
	double Storage_Snow_underTreeCanopy_m = folder->VarDict["Storage_Snow_underTreeCanopy_m"] + folder->VarDict["ThroughFall_Snow_TreeCanopy_m"];
	folder->VarDict["Storage_Snow_underTreeCanopy_m"] = Storage_Snow_underTreeCanopy_m;


	// Placing the input air temp into Tair_F
	double Tair_F = input->Tair_F[timeStep];  
	// Placing the tree throughfall into ThroughFall_Rain_TreeCanopy_m (this is folder (instead of timestep) because it is stored for each timestep once in each folder - this is not the vector that stores every timestep)
	double ThroughFall_Rain_TreeCanopy_m = folder->VarDict["ThroughFall_Rain_TreeCanopy_m"];  
	// Placing the input solar radiation into RadiationNet_Wpm2
	double RadiationNet_Wpm2 = input->RadiationNet_Wpm2[timeStep];		
	// Placing the input dew Temperature into Tdew_F 
	double Tdew_F = input->Tdew_F[timeStep]; 
	// Placing the input windspeed into UV
	double UV = input->WindSpd_mps[timeStep];                     

	// Kp, Za, Zb are initialized in the SnowMeltUnderTreeRepo constructor - Kp(0.8), Za(2.0), Zb(2.0)
	// kp is the ratio of atmospheric pressure at the location to the atmospheric pressure at the sea level
	// za (m) is the height at which the temperature and the air vapor pressure are measured
	// zb (m) is the height of wind velocity measurement

	double Kp = 0.8;
	double Za = 2.0;
	double Zb = 2.0;

	// Mp = snow melted by rain
	double Mp = 0.0129 * ThroughFall_Rain_TreeCanopy_m * (Tair_F - 32);
	// Ms (m/hr) = snow melt by short wave radiation
	double Ms = 0.0000317;
	// Ml (m/hr) = snow melt by long wave radiation 
	double Ml = 0.0000307 * (Tair_F - 32);
	// Mh (m/hr) = the snow melt by sensible heat = 0.0000044*Kp*(Za*Zb)^(-1/6) * (Tair_F - 32) * UV
	// The original equation used this constant: 0.00000792.  This equation applies the fraction (5.0/9.0), from conversion of F to C, to get 0.0000044
	double Mh = 0.0000044 * Kp * pow(Za*Zb, -(1.0/6.0)) * (Tair_F - 32) * UV;
	// Me (m/hr) = snowmelt by latent heat.  0.0000124 has been arrived by applying (5.0/9.0) to 0.00000223
	double Me = 0.0000124 * pow(Za*Zb, -(1.0/6.0)) * (Tdew_F - 32) * UV;
	// Mg (m/hr) = snow melt from the ground heat
	double Mg = 0.000021;
	
	// Modify the MeltUnderTree amount = melted snow from all the different melt pathways
	folder->VarDict["Melt_Snow_underTreeCanopy_m"] = Ms + Ml + Mp + Mg + 0.3*(Mh + Me);       
	//0.3 is the wind coefficient and can be changed to other value in range 0.3 to 1.0

	// If the melted snow is somehow less than 0 (meaning snow was gained through the melt processes) set it to 0
	if (folder->VarDict["Melt_Snow_underTreeCanopy_m"] < 0)
		folder->VarDict["Melt_Snow_underTreeCanopy_m"] = 0.0;

	// If the melted snow is > Storage_Snow_noCanopy_m, set it equal to Storage_Snow_noCanopy_m
	if (folder->VarDict["Melt_Snow_underTreeCanopy_m"] > folder->VarDict["Storage_Snow_underTreeCanopy_m"])
		folder->VarDict["Melt_Snow_underTreeCanopy_m"] = folder->VarDict["Storage_Snow_underTreeCanopy_m"];

	// Update the Storage_Snow_underTreeCanopy_m to reflect the loss of the melted snow
	Storage_Snow_underTreeCanopy_m = folder->VarDict["Storage_Snow_underTreeCanopy_m"] - folder->VarDict["Melt_Snow_underTreeCanopy_m"];
	folder->VarDict["Storage_Snow_underTreeCanopy_m"] = Storage_Snow_underTreeCanopy_m;

	//SublimationSnow_Potential_m (m) defined as VarDict SublimationSnow_Potential_m, which is generated by functor calculateGroundSublimation in OpenWaterEvaporationCalc::calculateGroundSublimationStatistical or calculateGroundSublimationTemperature
	//Note: From SimulationCoordinator::runCalculations calculateGroundSublimation functor was called, which computes SublimationSnow_Potential_m (m) of ground sublimation 
	//Note: To call functor from this function, function must contain (.., int timeStep, std::map<std::string, void(*)(Inputs* input, DataFolder* folder, int timeStep) >& functors)
	double SublimationSnow_Potential_m = folder->VarDict["SublimationSnow_Potential_m"];

	// The amount of snow that could sublimate is > snow available, set equal to Storage_Snow_underTreeCanopy_m (snow available)
	if (SublimationSnow_Potential_m > folder->VarDict["Storage_Snow_underTreeCanopy_m"])
		SublimationSnow_Potential_m = folder->VarDict["Storage_Snow_underTreeCanopy_m"];

	// Update Storage_Snow_underTreeCanopy_m area to reflect the loss of the sublimated snow
	Storage_Snow_underTreeCanopy_m = folder->VarDict["Storage_Snow_underTreeCanopy_m"] - SublimationSnow_Potential_m;
	folder->VarDict["Storage_Snow_underTreeCanopy_m"] = Storage_Snow_underTreeCanopy_m;

	// Update the Precipitation_RainSnowMelt_underTreeCanopy_m (this folder, this timestep) to be = (RainOpen (Precip_mpdt falling on the open area) + Melt_Snow_underTreeCanopy_m)
	folder->VarDict["Precipitation_RainSnowMelt_underTreeCanopy_m"] = folder->VarDict["Melt_Snow_underTreeCanopy_m"] + ThroughFall_Rain_TreeCanopy_m;

	//SublimationSnow_underTreeCanopy_m (m) updated with SublimationSnow_Potential_m
	folder->VarDict["SublimationSnow_underTreeCanopy_m"] = SublimationSnow_Potential_m;
	//Control for negative values
	if (folder->VarDict["SublimationSnow_underTreeCanopy_m"] < 0) { folder->VarDict["SublimationSnow_underTreeCanopy_m"] = 0; }

	folder->VarDict["Storage_RainSnow_TreeCanopy_m"] = folder->VarDict["Storage_Rain_TreeCanopy_m"] + folder->VarDict["Storage_Snow_TreeCanopy_m"];

}

