﻿#include "Inputs.h"
#include "TerrainProcessor.h"
#include "WeatherPreprocessor.h"
#include "../Buffer/Buffer.h"
#include <random>
#include <iostream>
#include <string>
#include <algorithm>
#include <vector>

unordered_map<string, double> Inputs::LocationParams;
unordered_map<string, string> Inputs::BufferSpatialWeightingStringParams;
vector<double> Inputs::FlowAccum;
int  Inputs::ncols;
int  Inputs::nrows;
double Inputs::cellsize;
double Inputs::xllcorner;
double Inputs::yllcorner;
double Inputs::Length_Pixel_Side_m;
vector<double> Inputs::Waters01, Inputs::Waters0;
vector<double> Inputs::NHDfdr_reverse, Inputs::NHDfdr, Inputs::DEMfdr, Inputs::negDEMfdr;
unordered_map<string, string> Inputs::SimulationStringParams;
vector<double> Inputs::ImperCoverData, Inputs::PerCoverdata;
vector<double> Inputs::s_trans;
double  Inputs::s_trans_avg, Inputs::s_depth_avg;
double Inputs::NODATA;
vector<double> Inputs::s_depth;
vector<double>  Inputs::s_Ksat;
vector<double> Inputs::s_wtable;
vector<double>  Inputs::gi_area, Inputs::gi_c_area;


//Inputs constructor receives string Directory_Input_CLArg which is command line path with backslash
Inputs::Inputs(string& path) : Directory_Input_CLArg(path)
{
	cout << "============================================" << endl;
	cout << "Launching i-Tree HydroPlus. " << endl;
	cout << "Reading input from: " << Directory_Input_CLArg << endl;
	cout << "============================================" << endl;

	//Call function to read HydroPlusConfig.xml, 
	//Note: This SimulationStringParam[""], SimulationNumericalParams[""], and LocationParams[""] terms
	readHydroPlusConfigFile();

	//SimulationStringParams["InputDirectory"] set to value Directory_Input_CLArg
	SimulationStringParams["InputDirectory"] = Directory_Input_CLArg;

	//If Model_Selection is SpatialBuffer or SpatialBufferwGI then delete the z_ files
	if (SimulationStringParams["Model_Selection"] == "SpatialBuffer" || SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {

		// List of files to delete
		vector<string> filesToDelete = {
			Directory_Input_CLArg + "z_TIorganizer.asc",
			Directory_Input_CLArg + "z_SlopeGround_rad.asc",
			Directory_Input_CLArg + "z_AspectGround_N_0_rad.asc",
			Directory_Input_CLArg + "z_FDorganizer.asc",
			Directory_Input_CLArg + "z_FlowAccum.asc",
			Directory_Input_CLArg + "z_expdecayTI.dat",
			Directory_Input_CLArg + "z_powdecayTI.dat"
		};

		// Loop through the files and delete if they exist
		for (const auto& file : filesToDelete) {
			//If EnsureFileExists is true then enter
			if (EnsureFileExists(file)) {
				//If remove(file.c_str() is zero then notify it was deleted
				if (remove(file.c_str()) == 0) {
					cout << "Notice: Deleted: " << file << endl;
				}
			}
		}
	}
	//Note: erase function shortens the length of a string; begin and end return iterators pointing to position in string; 
	//OutputDirectory_String initialized as SimulationStringParams["OutputFolder_Path"] from HydroPlusConfig.xml 
	string OutputDirectory_String = SimulationStringParams["OutputFolder_Path"];
	//OutputDirectory_String is updated by EnsureTrailingBackslash function to ensure it has no parenthesis and ends with backslash
	OutputDirectory_String = EnsureTrailingBackslash(OutputDirectory_String);
	//SimulationStringParams["OutputFolder_Path"] redefined as OutputDirectory_String
	SimulationStringParams["OutputFolder_Path"] = OutputDirectory_String;

	//If SimulationStringParams["Model_Selection"] not ECDynamic then readWeatherFile, readEvaporationFile, and readPollutionData functions
	if (SimulationStringParams["Model_Selection"] != "ECDynamic" && SimulationStringParams["Model_Selection"] != "SpatialBuffer")	{
		if ((SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro" && SimulationStringParams["Flag_MultipleStation"] == "0" && SimulationStringParams["Flag_HottestDaySimulated"] == "1")) {
			cout << "Warning: The HydroPlusConfig.xml has parameter Flag_HottestDaySimulated = 1 when Flag_MultipleStation = 0." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: Hottest day option only works for multiple station scenarios in CoolAir." << endl;
			cout << "Correction: If you do not wish to auto simulate the hottest day for one weather station, then ..." << endl;
			cout << "... in the HydroPlusConfig.xml file, set the Flag_HottestDaySimulated = 0." << endl;
			cout << "Correction: If you wish to auto simulate the hottest day for one weather station, then ..." << endl;
			cout << "... in the HydroPlusConfig.xml file, set the Flag_MultipleStation = 1, and provide the inputs for one station ..." << endl;
			cout << "... in a file linked by its path in the HydroPlusConfig.xml element WeatherStationAttributeFile." << endl;
			cout << "... An example is available in the folder TestingFilesAndScript\TestCases\SpatialCoolAir\MultipleStations\." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}
		//If (Model = SpatialTemperatureHydro or StatisticalHydro) & Flag_MultipleStation = 1 then call readMultipleWeatherStations_List_CSV function
		if ((SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro" || SimulationStringParams["Model_Selection"] == "StatisticalHydro") && SimulationStringParams["Flag_MultipleStation"] == "1") {
			//Call readMultipleWeatherStations_List_CSV function
			readMultipleWeatherStations_List_CSV();
		}
		//Else If Model_Selection = SpatialTemperatureHydro or StatisticalHydro without Flag_MultipleStation then
		else {
			//cte 2025 Refactor to remove bool nature of readWeatherFile and readRadiationFile and readEvaporationFile files
			//WeatherPreprocessor::readWeatherFile function sends this as Inputs pointer to process Weather.csv input data
			WeatherPreprocessor::readWeatherFile(this);
			//WeatherPreprocessor::readRadiationFile function sends this as Inputs pointer to process Radiation.csv input data
			WeatherPreprocessor::readRadiationFile(this);
			//HydroPlusConfig_StopDay_str_YYYYMMDD is string form of HydroPlusConfig.xml input StopDate_YYYYMMDD
			string HydroPlusConfig_StopDay_str_YYYYMMDD = to_string(SimulationNumericalParams["StopDate_YYYYMMDD"]);
			//HydroPlusConfig_StopDay_YYYYMMDD is integer form of substring from HydroPlusConfig_StopDay_str_YYYYMMDD, taking first 8 elements
			int HydroPlusConfig_StopDay_YYYYMMDD = atoi(HydroPlusConfig_StopDay_str_YYYYMMDD.substr(0, 8).c_str());
			//WeatherInput_StopDay_YYYYMMDD is integer form of Weather.csv input vector last item
			int WeatherInput_StopDay_YYYYMMDD = SimulationDate_GD.back();
			//If WeatherInput_StopDay_YYYYMMDD < HydroPlusConfig_StopDay_YYYYMMDD then alert user and abort
			if (WeatherInput_StopDay_YYYYMMDD < HydroPlusConfig_StopDay_YYYYMMDD) {
				cout << "Warning: The Weather.csv 1st column for date, YYYYMMDD, has a ending date of " << WeatherInput_StopDay_YYYYMMDD << " ..." << endl;
				cout << "... which is earlier than the HydroPlusConfig.xml parameter StopDate_YYYYMMDD, set to " << HydroPlusConfig_StopDay_YYYYMMDD << "." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: The HydroPlus model needs Weather.csv to contain all dates up to the StopDate_YYYYMMDD." << endl;
				cout << "Correction: Either generate new Weather.csv or change the StopDate_YYYYMMDD in the HydroPlusConfig.xml file." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}
			//Check if Radiation and Weather files have same length
			//Length_RadiationInputs is integer length of 1st variable from Radiation.csv
			int Length_RadiationInputs = Radiation_Shortwave_Direct_Wpm2.size();
			//Length_WeatherInputs is integer length of 1st variable from Weather.csv
			int Length_WeatherInputs = Tair_F.size();

			//If Length_WeatherInputs not equal to Length_RadiationInputs then alert user and abort
			if (Length_WeatherInputs != Length_RadiationInputs) {
				cout << "Warning: The Weather.csv has " << Length_WeatherInputs << " items, but Radiation.csv has " << Length_RadiationInputs << " items." << endl;
				cout << "This program cannot verify if they are correct datasets properly overlapping in time." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: The HydroPlus model needs Weather.csv and Radiation.csv to contain identical rows of data." << endl;
				cout << "Correction: Generate new Weather.csv or Radiation.csv data, or make those datasets have the same row count and dates." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}

			//If Flag_ReadEvaporationData = 1 then proceed to call readEvaporationFile function
			if (SimulationStringParams["Flag_ReadEvaporationData"] == "1" && SimulationStringParams["Model_Selection"] != "SpatialTemperatureHydro") {
				//WeatherPreprocessor::readEvaporationFile function sends this as Inputs pointer to process Evaporation.csv input data
				WeatherPreprocessor::readEvaporationFile(this);
				//Check if Evaporation and Weather files have same length
				//Length_EvaporationInputs is integer length of 1st variable from Evaporation.csv
				int Length_EvaporationInputs = PotentialEvapotranspiration_TreeCover_m.size();
				//Length_WeatherInputs is integer length of 1st variable from Weather.csv
				int Length_WeatherInputs = Tair_F.size();
				//cte 2024 Debug to determine why this does not work for CoolRiver_Unsteady TestCase; Perhaps not best test, given lengths could be different as long as both contain the proper simulation dates
				/*
				//If Length_WeatherInputs not equal to Length_EvaporationInputs then alert user and abort
				if (Length_WeatherInputs != Length_EvaporationInputs) {
					cout << "Warning: The Weather.csv has " << Length_WeatherInputs << " items, but Evaporation.csv has " << Length_EvaporationInputs << " items." << endl;
					cout << "This program cannot verify if they are correct datasets properly overlapping in time." << endl;
					cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
					cout << "Explanation: The HydroPlus model needs Weather.csv and Evaporation.csv to contain identical rows of data." << endl;
					cout << "Correction: Generate new Weather.csv or Evaporation.csv data, or make those datasets have the same row count and dates." << endl;
					//Call abort function, which ends the HydroPlus.exe simulation
					abort();
				}
				*/
			}
			//If HydroPlusConfig.xml element Flag_CompareToObservedRunoff not equal to 0 then read Observed Runoff
			//Note: HydroPlusConfig.xmls element Flag_CompareToObservedRunoff can equal 0, 1 (routed runoff), or 2 (unrouted runoff)
			if (SimulationStringParams["Flag_CompareToObservedRunoff"] != "0") {
				//readRunoff_ObservedFile function will process Runoff_Observed_for_Calibration.csv input data
				readRunoff_ObservedFile();
			}
		}
		//If Model_Selection not CoolRiver then call readPollutionData function to process input data
		//Note: If condition above included CoolRiver with SpatialTemperatureHydro and Statistical Hydro to read in Weather.csv etc.
		if (SimulationStringParams["Model_Selection"] != "CoolRiver") {
			//Note: Pollution input data are optional, as default values are stored within code
			readPollutionData(Directory_Input_CLArg);
		}
	}

	if (SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro")	{
		//Call ProcessTemperatureExecutionParams function to read HydroPlusConfig.xml inputs from TemperatureExecutionParams section
		//Note: This call defines MapPixel_ReferenceStation_ID as function of HydroPlusConfig.xml RefWeatherLocationRowCol and nCols
		//Note: MapPixel_ReferenceStation_ID = row_ReferenceStation * int(LocationParams["nCols"]) + col_ReferenceStation
		//Note: This call must occur before Inputs::calcNLCDLandCoverData function assigns land cover properties in calcNLCDLandCoverData
		ProcessTemperatureExecutionParams();
	
		//Call readNLCDLandCoverFile function to read landcover.asc file containing NLCD codes (e.g., 11, 21, 22, .., 41, etc.) for all pixels with data
		readNLCDLandCoverFile(Directory_Input_CLArg, LocationParams["NODATA"]);

		//Call readImperviousFile function to read imperviouscover.asc file containing impervious cover percent for all pixels with data
		//Note: Area_Impervious_frac computed for use in TerrainProcessor to compute TopUrban form of topographic index
		readImperviousFile(Directory_Input_CLArg, LocationParams["NODATA"]);

		//Call readTreeCoverFile function
		readTreeCoverFile(Directory_Input_CLArg, LocationParams["NODATA"]);
		
		//Call readAnthropogenicHeatMap to read anthropenic heat maps obtained from AH4GUC
		//Note: If Flag_CoolAir_AnthropogenicHeat equals 0, and no AH4GUC maps are available, zeros will be assigned
		readAnthropogenicHeatMap(Directory_Input_CLArg, LocationParams["NODATA"]);

		//If HydroPlusConfig.xml element RefWeatherLocationNLCD_Class > 0 then enter
		//Note: Else it was text for Map and reset to -1 in Inputs::ProcessTemperatureExecutionParams 
		//Note: This conditional must follow Inputs::readNLCDLandCoverFile, and precede Inputs::calcNLCDLandCoverData
		//Note: DEM[MapPixel_ReferenceStation_ID], slope, aspect for reference station defined in BuildDataOrganizer::BuildSpatialDataOrganizer
		if (TemperatureCalculationParams["RefWeatherLocationNLCD_Class"] > 0) {
			//NLCDLandCoverData at MapPixel_ReferenceStation_ID converted from value in landcover.asc to value in HydroPlusConfig.xml
			NLCDLandCoverData[MapPixel_ReferenceStation_ID] = TemperatureCalculationParams["RefWeatherLocationNLCD_Class"];
		}

		//If HydroPlusConfig.xml element RefWeatherLocationTreeCover_percent >= 0 then enter
		//Note: Else it was text for Map and reset to -1 in Inputs::ProcessTemperatureExecutionParams 
		//Note: This conditional must follow Inputs::readNLCDLandCoverFile, and precede Inputs::calcNLCDLandCoverData
		//Note: DEM[MapPixel_ReferenceStation_ID], slope, aspect for reference station defined in BuildDataOrganizer::BuildSpatialDataOrganizer
		if (TemperatureCalculationParams["RefWeatherLocationTreeCover_percent"] >= 0) {
			//TreeCoverData (fraction) is RefWeatherLocationTreeCover_percent (%) * Ratio_Decimal_to_Percent; consistent with Inputs::readTreeCoverFile units
			TreeCoverData[MapPixel_ReferenceStation_ID] = TemperatureCalculationParams["RefWeatherLocationTreeCover_percent"] * Ratio_Decimal_to_Percent;
		}
		//If HydroPlusConfig.xml element RefWeatherLocationImperviousCover_percent >= 0 then enter
		//Note: Else it was text for Map and reset to -1 in Inputs::ProcessTemperatureExecutionParams 
		//Note: This conditional must follow Inputs::readNLCDLandCoverFile, and precede Inputs::calcNLCDLandCoverData
		//Note: DEM[MapPixel_ReferenceStation_ID], slope, aspect for reference station defined in BuildDataOrganizer::BuildSpatialDataOrganizer
		if (TemperatureCalculationParams["RefWeatherLocationImperviousCover_percent"] >= 0) {
			//ImperCoverData (fraction) is RefWeatherLocationImperviousCover_percent (%) * Ratio_Decimal_to_Percent; consistent with Inputs::readImperviousFile units
			ImperCoverData[MapPixel_ReferenceStation_ID] = TemperatureCalculationParams["RefWeatherLocationImperviousCover_percent"] * Ratio_Decimal_to_Percent;
		}
		//If HydroPlusConfig.xml element RefWeatherLocationAnthropogenicHeatMax_Wpm2 >= 0 then enter
		//Note: Else it was text for Map and reset to -1 in Inputs::ProcessTemperatureExecutionParams 
		//Note: This conditional must follow Inputs::readAnthropogenicHeatMap, and precede Inputs::calcNLCDLandCoverData
		//Note: DEM[MapPixel_ReferenceStation_ID], slope, aspect for reference station defined in BuildDataOrganizer::BuildSpatialDataOrganizer
		if (SimulationStringParams["Flag_CoolAir_AnthropogenicHeat"] == "1" && TemperatureCalculationParams["RefWeatherLocationAnthropogenicHeatMax_Wpm2"] >= 0) {
			//ImperCoverData (fraction) is RefWeatherLocationImperviousCover_percent (%) * Ratio_Decimal_to_Percent; consistent with Inputs::readImperviousFile units
			AnthropogenicHeat_Max_W_p_m2[MapPixel_ReferenceStation_ID] = TemperatureCalculationParams["RefWeatherLocationAnthropogenicHeatMax_Wpm2"];
		}

		//Call readBlockGroupOutputFileMap function
		readBlockGroupOutputFileMap(Directory_Input_CLArg, LocationParams["NODATA"]);

		//Call calcNLCDLandCoverData function
		//Note: Depending on HydroPlusConfig.xml elements, function calcNLCDLandCoverData may change land cover properties
		calcNLCDLandCoverData(int(LocationParams["NumberOfLocations"]), LocationParams["NODATA"]);

		//If Flag_ExtendedOutputs = 1 And land cover change is designated by 1 of 4 elements then write maps of changed land cover
		//Note: Land cover changes when any of these elements are > 0: TreeCanopyCover_overImpervious_frac, Scenario_CoolAir_Base_TCI_to_TC_frac, Scenario_CoolAir_Alternative_TC_frac; or if this element is < 1: Scenario_CoolAir_Alternative_IC_frac
		if (SimulationStringParams["Flag_ExtendedOutputs"] == "1" && (RefParams["TreeCanopyCover_overImpervious_frac"] > 0 || Scenario_CoolAir_Base_TCI_to_TC_frac > 0 || Scenario_CoolAir_Alternative_TC_frac > 0 || Scenario_CoolAir_Alternative_IC_frac < 1)) {
			//Call function Inputs::writeAsciiMap("TreeOnPervious", TreeOnPervious), sending variable name and data to write map
			Inputs::writeAsciiMap("TreeOnPervious", TreeOnPervious);
			Inputs::writeAsciiMap("ImperviousNoTree", ImperviousNoTree);
			Inputs::writeAsciiMap("TreeOnImpervious", TreeOnImpervious);
			Inputs::writeAsciiMap("ShortVegCover", ShortVegCover);
			Inputs::writeAsciiMap("SoilCover", SoilCover);
			Inputs::writeAsciiMap("WaterCover", WaterCover);
		}
		//cte 2024 Refactor to handle cases when Flag_MultipleStation = 1 and using readMultipleWeatherStations_List_CSV
		/*
		//Latitude_NH_pos_dd and Longitude_WH_pos_dd computed
		//If not Flag_LatitudeLongitude_Computed then compute Latitude_Longitude_vectorPair_dd values
		if (!Flag_LatitudeLongitude_Computed) {
			//Inputs::LatitudeLongitude_Projection_Manager called
			Inputs::LatitudeLongitude_Projection_Manager(Latitude_Longitude_vectorPair_dd);
			//Flag_LatitudeLongitude_Computed set to true so this is not entered again
			Flag_LatitudeLongitude_Computed = true;
		}
		*/
	}
	//If Model_Selection equals StatisticalHydro then define Area_Impervious_frac
	if (SimulationStringParams["Model_Selection"] == "StatisticalHydro") {
		//Area_Impervious_frac (fraction) is sum of ImperviousCover_noTreeCanopy_frac, TreeCanopyCover_overImpervious_frac, WaterCover_noTreeCanopy_frac
		LocationParams["Area_Impervious_frac"] = RefParams["ImperviousCover_noTreeCanopy_frac"] + RefParams["TreeCanopyCover_overImpervious_frac"] + RefParams["WaterCover_noTreeCanopy_frac"];
	}

	//If Model_Selection equals CoolRiver then call readHecRasFile, readRiverSWTimeSeriesFile, readRiverGWTimeSeriesFile, readRiverTimeSeriesFile and readRiverStationSeriesFile functions
	if (SimulationStringParams["Model_Selection"] == "CoolRiver") {
		//Call readHecRasFile; 
		readHecRasFile();
		readRiverSWTimeSeriesFile(SimulationNumericalParams["StartDate_YYYYMMDD"], SimulationNumericalParams["StopDate_YYYYMMDD"]);
		readRiverGWTimeSeriesFile(SimulationNumericalParams["StartDate_YYYYMMDD"], SimulationNumericalParams["StopDate_YYYYMMDD"]);
		readRiverTimeSeriesFile(SimulationNumericalParams["StartDate_YYYYMMDD"], SimulationNumericalParams["StopDate_YYYYMMDD"]);
		readRiverStationSeriesFile(Directory_Input_CLArg, LocationParams["NODATA"]);
	}
	
	//If Model_Selection equals SpatialBuffer then call readNLCDLandCoverFile function
	if (SimulationStringParams["Model_Selection"] == "SpatialBuffer"|| SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
		if (SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
			//reads HydroPlusConfig.xml inputs for Spatial Buffer with GI
			ProcessSpatialBufferGIExecutionParams();
		}
		//Call readNLCDLandCoverFile function
		readNLCDLandCoverFile(Directory_Input_CLArg, LocationParams["NODATA"]);

		//If Flag_FlowDirection_Source equals NHD then call readWaterFeaturesMap and readNHDFlowDirectionMap functions
		//Note: Uses data from NHDplus (National Hydrography Data, managed by USEPA)
		if (BufferSpatialWeightingStringParams["Flag_FlowDirection_Source"] == "NHD") {
			//force flow routing method to be D8
			if (SimulationStringParams["Algorithm_FlowDirection"] != "D8") {
				SimulationStringParams["Algorithm_FlowDirection"] = "D8";
				//write the log message and warning message to user
				cout << "Warning: You've specified Flag_FlowDirection_Source as NHD, but Algorithm_FlowDirection is not set to D8." << endl;
				cout << "Algorithm_FlowDirection is now being forced to D8." << endl;
				cout << "Explanation: NHDPlus provides flow direction in D8 format, making Flag_FlowDirection_Source = D8 the only valid option for the subsequent computation." << endl;
			}
			//readWaterFeaturesMap function reads a pixel map of hydrography, such as rivers and lakes
			readWaterFeaturesMap();
			//readNHDFlowDirectionMap function reads NHD (National Hydrography Database) map of flow directions, corrected by NHD to allign with planimetric maps of hydrography
			readNHDFlowDirectionMap();
		}

		// Determine if subsurface calculations are enabled based on surface contribution flags
		bool isSubsurfaceCalculationEnabled = (
			BufferSpatialWeightingNumericalParams["ECTP_surface_contribution_frac"] < 1 ||
			BufferSpatialWeightingNumericalParams["ECTN_surface_contribution_frac"] < 1 ||
			BufferSpatialWeightingNumericalParams["ECTSS_surface_contribution_frac"] < 1
			);

		// Check if specific subsurface conditions require SSURGO data
		if (isSubsurfaceCalculationEnabled) {
			//readHydraulicConductivityMap_umps function obtains SSURGO hydraulic conductivity (um/s)
			readHydraulicConductivityMap_umps();
			//readSoilDepthMap_cm function obtains SSURGO soil depth (cm)
			readSoilDepthMap_cm();
			//calculateSoilTransmissivityChangeUnits function converts transmissivity from SSURGO units to HydroPlus model units
			calculateSoilTransmissivityChangeUnits();

			// Check if subsurface slope method uses the water table slope, which requires SSURGO data
			if (BufferSpatialWeightingStringParams["Flag_ElevationMethod_Subsurface"] == "WEM") {
				//if the p and s_WTable_avg can't be read, then a s_wtable map is needed
				if (!(BufferSpatialWeightingNumericalParams["f_param"] > 0 && BufferSpatialWeightingNumericalParams["S_WTable_avg"] > 0)){
					//read watertable map;
					if (!readWatertableMap_cm()) {
						// read f, k_avg or read watertable map
						cout << "Warning: Flag_ElevationMethod_Subsurface is set to WEM, but required parameters or watertable map are missing." << endl;
						cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
						cout << "Explanation: Using of watertable elevation requires either a water table map file (s_wtable.asc) or values for 'f_param' and 'S_WTable_avg' (average soil water table depth and transmissivity)." << endl;
						cout << "Correction: Ensure that 's_wtable.asc' is available in the input directory, or provide both 'f_param' and 'S_WTable_avg' values in the configuration file." << endl;
						abort();
					}
				}	
			}
		}
	}

	//If Model_Selection other than ECDynamic and CoolRiver then call ProcessTopographyData function
	if (SimulationStringParams["Model_Selection"] != "ECDynamic" && SimulationStringParams["Model_Selection"] != "CoolRiver") {
		//cte 2025 Refactor and determine if imperviousArea_mapAverage_frac is used, and whether ImperviousArea_total_frac was properly defined
		//Call ProcessTopographyData function
		ProcessTopographyData();
	}

	//If SimulationStringParams["Model_Selection"] equals ECDynamic then enter 
	if (SimulationStringParams["Model_Selection"] == "ECDynamic" || SimulationStringParams["Model_Selection"] == "SpatialBuffer" || SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
		//If inputs do not include NLCD map or discharge input, then just generate export coefficient database
		if (SimulationStringParams["Model_Selection"] == "ECDynamic" && (readNLCDLandCoverFile(Directory_Input_CLArg, LocationParams["NODATA"]) == false || readDischargeData(Directory_Input_CLArg) == false)) {
			cout << "Calculating export coefficient database without running simulation ..." << endl;
			//run_ECDynamicModel set to zero
			run_ECDynamicModel = 0;
		}
		//Else if NLCD map and discharge data are included then were already been read in the previous if condition then
		else {
			cout << "Calculating export coefficient database." << endl;
			
		}
		//Call readExportCoefficientTable function to read the White et al. (2015) database
		readExportCoefficientTable();
	}
}

//void Inputs::ProcessTopographyData function will manage calls to TerrainProcessor function
void Inputs::ProcessTopographyData()
{
	//initialize numfolders as number of folders, each folder with a unique water balance (e.g, map pixel in spatial distribution, or bulk area in statistical distribution)
	double numfolders = LocationParams["NumberOfLocations"];
	//initialize NODATA_code as NODATA
	double NODATA_code = LocationParams["NODATA"];
	//initialize NAC as number of topographic index bins TopographicIndexBins
	//Note: TI_Value.size() determined by NAC when reading in dem.asc, 
	//Note: otherwise, TI_Value.size() determined based on rows of data within z_expdecayTI.dat or z_powdecayTI.dat
	int NAC = int(SimulationNumericalParams["TopographicIndexBins"]);
	//if NAC value is not greater than 0 
	if (NAC * 1 > 0) {}
	//reset NAC to default size of 30 bins
	else {
		NAC = 30;
		cout << "Inputs: The number of topographic index bins, NAC, was reset to: " << NAC << endl;
	}

	//TerrainProcessor object creates instance terrainProcessor and uses keyword = this to send input pointer
	TerrainProcessor terrainProcessor(this);

	//If Model_Selection is StatisticalHydro or SpatialBufferwGI then call TopographicIndex_Histogram_Read for TI Histogram
	if (SimulationStringParams["Model_Selection"] == "StatisticalHydro" || SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
		//terrainProcessor.TopographicIndex_Histogram_Read function called sending this as pointer to Inputs
		//Note: If z_powdecayTI.dat or z_expdecayTI.dat do not exist, or Flag_Recompute_TopographicIndex true, then TI is computed
		terrainProcessor.TopographicIndex_Histogram_Read(this);
	}
	//Else If Model_Selection is SpatialTemperatureHydro or CoolRiver or SpatialBuffer then call TopographicIndex_Map_Read for TI map
	else if (SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro" || SimulationStringParams["Model_Selection"] == "CoolRiver" || SimulationStringParams["Model_Selection"] == "SpatialBuffer") {
		//terrainProcessor.TopographicIndex_Map_Read function called sending this as pointer to Inputs
		//Note: If z_TIorganizer.asc or z_FDorganizer.asc do not exist, or Flag_Recompute_TopographicIndex true, then TI is computed
		terrainProcessor.TopographicIndex_Map_Read(this);
	}

	//TI_Value_Average is watershed area weighted average of Topographic Index, often called Lambda, computed with terrainProcessor.averageIndex
	TI_Value_Average = terrainProcessor.averageIndex;

	//cte 202403 Refactor TI_Value_Average 
	//Ratio_TI_Avg_Target_to_DEM is initialized to 1
	//Note: Ratio_TI_Avg_Target_to_DEM is ratio of topographic index average for drainage basin to dem.asc, equals 1 in ideal case
	double Ratio_TI_Avg_Target_to_DEM = 1.0;
	//If RefParams["Ratio_TI_Avg_Target_to_DEM"] is > 0 & <= 3 and defined then allow for redefinition of Ratio_TI_Avg_Target_to_DEM
	//Note: RefParams["Ratio_TI_Avg_Target_to_DEM"] is optional element in HydroPlusConfig.xml
	if (RefParams["Ratio_TI_Avg_Target_to_DEM"] > 0 && RefParams["Ratio_TI_Avg_Target_to_DEM"] <= 3) {
		//Ratio_TI_Avg_Target_to_DEM redefined based on HydroPlusConfig.xml element Ratio_TI_Avg_Target_to_DEM 
		Ratio_TI_Avg_Target_to_DEM = RefParams["Ratio_TI_Avg_Target_to_DEM"];
		//Notify user of adjustment
		cout << "Note: HydroPlusConfig.xml element Ratio_TI_Avg_Target_to_DEM will adjust TI_Value_Average by " << Ratio_TI_Avg_Target_to_DEM << "." << endl;
	}
	
	//TI_Value_Average = TI_Value_Average * Ratio_TI_Avg_Target_to_DEM, where Ratio_TI_Avg_Target_to_DEM is typically unity, 1
	//Note: If Ratio_TI_Avg_Target_to_DEM != 1 then TI_Value_Average adjusted to represent value from drainage basin rather than dem.asc map
	//Note: Concern is dem.asc may represent a non-drainage basin area, resulting in an TI_Value_Average value skewed to small or large 
	TI_Value_Average = TI_Value_Average * Ratio_TI_Avg_Target_to_DEM;

	//cte 2025 Refactor if there is a need for returning SZQ parameter
	//AveSMD (m) is terrainProcessor.getAveSoilMoistureDeficit(this) with this as pointer to Inputs for average soil moisture deficit
	AveSMD = terrainProcessor.getAveSoilMoistureDeficit(this);

	//Size_TI_Value_Vector is number of index values in TI_Value vector 
	//Note: equal to bins in statistically distributed, equal to count of map pixels with real data in spatially distributed
	Size_TI_Value_Vector = int(terrainProcessor.TI_Value.size());
	//TI_Value is topographic index (TI) vector
	TI_Value = terrainProcessor.TI_Value;
	//Size_TI_Area_Vector is number of topographic index values in TI fractional area vector (equal to bins + 1 in statistically distributed, equal to count of map pixels with real data in spatially distributed)
	Size_TI_Area_Vector = int(terrainProcessor.TI_Area_frac.size());
	//TI_Area_frac is fraction of catchment area associated with TI_Value value
	TI_Area_frac = terrainProcessor.TI_Area_frac;

	setrows(terrainProcessor.getrows());
	setcols(terrainProcessor.getcols());
	setxllcorner(terrainProcessor.getxllcorner());
	setyllcorner(terrainProcessor.getyllcorner());

	//cte 2025 Refactor to have TerrainProcessor::DEM_Read_and_Prepare create a DEM_raw_m file so we are not reading here
	//If Model is SpatialTemperatureHydro then enter to get maps of DEM and derived slope and aspect products
	if (SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro") {

		//If Flag_DEM_RemovePitsFlats equals 1 then read dem.asc again, after having filled pits
		//Note: Filled elevation data creates errors in Cool Air aspect and elevation, affecting estimated Tair_K
		if (Inputs::SimulationStringParams["Flag_DEM_RemovePitsFlats"] == "1") {
			//tempElv temporarily stores the read in elevation data (m)
			double tempElv;
			//initialize buffer vector, which contains characters along a row of DEM file, separated by commas
			char buffer[80];
			//file name is dem.asc, format recognized by QGIS and ArcGIS as input type for given header
			string location = Directory_Input_CLArg + "dem.asc";
			//ifstream function reads infile from location
			ifstream infile(location);
			//buffer vector takes on values of infile, up to first 80 columns set by MAX, separated by commas
			infile >> buffer;
			//Ascii_Header_str used to hold map header data read from infile
			string Ascii_Header_str = buffer;
			//if first line of header is ncols or NCOLS then read infile header values
			if (Ascii_Header_str == "ncols" || Ascii_Header_str == "NCOLS") {
				//infile defines nrows, columns, lower left xllcorner, yllcorner, cell size, nodata value
				infile >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> Length_Pixel_Side_m >> buffer >> NODATA;
			}
			//loop through rows in DEM file along row counter
			for (int row = 0; row < nrows; row++) {
				//loop through cols in DEM file along col counter
				for (int col = 0; col < ncols; col++) {
					//read elevation data from DEM file
					infile >> tempElv;
					DEM.push_back(tempElv);
				}
			}
		}
		//Else if Flag_DEM_RemovePitsFlats equals 0 then define DEM as elevation1D
		else {
			//DEM (m) vector derived from terrainProcessor.elevation1D
			DEM = terrainProcessor.elevation1D;
		}
		//cte 2025 Refactor to create SlopeGround_rad and AspectGround_N_0_rad within TerrainProcessor
		//terrainProcessor.Ground_Slope_and_Aspect_File_Read function to read in z_SlopeGround_rad.asc and z_AspectGround_N_0_rad.asc
		terrainProcessor.Ground_Slope_and_Aspect_File_Read();
		//SlopeGround_rad is vector derived from terrainProcessor.slope1D, read from z_SlopeGround_rad.asc
		SlopeGround_rad = terrainProcessor.slope1D;
		//AspectGround_N_0_rad is vector derived from terrainProcessor.aspect1D, read from z_AspectGround_N_0_rad.asc
		AspectGround_N_0_rad = terrainProcessor.aspect1D;
		
	}
	//for Buffer only
	//cte 2025 Li Zhang: Why not use elevation1D and flowAccum1D in Buffer? Are you really using DEM? You weren't using Slope.
	//cte 2025 Refactor to create DEM and FlowAccum within TerrainProcessor
	if (SimulationStringParams["Model_Selection"] == "SpatialBuffer"|| SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
		DEM = terrainProcessor.elevation1D;
		FlowAccum = terrainProcessor.flowAccum1D;
		//FlowDir = terrainProcessor.flowDir1D;
		//Slope = terrainProcessor.slope1D;
	}
}

void Inputs::readHydroPlusConfigFile() {

	//Note: XMLReader contains six interacting .cpp files intended to extract data from inputs, such as HydroPlusConfig.xml, and build new document tree of element nodes: 
	//Note: Tokenizer.cpp remove whitespace and constructs new document using XMLElementParts.cpp
	//Note: Node.cpp creates nodes which Tree.cpp uses to create document tree with XML_Document.cpp

	//Create t as Tree class for organizing HydroPlusConfig.xml input
	Tree t;
	//Create doc as XML_Document class for containing HydroPlusConfig.xml file
	XML_Document doc;
	//Create root as pointer to Node within XML document
	Node* root;
	//Define root as XML_Document HydroPlusConfig.xml file, loaded from directory given by command prompt
	root = doc.Load(Directory_Input_CLArg + "HydroPlusConfig.xml", true);
	//Define temp as pointer to HydroPlusConfig node, known as child within XML_Document
	Node* temp = root->getChildByTagName("HydroPlusConfig");

	//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
	cout << "Reading HydroPlusConfig.xml file for element: SimulationStringParams" << endl;
	//Define SimulationStringParams and SimulationNumericalParams nodes in HydroPlusConfig.xml, and once defined they are used to call specific keys and their values
	//Define SimulationStringParams as branch off t Tree using WalkTree, found as child within HydroPlusConfig node called temp
	t.WalkTree(temp->getChildByTagName("SimulationStringParams"), SimulationStringParams);
	//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
	cout << "Reading HydroPlusConfig.xml file for element: SimulationNumericalParams" << endl;
	//Define SimulationNumericalParams as branch off t Tree using WalkTree, found as child within HydroPlusConfig node called temp
	t.WalkTree(temp->getChildByTagName("SimulationNumericalParams"), SimulationNumericalParams);

	//If SimulationStringParams["OutputFolder_Path"].size() or Model_Selection or Algorithm_SoilKsatDecay has no length, then the HydroPlusConfig.xml file is erroneous, likely outdated
	if (SimulationStringParams["OutputFolder_Path"].size() <= 0 || SimulationStringParams["Model_Selection"].size() <= 0 || SimulationStringParams["Algorithm_SoilKsatDecay"].size() <= 0) {
		cout << "Warning: HydroPlusConfig.xml elements needed in the SimulationStringParams section are not present." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: You may be trying to use an outdated HydroPlusConfig with a newer HydroPlus.exe." << endl;
		cout << "Correction: Update your HydroPlusConfig.xml elements using a ..\\TestingFilesAndScript\\TestCases template." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//String reader functions
	//refParamLoc string defined as key RefParamFolder, contained within node SimulationStringParams of HydroPlusConfig.xml
	string refParamLoc = SimulationStringParams["RefParamFolder"];
	//row defined using atoi function from substring of refParamLoc key, as value between starting point 0 and comma string
	int row = atoi(refParamLoc.substr(0, refParamLoc.find(",")).c_str());
	//col defined using atoi function from substring of refParamLoc key, as value between comma string + 1 size of refParamLoc string minus location of comma
	int col = atoi(refParamLoc.substr(refParamLoc.find(",") + 1, refParamLoc.size() - refParamLoc.find(",")).c_str());

	//Routine to establish StartDate_YYYYMMDD and StopDate_YYYYMMDD parameters for use in handling model simulation inputs, runtime, etc.
	//DateParts_Gregorian[4] integer array initialized to hold parts of date for ExtractGregorianDateParts function
	int DateParts_Gregorian[4];
	//ExtractGregorianDateParts function extracts StopDate_YYYYMMDD year, month, day, and hour date parts from Gregorian date, format YYYYMMDDHH
	ExtractGregorianDateParts(int(SimulationNumericalParams["StopDate_YYYYMMDD"]), DateParts_Gregorian);
	//SimulationDateStop_GD defined as combination of YYYYY0000 + MM00 + DD
	SimulationNumericalParams["SimulationDateStop_GD"] = DateParts_Gregorian[0] * 10000 + DateParts_Gregorian[1] * 100 + DateParts_Gregorian[2];
	//if DateParts_Gregorian[3] < 0 for StopDate_YYYYMMDD, then hour was not included in StopDate_YYYYMMDD, and it will be set to 23, last hour of day
	if (DateParts_Gregorian[3] < 0) {
		if (SimulationStringParams["Flag_HottestDaySimulated"] == "1") {
			//If StopDate_YYYYMMDD does not have an hour given, then the default is for 19:00. 
			DateParts_Gregorian[3] = 19;
		}
		else {
			//DateParts_Gregorian[3] of hour HH is defined as 23 for StopDate_YYYYMMDD, presuming user wants to simulate a complete day
			DateParts_Gregorian[3] = 23;
		}
		//SimulationDateStop_GDH contains Gregorian Date and Hour, extending 8 to 10 digits by multiplication with 100, adding hour
		SimulationNumericalParams["SimulationDateStop_GDH"] = (SimulationNumericalParams["StopDate_YYYYMMDD"] * 100) + DateParts_Gregorian[3];
	}
	else {
		//SimulationDateStop_GDH contains Gregorian Date and Hour as 10 digit number from input
		SimulationNumericalParams["SimulationDateStop_GDH"] = SimulationNumericalParams["StopDate_YYYYMMDD"];
	}
	//SimulationDateStop_JD created with GregorianToJulianDate function, converting StopDate_YYYYMMDD Gregorian date parts to Julian Date integer
	SimulationNumericalParams["SimulationDateStop_JD"] = GregorianToJulianDate(DateParts_Gregorian);
	//SimulationStop_Hour_HH created to contain EndDate hour
	SimulationNumericalParams["SimulationStop_Hour_HH"] = DateParts_Gregorian[3];

	//ExtractGregorianDateParts function extracts StartDate_YYYYMMDD year, month, day, and hour date parts from Gregorian date, format YYYYMMDDHH
	ExtractGregorianDateParts(int(SimulationNumericalParams["StartDate_YYYYMMDD"]), DateParts_Gregorian);
	//SimulationDateStart_GD defined as combination of YYYYY0000 + MM00 + DD
	//Note: DateParts_Gregorian[0] = YYYY, DateParts_Gregorian[1] = MM, DateParts_Gregorian[2] = DD
	SimulationNumericalParams["SimulationDateStart_GD"] = DateParts_Gregorian[0] * 10000 + DateParts_Gregorian[1] * 100 + DateParts_Gregorian[2];
	//if DateParts_Gregorian[3] < 0 for StartDate_YYYYMMDD, then hour was not included in StartDate_YYYYMMDD, and it will be set to 0, first hour of day
	//Note: DateParts_Gregorian[3] = HH
	if (DateParts_Gregorian[3] < 0) {
		//DateParts_Gregorian[3] of hour HH is defined as 0 for StartDate_YYYYMMDD, presuming user wants to simulate a complete day
		DateParts_Gregorian[3] = 0;
		//SimulationDateStart_GDH contains Gregorian Date and Hour, extending 8 to 10 digits by multiplication with 100, adding hour
		SimulationNumericalParams["SimulationDateStart_GDH"] = (SimulationNumericalParams["StartDate_YYYYMMDD"] * 100) + DateParts_Gregorian[3];
	}
	else {
		//SimulationDateStart_GDH contains Gregorian Date and Hour as 10 digit number from input
		SimulationNumericalParams["SimulationDateStart_GDH"] = SimulationNumericalParams["StartDate_YYYYMMDD"];
	}
	//SimulationDateStart_JD created with GregorianToJulianDate function, converting StartDate_YYYYMMDD Gregorian date parts to Julian Date integer
	SimulationNumericalParams["SimulationDateStart_JD"] = GregorianToJulianDate(DateParts_Gregorian);
	//SimulationStop_Hour_HH created to contain StartDate hour
	SimulationNumericalParams["SimulationStart_Hour_HH"] = DateParts_Gregorian[3];

	//SimulationDuration_Integer_Days is an integer comprised of the whole days within the simulation dates
	int SimulationDuration_Integer_Days;
	//SimulationStartStopDate_Hours is an integer adjusted down for missing hours in StartDate_YYYYMMDD, up for hours in StopDate_YYYYMMDD
	int SimulationStartStopDate_Hours = 0;
	//SimulationDuration_Integer_Hours is an integer comprised of the whole hours within the simulation dates
	int SimulationDuration_Integer_Hours;

	//TotalTimeSteps parameter will be function of StartDate and EndDate, accumulating simulation days, hours, and adjusting for time step duration
	//SimulationDuration_Integer_Days defined as difference between SimulationDateStop_JD and SimulationDateStart_JD
	//If run multiple stations and just for the hottest day, SimulationDuration_Integer_Days will always be 0
	if (SimulationStringParams["Flag_MultipleStation"] == "1" && SimulationStringParams["Flag_HottestDaySimulated"] == "1") {
		SimulationDuration_Integer_Days = 0;
	}
	else {
		SimulationDuration_Integer_Days = int(SimulationNumericalParams["SimulationDateStop_JD"] - SimulationNumericalParams["SimulationDateStart_JD"]);
	}

	//if SimulationDuration_Integer_Days < 0 then abort program
	if (SimulationDuration_Integer_Days < 0) {
		cout << "Warning: The HydroPlusConfig.xml has parameter StartDate_YYYYMMDD < StartDate_YYYYMMDD, and HydroPlus needs time to progress forward." << endl;
		cout << "Warning: To avoid this unrealistic parameterization, the StartDate_YYYYMMDD or StartDate_YYYYMMDD inputs should be adjusted." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//Analyze contribution of hours to number of simulation time steps
	//if SimulationDuration_Integer_Days equals zero, only 1 day or less is simulated and only simulation hours comprise the TotalTimeSteps
	if (SimulationDuration_Integer_Days == 0) {

		//if SimulationStop_Hour_HH < SimulationStart_Hour_HH for single day of simulation then abort program
		if (SimulationNumericalParams["SimulationStop_Hour_HH"] - SimulationNumericalParams["SimulationStart_Hour_HH"] < 0) {
			cout << "Warning: The HydroPlusConfig.xml has parameter StartDate_YYYYMMDD < StartDate_YYYYMMDD." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: The HydroPlus model needs to progress forward in time." << endl;
			cout << "Correction: In the HydroPlusConfig.xml file, set the StartDate_YYYYMMDD to equal or precede StartDate_YYYYMMDD." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}

		//SimulationStartStopDate_Hours is difference between hours in single day of simulation, plus 1 hour for completion of first hour
		SimulationStartStopDate_Hours = int(SimulationNumericalParams["SimulationStop_Hour_HH"] -
			SimulationNumericalParams["SimulationStart_Hour_HH"]) + 1;
	}

	//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
	cout << "Reading HydroPlusConfig.xml file for element: DataOrganizer" << endl;
	//Define DataOrganizer nodes in HydroPlusConfig.xml, and once defined they are used to call specific keys and their values
	//Define DataOrganizer as branch off t Tree using WalkTree, found as child within HydroPlusConfig node called temp
	t.WalkTree(temp->getChildByTagName("DataOrganizer"), DataDrawers, StringDataDrawers);
	if (row > DataDrawers.size() - 1 || row > StringDataDrawers.size() - 1) {
		cout << "RefParamFolder coordinates out of bounds. Check drawer index." << endl;
		exit(0);
	}
	if (col > DataDrawers[row].size() - 1 || col > StringDataDrawers[row].size() - 1) {
		cout << "RefParamFolder coordinates out of bounds. Check drawer index." << endl;
		exit(0);
	}
	RefParams = DataDrawers[row][col];

	RefStringParams = StringDataDrawers[row][col];

	//if Model equals SpatialTemperatureHydro then XML elements read into memory from HydroPlusConfig.xml sections
	if (SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro")
	{
		//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
		cout << "Reading HydroPlusConfig.xml file for CoolAir element: TemperatureCalculationParams" << endl;
		//TemperatureCalculationParams element from HydroPlusConfig.xml is called TemperatureCalculationParams, and all sub-elements are read into memory
		t.WalkTree(temp->getChildByTagName("TemperatureCalculationParams"), TemperatureCalculationParams);

		//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
		cout << "Reading HydroPlusConfig.xml file for CoolAir element: TemperatureExecutionParams" << endl;
		//TemperatureExecutionParams element from HydroPlusConfig.xml is called TemperatureExecutionParams, and all sub-elements are read into memory
		t.WalkTree(temp->getChildByTagName("TemperatureExecutionParams"), TemperatureExecutionParams);

		//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
		cout << "Reading HydroPlusConfig.xml file for CoolAir element: SimulationLocationParams" << endl;
		//locationParamsNode Node pointer takes on returned getChildByTagName function value of tagName SimulationLocationParams
		//Note: Once no HydroPlusConfig.xml files use TemperatureLocationParams as element tagName, then replace this line and if conditionals w ..
		//Note: ... t.WalkTree(temp->getChildByTagName("SimulationLocationParams"), LocationParams);
		Node* locationParamsNode = temp->getChildByTagName("SimulationLocationParams");

		//If locationParamsNode equals nullpter from getChildByTagName with tagName = SimulationLocationParams, then try older tagName 
		if (locationParamsNode == nullptr) {
			//locationParamsNode takes on returned getChildByTagName function value of tagName TemperatureLocationParams
			locationParamsNode = temp->getChildByTagName("TemperatureLocationParams");
		}
		//If locationParamsNode not equal to nullpter then t.WalkTree will function properly 
		if (locationParamsNode != nullptr) {
			//Note: t.WalkTree function assigns locationParamsNode to LocationParams, and all HydroPlusConfig.xml sub-elements are read into memory
			//Note: LocationParams["NODATA"] defined as value from HydroPlusConfig.xml LocationParams section, SimulationLocationParams
			t.WalkTree(locationParamsNode, LocationParams);
		}

		//NumberOfLocations defined as product of nCols and nRows from HydroPlusConfig.xml LocationParams section, SimulationLocationParams
		LocationParams["NumberOfLocations"] = LocationParams["nCols"] * LocationParams["nRows"];

	}
	else if (SimulationStringParams["Model_Selection"] == "SpatialBuffer" || SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {

		//cout message to notify user HydroPlusConfig.xml is being read for SimulationLocationParams
		cout << "Reading HydroPlusConfig.xml file for CoolAir element: SimulationLocationParams" << endl;
		//SimulationLocationParams element from HydroPlusConfig.xml is called LocationParams, and all sub-elements are read into memory
		t.WalkTree(temp->getChildByTagName("SimulationLocationParams"), LocationParams);
	}
	//Else, Model must equal StatisticalHydro and LocationParams not available from HydroPlusConfig.xml and will need to be defined below
	else
	{
		LocationParams["NumberOfLocations"] = 1.0;
		LocationParams["nCols"] = 1.0;
		LocationParams["nRows"] = 1.0;
		LocationParams["NODATA"] = -9999.0;
		LocationParams["cellsize"] = 1.0;
		LocationParams["xllcorner"] = 1.0;
		LocationParams["yllcorner"] = 1.0;
	}

	//if Model equals CoolRiver then XML elements read into memory from HydroPlusConfig.xml sections
	if (SimulationStringParams["Model_Selection"] == "CoolRiver")
	{
		//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
		cout << "Reading HydroPlusConfig.xml file for CoolRiver element: CoolRiver_Variables" << endl;
		//CoolRiver_Variables element from HydroPlusConfig.xml is called CoolRiverVariables, and all sub-elements are read into memory
		t.WalkTree(temp->getChildByTagName("CoolRiver_Variables"), CoolRiverVariables);
	}
	//if Model equals ECDynamic then XML elements read into memory from HydroPlusConfig.xml sections
	if (SimulationStringParams["Model_Selection"] == "ECDynamic")
	{
		//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
		cout << "Reading HydroPlusConfig.xml file for SpatialBuffer element: BufferNutrientsStringParams" << endl;
		//BufferNutrientsStringParams HydroPlusConfig.xml element called BufferNutrientsStringParams, sub-elements read into memory
		t.WalkTree(temp->getChildByTagName("BufferNutrientsStringParams"), BufferNutrientsStringParams);
		//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
		cout << "Reading HydroPlusConfig.xml file for ECDynamic element: BufferTemporalLoadingNumericalParams" << endl;
		//BufferTemporalLoadingNumericalParams element from HydroPlusConfig.xml is called BufferTemporalLoadingNumericalParams, and all sub-elements are read into memory
		t.WalkTree(temp->getChildByTagName("BufferTemporalLoadingNumericalParams"), BufferTemporalLoadingNumericalParams);
	}
	//If Model equals SpatialBuffer then XML elements read into memory from HydroPlusConfig.xml sections
	//cte 2024 Refactor to allow getChildByTagName to work when TemperatureExecutionParams precedes Buffer elements in HydroPlusConfig.xml
	if (SimulationStringParams["Model_Selection"] == "SpatialBuffer" || SimulationStringParams["Model_Selection"] == "SpatialBufferwGI") {
		//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
		cout << "Reading HydroPlusConfig.xml file for SpatialBuffer element: BufferNutrientsStringParams" << endl;
		//BufferNutrientsStringParams HydroPlusConfig.xml element called BufferNutrientsStringParams, sub-elements read into memory
		t.WalkTree(temp->getChildByTagName("BufferNutrientsStringParams"), BufferNutrientsStringParams);
		//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
		cout << "Reading HydroPlusConfig.xml file for SpatialBuffer element: BufferSpatialWeightingStringParams" << endl;
		//BufferSpatialWeightingStringParams HydroPlusConfig.xml element called BufferSpatialWeightingStringParams, sub-elements read into memory
		t.WalkTree(temp->getChildByTagName("BufferSpatialWeightingStringParams"), BufferSpatialWeightingStringParams);
		//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
		cout << "Reading HydroPlusConfig.xml file for SpatialBuffer element: BufferSpatialWeightingNumericalParams" << endl;
		//BufferSpatialWeightingNumericalParams HydroPlusConfig.xml element called BufferSpatialWeightingNumericalParams, sub-elements read into memory
		t.WalkTree(temp->getChildByTagName("BufferSpatialWeightingNumericalParams"), BufferSpatialWeightingNumericalParams);
	}
	//If Model equals CoolBuilding then XML elements read into memory from HydroPlusConfig.xml sections
	if (SimulationStringParams["Model_Selection"] == "CoolBuilding") {
		//cout message to notify user HydroPlusConfig.xml is being read, in case HydroPlusConfig.xml causes program to exit
		cout << "Reading HydroPlusConfig.xml file for CoolBuilding element: CoolBuildingStringParams" << endl;
		//CoolBuildingStringParams HydroPlusConfig.xml element called CoolBuildingStringParams, sub-elements read into memory
		t.WalkTree(temp->getChildByTagName("CoolBuildingStringParams"), CoolBuildingStringParams);
		//CoolBuildingNumericalParams HydroPlusConfig.xml element called CoolBuildingNumericalParams, sub-elements read into memory
		t.WalkTree(temp->getChildByTagName("CoolBuildingNumericalParams"), CoolBuildingNumericalParams);
	}
}

//Inputs::ProcessTemperatureExecutionParams function reads HydroPlusConfig.xml inputs for CoolAir 
//Note: TemperatureExecutionParams string variables converted to TemperatureCalculationParams numeric variables
void Inputs::ProcessTemperatureExecutionParams()
{
	//If TemperatureExecutionParams["PrintBlockGroupRange"] = all or All then HydroPlusConfig.xml has indicated all blockgroups should be printed, set flag to 1
	if (TemperatureExecutionParams["PrintBlockGroupRange"] == "all" || TemperatureExecutionParams["PrintBlockGroupRange"] == "All") {
		TemperatureCalculationParams["PrintAllBlockGroups"] = 1; }
	else {
		TemperatureCalculationParams["PrintAllBlockGroups"] = 0; 
	}
	//If TemperatureExecutionParams["RowColOutputPeriod"] = all or All then HydroPlusConfig.xml has indicated all time steps should be printed, set flag to 1
	if (TemperatureExecutionParams["RowColOutputPeriod"] == "all" || TemperatureExecutionParams["RowColOutputPeriod"] == "All")	{
		TemperatureCalculationParams["RowColOutputAllTimeSteps"] = 1;
	}
	//If TemperatureExecutionParams["PrintRowCol"] = all or All then HydroPlusConfig.xml has indicated all row col pairs should be printed, set flag to 1
	if (TemperatureExecutionParams["PrintRowCol"] == "all" || TemperatureExecutionParams["PrintRowCol"] == "All") {
		TemperatureCalculationParams["PrintAllRowColOutput"] = 1; 
	}
	//If TemperatureExecutionParams["TimeBasedOutputPeriod"] = all or All then HydroPlusConfig.xml has indicated all time steps should be printed, set flag to 1
	if (TemperatureExecutionParams["TimeBasedOutputPeriod"] == "all" || TemperatureExecutionParams["TimeBasedOutputPeriod"] == "All") {
		TemperatureCalculationParams["TimeBasedOutputAllTimeSteps"] = 1; 
	}
	//If TemperatureExecutionParams["PrintTimeBasedResults"] = all or All then HydroPlusConfig.xml has indicated all time based maps should be printed, set flag to 1
	if (TemperatureExecutionParams["PrintTimeBasedResults"] == "all" || TemperatureExecutionParams["PrintTimeBasedResults"] == "All") {
		TemperatureCalculationParams["PrintAllTimeBasedResults"] = 1; 
	}

	//Note: Reference Station DEM based properties (ground elevation, ground slope, ground aspect, TI) are read from HydroPlusConfig.xml or derived from the dem.asc file
	//Note: The use of the HydroPlusCongig.xml to select option facilitates simulations where meteorological station is not in simulated area.
	//cte 2024 Consider refactor to read "DEM" from HydroPlusConfig.xml inputs RefWeatherLocationElevation_m, RefWeatherLocationSlope_deg, RefWeatherLocationAspect_deg, and then set to -1, which is used in place of 0 for BuildDataOrganizer assignment of values

	//If RefWeatherLocationTopographicIndex == Avg or avg then set value equal to 1 as flag in SimulationCoordinator::RunTemperatureSpatialModel function
	//In RunTemperatureSpatialModel function, if RefWeatherLocationTopographicIndex == 1 then use TI_Value_Average value, else input->TI_Value defined by dem.asc in Inputs::ProcessTopographyData
	//Note: HydroPlusConfig.xml element RefWeatherLocationTopographicIndex is set to "DEM" or "Avg", and Avg is converted to int 1 below
	if (TemperatureExecutionParams["RefWeatherLocationTopographicIndex"] == "avg" || TemperatureExecutionParams["RefWeatherLocationTopographicIndex"] == "Avg")	{
		//RefWeatherLocationTopographicIndex set to 1 as indicator to use TI_Value_Average in SimulationCoordinator::RunTemperatureSpatialModel function
		TemperatureCalculationParams["RefWeatherLocationTopographicIndex"] = 1;	
	}
	//If RefWeatherLocationNLCD_Class == Map or map or MAP then set value equal to -1 as flag in Inputs::Inputs function to read data from landcover.asc
	if (TemperatureExecutionParams["RefWeatherLocationNLCD_Class"] == "Map" || TemperatureExecutionParams["RefWeatherLocationNLCD_Class"] == "map" || TemperatureExecutionParams["RefWeatherLocationNLCD_Class"] == "MAP") {
		TemperatureCalculationParams["RefWeatherLocationNLCD_Class"] = - 1;
	}
	//If RefWeatherLocationTreeCover_percent == Map or map or MAP then set value equal to -1 as flag in Inputs::Inputs function to read data from treecover.asc 
	if (TemperatureExecutionParams["RefWeatherLocationTreeCover_percent"] == "Map" || TemperatureExecutionParams["RefWeatherLocationTreeCover_percent"] == "map" || TemperatureExecutionParams["RefWeatherLocationTreeCover_percent"] == "MAP") {
		TemperatureCalculationParams["RefWeatherLocationTreeCover_percent"] = -1;
	}
	//If RefWeatherLocationImperviousCover_percent == Map or map or MAP then set value equal to -1 as flag in Inputs::Inputs function to read data from imperviouscover.asc
	if (TemperatureExecutionParams["RefWeatherLocationImperviousCover_percent"] == "Map" || TemperatureExecutionParams["RefWeatherLocationImperviousCover_percent"] == "map" || TemperatureExecutionParams["RefWeatherLocationImperviousCover_percent"] == "MAP") {
		TemperatureCalculationParams["RefWeatherLocationImperviousCover_percent"] = -1;
	}
	//If RefWeatherLocationAnthropogenicHeatMax_Wpm2 == Map or map or MAP then set value equal to -1 as flag in Inputs::Inputs function to read data from imperviouscover.asc
	if (TemperatureExecutionParams["RefWeatherLocationAnthropogenicHeatMax_Wpm2"] == "Map" || TemperatureExecutionParams["RefWeatherLocationAnthropogenicHeatMax_Wpm2"] == "map" || TemperatureExecutionParams["RefWeatherLocationAnthropogenicHeatMax_Wpm2"] == "MAP") {
		TemperatureCalculationParams["RefWeatherLocationAnthropogenicHeatMax_Wpm2"] = -1;
	}
	//Looping through all values stored by HydroPlusConfig.xml keys within child <TemperatureExecutionParams>; each RowCol or DateTime pair count as two values
	for (auto i : TemperatureExecutionParams) {
		if (i.second == "")
			continue;
		//Else If Not PrintAllRowColOutput and If PrintRowCol (with number appended) is present, and Not npos, then
		if (!TemperatureCalculationParams["PrintAllRowColOutput"] && i.first.find("PrintRowCol") != string::npos) {
			int row = atoi(i.second.substr(0, i.second.find(",")).c_str());
			int col = atoi(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str());
			int MapPixel_ID = row * int(LocationParams["nCols"]) + col;
			LocationIdsToPrintTempOutput.insert(MapPixel_ID);
		}
		//Else If Not RowColOutputAllTimeSteps and If RowColOutputPeriod (with number appended) is present, and Not npos, then
		else if (!TemperatureCalculationParams["RowColOutputAllTimeSteps"] && i.first.find("RowColOutputPeriod") != string::npos) {
			int startDateHour = atoi(i.second.substr(0, i.second.find(",")).c_str());
			int endDateHour = atoi(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str());
			StartDatesForRowColTempOutput.push_back(startDateHour);
			EndDatesForRowColTempOutput.push_back(endDateHour);
		}
		//Else If Not TimeBasedOutputAllTimeSteps and If TimeBasedOutputPeriod (with number appended) is present, and Not npos, then
		else if (!TemperatureCalculationParams["TimeBasedOutputAllTimeSteps"] && i.first.find("TimeBasedOutputPeriod") != string::npos) {
			int startDateHour = atoi(i.second.substr(0, i.second.find(",")).c_str());
			int endDateHour = atoi(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str());
			StartDatesForTimeBasedTempOutput.push_back(startDateHour);
			EndDatesForTimeBasedTempOutput.push_back(endDateHour);
		}
		//Else If Not PrintAllBlockGroups and If PrintBlockGroupRange (with number appended) is present, and Not npos, then
		else if (!TemperatureCalculationParams["PrintAllBlockGroups"] && i.first.find("PrintBlockGroupRange") != string::npos)	{
			int startBlockId = atoi(i.second.substr(0, i.second.find(",")).c_str());
			int endBlockId = atoi(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str());
			StartingBlockIdsToPrintTempOutput.push_back(startBlockId);
			EndingBlockIdsToPrintTempOutput.push_back(endBlockId);
		}
		//RefWeatherLocationRowCol has HydroPlusConfig.xml reference station (row, column) pair and is converted into a vector MapPixel_ID 
		else if (i.first.find("RefWeatherLocationRowCol") != string::npos)	{
			row_ReferenceStation = atoi(i.second.substr(0, i.second.find(",")).c_str());
			col_ReferenceStation = atoi(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str());

			//MapPixel_ReferenceStation_ID = row_ReferenceStation * col_max + col_ReferenceStation, where row and column start at 0 when defining reference station in HydroPlusCofig.xml
			MapPixel_ReferenceStation_ID = row_ReferenceStation * int(LocationParams["nCols"]) + col_ReferenceStation;

		}
		//Note: Reference Station DEM based properties (ground elevation, ground slope, ground aspect, TI) are read from HydroPlusConfig.xml or derived from dem.asc filestation is not in simulated area
		//Note: The use of the HydroPlusCongig.xml to select option facilitates simulations where meteorological station is not in simulated area.
		
		//RefWeatherLocationElevation_m defined as elevation_m from HydroPlusConfig.xml; atof will convert "DEM" to 0
		//In RunTemperatureSpatialModel function, if RefWeatherLocationElevation_m != 0 then use HydroPlusConfig.xml value, else input->DEM defined by dem.asc in Inputs::ProcessTopographyData
		//Note: HydroPlusConfig.xml element RefWeatherLocationElevation_m is set to "DEM" or Value <> 0 m
		else if (i.first.find("RefWeatherLocationElevation_m") != string::npos) {
			double elevation_m = atof(i.second.substr(0, i.second.size()).c_str());
			//"DEM" string in HydroPlusConfig.xml will generate a value of 0 for elevation
			TemperatureCalculationParams["RefWeatherLocationElevation_m"] = elevation_m;
		}
		//RefWeatherLocationSlope_deg defined as slope_deg from HydroPlusConfig.xml; atof will convert "DEM" to 0
		//In RunTemperatureSpatialModel function, if RefWeatherLocationSlope_deg > 0 then use HydroPlusConfig.xml value, else input->SlopeGround_rad defined by dem.asc in Inputs::ProcessTopographyData
		//Note: HydroPlusConfig.xml element RefWeatherLocationSlope_deg is set to "DEM" or Value 1 to 360 degrees
		else if (i.first.find("RefWeatherLocationSlope_deg") != string::npos) {
			double slope_deg = atof(i.second.substr(0, i.second.size()).c_str());
			//"DEM" string in HydroPlusConfig.xml will generate a value of 0 for slope_deg
			TemperatureCalculationParams["RefWeatherLocationSlope_deg"] = slope_deg;
		}
		//RefWeatherLocationAspect_deg defined as aspect_deg from HydroPlusConfig.xml; atof will convert "DEM" to 0
		//In RunTemperatureSpatialModel function, if RefWeatherLocationAspect_deg > 0 then use HydroPlusConfig.xml value, else input->AspectGround_N_0_rad defined by dem.asc in Inputs::ProcessTopographyData
		//Note: HydroPlusConfig.xml element RefWeatherLocationSlope_deg is set to "DEM" or Value > 0.001 degrees
		else if (i.first.find("RefWeatherLocationAspect_deg") != string::npos)	{
			double aspect_deg = atof(i.second.substr(0, i.second.size()).c_str());
			//"DEM" string in HydroPlusConfig.xml will generate a value of 0 for aspect_deg. While N=0, the zero ground slope results in no effect of ground aspect
			TemperatureCalculationParams["RefWeatherLocationAspect_deg"] = aspect_deg;
		}
		//RefWeatherLocationNLCD_Class defined as LandCover_Class from HydroPlusConfig.xml
		//Note: HydroPlusConfig.xml element RefWeatherLocationNLCD_Class is set to -1 when "Map", otherwise Value >= 0 percent
		else if (TemperatureCalculationParams["RefWeatherLocationNLCD_Class"] != -1 && i.first.find("RefWeatherLocationNLCD_Class") != string::npos) {
			double LandCover_Class = atof(i.second.substr(0, i.second.size()).c_str());
			//"MAP" string in HydroPlusConfig.xml will generate a value of 0 for LandCover_Class.
			TemperatureCalculationParams["RefWeatherLocationNLCD_Class"] = LandCover_Class;
		}
		//RefWeatherLocationTreeCover_percent defined as TreeCover_percent from HydroPlusConfig.xml
		//Note: HydroPlusConfig.xml element RefWeatherLocationTreeCover_percent is set to -1 when "Map", otherwise Value >= 0 percent
		else if (TemperatureCalculationParams["RefWeatherLocationTreeCover_percent"] != -1 && i.first.find("RefWeatherLocationTreeCover_percent") != string::npos) {
			double TreeCover_percent = atof(i.second.substr(0, i.second.size()).c_str());
			//"MAP" string in HydroPlusConfig.xml will generate a value of 0 for TreeCover_percent. 
			TemperatureCalculationParams["RefWeatherLocationTreeCover_percent"] = TreeCover_percent;
		}
		//RefWeatherLocationImperviousCover_percent defined as ImperviousCover_percent from HydroPlusConfig.xml
		//Note: HydroPlusConfig.xml element RefWeatherLocationImperviousCover_percent is set to -1 when "Map", otherwise Value >= 0 percent
		else if (TemperatureCalculationParams["RefWeatherLocationImperviousCover_percent"] != -1 && i.first.find("RefWeatherLocationImperviousCover_percent") != string::npos) {
			double ImperviousCover_percent = atof(i.second.substr(0, i.second.size()).c_str());
			//"MAP" string in HydroPlusConfig.xml will generate a value of 0 for ImperviousCover_percent. 
			TemperatureCalculationParams["RefWeatherLocationImperviousCover_percent"] = ImperviousCover_percent;
		}
		//RefWeatherLocationAnthropogenicHeatMax_Wpm2 defined as AnthropogenicHeatMax_Wpm2 from HydroPlusConfig.xml
		//Note: HydroPlusConfig.xml element RefWeatherLocationAnthropogenicHeatMax_Wpm2 is set to -1 when "MAP" or Value >= 0 W/m2
		else if (TemperatureCalculationParams["RefWeatherLocationAnthropogenicHeatMax_Wpm2"] != -1 && i.first.find("RefWeatherLocationAnthropogenicHeatMax_Wpm2") != string::npos) {
			double AnthropogenicHeatMax_Wpm2 = atof(i.second.substr(0, i.second.size()).c_str());
			//"MAP" string in HydroPlusConfig.xml will generate a value of 0 for AnthropogenicHeatMax_Wpm2. 
			TemperatureCalculationParams["RefWeatherLocationAnthropogenicHeatMax_Wpm2"] = AnthropogenicHeatMax_Wpm2;
		}

		//AH4GUC_HourlyValue_to_MaxValue_frac defined in HydroPlusConfig.xml to contain 24 values of the ratio AH4GUC_##H (W/m2) to AH4GUC_MaxH (W/m2)
		//Note: AH4GUC_MaxH (W/m2) is the maximum value in the 24 hour period, therefor it will have AH4GUC_HourlyValue_to_MaxValue_frac = 1
		else if (i.first.find("AH4GUC_HourlyValue_to_MaxValue_frac") != string::npos) {
			AH4GUC_HourlyValue_to_MaxValue_frac_line = i.second.c_str();
			//stringstream function to create AH4GUC_HourlyValue_to_MaxValue_frac_ss from AH4GUC_HourlyValue_to_MaxValue_frac_line
			stringstream AH4GUC_HourlyValue_to_MaxValue_frac_ss(AH4GUC_HourlyValue_to_MaxValue_frac_line);
			//While loop through AH4GUC_HourlyValue_to_MaxValue_frac_ss using good() function
			while (AH4GUC_HourlyValue_to_MaxValue_frac_ss.good()) {
				string AH_ratio_HrVal_to_MaxVal;
				//getline function to extract value from AH4GUC_HourlyValue_to_MaxValue_frac_ss and assign to AH_ratio_HrVal_to_MaxVal 
				getline(AH4GUC_HourlyValue_to_MaxValue_frac_ss, AH_ratio_HrVal_to_MaxVal, ',');
				//AH4GUC_HourlyValue_to_MaxValue_frac_str uses push_back function to append AH_ratio_HrVal_to_MaxVal
				AH4GUC_HourlyValue_to_MaxValue_frac_str.push_back(AH_ratio_HrVal_to_MaxVal);
			}
		}
	}
	//Flag_BlockGroupsProcessed initialized to zero, no block group data processed
	Flag_BlockGroupsProcessed = 0;
	//if TemperatureCalculationParams["PrintAllBlockGroups"] equals 1 or StartingBlockIdsToPrintTempOutput.size() > 0 then
	if (TemperatureCalculationParams["PrintAllBlockGroups"] == 1 || StartingBlockIdsToPrintTempOutput.size() > 0) {
		//Flag_BlockGroupsProcessed set to 1, block group data processed
		Flag_BlockGroupsProcessed = 1;
	}

	//If Flag_CoolAir_AnthropogenicHeat equals 1 and AH4GUC_HourlyValue_to_MaxValue_frac_str not equal to 24 then terminate program
	if (SimulationStringParams["Flag_CoolAir_AnthropogenicHeat"] == "1" && AH4GUC_HourlyValue_to_MaxValue_frac_str.size() != 24) {
		cout << "Warning: The HydroPlusConfig.xml cannot find element AH4GUC_HourlyValue_to_MaxValue_frac with 24 comma separated values." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: If Flag_CoolAir_AnthropogenicHeat = 1 then AH4GUC_HourlyValue_to_MaxValue_frac must contain 24 comma separated values." << endl;
		cout << "Correction: Either set Flag_CoolAir_AnthropogenicHeat = 0 or set AH4GUC_HourlyValue_to_MaxValue_frac to 24 distinct values." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	//If Flag_CoolAir_AnthropogenicHeat equals 1 then populate AH4GUC_HourlyValue_to_MaxValue_frac vector with AH4GUC_HourlyValue_to_MaxValue_frac_str
	else if (SimulationStringParams["Flag_CoolAir_AnthropogenicHeat"] == "1") {
		for (int AH_cnt = 0; AH_cnt < AH4GUC_HourlyValue_to_MaxValue_frac_str.size(); AH_cnt++) {
			//initialize char AH_frac_char to 10 digits
			char AH_frac_char[10];
			//strcpy function to define AH_frac_char as item AH_cnt from vector AH4GUC_HourlyValue_to_MaxValue_frac_str[]
			//Note: strcpy funtion converts string to char
			strcpy(AH_frac_char, AH4GUC_HourlyValue_to_MaxValue_frac_str[AH_cnt].c_str());
			//double AH4GUC_HourlyValue_to_MaxValue_frac has AH_frac_char appended using atof function, converting char to double
			AH4GUC_HourlyValue_to_MaxValue_frac.push_back(atof(AH_frac_char));
		}
	}
	//If Flag_CoolAir_AnthropogenicHeat not equal to 1 then populate AH4GUC_HourlyValue_to_MaxValue_frac vector with zero
	else if (SimulationStringParams["Flag_CoolAir_AnthropogenicHeat"] != "1") {
		int ratio_hours_to_day = 24;
		//For Loop with AH_cnt through hours in day
		for (int AH_cnt = 0; AH_cnt < ratio_hours_to_day; AH_cnt++) {
			//AH4GUC_HourlyValue_to_MaxValue_frac has 0.0 appended
			AH4GUC_HourlyValue_to_MaxValue_frac.push_back(0.0);
		}
	}
}

//To define static variables we should do this in the .CPP again
unordered_map<string, double> Inputs::RefParams;
unordered_map<string, double> Inputs::LAI_BAI_map;
vector<double> Inputs::LAI_BAI_Tree_m2_p_m2;
vector<double> Inputs::LAI_BAI_SVeg_m2_p_m2;
vector<double> Inputs::LAI_Tree_m2_p_m2;
vector<double> Inputs::LAI_SVeg_m2_p_m2;

void Inputs::InitializeLAIVariables() {

	//LAI variables are initialized

	//LeafOn_Start_JD (Julian Day) is LeafOnDay minus 0.5 * LeafTransDays, half of leaf transition days
	LAI_BAI_map["LeafOn_Start_JD"] = RefParams["LeafOnDate_JDay"] - RefParams["LeafTransition_Day"] * 0.5;
	//LeafOn_Stop_JD (Julian Day) is LeafOnDay plus 0.5 * LeafTransDays, half of leaf transition days
	LAI_BAI_map["LeafOn_Stop_JD"] = RefParams["LeafOnDate_JDay"] + RefParams["LeafTransition_Day"] * 0.5;
	//LeafOff_Start_JD (Julian Day) is LeafOffDay minus 0.5 * LeafTransDays, half of leaf transition days
	LAI_BAI_map["LeafOff_Start_JD"] = RefParams["LeafOffDate_JDay"] - RefParams["LeafTransition_Day"] * 0.5;
	LAI_BAI_map["LeafOff_Stop_JD"] = RefParams["LeafOffDate_JDay"] + RefParams["LeafTransition_Day"] * 0.5;
	//LAI_BAI_Tree_min_m2pm2 (m2/m2) is minimum tree leaf area index plus bark area index
	LAI_BAI_map["LAI_BAI_Tree_min_m2pm2"] = RefParams["LeafAreaIndex_Tree_m2pm2"] * RefParams["TreeCover_Evergreen_frac"] + RefParams["BarkAreaIndex_Tree_m2pm2"];
	//LAI_BAI_Tree_max_m2pm2 (m2/m2) is maximum tree leaf area index plus bark area index
	LAI_BAI_map["LAI_BAI_Tree_max_m2pm2"] = RefParams["LeafAreaIndex_Tree_m2pm2"] + RefParams["BarkAreaIndex_Tree_m2pm2"];
	//LAI_BAI_SVeg_min_m2pm2 (m2/m2) is minimum short vegetation leaf area index plus bark area index
	LAI_BAI_map["LAI_BAI_SVeg_min_m2pm2"] = RefParams["LeafAreaIndex_SVeg_m2pm2"] * RefParams["SVegCover_Evergreen_frac"] + RefParams["BarkAreaIndex_SVeg_m2pm2"];
	//LAI_BAI_SVeg_min_m2pm2 (m2/m2) is maximum short vegetation leaf area index
	LAI_BAI_map["LAI_BAI_SVeg_max_m2pm2"] = RefParams["LeafAreaIndex_SVeg_m2pm2"] + RefParams["BarkAreaIndex_SVeg_m2pm2"];

	//LAI_BAI_Tree_min_m2pm2 (m2/m2) is minimum tree leaf area index
	LAI_BAI_map["LAI_Tree_min_m2pm2"] = RefParams["LeafAreaIndex_Tree_m2pm2"] * RefParams["TreeCover_Evergreen_frac"];
	//LAI_BAI_Tree_max_m2pm2 (m2/m2) is maximum tree leaf area index
	LAI_BAI_map["LAI_Tree_max_m2pm2"] = RefParams["LeafAreaIndex_Tree_m2pm2"];
	//LAI_BAI_SVeg_min_m2pm2 (m2/m2) is minimum short vegetation leaf area index
	LAI_BAI_map["LAI_SVeg_min_m2pm2"] = RefParams["LeafAreaIndex_SVeg_m2pm2"] * RefParams["SVegCover_Evergreen_frac"];
	//LAI_BAI_Tree_max_m2pm2 (m2/m2) is maximum short vegetation leaf area index
	LAI_BAI_map["LAI_SVeg_max_m2pm2"] = RefParams["LeafAreaIndex_SVeg_m2pm2"];

}

//cte 2024 Refactor to remove LAI_BAI_Tree_m2pm2, only use phenology LAI_Tree_m2pm2, and maintain LAI_Tree_m2pm2 and BAI_Tree_m2pm2
//CalcTreeLAI function should compute tree leaf area index for any model time step for Northern Hemisphere
//Note: i-Tree LocationSpecies Database uses Last Frost and First Frost dates to indicate LeafOn_MidPoint_JD and LeafOff_MidPoint_JD
//Note: LeafOn_Start_JD = LeafOn_MidPoint_JD - 14; LeafOn_Stop_JD = LeafOn_MidPoint_JD + 14;
//Note: LeafOff_Start_JD = LeafOff_MidPoint_JD - 14; LeafOff_Stop_JD = LeafOff_MidPoint_JD + 14;
void Inputs::CalcTreeLAI(int JulianDay)
{
	double LAI_BAI_Tree_m2pm2 = 0;
	double LAI_Tree_m2pm2 = 0;
	double Coeff_LeafTransition = 0;
	//If JulianDay less than or equal to LeafOn_Start_JD then 
	if (JulianDay <= LAI_BAI_map["LeafOn_Start_JD"]) {
		LAI_BAI_Tree_m2pm2 = LAI_BAI_map["LAI_BAI_Tree_min_m2pm2"];
		LAI_Tree_m2pm2 = LAI_BAI_map["LAI_Tree_min_m2pm2"];
	}
	//Else If JulianDay > LeafOn_Start_JD and < LeafOn_Stop_JD then transition leaf on
	else if (JulianDay > LAI_BAI_map["LeafOn_Start_JD"] && JulianDay < LAI_BAI_map["LeafOn_Stop_JD"]) {
		//Coeff_LeafTransition term is derived using coefficient 0.37 from Nowak and Hirabayashi function to approximate sinosoidal leaf transition with exponential function
		Coeff_LeafTransition = (JulianDay - RefParams["LeafOnDate_JDay"]) * 0.37;
		LAI_BAI_Tree_m2pm2 = 1 / (1 + exp(-Coeff_LeafTransition)) * (LAI_BAI_map["LAI_BAI_Tree_max_m2pm2"] - LAI_BAI_map["LAI_BAI_Tree_min_m2pm2"]) + LAI_BAI_map["LAI_BAI_Tree_min_m2pm2"];
		LAI_Tree_m2pm2 = 1 / (1 + exp(-Coeff_LeafTransition)) * (LAI_BAI_map["LAI_Tree_max_m2pm2"] - LAI_BAI_map["LAI_Tree_min_m2pm2"]) + LAI_BAI_map["LAI_Tree_min_m2pm2"];
	}
	//Else If JulianDay >= LeafOn_Stop_JD and less than LeafOff_Start_JD then set to maximum LAI + BAI
	else if (JulianDay >= LAI_BAI_map["LeafOn_Stop_JD"] && JulianDay <= LAI_BAI_map["LeafOff_Start_JD"]) {
		LAI_BAI_Tree_m2pm2 = LAI_BAI_map["LAI_BAI_Tree_max_m2pm2"];
		LAI_Tree_m2pm2 = LAI_BAI_map["LAI_Tree_max_m2pm2"];
	}
	//Else If JulianDay > LeafOff_Start_JD and < LeafOff_Stop_JD then transition leaf off
	else if (JulianDay > LAI_BAI_map["LeafOff_Start_JD"] && JulianDay < LAI_BAI_map["LeafOff_Stop_JD"])	{
		Coeff_LeafTransition = (RefParams["LeafOffDate_JDay"] - JulianDay) * 0.37;
		LAI_BAI_Tree_m2pm2 = 1 / (1 + exp(-Coeff_LeafTransition)) * (LAI_BAI_map["LAI_BAI_Tree_max_m2pm2"] - LAI_BAI_map["LAI_BAI_Tree_min_m2pm2"]) + LAI_BAI_map["LAI_BAI_Tree_min_m2pm2"];
		LAI_Tree_m2pm2 = 1 / (1 + exp(-Coeff_LeafTransition)) * (LAI_BAI_map["LAI_Tree_max_m2pm2"] - LAI_BAI_map["LAI_Tree_min_m2pm2"]) + LAI_BAI_map["LAI_Tree_min_m2pm2"];
	}
	//Else If JulianDay >= LeafOff_Stop_JD then set to minimum LAI + BAI
	else if (JulianDay >= LAI_BAI_map["LeafOff_Stop_JD"]) {
		LAI_BAI_Tree_m2pm2 = LAI_BAI_map["LAI_BAI_Tree_min_m2pm2"];
		LAI_Tree_m2pm2 = LAI_BAI_map["LAI_Tree_min_m2pm2"];
	}
	LAI_BAI_Tree_m2_p_m2.push_back(LAI_BAI_Tree_m2pm2);
	LAI_Tree_m2_p_m2.push_back(LAI_Tree_m2pm2);
}

//cte 2024 Refactor to remove LAI_BAI_SVeg_m2pm2, only use phenology LAI_SVeg_m2pm2, and maintain LAI_SVeg_m2pm2 and BAI_SVeg_m2pm2
//CalcShortVegLAI function should compute tree leaf area index for any model time step for Northern Hemisphere
//Note: i-Tree LocationSpecies Database uses Last Frost and First Frost dates to indicate LeafOn_MidPoint_JD and LeafOff_MidPoint_JD
//Note: LeafOn_Start_JD = LeafOn_MidPoint_JD - 14; LeafOn_Stop_JD = LeafOn_MidPoint_JD + 14;
//Note: LeafOff_Start_JD = LeafOff_MidPoint_JD - 14; LeafOff_Stop_JD = LeafOff_MidPoint_JD + 14;
void Inputs::CalcShortVegLAI(int JulianDay)
{
	double LAI_BAI_SVeg_m2pm2 = 0;
	double LAI_SVeg_m2pm2 = 0;
	double Coeff_LeafTransition = 0;
	if (JulianDay <= LAI_BAI_map["LeafOn_Start_JD"]) {
		LAI_BAI_SVeg_m2pm2 = LAI_BAI_map["LAI_BAI_SVeg_min_m2pm2"];
		LAI_SVeg_m2pm2 = LAI_BAI_map["LAI_SVeg_min_m2pm2"];
	}
	else if (JulianDay > LAI_BAI_map["LeafOn_Start_JD"] && JulianDay < LAI_BAI_map["LeafOn_Stop_JD"]) {
		Coeff_LeafTransition = (JulianDay - RefParams["LeafOnDate_JDay"]) * 0.37;
		LAI_BAI_SVeg_m2pm2 = 1 / (1 + exp(-Coeff_LeafTransition)) * (LAI_BAI_map["LAI_BAI_SVeg_max_m2pm2"] - LAI_BAI_map["LAI_BAI_SVeg_min_m2pm2"]) + LAI_BAI_map["LAI_BAI_SVeg_min_m2pm2"];
		LAI_SVeg_m2pm2 = 1 / (1 + exp(-Coeff_LeafTransition)) * (LAI_BAI_map["LAI_BAI_SVeg_max_m2pm2"] - LAI_BAI_map["LAI_BAI_SVeg_min_m2pm2"]) + LAI_BAI_map["LAI_BAI_SVeg_min_m2pm2"];
	}
	else if (JulianDay >= LAI_BAI_map["LeafOn_Stop_JD"] && JulianDay <= LAI_BAI_map["LeafOff_Start_JD"]) {
		LAI_BAI_SVeg_m2pm2 = LAI_BAI_map["LAI_BAI_SVeg_max_m2pm2"];
		LAI_SVeg_m2pm2 = LAI_BAI_map["LAI_SVeg_max_m2pm2"];
	}
	else if (JulianDay > LAI_BAI_map["LeafOff_Start_JD"] && JulianDay < LAI_BAI_map["LeafOff_Stop_JD"]) {
		Coeff_LeafTransition = (RefParams["LeafOffDate_JDay"] - JulianDay) * 0.37;
		LAI_BAI_SVeg_m2pm2 = 1 / (1 + exp(-Coeff_LeafTransition)) * (LAI_BAI_map["LAI_BAI_SVeg_max_m2pm2"] - LAI_BAI_map["LAI_BAI_SVeg_min_m2pm2"]) + LAI_BAI_map["LAI_BAI_SVeg_min_m2pm2"];
		LAI_SVeg_m2pm2 = 1 / (1 + exp(-Coeff_LeafTransition)) * (LAI_BAI_map["LAI_SVeg_max_m2pm2"] - LAI_BAI_map["LAI_SVeg_min_m2pm2"]) + LAI_BAI_map["LAI_SVeg_min_m2pm2"];
	}
	else if (JulianDay >= LAI_BAI_map["LeafOff_Stop_JD"]) {
		LAI_BAI_SVeg_m2pm2 = LAI_BAI_map["LAI_BAI_SVeg_min_m2pm2"];
	}
	LAI_BAI_SVeg_m2_p_m2.push_back(LAI_BAI_SVeg_m2pm2);
	LAI_SVeg_m2_p_m2.push_back(LAI_SVeg_m2pm2);
}

//readNLCDLandCoverFile function will read input map file landcover.asc (#) containing NLCD Classes
bool Inputs::readNLCDLandCoverFile(string& Directory_Input_CLArg, double NODATA_code)
{
	//readLandCover_map assigned to landcover.asc file from Directory_Input_CLArg; if file does not exist it is given a value of false
	ifstream readLandCover_map(Directory_Input_CLArg + "landcover.asc");
	//readLandCover_table assigned to landcover.asc file from Directory_Input_CLArg; if file does not exist it is given a value of false
	ifstream readLandCover_table(Directory_Input_CLArg + "LandCover_table.txt");

	//If readLandCover_map is false and readLandCover_table is false, then
	if (!readLandCover_map && !readLandCover_table) {
		//In EC dynamic model, it will calculate EC and EMC database without running Buffer simulation if not providing LC and Discharge.
		if (SimulationStringParams["Model_Selection"] != "ECDynamic") {
			cout << "Warning: landcover.asc or LandCover_table.txt could not be opened." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: When the landcover.asc or LandCover_table.txt file is missing there is no land cover data to drive the simulation." << endl;
			cout << "Correction: Create a landcover.asc or LandCover_table.txt file, either manually or perhaps using the i-Tree Cool Air python utility." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}
		//if Model_Selection == ECDynamic
		else {
			cout << "Warning:landcover.asc or LandCover_table.txt could not be opened." << endl;
			//false reading NLCD will trigger the generation of EC database without executing the entire model; don't remove this line
			return false;
		}
	}
	//buffer defined as string to take value of header data
	string buffer;

	//if readLandCover_map is not false, then read header into buffer string
	if (readLandCover_map) {
		readLandCover_map >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> cellsize >> buffer >> NODATA;

		//If int(LocationParams["nCols"]) not equal to ncols or int(LocationParams["nRows"]) != nrows then abort due to error in inputs
		if ((int(LocationParams["nCols"]) != ncols || int(LocationParams["nRows"]) != nrows) && SimulationStringParams["Model_Selection"] == "SpatialTemperatureHydro") {
			cout << "Warning: landcover.asc header ncols or nrows value is different from HydroPlusConfig.xml element nCols or nRows." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: When the landcover.asc file ncols or nrows value differs, file size may be faulty." << endl;
			cout << "Correction: Create a landcover.asc file that agrees with HydroPlusConfig.xml nCols and nRows." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}
		//tempdata and count initialized as zero
		int tempdata;
		int count = 0;

		//while loop continues executing as long as two conditions are met:
		//1. The data is successfully read from the file stream object into a variable named "tempdata" using the stream extraction operator (>>).
		//2. The variable count is less than the product of nrows and ncols representing map size, and count represents the number of data points read so far.
		while (readLandCover_map >> tempdata && count < nrows * ncols) {
			//If tempdata is not NODATA, then read NLCD Class value to NLCDLandCoverData, 
			//Note: Initialize other land cover fraction vectors with 0 value
			if (tempdata != NODATA) {
				//If tempdata not equal to static_cast<int>(tempdata) then the real value is not equal to its integer value
				if (tempdata != static_cast<int>(tempdata)) {
					cout << "Warning: landcover.asc contains data with non-integer values." << endl;
					cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
					cout << "Explanation: When the landcover.asc file contains non-integer values, it breaks from structure." << endl;
					cout << "Correction: Create a landcover.asc file with integer values, minimum values of zero." << endl;
					//Call abort function, which ends the HydroPlus.exe simulation
					abort();
				}

				NLCDLandCoverData.push_back(tempdata);
				//TotalCover defined as total TreeCover and ImperviousCover
				TotalCover.push_back(0.0);
				SoilCover.push_back(0.0);
				WaterCover.push_back(0.0);
				ImperviousNoTree.push_back(0.0);
				ShortVegCover.push_back(0.0);
				ConnectedImpervious.push_back(0.0);
				FracEgreenTree.push_back(0.0);
				FracEgreenShortVeg.push_back(0.0);
				TreeOnPervious.push_back(0.0);
				TreeOnImpervious.push_back(0.0);
				PermeablePavementCover.push_back(0.0);
			}
			//If else NODATA is value, then assign NODATA_code
			else {
				NLCDLandCoverData.push_back(NODATA_code);
				TotalCover.push_back(NODATA_code);
				SoilCover.push_back(NODATA_code);
				WaterCover.push_back(NODATA_code);
				ImperviousNoTree.push_back(NODATA_code);
				ShortVegCover.push_back(NODATA_code);
				ConnectedImpervious.push_back(NODATA_code);
				FracEgreenTree.push_back(NODATA_code);
				FracEgreenShortVeg.push_back(NODATA_code);
				TreeOnPervious.push_back(NODATA_code);
				TreeOnImpervious.push_back(NODATA_code);
				PermeablePavementCover.push_back(NODATA_code);
			}

			//if tempdata equals NLCD value 21, then increase LC21 scalar
			if (tempdata == 21) { LC21 += 1; }
			if (tempdata == 22) { LC22 += 1; }
			if (tempdata == 23) { LC23 += 1; }
			if (tempdata == 24) { LC24 += 1; }
			if (tempdata == 31) { LC31 += 1; }
			if (tempdata == 41) { LC41 += 1; }
			if (tempdata == 42) { LC42 += 1; }
			if (tempdata == 43) { LC43 += 1; }
			if (tempdata == 52) { LC52 += 1; }
			if (tempdata == 71) { LC71 += 1; }
			if (tempdata == 81) { LC81 += 1; }
			if (tempdata == 82) { LC82 += 1; }
			if (tempdata == 11) { LC11 += 1; }
			if (tempdata == 90) { LC90 += 1; }
			if (tempdata == 95) { LC95 += 1; }

			//count advanced by 1
			++count;
		}
		
		//if NLCDLandCoverData has size 0, then abort program
		//Note: This often happens when values are negative or the file has other non-numeric symbols
		if (NLCDLandCoverData.size() == 0 || NLCDLandCoverData.size() < ncols * nrows) {
			cout << "Warning: landcover.asc contains data with negative values or other non-numeric symbols." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: When the landcover.asc file contains non-numeric symbols, space-time becomes warped." << endl;
			cout << "Correction: Create a landcover.asc file with minimum values of zero, within the realm of reality." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}
		readLandCover_map.close();

	}
	else if (readLandCover_table) {
		string LCtitle;//to save the row name
		double LCnum;//to save land cover number
		//read the header
		getline(readLandCover_table, buffer);
		while (readLandCover_table >> LCtitle) {
			readLandCover_table >> LCnum;
			if (LCtitle.find("21") != string::npos) {
				LC21 = LCnum;
			}
			if (LCtitle.find("22") != string::npos) {
				LC22 = LCnum;
			}
			if (LCtitle.find("23") != string::npos) {
				LC23 = LCnum;
			}
			if (LCtitle.find("24") != string::npos) {
				LC24 = LCnum;
			}
			if (LCtitle.find("31") != string::npos) {
				LC31 = LCnum;
			}
			if (LCtitle.find("41") != string::npos) {
				LC41 = LCnum;
			}
			if (LCtitle.find("42") != string::npos) {
				LC42 = LCnum;
			}
			if (LCtitle.find("43") != string::npos) {
				LC43 = LCnum;
			}
			if (LCtitle.find("52") != string::npos) {
				LC52 = LCnum;
			}
			if (LCtitle.find("71") != string::npos) {
				LC71 = LCnum;
			}
			if (LCtitle.find("81") != string::npos) {
				LC81 = LCnum;
			}
			if (LCtitle.find("82") != string::npos) {
				LC82 = LCnum;
			}
			//water area
			if (LCtitle.find("11") != string::npos) {
				LC11 = LCnum;
			}
			if (LCtitle.find("90") != string::npos) {
				LC90 = LCnum;
			}
			if (LCtitle.find("95") != string::npos) {
				LC95 = LCnum;
			}
		}
		readLandCover_table.close();		
	}
	cell_count = LC21 + LC22 + LC23 + LC24 + LC31 + LC41 + LC42 + LC43 + LC52 + LC71 + LC81 + LC82 + LC11 + LC90 + LC95;
	cell_count_Nowater = LC21 + LC22 + LC23 + LC24 + LC31 + LC41 + LC42 + LC43 + LC52 + LC71 + LC81 + LC82;
	// LC 31 Barren Land -> LC71 Grassland
	// LC 52 Shrub/Scrub ->LC 41 Deciduous Forest
	Cell_number.push_back(LC21);
	Cell_number.push_back(LC22);
	Cell_number.push_back(LC23);
	Cell_number.push_back(LC24);
	Cell_number.push_back(int(LC41 + LC52));
	Cell_number.push_back(LC42);
	Cell_number.push_back(LC43);
	Cell_number.push_back(int(LC71 + LC31));
	Cell_number.push_back(LC81);
	Cell_number.push_back(LC82);

	return true;
	//cout << LC21 << " " << LC22 << " " << LC23 << " " << LC24 << " " << LC41 << " " << LC42 << " " << LC43 << " " << LC71 << " " << LC81 << " " << LC82 << endl;
}

//readTreeCoverFile function will read input map file of tree cover (percent), treecover.asc (%) 
bool Inputs::readTreeCoverFile(string& Directory_Input_CLArg, double NODATA_code)
{
	ifstream readTreeCover(Directory_Input_CLArg + "treecover.asc");
	if (!readTreeCover) {
		cout << "Warning: treecover.asc could not be opened." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the treecover.asc file is missing there is no tree cover data to drive the simulation." << endl;
		cout << "Correction: Create a treecover.asc file, either manually or perhaps using the i-Tree Cool Air python utility." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	string buffer;
	readTreeCover >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;

	//If int(LocationParams["nCols"]) not equal to ncols or int(LocationParams["nRows"]) != nrows then abort due to error in inputs
	if (int(LocationParams["nCols"]) != ncols || int(LocationParams["nRows"]) != nrows) {
		cout << "Warning: treecover.asc header ncols or nrows value is different from HydroPlusConfig.xml element nCols or nRows." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the treecover.asc file ncols or nrows value differs, file size may be faulty." << endl;
		cout << "Correction: Create a treecover.asc file that agrees with HydroPlusConfig.xml nCols and nRows." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	double tempdata = 0.0;
	int count = 0;

	//while loop to read tree cover as tempdata (fraction) from readTreeCover while count < product of nrows and ncols
	while (readTreeCover >> tempdata && count < nrows * ncols) {
		//If tempdata not equal to NODATA then 
		if (tempdata != NODATA) {
			//If tempdata < 0 then abort, as positive values needed
			if (tempdata < 0) {
				cout << "Warning: treecover.asc contains data with negative values." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: When the treecover.asc file contains non-numeric symbols, space-time becomes warped." << endl;
				cout << "Correction: Create a treecover.asc file with minimum values of zero, within the realm of reality." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}
			//TreeCoverData  (fraction) is converted from tempdata (percent) to decimal using Ratio_Decimal_to_Percent 
			TreeCoverData.push_back(tempdata * Ratio_Decimal_to_Percent);
		}
		//else tempdata equals NODATA 
		else {
			//TreeCoverData (%) is defined as NODATA_code
			TreeCoverData.push_back(NODATA_code);
		}
		++count;
	}

	//if TreeCoverData has size 0, then abort program
	//Note: This often happens when values are negative or the file has other non-numeric symbols
	if (TreeCoverData.size() == 0 || TreeCoverData.size() < ncols * nrows) {
		cout << "Warning: treecover.asc contains data with negative values or other non-numeric symbols." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the treecover.asc file contains non-numeric symbols, space-time becomes warped." << endl;
		cout << "Correction: Create a treecover.asc file with minimum values of zero, within the realm of reality." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	readTreeCover.close();
	return true;
}

//readImperviousFile function will read input map file of impervious cover (percent), imperviouscover.asc (%) 
bool Inputs::readImperviousFile(string& Directory_Input_CLArg, double NODATA_code)
{
	ifstream readImperviousFile(Directory_Input_CLArg + "imperviouscover.asc");
	if (!readImperviousFile) {
		cout << "Warning: imperviouscover.asc could not be opened." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the imperviouscover.asc file is missing there is no impervious cover data to drive the simulation." << endl;
		cout << "Correction: Create a imperviouscover.asc file, either manually or perhaps using the i-Tree Cool Air python utility." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	string buffer;
	readImperviousFile >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;

	//If int(LocationParams["nCols"]) not equal to ncols or int(LocationParams["nRows"]) != nrows then abort due to error in inputs
	if (int(LocationParams["nCols"]) != ncols || int(LocationParams["nRows"]) != nrows) {
		cout << "Warning: imperviouscover.asc header ncols or nrows value is different from HydroPlusConfig.xml element nCols or nRows." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the imperviouscover.asc file ncols or nrows value differs, file size may be faulty." << endl;
		cout << "Correction: Create a imperviouscover.asc file that agrees with HydroPlusConfig.xml nCols and nRows." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	double tempdata = 0.0;
	int count = 0;
	//imperviousArea_mapCells_count initialized to zero
	int imperviousArea_mapCells_count = 0;
	//imperviousArea_mapTotal_frac initialized to zero
	double imperviousArea_mapTotal_frac = 0;
	double imperviousArea_mapAverage_frac = 0;
	//while loop to read impervious cover as tempdata (percent) from readImperviousFile, while count is less than product of ncols and nrows
	while (readImperviousFile >> tempdata && count < ncols * nrows)	{
		//If tempdata not equal to NODATA then 
		if (tempdata != NODATA) {
			//If tempdata < 0 then abort, as positive values needed
			if (tempdata < 0) {
				cout << "Warning: imperviouscover.asc contains data with negative values." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: When the imperviouscover.asc file contains non-numeric symbols, space-time becomes warped." << endl;
				cout << "Correction: Create a imperviouscover.asc file with minimum values of zero, within the realm of reality." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}

			//ImperCoverData (fraction) is converted from tempdata (percent) to decimal using Ratio_Decimal_to_Percent 
			ImperCoverData.push_back(tempdata * Ratio_Decimal_to_Percent);
			PerCoverdata.push_back(1- tempdata * Ratio_Decimal_to_Percent);
			imperviousArea_mapCells_count = imperviousArea_mapCells_count + 1;
			//imperviousArea_mapTotal_frac (fraction) is summation for quotient of tempdata (percent) and 100.0
			imperviousArea_mapTotal_frac = imperviousArea_mapTotal_frac + tempdata * Ratio_Decimal_to_Percent;
		}
		//else tempdata equals NODATA 
		else {
			//ImperCoverData (percent) is defined as NODATA_code
			ImperCoverData.push_back(NODATA_code);
			PerCoverdata.push_back(NODATA_code);
		}
		++count;
	}

	//if ImperCoverData has size 0, then abort program
	//Note: This often happens when values are negative or the file has other non-numeric symbols
	if (ImperCoverData.size() == 0 || ImperCoverData.size() < ncols * nrows) {
		cout << "Warning: imperviouscover.asc contains data with negative values or other non-numeric symbols." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the imperviouscover.asc file contains non-numeric symbols, space-time becomes warped." << endl;
		cout << "Correction: Create a imperviouscover.asc file with minimum values of zero, within the realm of reality." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//imperviousArea_mapAverage_frac (fraction) = imperviousArea_mapTotal_frac / imperviousArea_mapCells_count
	//Note: HydroPlusConfig.xml Flag_TI_PerviousOnly=1 excludes impervious area in FlowAccumulation_Map ...
	//... keeping with TopUrban theory of Valeo and Moin (2000)
	imperviousArea_mapAverage_frac = imperviousArea_mapTotal_frac / imperviousArea_mapCells_count;
	//Area_Impervious_frac (fraction) is imperviousArea_mapAverage_frac, all impervious area in imperviouscover.asc
	LocationParams["Area_Impervious_frac"] = imperviousArea_mapAverage_frac;

	//close function to close readImperviousFile 
	readImperviousFile.close();
	return true;
}

//readBlockGroupOutputFileMap function will create vector integer BlockGoupData holding blockgroup.asc values
bool Inputs::readBlockGroupOutputFileMap(string& Directory_Input_CLArg, double NODATA_code)
{
	//Flag_BlockGroupIsLandCover used to control when blockgroup.asc missing
	ifstream readBlockGroupOutputFileMap(Directory_Input_CLArg + "blockgroup.asc");
	if (!readBlockGroupOutputFileMap) {
		//Note: input->Flag_BlockGroupsProcessed variable set to 0 when HydroPlusConfig.xml has no indication of reading Block Group data
		cout << "Notice: If simulating Block Groups, then place a blockgroup.asc file into the input folder." << endl;
		return true;
	}

	string buffer;
	readBlockGroupOutputFileMap >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;

	//If int(LocationParams["nCols"]) not equal to ncols or int(LocationParams["nRows"]) != nrows then abort due to error in inputs
	if (int(LocationParams["nCols"]) != ncols || int(LocationParams["nRows"]) != nrows) {
		cout << "Warning: blockgroup.asc header ncols or nrows value is different from HydroPlusConfig.xml element nCols or nRows." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the blockgroup.asc file ncols or nrows value differs, file size may be faulty." << endl;
		cout << "Correction: Create a blockgroup.asc file that agrees with HydroPlusConfig.xml nCols and nRows." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	int tempdata;
	int count = 0;

	while (readBlockGroupOutputFileMap >> tempdata && count < ncols * nrows) {
		//If tempdata not equal to NODATA then 
		if (tempdata != NODATA) {
			//If tempdata not equal to static_cast<int>(tempdata) then the real value is not equal to its integer value
			if (tempdata != static_cast<int>(tempdata) || tempdata < 0) {
				cout << "Warning: blockgroup.asc contains data with non-integer or negative values." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: When the blockgroup.asc file contains non-integer or negative values, structure breaks." << endl;
				cout << "Correction: Create a blockgroup.asc file with integer values, minimum values of zero." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}
			//BlockGoupData appended with tempdata
			BlockGoupData.push_back(tempdata);
		}
		//else tempdata equals NODATA 
		else {
			//BlockGoupData is defined as NODATA_code
			BlockGoupData.push_back(NODATA_code);
		}
		//advance count
		count = count + 1;
	}
	//if BlockGoupData has size 0 or size < ncols * nrows, then abort program
	//Note: This often happens when values are negative or the file has other non-numeric symbols
	if (BlockGoupData.size() == 0 || BlockGoupData.size() < ncols * nrows) {
		cout << "Warning: blockgroup.asc contains data that are non-integer or negative, or other non-numeric symbols." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the blockgroup.asc file contains such values, space-time becomes warped." << endl;
		cout << "Correction: Create a blockgroup.asc file with integer values, minimum values of zero." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	readBlockGroupOutputFileMap.close();
	return true;
}

//Inputs::readAnthropogenicHeatMap bool function to read anthropogenic heat maps from AH4GUC for maximum hourly value
//Note: AH4GUC maximum value is used with vector of ratios value_hour / value_max to obtain AH4GUC value for each hour
bool Inputs::readAnthropogenicHeatMap(string& Directory_Input_CLArg, double NODATA_code)
{
	//If Flag_CoolAir_AnthropogenicHeat equals 1 then
	if (SimulationStringParams["Flag_CoolAir_AnthropogenicHeat"] == "1") {
		//ifstream function to assign readAnthropogenicHeatMapFile as object containing input file .asc
		ifstream readAnthropogenicHeatMapFile(Directory_Input_CLArg + "ah_max_w_p_m2.asc");

		//If not readAnthropogenicHeatMap then
		if (!readAnthropogenicHeatMapFile) {
			cout << "Warning: ah_max_w_p_m2.asc could not be opened." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: When the ah_max_w_p_m2.asc file is missing there is no anthropogenic heating data to drive the simulation." << endl;
			cout << "Correction: Create a ah_max_w_p_m2.asc file, either manually or perhaps using the i-Tree Cool Air python utility, or ..." << endl;
			cout << "Correction: ... set HydroPlusConfig.xml element Flag_CoolAir_AnthropogenicHeat = 0 to turn off this input option." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}
		//buffer string initialized
		string buffer;
		//readAnthropogenicHeatMapFile reads to buffer the header columns, which are ArcMAP ASCII standards
		readAnthropogenicHeatMapFile >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;

		//If int(LocationParams["nCols"]) not equal to ncols or int(LocationParams["nRows"]) != nrows then abort due to error in inputs
		if (int(LocationParams["nCols"]) != ncols || int(LocationParams["nRows"]) != nrows) {
			cout << "Warning: ah_max_w_p_m2.asc header ncols or nrows value is different from HydroPlusConfig.xml element nCols or nRows." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: When the ah_max_w_p_m2.asc file ncols or nrows value differs, file size may be faulty." << endl;
			cout << "Correction: Create a ah_max_w_p_m2.asc file that agrees with HydroPlusConfig.xml nCols and nRows." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}
		//tempdata reset to empty with ""
		double tempdata;
		//count reset to zero
		int count = 0;

		//while loop through readAnthropogenicHeatMapFile, extracting tempdata, while count is less than mcols * nrows
		while (readAnthropogenicHeatMapFile >> tempdata && count < ncols * nrows) {
			//If tempdata not equal to NODATA then 
			if (tempdata != NODATA) {
				//If tempdata < 0 then abort, as positive values needed
				if (tempdata < 0) {
					cout << "Warning: imperviouscover.asc contains data with negative values." << endl;
					cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
					cout << "Explanation: When the imperviouscover.asc file contains non-numeric symbols, space-time becomes warped." << endl;
					cout << "Correction: Create a imperviouscover.asc file with minimum values of zero, within the realm of reality." << endl;
					//Call abort function, which ends the HydroPlus.exe simulation
					abort();
				}
				//AnthropogenicHeat_Max_W_p_m2 (W/m2) vector is appended using atof function, float value of tempdata.c_str()
				AnthropogenicHeat_Max_W_p_m2.push_back(tempdata);
			}
			//else tempdata equals NODATA 
			else {
				//AnthropogenicHeat_Max_W_p_m2 is defined as NODATA_code
				AnthropogenicHeat_Max_W_p_m2.push_back(NODATA_code);
			}
			//count is advanced
			++count;
		}
		//if AnthropogenicHeat_Max_W_p_m2 has size 0 or size < ncols * nrows, then abort program
		//Note: This often happens when values are negative or the file has other non-numeric symbols
		if (AnthropogenicHeat_Max_W_p_m2.size() == 0 || AnthropogenicHeat_Max_W_p_m2.size() < ncols * nrows) {
			cout << "Warning: ah_max_w_p_m2.asc contains data with negative values or other non-numeric symbols." << endl;
			cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
			cout << "Explanation: When the ah_max_w_p_m2.asc file contains non-numeric symbols, space-time becomes warped." << endl;
			cout << "Correction: Create a ah_max_w_p_m2.asc file with minimum values of zero, within the realm of reality." << endl;
			//Call abort function, which ends the HydroPlus.exe simulation
			abort();
		}

		//readAnthropogenicHeatMapFile close function
		readAnthropogenicHeatMapFile.close();
		//return true to close bool function
		return true;
	}
	//Else If Flag_CoolAir_AnthropogenicHeat not equal to 1 then
	else {
		//count reset to zero
		int count = 0;
		//ncols is defined from HydroPlusConfig.xml LocationParams["nCols"]
		ncols = int(LocationParams["nCols"]);
		//nrows is defined from HydroPlusConfig.xml LocationParams["nRows"]
		nrows = int(LocationParams["nRows"]);
			while (count < ncols * nrows) {
				//AnthropogenicHeat_Max_W_p_m2 (W/m2) is defined as zero
				AnthropogenicHeat_Max_W_p_m2.push_back(0.0);
				//count is advanced
				++count;
			}
		//return true to close bool function
		return true;
	}
}

//calcNLCDLandCoverData function for Model = SpatialTemperatureHydro uses NLCD Class values to determine land cover fractions
//Note: Land Cover vectors are passed to BuildDataOrganizer::AddFolderAreaSpatialCoverData 
void Inputs::calcNLCDLandCoverData(int NumberOfLocations, double NODATA_code)
{
	//HydroPlusConfig.xml DataFolder elements provide values for FracEgreenTree, FracEgreenShortVeg, ConnectedImpervious
	double DCIA_frac = RefParams["DirectlyConnectedImperviousArea_frac"];
	double EvergreenTreeCover_frac = RefParams["TreeCover_Evergreen_frac"];
	double EverGreenShrubCover_frac = RefParams["SVegCover_Evergreen_frac"];

	//Flag_CoolAir_ImperviousCoverAdjust (flag) is initialized to 0, which will result in no change to land cover
	//If SimulationStringParams map contains the key Flag_CoolAir_ImperviousCoverAdjust then enter
	if (SimulationStringParams.find("Flag_CoolAir_ImperviousCoverAdjust") != SimulationStringParams.end()) {
		//SimulationStringParams["Flag_CoolAir_ImperviousCoverAdjust"] defined in HydroPlusConfig.xml for element TreeCanopyCover_overImpervious_frac
		//Note: All adjustments maintain the ranges for fractional cover between 0 and 1
		//Note: If Flag_CoolAir_ImperviousCoverAdjust = 0 then TreeCanopyCover_overImpervious_frac causes no change in imperviouscover.asc values ...
		//Note: ... with IC = IC, TCI = TC * TreeCanopyCover_overImpervious_frac, TCP = TC - TCI, IC_underT = TCI, IC_noT = IC - TCI
		//Note: If Flag_CoolAir_ImperviousCoverAdjust = 1 then TreeCanopyCover_overImpervious_frac causes increase in imperviouscover.asc values ...
		//Note: ... with IC = IC + TCI, TCI = TC * TreeCanopyCover_overImpervious_frac, TCP = TC - TCI, IC_underT = TCI, IC_noT = IC - TCI
		//Note: If Flag_CoolAir_ImperviousCoverAdjust = 2 then comparison is made with base case that used Flag_CoolAir_ImperviousCoverAdjust = 0 ...
		//Note: ... with TC increased by Scenario_CoolAir_Alternative_TC, distributed by Scenario_CoolAir_Base_TCI_to_TC ...
		//Note: ... without increasing IC to include TCI and perhaps decreasing IC to implement Scenario_CoolAir_Base_TCI_to_TC 
		//Note: If Flag_CoolAir_ImperviousCoverAdjust = 3 then comparison is made with base case that used Flag_CoolAir_ImperviousCoverAdjust = 1 ...
		//Note: ... with TC increased by Scenario_CoolAir_Alternative_TC, distributed by Scenario_CoolAir_Base_TCI_to_TC ...
		//Note: ... with increasing IC to include TCI and perhaps decreasing IC to implement Scenario_CoolAir_Base_TCI_to_TC 
		//Note: If Flag_CoolAir_ImperviousCoverAdjust = 4 then comparison is made with base case that used Flag_CoolAir_ImperviousCoverAdjust = 1 ...
		//Note: ... with TC increased by Scenario_CoolAir_Alternative_TC, distributed by TreeCanopyCover_overImpervious_frac ...
		//Note: ... with increasing Permeable Pavement to include TCI suggested in implement Scenario_CoolAir_Base_TCI_to_TC 
		Flag_CoolAir_ImperviousCoverAdjust = stoi(SimulationStringParams["Flag_CoolAir_ImperviousCoverAdjust"]);
		//If Flag_CoolAir_ImperviousCoverAdjust not 0, 1, 2, 3, or 4 then set to 0
		if (Flag_CoolAir_ImperviousCoverAdjust != 0 && Flag_CoolAir_ImperviousCoverAdjust != 1 && Flag_CoolAir_ImperviousCoverAdjust != 2 && Flag_CoolAir_ImperviousCoverAdjust != 3 && Flag_CoolAir_ImperviousCoverAdjust != 4) {
			//Flag_CoolAir_ImperviousCoverAdjust default setting is 0, no change
			Flag_CoolAir_ImperviousCoverAdjust = 0;
		}
	}

	//Scenario_CoolAir_Base_TCI_to_TC (frac) is initialized to -1, which will result in no change to land cover
	//If SimulationStringParams map contains the key Scenario_CoolAir_Base_TCI_to_TC_frac then enter
	if (SimulationStringParams.find("Scenario_CoolAir_Base_TCI_to_TC_frac") != SimulationStringParams.end()) {
		//SimulationStringParams["Scenario_CoolAir_Base_TCI_to_TC_frac"] defined in HydroPlusConfig.xml for scenarios
		Scenario_CoolAir_Base_TCI_to_TC_frac = stod(SimulationStringParams["Scenario_CoolAir_Base_TCI_to_TC_frac"]);
		//Conditionals to maintain limits
		if (Scenario_CoolAir_Base_TCI_to_TC_frac < 0) { Scenario_CoolAir_Base_TCI_to_TC_frac = 0.0; }
		if (Scenario_CoolAir_Base_TCI_to_TC_frac > 1) { Scenario_CoolAir_Base_TCI_to_TC_frac = 1.0; }
	}

	//Scenario_CoolAir_Alternative_TC_frac (frac) is initialized to -1, which will result in no change to land cover
	//If SimulationStringParams map contains the key Scenario_CoolAir_Alternative_TC_frac then enter
	if (SimulationStringParams.find("Scenario_CoolAir_Alternative_TC_frac") != SimulationStringParams.end()) {
		//SimulationStringParams["Scenario_CoolAir_Alternative_TC_frac"] defined in HydroPlusConfig.xml for scenarios
		//Note: If Flag_CoolAir_ImperviousCoverAdjust>1 then Scenario_CoolAir_Alternative_TC_frac (frac) represents minimum treecover (frac)
		Scenario_CoolAir_Alternative_TC_frac = stod(SimulationStringParams["Scenario_CoolAir_Alternative_TC_frac"]);
		//Conditionals to maintain limits
		if (Scenario_CoolAir_Alternative_TC_frac < 0) { Scenario_CoolAir_Alternative_TC_frac = 0.0; }
		if (Scenario_CoolAir_Alternative_TC_frac > 1) { Scenario_CoolAir_Alternative_TC_frac = 1.0; }
	}

	//Scenario_CoolAir_Alternative_IC_frac (frac) is initialized to -1, which will result in no change to land cover
	//If SimulationStringParams map contains the key Scenario_CoolAir_Alternative_IC_frac then enter
	if (SimulationStringParams.find("Scenario_CoolAir_Alternative_IC_frac") != SimulationStringParams.end()) {
		//SimulationStringParams["Scenario_CoolAir_Alternative_IC_frac"] defined in HydroPlusConfig.xml for scenarios
		//Note: If Flag_CoolAir_ImperviousCoverAdjust>1 then Scenario_CoolAir_Alternative_IC_frac (frac) represents maximum imperviouscover (frac)
		Scenario_CoolAir_Alternative_IC_frac = stod(SimulationStringParams["Scenario_CoolAir_Alternative_IC_frac"]);
		//Conditionals to maintain limits
		if (Scenario_CoolAir_Alternative_IC_frac < 0) { Scenario_CoolAir_Alternative_IC_frac = 0.0; }
		if (Scenario_CoolAir_Alternative_IC_frac > 1) { Scenario_CoolAir_Alternative_IC_frac = 1.0; }
	}

	//TreeCanopyCover_overImpervious_frac (frac) is defined from HydroPlusConfig.xml element TreeCanopyCover_overImpervious_frac
	//Note: TreeCanopyCover_overImpervious_frac in Model = SpatialTemperatureHydro, e.g., CoolAir, is fraction of total tree cover
	//Note: TreeCanopyCover_overImpervious_frac in Model = StatisticalHydro, e.g., Hydro, is fraction of total land cover
	double TreeCanopyCover_overImpervious_frac = RefParams["TreeCanopyCover_overImpervious_frac"];
	//If TreeCanopyCover_overImpervious_frac < 0 or > 1 then set to 0 or 1; required given fractional areas are not checked
	if (TreeCanopyCover_overImpervious_frac > 1) { TreeCanopyCover_overImpervious_frac = 1; }
	if (TreeCanopyCover_overImpervious_frac < 0) { TreeCanopyCover_overImpervious_frac = 0; }

	//If Flag_CoolAir_ImperviousCoverAdjust > 1 for alternative, and Scenario_CoolAir_Base_TCI_to_TC_frac undefined then set to base case value
	if (Flag_CoolAir_ImperviousCoverAdjust > 1 && Scenario_CoolAir_Base_TCI_to_TC_frac < 0) {
		Scenario_CoolAir_Base_TCI_to_TC_frac = TreeCanopyCover_overImpervious_frac;
	}		

	//PerviousCover_FullSkyView_frac (fraction) defined as area not within TotalCover (tree cover and impervious cover)
	double PerviousCover_FullSkyView_frac = 0;
	//PerviousCover_FullSkyView_with_SVeg_frac (fraction) is defined as the fraction of PerviousCover_FullSkyView_frac in short vegetation
	//Note: PerviousCover_FullSkyView_with_SVeg_frac value defined within each NLCD Class using PerviousCover_FullSkyView_frac
	//Note: ShortVegCover (frac) = PerviousCover_FullSkyView_frac * SVPerviousCover_FullSkyView_with_SVeg_frac
	//Note: For example, if NLCD Class 21-24, then SVPerviousCover_FullSkyView_with_SVeg_frac = 0.5
	double PerviousCover_FullSkyView_with_SVeg_frac = 0;

	//If LocationParams["NumberOfLocations"] = LocationParams["nCols"] * LocationParams["nRows"] !+ TreeCoverData.size() then errror with inputs
	//Note: If HydroPlusConfig.xml has SimulationLocationParams nCols and nRows =! with size of map inputs such as treecover.asc, then model will abort
	if (NumberOfLocations != TreeCoverData.size()) {
		cout << endl;
		cout << "Warning: HydroPlusConfig.xml parameters nCols and nRows disagree with the " << TreeCoverData.size() << " pixels in treecover.asc map." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the product of nCols and nRows does not equal pixels of map input data, the model cannot align data." << endl;
		cout << "Correction: In the HydroPlusConfig.xml file, ensure parameters nCols and nRows agree with the map input data." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//For loop for i to NumberOfLocations
	for (int i = 0; i < NumberOfLocations; i++)	{

		//Variables for scenario development, explained below 
		//cte 202410 Compute TreeOnImpervious_base_frac and TreeOnPervious_base_frac with Scenario_CoolAir_Base_TCI_to_TC_frac
		//TreeOnPervious_base_frac = TreeCoverData[i] * (1 - Scenario_CoolAir_Base_TCI_to_TC_frac)
		//TreeOnImpervious_base_frac = TreeCoverData[i] * Scenario_CoolAir_Base_TCI_to_TC_frac
		//TreeOnImperviousPorous_alt_frac = TreeOnImperviousPorous_alt_frac + (TreeOnImpervious_alt_frac - TreeOnImpervious_base_frac)

		double TreeCoverdata_base_frac = 0;
		double ImperCoverdata_base_frac = 0;
		double TotalCover_base_frac = 0;
		double TreeOnImpervious_base_frac = 0;
		double ImperviousNoTree_base_frac = 0;
		double TreeOnPervious_base_frac = 0;
		double TreeOnPervious_max_frac = 0;

		//If TreeCoverData (frac) and ImperCoverData (frac) are not NODATA_code then determine TotalCover (frac) and ...
		// ... depending on HydroPlusConfig.xml Flag_CoolAir_ImperviousCoverAdjust change TreeCoverData and ImperCoverData
		//Note: In some simulations the input map (e.g., treecover.asc) has a NODATA_code for the reference station pixel, but ...
		//Note: ... in a previous Input.cpp function the TreeCoverData[] and ImperCoverData[] vectors were updated with the reference station
		if (TreeCoverData[i] != NODATA_code && ImperCoverData[i] != NODATA_code) {

			//cte 202405 Refactor to try and reduce items below if they can reside within TotalCover > 1 or Else components
			//cte 202405 Refactor to ensure Scenario_CoolAir_Alternative_IC_frac is working
			//If NLCDLandCoverData Class > 20 and < 25 AND  && i != MapPixel_ReferenceStation_ID then enter for non-reference station urban areas ...
			if (NLCDLandCoverData[i] > 20 && NLCDLandCoverData[i] < 25 && i != MapPixel_ReferenceStation_ID) {

				//TreeCoverdata_base_frac (frac) equals TreeCoverData (frac)
				TreeCoverdata_base_frac = TreeCoverData[i];
				//ImperCoverdata_base_frac (frac) equals ImperCoverData (frac)
				ImperCoverdata_base_frac = ImperCoverData[i];
				//TotalCover_base_frac (frac) equals TreeCoverdata_base_frac (frac) + ImperCoverdata_base_frac (frac)
				TotalCover_base_frac = TreeCoverdata_base_frac + ImperCoverdata_base_frac;
				//TreeOnPervious_max_frac (frac) = 1.0 - ImperCoverdata_base_frac (frac)
				//Note: TreeOnPervious_max_frac used to nudge new tree cover over impervious cover, which can become permeable pavement
				TreeOnPervious_max_frac = 1.0 - ImperCoverdata_base_frac;

				//If Flag_CoolAir_ImperviousCoverAdjust = 2 or 3 then implement scenarios for land cover change
				//Note: Scenarios are described above where Flag_CoolAir_ImperviousCoverAdjust is read from HydroPlusConfig.xml
				if (Flag_CoolAir_ImperviousCoverAdjust > 1) {
					//If TreeCoverData (frac) < Scenario_CoolAir_Alternative_TC_frac (frac) then use larger value 
					if (TreeCoverData[i] < Scenario_CoolAir_Alternative_TC_frac) {
						//TreeCoverData (frac) equals Scenario_CoolAir_Alternative_TC_frac (frac) as the larger value
						TreeCoverData[i] = Scenario_CoolAir_Alternative_TC_frac;
						//If TreeCoverData > 1 then set to 1 to maintain upper limit
						if (TreeCoverData[i] > 1.0) { TreeCoverData[i] = 1.0; }
					}
					//If ImperCoverData (frac) > Scenario_CoolAir_Alternative_IC_frac (frac) then use smaller value 
					//Note: Scenario_CoolAir_Alternative_IC_frac (frac) is maximum allowable IC and defaults to 1.0
					if (ImperCoverData[i] > Scenario_CoolAir_Alternative_IC_frac) {
						//ImperCoverData (frac) equals Scenario_CoolAir_Alternative_TC_frac (frac) as the smaller value
						ImperCoverData[i] = Scenario_CoolAir_Alternative_IC_frac;
						//If ImperCoverData > 1 then set to 1 to maintain upper limit
						if (ImperCoverData[i] > 1.0) { ImperCoverData[i] = 1.0; }
					}
				}
			}
			//TotalCover (fraction) defined as sum of TreeCoverData and ImperCoverData read from input map files treecover.asc and imperviouscover.asc
			TotalCover[i] = TreeCoverData[i] + ImperCoverData[i];
		}

		//If TreeCoverData (frac), ImperCoverData (frac) and NLCDLandCoverData (frac) are not NODATA_code then determine cover types 
		//Note: Depending on HydroPlusConfig.xml Flag_CoolAir_ImperviousCoverAdjust change cover types, e.g., TCI, TCP, ICnoT
		//Note: In some simulations the input map (e.g., treecover.asc) has a NODATA_code for the reference station pixel, but ...
		//Note: ... in a previous Input.cpp function the TreeCoverData[] and ImperCoverData[] vectors were updated with the reference station
		if (TreeCoverData[i] != NODATA_code && ImperCoverData[i] != NODATA_code && NLCDLandCoverData[i] != NODATA_code)	{
			//Note: FracEgreenTree and FracEgreenShortVeg not currently used by SpatialTemperatureHydro, could be used to set LAI_BAI_Tree_min_m2pm2, minimum Tree LAI
			FracEgreenTree[i] = EvergreenTreeCover_frac;
			FracEgreenShortVeg[i] = EverGreenShrubCover_frac;
			//ConnectedImpervious (frac) is used in AddFolderAreaSpatialCoverData function to create ImperviousCover_DrainsToOutlet_frac to partition runoff
			ConnectedImpervious[i] = DCIA_frac;
			//If NLCD Class within 20 to 25 range then within developed pixels
			//Note: Class 21 = Developed open space, 22 = Developed low intensity, 23 = Developed medium intensity, 24 = Developed high intensity
			if (NLCDLandCoverData[i] > 20 && NLCDLandCoverData[i] < 25) {
				//PermeablePavementCover (frac) is zero given NLCD Class and scenario
				PermeablePavementCover[i] = 0;
				//WaterCover (fraction) is zero given NLCD class is terrestrial
				WaterCover[i] = 0;
				//PerviousCover_FullSkyView_with_SVeg_frac (frac) = SVeg_in_Gap_DevelopedArea_frac (frac) 
				//Note: PerviousCover_FullSkyView_with_SVeg_frac is HydroPlusConfig.xml element SVeg_in_Gap_DevelopedArea_frac (frac); typically 0.5
				//Note: PerviousCover_FullSkyView_with_SVeg_frac for non-urban defined below for each NLCD Class based on sub-grid heterogeneity
				PerviousCover_FullSkyView_with_SVeg_frac = TemperatureCalculationParams["SVeg_in_Gap_DevelopedArea_frac"];
				//If PerviousCover_FullSkyView_with_SVeg_frac <= 0 then redefine to 0.5; HydroPlusConfig.xml missing SVeg_in_Gap_DevelopedArea_frac
				if (PerviousCover_FullSkyView_with_SVeg_frac <= 0) { PerviousCover_FullSkyView_with_SVeg_frac = 0.5; }	
				//PerviousCover_FullSkyView_frac (frac) = 1 - TotalCover (frac); area not within TotalCover (tree cover and impervious cover)
				PerviousCover_FullSkyView_frac = 1.0 - TotalCover[i];
				//If PerviousCover_FullSkyView_frac (frac) < 0 then set to 0 conditional to control for lower limit
				if (PerviousCover_FullSkyView_frac < 0.0) { PerviousCover_FullSkyView_frac = 0.0; }
				//SoilCover (frac) = PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac)
				SoilCover[i] = PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac);
				//ShortVegCover (frac) = PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac
				ShortVegCover[i] = PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac;
				//If TotalCover (frac) > 1.0 then initially assign TreeOnImpervious (frac) = TotalCover (frac) - 1.0
				if (TotalCover[i] > 1.0) {
					//TreeOnImpervious (frac) is tree cover above impervious area, defined as all cover greater than 1.0
					//Note: For Example: If 1.2 TotalCover = 0.6 TC + 0.6 IC, then 0.2 TreeOnImpervious = 1.2 - 1.0
					TreeOnImpervious[i] = TotalCover[i] - 1.0;
					//TreeOnPervious (frac) is tree cover above pervious, defined as that which is not above impervious
					//Note: For Example: 0.4 TreeOnPervious = 0.6 TC - 0.2 TreeOnImpervious 
					TreeOnPervious[i] = TreeCoverData[i] - TreeOnImpervious[i];
					//ImperviousNoTree (frac) is impervious cover with full sky view, defined as all impervious cover without trees 
					//Note: For Example: 0.4 ImperviousNoTree = 0.6 ImperCoverData - 0.2 TreeOnImpervious 
					ImperviousNoTree[i] = ImperCoverData[i] - TreeOnImpervious[i];
				}
				//Else If TotalCover (frac) <= 1.0 then initially assign TreeOnImpervious (frac) = 0
				else {
					//TreeOnImpervious (fraction) is zero given TotalCover < 100%
					TreeOnImpervious[i] = 0;
					//TreeOnPervious (fraction) is all TreeCoverData given TotalCover < 100%
					TreeOnPervious[i] = TreeCoverData[i];
					//ImperviousNoTree (fraction) is all impervious cover given TotalCover < 100%
					ImperviousNoTree[i] = ImperCoverData[i];
				}

				//If i != MapPixel_ReferenceStation_ID then 
				//Note: Reclassification of tree cover onto impervious cover is not allowed at the reference station
				if (i != MapPixel_ReferenceStation_ID) {

					//tci_actual_frac (frac) = TreeCoverdata_base_frac (frac) * TreeCanopyCover_overImpervious_frac (frac)
					double tci_actual_frac = TreeCoverdata_base_frac * TreeCanopyCover_overImpervious_frac;
					//If Flag_CoolAir_ImperviousCoverAdjust >= 2 then 
					if (Flag_CoolAir_ImperviousCoverAdjust >= 2) {
						//tci_actual_frac (frac) = TreeCoverdata_base_frac (frac) * Scenario_CoolAir_Base_TCI_to_TC_frac (frac)
						tci_actual_frac = TreeCoverdata_base_frac * Scenario_CoolAir_Base_TCI_to_TC_frac;
					}
					//tcp_actual_frac (frac) = TreeCoverdata_base_frac (frac) - tci_actual_frac (frac)
					double tcp_actual_frac = TreeCoverdata_base_frac - tci_actual_frac;
					//ic_no_tc_actual_frac (frac) = 0 is impervious cover with no tree cover above
					double ic_no_tc_actual_frac = 0;
					//tci_target_frac (frac) = 0 is tree cover over impervious cover
					double tci_target_frac = 0;
					//cte 202410 Remove
					//tci_permeablepavement_target_frac (frac) = 0 is tree cover over impervious cover that becomes permeable pavement
					double tci_permeablepavement_target_frac = 0;
					//tcp_target_frac (frac) = 0 is tc over pervious cover
					double tcp_target_frac = 0;
					//ic_no_tc_target_frac (frac) = 0 is impervious cover with no tree cover above
					double ic_no_tc_target_frac = 0;
					//cte 202410 Remove
					//ic_no_tc_available_frac (frac) = 0 is impervious cover with no tree cover above used in conditional 
					double ic_no_tc_available_frac = 0;

					//If tcp_actual_frac (frac) > TreeOnPervious_max_frac (frac) then 
					if (tcp_actual_frac > TreeOnPervious_max_frac) {
						//tcp_actual_frac (frac) = TreeOnPervious_max_frac (frac)
						tcp_actual_frac = TreeOnPervious_max_frac;
						//tci_actual_frac (frac) = TreeCoverdata_base_frac (frac) - tcp_actual_frac (frac)
						tci_actual_frac = TreeCoverdata_base_frac - tcp_actual_frac;
						//If tci_actual_frac (frac) < 0.0 then set to 0
						if (tci_actual_frac < 0.0) { tci_actual_frac = 0.0; }
					}
					//If (ImperCoverData (frac) + tci_actual_frac (frac) + tcp_actual_frac (frac)) > 1.0 then
					//Note: Conditional indicates IC_no_TC + TCI + TCP > 1, therefor IC_no_TC is reduced, becoming TCI
					if ((ImperCoverData[i] + tci_actual_frac + tcp_actual_frac) > 1.0) {
						//ic_no_tc_actual_frac (frac) = 1.0 - (tci_actual_frac (frac) + tcp_actual_frac (frac))
						ic_no_tc_actual_frac = 1.0 - (tci_actual_frac + tcp_actual_frac);
					}
					//Else If Flag_CoolAir_ImperviousCoverAdjust = 0 or Flag_CoolAir_ImperviousCoverAdjust = 2 then 
					else if (Flag_CoolAir_ImperviousCoverAdjust == 0 || Flag_CoolAir_ImperviousCoverAdjust == 2) {
						//ic_no_tc_actual_frac (frac) =  ImperCoverdata_base_frac (frac) - tci_actual_frac (frac)
						ic_no_tc_actual_frac = ImperCoverdata_base_frac - tci_actual_frac;
					}
					//Else If Flag_CoolAir_ImperviousCoverAdjust = 1 or Flag_CoolAir_ImperviousCoverAdjust >= 3 then 
					else if (Flag_CoolAir_ImperviousCoverAdjust == 1 || Flag_CoolAir_ImperviousCoverAdjust >= 3) {
						//ic_no_tc_actual_frac (frac) = ImperCoverData (frac)
						ic_no_tc_actual_frac = ImperCoverData[i];
					}
					//If ic_no_tc_actual_frac (frac) < 0 then set to 0
					if (ic_no_tc_actual_frac < 0.0) { ic_no_tc_actual_frac = 0.0; }

					//If Flag_CoolAir_ImperviousCoverAdjust <= 1 then enter for scenario base, e.g., 0 or 1
					if (Flag_CoolAir_ImperviousCoverAdjust <= 1) {
						//TreeOnPervious (frac) = tcp_actual_frac (frac)
						TreeOnPervious[i] = tcp_actual_frac;
						//TreeOnImpervious (frac) = tci_actual_frac (frac)
						TreeOnImpervious[i] = tci_actual_frac;
						//ImperviousNoTree (frac) = ic_no_tc_actual_frac (frac)
						ImperviousNoTree[i] = ic_no_tc_actual_frac;
					}

					//If Flag_CoolAir_ImperviousCoverAdjust >= 2 then enter for scenario alternative, e.g., 2 or 3 or 4
					if (Flag_CoolAir_ImperviousCoverAdjust >= 2) {
						//If (TreeCoverData (frac) > TreeCoverdata_base_frac (frac) then
						if (TreeCoverData[i] > TreeCoverdata_base_frac) {
							//If Flag_CoolAir_ImperviousCoverAdjust = 2 or 3 then enter for scenario alternative
							if (Flag_CoolAir_ImperviousCoverAdjust == 2 || Flag_CoolAir_ImperviousCoverAdjust == 3) {
								//tci_target_frac (frac) = tci_actual_frac (frac) + (TreeCoverData (frac) - TreeCoverdata_base_frac (frac)) * TreeCanopyCover_overImpervious_frac (frac)
								tci_target_frac = tci_actual_frac + (TreeCoverData[i] - TreeCoverdata_base_frac) * TreeCanopyCover_overImpervious_frac;
							}
							//If Flag_CoolAir_ImperviousCoverAdjust = 4 then enter for scenario alternative
							if (Flag_CoolAir_ImperviousCoverAdjust == 4) {
								//tci_target_frac (frac) = tci_actual_frac (frac)
								tci_target_frac = tci_actual_frac; 
							}
							//If (tci_target_frac (frac) > (tci_actual_frac (frac) + ic_no_tc_actual_frac (frac)))
							if (tci_target_frac > (tci_actual_frac + ic_no_tc_actual_frac)) {
								//tci_target_frac (frac) = (tci_actual_frac (frac) + ic_no_tc_actual_frac (frac))
								tci_target_frac = (tci_actual_frac + ic_no_tc_actual_frac);
							}
						}
						//Else (TreeCoverData (frac) <= TreeCoverdata_base_frac (frac) then
						else {
							//tci_target_frac (frac) = tci_actual_frac (frac)
							tci_target_frac = tci_actual_frac;
						}

						//tcp_target_frac (frac) = TreeCoverData (frac) - tci_target_frac (frac)
						tcp_target_frac = TreeCoverData[i] - tci_target_frac;

						//If tcp_target_frac (frac) < tcp_actual_frac (frac) then
						//Note: Consider refactor if this conditional not needed
						if (tcp_target_frac < tcp_actual_frac) {
							//tcp_target_frac (frac) = tcp_actual_frac (frac)
							tcp_target_frac = tcp_actual_frac;
							//tci_target_frac (frac) = TreeCoverData (frac) - tcp_target_frac (frac)
							tci_target_frac = TreeCoverData[i] - tcp_target_frac;
							//If tci_target_frac (frac) < 0.0 then set to 0
							if (tci_target_frac < 0.0) { tci_target_frac = 0.0; }
						}
						//If (ImperCoverdata_base_frac (frac) + tci_target_frac (frac) + tcp_target_frac (frac)) > 1.0 then
						//Note: Conditional indicates IC_no_TC + TCI + TCP > 1, therefor IC_no_TC is reduced, becoming TCI
						if ((ImperCoverdata_base_frac + tci_target_frac + tcp_target_frac) > 1.0) {
							//ic_no_tc_target_frac (frac) = 1 - (tci_target_frac (frac) + tcp_target_frac (frac))
							ic_no_tc_target_frac = 1 - (tci_target_frac + tcp_target_frac);
							//If ic_no_tc_actual_frac (frac) < 0 then set to 0
							if (ic_no_tc_target_frac < 0.0) { ic_no_tc_target_frac = 0.0; }
						}
						//Else If Flag_CoolAir_ImperviousCoverAdjust = 2 then
						else if (Flag_CoolAir_ImperviousCoverAdjust == 2) {
							//ic_no_tc_target_frac (frac) = ImperCoverdata_base_frac (frac) - tci_target_frac (frac)
							ic_no_tc_target_frac = ImperCoverdata_base_frac - tci_target_frac;
						}
						//Else If Flag_CoolAir_ImperviousCoverAdjust = 3 then
						else if (Flag_CoolAir_ImperviousCoverAdjust == 3) {
							//ic_no_tc_target_frac (frac) = ic_no_tc_actual_frac
							ic_no_tc_target_frac = ic_no_tc_actual_frac;
						}
						//Else If Flag_CoolAir_ImperviousCoverAdjust = 4 then
						else if (Flag_CoolAir_ImperviousCoverAdjust == 4) {
							//ic_no_tc_target_frac (frac) = ImperCoverdata_base_frac - tci_target_frac - (TreeCoverData[i] - TreeCoverdata_base_frac) * TreeCanopyCover_overImpervious_frac
							ic_no_tc_target_frac = ImperCoverdata_base_frac - tci_target_frac - (TreeCoverData[i] - TreeCoverdata_base_frac) * TreeCanopyCover_overImpervious_frac;
						}
						
						//If ic_no_tc_target_frac (frac) < 0 then set to 0
						if (ic_no_tc_target_frac < 0.0) { ic_no_tc_target_frac = 0.0; }

						//If TreeCoverData (frac) > TreeCoverdata_base_frac (frac) And ((tci_target_frac + ic_no_tc_target_frac) > (tci_actual_frac + ic_no_tc_actual_frac)) then
						if ((TreeCoverData[i] > TreeCoverdata_base_frac) && ((tci_target_frac + ic_no_tc_target_frac) > (tci_actual_frac + ic_no_tc_actual_frac))) {
							//ic_no_tc_target_frac (frac) = (tci_actual_frac (frac) + ic_no_tc_actual_frac (frac)) - tci_target_frac (frac)
							ic_no_tc_target_frac = (tci_actual_frac + ic_no_tc_actual_frac) - tci_target_frac;
						}
						//If ic_no_tc_target_frac (frac) < 0 then ic_no_tc_target_frac (frac) set to 0
						if (ic_no_tc_target_frac < 0) { ic_no_tc_target_frac = 0.0; }

						//If (tci_target_frac + ic_no_tc_target_frac) > 1.0 then 
						//Note: Consider refactor if this conditional not needed
						if ((tci_target_frac + ic_no_tc_target_frac) > 1.0) {
							//ic_no_tc_target_frac (frac) = 1.0 - tci_target_frac (frac)
							ic_no_tc_target_frac = 1.0 - tci_target_frac;
						}

						//TreeOnPervious (frac) = tcp_target_frac (frac)
						TreeOnPervious[i] = tcp_target_frac;
						//TreeOnImpervious (frac) = tci_target_frac (frac)
						TreeOnImpervious[i] = tci_target_frac;
						//ImperviousNoTree (frac) = ic_no_tc_target_frac (frac)
						ImperviousNoTree[i] = ic_no_tc_target_frac;
					}

					//If (1 - ImperviousNoTree (frac) - TreeOnImpervious (frac) - TreeOnPervious (frac)) > 0 then
					//Note: Conditional indicates IC_no_TC + TCI + TCP > 1, therefor IC_no_TC is reduced, becoming TCI
					if ((1.0 - ImperviousNoTree[i] - TreeOnImpervious[i] - TreeOnPervious[i]) > 0) {
						//PerviousCover_FullSkyView_with_SVeg_frac (frac) defined for NLCD Class based on best estimates of sub-grid heterogeneity
						//PerviousCover_FullSkyView_with_SVeg_frac is HydroPlusConfig.xml element SVeg_in_Gap_DevelopedArea_frac (frac); typically 0.5
						PerviousCover_FullSkyView_with_SVeg_frac = TemperatureCalculationParams["SVeg_in_Gap_DevelopedArea_frac"];
						//If PerviousCover_FullSkyView_with_SVeg_frac <= 0 then redefine; likely due to HydroPlusConfig.xml not containing SVeg_in_Gap_DevelopedArea_frac
						if (PerviousCover_FullSkyView_with_SVeg_frac <= 0) {
							//Note: PerviousCover_FullSkyView_with_SVeg_frac defined here as 0.5 
							PerviousCover_FullSkyView_with_SVeg_frac = 0.5;
						}
						//PerviousCover_FullSkyView_frac defined as area not within tree cover and impervious cover
						PerviousCover_FullSkyView_frac = 1.0 - ImperviousNoTree[i] - TreeOnImpervious[i] - TreeOnPervious[i];
						//SoilCover (frac) defined as PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac)
						SoilCover[i] = PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac);
						//ShortVegCover (frac) defined as PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac
						ShortVegCover[i] = PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac;
					}
					//Else (1 - ImperviousNoTree (frac) - TreeOnImpervious (frac) - TreeOnPervious (frac)) < 0 then
					else {
						//SoilCover (frac) = 0
						SoilCover[i] = 0;
						//ShortVegCover (frac) = 0
						ShortVegCover[i] = 0;
					}
				}
			}
			//Else If NLCD Class is water type
			//Note: Class 11 = Open Water or 12 = Perennial Ice/Snow then
			else if (NLCDLandCoverData[i] == 11 || NLCDLandCoverData[i] == 12) {
				//PermeablePavementCover (frac) is zero given NLCD Class and scenario
				PermeablePavementCover[i] = 0;
				//SoilCover (fraction) is zero given NLCD class is water
				SoilCover[i] = 0;
				//ShortVegCover (fraction) is zero given NLCD class is water
				ShortVegCover[i] = 0;
				//WaterCover (fraction) occupies all remaining space not occupied by TotalCover
				WaterCover[i] = (1.0 - TotalCover[i]);
				//TreeOnImpervious (fraction) is zero given TotalCover < 100%
				TreeOnImpervious[i] = 0;
				//TreeOnPervious (fraction) is all TreeCoverData given TotalCover < 100%
				TreeOnPervious[i] = TreeCoverData[i];
				//ImperviousNoTree (fraction) is all impervious cover given TotalCover < 100%
				ImperviousNoTree[i] = ImperCoverData[i];
			}
			//Else If NLCD Class is barren or rocky 
			//Note: Class 31 = Barren land and rocky then
			else if (NLCDLandCoverData[i] == 31) {
				//PerviousCover_FullSkyView_with_SVeg_frac (frac) defined for each NLCD Class based on best estimates of sub-grid heterogeneity
				//Note: PerviousCover_FullSkyView_with_SVeg_frac defined here as 0.0 
				PerviousCover_FullSkyView_with_SVeg_frac = 0.0;
				//PermeablePavementCover (frac) is zero given NLCD Class and scenario
				PermeablePavementCover[i] = 0;
				//WaterCover (fraction) is zero given NLCD class is terrestrial
				WaterCover[i] = 0;
				//PerviousCover_FullSkyView_frac defined as area not within TotalCover (tree cover and impervious cover)
				PerviousCover_FullSkyView_frac = 1.0 - TotalCover[i];
				//SoilCover (frac) defined as PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac)
				SoilCover[i] = PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac);
				//ShortVegCover (frac) defined as PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac
				ShortVegCover[i] = PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac;
				//TreeOnImpervious (fraction) is zero given TotalCover < 100%
				TreeOnImpervious[i] = 0;
				//TreeOnPervious (fraction) is all TreeCoverData given TotalCover < 100%
				TreeOnPervious[i] = TreeCoverData[i];
				//ImperviousNoTree (fraction) is all impervious cover given TotalCover < 100%
				ImperviousNoTree[i] = ImperCoverData[i];
			}
			//Else If NLCD Class is forest types
			//Note: CLass 41 = Deciduous forest, 42 = Evergreen forest, or 43 = Mixed forest then
			else if (NLCDLandCoverData[i] == 41 || NLCDLandCoverData[i] == 42 || NLCDLandCoverData[i] == 43) {
				//PerviousCover_FullSkyView_with_SVeg_frac (frac) defined for each NLCD Class based on best estimates of sub-grid heterogeneity
				//Note: PerviousCover_FullSkyView_with_SVeg_frac defined here as 1.0 
				PerviousCover_FullSkyView_with_SVeg_frac = 1.0;
				//PermeablePavementCover (frac) is zero given NLCD Class and scenario
				PermeablePavementCover[i] = 0;
				//WaterCover (fraction) is zero given NLCD class is terrestrial
				WaterCover[i] = 0;
				//SoilCover (fraction) is zero given NLCD class is forest
				SoilCover[i] = 0;
				//PerviousCover_FullSkyView_frac defined as area not within TotalCover (tree cover and impervious cover)
				PerviousCover_FullSkyView_frac = 1.0 - TotalCover[i];
				//ShortVegCover (frac) defined as PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac
				ShortVegCover[i] = PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac;
				//TreeOnImpervious (fraction) is zero given TotalCover < 100%
				TreeOnImpervious[i] = 0;
				//TreeOnPervious (fraction) is all TreeCoverData given TotalCover < 100%
				TreeOnPervious[i] = TreeCoverData[i];
				//ImperviousNoTree (fraction) is all impervious cover given TotalCover < 100%
				ImperviousNoTree[i] = ImperCoverData[i];
				//Note: If NLCD value 41 = Deciduous forest, fraction of evergreen remains as defined in HydroPlusConfig.xml
				//If NLCD value 42 = Evergreen forest, fraction of evergreen in respective covers redfined as 1
				if (NLCDLandCoverData[i] == 42) {
					FracEgreenTree[i] = 1;
					FracEgreenShortVeg[i] = 1;
				}
				//If NLCD value 43 = Mixed forest, fraction of evergreen in respective covers redefined as 0.5
				if (NLCDLandCoverData[i] == 43) {
					FracEgreenTree[i] = 0.5;
					FracEgreenShortVeg[i] = 0.5;
				}
			}
			//Else If NLCD Class is agricultural types
			//Note: Class 51 = Dwarf Scrub, 52 = Shrub / Scrub, 71 = Grassland / Herbaceous, 72 = Sedge / Herbaceous, 73 = Lichens, 74 = Moss, ...
			//Note: ... 81 = Pasture / Hay, or 82 = Cultivated Crops
			else if (NLCDLandCoverData[i] == 51 || NLCDLandCoverData[i] == 52 || NLCDLandCoverData[i] == 71 || NLCDLandCoverData[i] == 72 ||
				NLCDLandCoverData[i] == 73 || NLCDLandCoverData[i] == 74 || NLCDLandCoverData[i] == 81 || NLCDLandCoverData[i] == 82) {
				//PerviousCover_FullSkyView_with_SVeg_frac (frac) defined for each NLCD Class based on best estimates of sub-grid heterogeneity
				//Note: PerviousCover_FullSkyView_with_SVeg_frac defined here as 0.85 
				PerviousCover_FullSkyView_with_SVeg_frac = 0.85;
				//PermeablePavementCover (frac) is zero given NLCD Class and scenario
				PermeablePavementCover[i] = 0;
				//WaterCover (fraction) is zero given NLCD class is terrestrial
				WaterCover[i] = 0;
				//PerviousCover_FullSkyView_frac defined as area not within TotalCover (tree cover and impervious cover)
				PerviousCover_FullSkyView_frac = 1.0 - TotalCover[i];
				//SoilCover (frac) defined as PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac)
				SoilCover[i] = PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac);
				//ShortVegCover (frac) defined as PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac
				ShortVegCover[i] = PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac;
				//TreeOnImpervious (fraction) is zero given TotalCover < 100%
				TreeOnImpervious[i] = 0;
				//TreeOnPervious (fraction) is all TreeCoverData given TotalCover < 100%
				TreeOnPervious[i] = TreeCoverData[i];
				//ImperviousNoTree (fraction) is all impervious cover given TotalCover < 100%
				ImperviousNoTree[i] = ImperCoverData[i];
			}
			//Else If NLCD Class is woody wetlands type
			//Note: Class 90 = Woody wetlands then 
			else if (NLCDLandCoverData[i] == 90) {
				//PerviousCover_FullSkyView_with_SVeg_frac (frac) defined for each NLCD Class based on best estimates of sub-grid heterogeneity
				//Note: PerviousCover_FullSkyView_with_SVeg_frac defined here as 0.65 
				PerviousCover_FullSkyView_with_SVeg_frac = 0.65;
				//PermeablePavementCover (frac) is zero given NLCD Class and scenario
				PermeablePavementCover[i] = 0;
				//SoilCover (fraction) is zero given NLCD class is wetland
				SoilCover[i] = 0;
				//PerviousCover_FullSkyView_frac defined as area not within TotalCover (tree cover and impervious cover)
				PerviousCover_FullSkyView_frac = 1.0 - TotalCover[i];
				//WaterCover (frac) defined as PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac)
				//Note: If WaterCover <= 55%, then scenarios where IC is changed should avoid Tair_IC_0% > Tair_IC_2% 
				WaterCover[i] = PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac);
				//ShortVegCover (frac) defined as PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac
				ShortVegCover[i] = PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac;
				//TreeOnImpervious (fraction) is zero given TotalCover < 100%
				TreeOnImpervious[i] = 0;
				//TreeOnPervious (fraction) is all TreeCoverData given TotalCover < 100%
				TreeOnPervious[i] = TreeCoverData[i];
				//ImperviousNoTree (fraction) is all impervious cover given TotalCover < 100%
				ImperviousNoTree[i] = ImperCoverData[i];
			}
			//Else If NLCD Class is emergent wetlands type
			//Note: Class 95 = Emergent herbaceous wetlands then 
			else if (NLCDLandCoverData[i] == 95) {
				//PerviousCover_FullSkyView_with_SVeg_frac (frac) defined for each NLCD Class based on best estimates of sub-grid heterogeneity
				//Note: PerviousCover_FullSkyView_with_SVeg_frac defined here as 0.85 
				PerviousCover_FullSkyView_with_SVeg_frac = 0.85;
				//PermeablePavementCover (frac) is zero given NLCD Class and scenario
				PermeablePavementCover[i] = 0;
				//SoilCover (fraction) is zero given NLCD class is wetland
				SoilCover[i] = 0;
				//PerviousCover_FullSkyView_frac defined as area not within TotalCover (tree cover and impervious cover)
				PerviousCover_FullSkyView_frac = 1.0 - TotalCover[i];
				//WaterCover (frac) defined as PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac)
				//Note: If WaterCover <= 55%, then scenarios where IC is changed should avoid Tair_IC_0% > Tair_IC_2% 
				WaterCover[i] = PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac);
				//ShortVegCover (frac) defined as PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac
				ShortVegCover[i] = PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac;
				//TreeOnImpervious (fraction) is zero given TotalCover < 100%
				TreeOnImpervious[i] = 0;
				//TreeOnPervious (fraction) is all TreeCoverData given TotalCover < 100%
				TreeOnPervious[i] = TreeCoverData[i];
				//ImperviousNoTree (fraction) is all impervious cover given TotalCover < 100%
				ImperviousNoTree[i] = ImperCoverData[i];
			}
			//Else NLCD Class is some class not defined above and treat as NLCD Class 21 to 24
			else {
				//PerviousCover_FullSkyView_with_SVeg_frac (frac) defined for each NLCD Class based on best estimates of sub-grid heterogeneity
				//Note: PerviousCover_FullSkyView_with_SVeg_frac defined here as 0.5 
				PerviousCover_FullSkyView_with_SVeg_frac = 0.5;
				//PermeablePavementCover (frac) is zero given NLCD Class and scenario
				PermeablePavementCover[i] = 0;
				//WaterCover (fraction) is zero given NLCD class is presumed terrestrial
				WaterCover[i] = 0;
				//PerviousCover_FullSkyView_frac defined as area not within TotalCover (tree cover and impervious cover)
				PerviousCover_FullSkyView_frac = 1.0 - TotalCover[i];
				//SoilCover (frac) defined as PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac)
				SoilCover[i] = PerviousCover_FullSkyView_frac * (1 - PerviousCover_FullSkyView_with_SVeg_frac);
				//ShortVegCover (frac) defined as PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac
				ShortVegCover[i] = PerviousCover_FullSkyView_frac * PerviousCover_FullSkyView_with_SVeg_frac;
				//TreeOnImpervious (fraction) is zero given TotalCover < 100%
				TreeOnImpervious[i] = 0;
				//TreeOnPervious (fraction) is all TreeCoverData given TotalCover < 100%
				TreeOnPervious[i] = TreeCoverData[i];
				//ImperviousNoTree (fraction) is all impervious cover given TotalCover < 100%
				ImperviousNoTree[i] = ImperCoverData[i];
			}
		}

		//Check for small numbers, using Constant_1E_negative5 threshold used for land area fractions, if < 1E-5 then = 0.0
		if (ImperviousNoTree[i] < Constant_1E_negative5) { ImperviousNoTree[i] = 0.0; }
		if (TreeOnPervious[i] < Constant_1E_negative5) { TreeOnPervious[i] = 0.0; }
		if (TreeOnImpervious[i] < Constant_1E_negative5) { TreeOnImpervious[i] = 0.0; }
		if (WaterCover[i] < Constant_1E_negative5) { WaterCover[i] = 0.0; }
		if (SoilCover[i] < Constant_1E_negative5) { SoilCover[i] = 0.0; }
		if (ShortVegCover[i] < Constant_1E_negative5) { ShortVegCover[i] = 0.0; }
		if (PermeablePavementCover[i] < Constant_1E_negative5) { PermeablePavementCover[i] = 0.0; }
		
		//ImperviousNoTree (frac) = int((1000.0 * ImperviousNoTree[i]) + 0.5) / 1000.0
		//Note: RHS function divides by 1000 after taking integer of the term that takes product with 1000 and sums 0.5 to round up
		ImperviousNoTree[i] = int((1000.0 * ImperviousNoTree[i]) + 0.5) / 1000.0;
		TreeOnImpervious[i] = int((1000.0 * TreeOnImpervious[i]) + 0.5) / 1000.0;
		TreeOnPervious[i] = int((1000.0 * TreeOnPervious[i]) + 0.5) / 1000.0;
		SoilCover[i] = int((1000.0 * SoilCover[i]) + 0.5) / 1000.0;
		ShortVegCover[i] = int((1000.0 * ShortVegCover[i]) + 0.5) / 1000.0;
		PermeablePavementCover[i] = int((1000.0 * PermeablePavementCover[i]) + 0.5) / 1000.0;
		WaterCover[i] = int((1000.0 * WaterCover[i]) + 0.5) / 1000.0;

		//TotalCover_frac (frac) initialized to equal sum of ImperviousNoTree (frac), TreeOnImpervious (frac), TreeOnPervious (frac), SoilCover (frac), ShortVegCover (frac), PermeablePavementCover (frac), WaterCover (frac)
		double TotalCover_frac = ImperviousNoTree[i] + TreeOnImpervious[i] + TreeOnPervious[i] + SoilCover[i] + ShortVegCover[i] + PermeablePavementCover[i] + WaterCover[i];
		//TotalCover_Remainder_frac (frac) initialized to equal abs(1.0 - TotalCover_frac)
		//Note: Absolute function not likely needed as TotalCover_frac is presumed always less than 1
		double TotalCover_Remainder_frac = abs(1.0 - TotalCover_frac);
		
		//If TotalCover_Remainder_frac != 0.0 then add the remaining gap in a sequence that lastly updates tree cover, presumed correct
		//Note: Algorithm could be improved to search for smallest unit of area (frac) before setting anything to zero but ...
		//Note: ... Algorithm will never likely set anything to zero due to (1.0 - TotalCover_frac) naturally positive, without need for abs
		if (TotalCover_Remainder_frac != 0.0) {
			if (PermeablePavementCover[i] > 0.0) {
				//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - PermeablePavementCover[i])
				double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - PermeablePavementCover[i]);
				//PermeablePavementCover (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
				PermeablePavementCover[i] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
				//If PermeablePavementCover[i] < 0 then set to zero
				if (PermeablePavementCover[i] < 0) { PermeablePavementCover[i] = 0.0; }
			}
			else if (SoilCover[i] > 0.0) {
				//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - SoilCover[i])
				double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - SoilCover[i]);
				//SoilCover (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
				SoilCover[i] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
				//If SoilCover[i] < 0 then set to zero
				if (SoilCover[i] < 0) { SoilCover[i] = 0.0; }
			}
			else if (WaterCover[i] > 0.0) {
				//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - WaterCover[i])
				double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - WaterCover[i]);
				//WaterCover (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
				WaterCover[i] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
				//If WaterCover[i] < 0 then set to zero
				if (WaterCover[i] < 0) { WaterCover[i] = 0.0; }
			}
			else if (ShortVegCover[i] > 0.0) {
				//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - ShortVegCover[i])
				double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - ShortVegCover[i]);
				//ShortVegCover (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
				ShortVegCover[i] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
				//If ShortVegCover[i] < 0 then set to zero
				if (ShortVegCover[i] < 0) { ShortVegCover[i] = 0.0; }
			}
			else if (ImperviousNoTree[i] > 0.0) {
				//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - ImperviousNoTree[i])
				double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - ImperviousNoTree[i]);
				//ImperviousNoTree (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
				ImperviousNoTree[i] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
				//If ImperviousNoTree[i] < 0 then set to zero
				if (ImperviousNoTree[i] < 0) { ImperviousNoTree[i] = 0.0; }
			}
			else if (TreeOnImpervious[i] > 0.0) {
				//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - TreeOnImpervious[i])
				double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - TreeOnImpervious[i]);
				//TreeOnImpervious (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
				TreeOnImpervious[i] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
				//If TreeOnImpervious[i] < 0 then set to zero
				if (TreeOnImpervious[i] < 0) { TreeOnImpervious[i] = 0.0; }
			}
			else if (TreeOnPervious[i] > 0.0) {
				//TotalCover_Remainder_wo_frac (frac) initialized to equal 1.0 - (TotalCover_frac - TreeOnPervious[i])
				double TotalCover_Remainder_wo_frac = 1.0 - (TotalCover_frac - TreeOnPervious[i]);
				//TreeOnPervious (frac) = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0 (frac)
				TreeOnPervious[i] = int((1000 * TotalCover_Remainder_wo_frac) + 0.5) / 1000.0;
				//If TreeOnPervious[i] < 0 then set to zero
				if (TreeOnPervious[i] < 0) { TreeOnPervious[i] = 0.0; }
			}
		}
	}
}

//readRunoff_ObservedFile function reads in Runoff_Observed_for_Calibration.csv time series data for HydroPlus models
//Note: i-Tree HydroPlus Runoff_Observed_for_Calibration.csv.csv header row: YYYYMMDD,HH:MM:SS,Discharge(m) which is basin total runoff depth for time step
bool Inputs::readRunoff_ObservedFile()
{
	//readWeather created as part of fstream, defined as the Directory_Input_CLArg and file name for Weather.csv
	fstream readRunoff_Observed(Directory_Input_CLArg + "Runoff_Observed_for_Calibration.csv");

	//If Runoff_Observed_for_Calibration.csv input data file does not exist, then alert user and abort
	if (!readRunoff_Observed) {
		cout << "Warning: Runoff_Observed_for_Calibration.csv could not be opened." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The Runoff_Observed_for_Calibration.csv file is required when HydroPlusConfig.xml element Flag_CompareToObservedRunoff > 0." << endl;
		cout << "Correction: Provide Runoff_Observed_for_Calibration.csv in the input folder, or set HydroPlusConfig.xml Flag_CompareToObservedRunoff = 0." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	//Initialize variables
	Runoff_Observed_m.clear();
	string startline;
	string temp;
	string HrMinSec;
	//initialize Date_YYYYMMDD as -1, other variables to -99999
	double Date_YYYYMMDD = -1.0;
	double tempRunoff_Observed_m = -99999;
	double date_InputDataRow_HH;
	double date_InputDataRow_GDH;
	int Date_JulianDay;
	int Counter_inputRow = 0;

	//Theory on getline function of fstream library; takes characters from 1st item, stores them in 2nd item, until delimitation character found
	//getline reads in header from readRunoff_Observed, stores in temp
	getline(readRunoff_Observed, temp);

	//while readRunoff_Observed uses fstream good function to return true unless error flag is thrown
	while (readRunoff_Observed.good())
	{
		//getline reads 1st column characters of readRunoff_Observed before reaching comma, assigns to temp
		//Note: Reads Gregorian Date YYYYMMDD 
		getline(readRunoff_Observed, temp, ',');
		//avoid reading the empty line at the end of the input file
		if (temp[0] != '\n') {
			//advance Counter_inputRow 
			Counter_inputRow = Counter_inputRow + 1;
			//istringstream stores an instance of the string temp in the new ss string
			istringstream ss(temp);
			//ss string is passed into Date_YYYYMMDD as double
			ss >> Date_YYYYMMDD;

			//If Counter_inputRow equals 1 then at start of file then check if Weather.csv first date is not greater than HydroPlusConfig.xml StartDate_YYYYMMDD
			if (Counter_inputRow == 1) {
				//Check if year matches HydroPlusConfig.xml StartDate_YYYYMMDD Year (and below if Weather.csv contains full YYYYMMDD called for in HydroPlusConfig.xml)
				//HydroPlusConfig_StartDay_str_YYYYMMDD is string form of HydroPlusConfig.xml input StartDate_YYYYMMDD
				string HydroPlusConfig_StartDay_str_YYYYMMDD = to_string(SimulationNumericalParams["StartDate_YYYYMMDD"]);
				//Runoff_ObservedInput_StartDay_str_YYYYMMDD  is string form of Weather.csv input date column
				string Runoff_ObservedInput_StartDay_str_YYYYMMDD = to_string(Date_YYYYMMDD);
				//Runoff_ObservedInput_StartDay_YYYYMMDD defined as integer of Date_YYYYMMDD input
				int Runoff_ObservedInput_StartDay_YYYYMMDD = Date_YYYYMMDD;
				//HydroPlusConfig_StartDay_YYYYMMDD is integer form of substring from HydroPlusConfig_StartDay_str_YYYYMMDD, taking first 8 elements
				int HydroPlusConfig_StartDay_YYYYMMDD = atoi(HydroPlusConfig_StartDay_str_YYYYMMDD.substr(0, 8).c_str());

				//if HydroPlusConfig_StartDay_YYYYMMDD > HydroPlusConfig_StartDay_YYYYMMDD then error with inputs or expectations
				if (Runoff_ObservedInput_StartDay_YYYYMMDD > HydroPlusConfig_StartDay_YYYYMMDD) {
					cout << "Warning: The Runoff_Observed_for_Calibration.csv 1st column for date, YYYYMMDD, has a starting date of " << Runoff_ObservedInput_StartDay_YYYYMMDD << " ..." << endl;
					cout << "... which is later than the HydroPlusConfig.xml parameter StartDate_YYYYMMDD, set to " << HydroPlusConfig_StartDay_YYYYMMDD << "." << endl;
					cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
					cout << "Explanation: The HydroPlus model needs the Runoff_Observed_for_Calibration.csv to contain the date given in HydroPlusConfig.xml." << endl;
					cout << "Correction: Either generate new Runoff_Observed_for_Calibration.csv or change the StartDate_YYYYMMDD in the HydroPlusConfig.xml file." << endl;
					//Call abort function, which ends the HydroPlus.exe simulation
					abort();
				}
			}

			//getline reads column characters of readRunoff_Observed before reaching comma, assigns to temp
			//Note: Reads Time HH:MM:SS 
			getline(readRunoff_Observed, temp, ',');
			//istringstream stores an instance of the string temp in the new ssTime string
			istringstream ssTime(temp);
			//ssTime string is passed into HrMinSec as double
			ssTime >> HrMinSec;

			//Algorithm to extract hour integer from section of HrMinSec string, used to refine location where input rows are read
			//date_InputDataRow_HH integer from string with atoi function, extracting 2 digits of temp w substr function; c_str() points to temp
			date_InputDataRow_HH = atoi(HrMinSec.substr(0, 2).c_str());
			//date_InputDataRow_GDH contains Gregorian Date and Hour, extending 8 to 10 digits by multiplication with 100, adding hour
			date_InputDataRow_GDH = (Date_YYYYMMDD * 100) + date_InputDataRow_HH;

			//If date_InputDataRow_GDH is between SimulationDateStop_GDH and SimulationDateStart_GDH then proceed to read input
			if (date_InputDataRow_GDH <= SimulationNumericalParams["SimulationDateStop_GDH"] &&
				date_InputDataRow_GDH >= SimulationNumericalParams["SimulationDateStart_GDH"]) {

				//Reading air observed runoff depth input (m)
				//Linebreak \n is read to signify end of row
				getline(readRunoff_Observed, temp, '\n');
				istringstream ssRunoff_Observed(temp);
				ssRunoff_Observed >> tempRunoff_Observed_m;
				Runoff_Observed_m.push_back(tempRunoff_Observed_m);
			}
			//Else If date_InputDataRow_GDH is not between SimulationDateStop_GDH and SimulationDateStart_GDH then advance to next date
			else {
				getline(readRunoff_Observed, temp);
			}
		}
	}
	//readRunoff_Observed is closed
	readRunoff_Observed.close();
	return true;
}

//readRiverTimeSeriesFile function reads time series input data for i-Tree CoolRiver model
bool Inputs::readRiverTimeSeriesFile(double startDate, double endDate)
{
	fstream readRiverTimeSeries(Directory_Input_CLArg + "RiverTimeSeries.csv");
	if (!readRiverTimeSeries) {
		cout << "Inputs: RiverTimeSeries.csv could not be opened. Please check and compare file with expected input from a test case before starting new simulation." << endl;
		return false;
	}
	string temp;
	double date = -1.0;
	string HrMinSec;
	double tempSedi = -99999;
	double tempHumid = -99999;
	double tempCloudy = -99999;

	string startline;
	getline(readRiverTimeSeries, temp);

	while (readRiverTimeSeries.good())
	{
		getline(readRiverTimeSeries, temp, ',');
		istringstream ss(temp);
		ss >> date;
		if (date <= endDate && date >= startDate) {
			if (readRiverTimeSeries.eof()) {
				break;
			}
			DatesRiverTime.push_back(static_cast<int>(date));
			getline(readRiverTimeSeries, temp, ',');
			istringstream ssTime(temp);
			ssTime >> HrMinSec;
			RiverTimeSeries.push_back(HrMinSec);

			getline(readRiverTimeSeries, temp, ',');
			istringstream ssSedimentTC(temp);
			ssSedimentTC >> tempSedi;
			Sediment_T_C.push_back(tempSedi);

			getline(readRiverTimeSeries, temp, ',');
			istringstream ssCloudinessF(temp);
			ssCloudinessF >> tempCloudy;
			Cloudiness_F.push_back(tempCloudy);

			getline(readRiverTimeSeries, temp, '\n');
			istringstream ssHumidityP(temp);
			ssHumidityP >> tempHumid;
			Humidity_Per.push_back(tempHumid);
		}
		else {
			getline(readRiverTimeSeries, temp);
		}
	}
	readRiverTimeSeries.close();
	return true;
}

//readRiverStationSeriesFile function reads spatial series input data for i-Tree CoolRiver model
bool Inputs::readRiverStationSeriesFile(string& Directory_Input_CLArg, double NODATA_code)
{
	fstream readRiverStationSeries(Directory_Input_CLArg + "RiverStationSeries.csv");
	if (!readRiverStationSeries) {
		cout << "Inputs: RiverStationSeries.csv could not be opened. Please check and compare file with expected input from a test case before starting new simulation." << endl;
		return false;
	}
	string temp;
	string tempType;
	double stationNumber = -1.0;
	double stationDistance = -1.0;
	double tempDepthOfMesu = -99999;
	double tempHorizontalBedCon = -99999;
	double tempBedParticle = -99999;
	double tempEmbeddedness = -99999;
	double tempArea = -99999;
	double tempWidth = -99999;
	double tempDepth = -99999;
	double tempSlope = -99999;
	double tempRow = -99999;
	double tempColumn = -99999;
	double tempLongitude = -99999;
	double tempLatitude = -99999;
	double tempssZRiver = -99999;

	double tempEastBankH = -99999;
	double tempEastTreeH = -99999;
	double tempEastBuildingH = -99999;
	double tempEastBankDist = -99999;
	double tempEastCanDist = -99999;
	double tempEastBuildingDist = -99999;
	double tempEastBuffer = -99999;
	double tempWestBankH = -99999;
	double tempWestTreeH = -99999;
	double tempWestBuildingH = -99999;
	double tempWestBankDist = -99999;
	double tempWestCanDist = -99999;
	double tempWestBuildingDist = -99999;
	double tempWestBuffer = -99999;
	double tempElevation = -99999;
	double tempStreamAzi = -99999;
	double tempShadeFactor = -99999;
	double tempViewToSky = -99999;
	double stationDistanceMorphology = -1.0;
	double stationDistanceShading = -1.0;
	double stationDistanceShadingPercent = -1.0;

	string startline;
	getline(readRiverStationSeries, temp);

	while (readRiverStationSeries.good())
	{
		if (readRiverStationSeries.eof()) {
			break;
		}
		getline(readRiverStationSeries, temp, ',');
		if (temp == "") {
			break;
		}
		istringstream ss(temp);
		ss >> stationNumber;
		StationNumber.push_back(static_cast<int>(stationNumber));
		getline(readRiverStationSeries, temp, ',');
		istringstream ssDistance(temp);
		ssDistance >> stationDistance;
		StationDistanceBedData.push_back(static_cast<int>(stationDistance));

		getline(readRiverStationSeries, temp, ',');
		istringstream ssDepthOfMes(temp);
		ssDepthOfMes >> tempDepthOfMesu;
		DepthMesurments.push_back(tempDepthOfMesu);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssType(temp);
		ssType >> tempType;
		TypeOfRiver.push_back(tempType);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssHorizontalBedCon(temp);
		ssHorizontalBedCon >> tempHorizontalBedCon;
		HorizontalBedConnectivity.push_back(tempHorizontalBedCon);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssBedParticle(temp);
		ssBedParticle >> tempBedParticle;
		BedParticleSize.push_back(tempBedParticle);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssEmbeddedness(temp);
		ssEmbeddedness >> tempEmbeddedness;
		Embeddedness.push_back(tempEmbeddedness);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssstationDistanceMorphology(temp);
		ssstationDistanceMorphology >> stationDistanceMorphology;
		StationDistanceMorphology.push_back(static_cast<int>(stationDistanceMorphology));

		getline(readRiverStationSeries, temp, ',');
		istringstream ssArea(temp);
		ssArea >> tempArea;
		AreaR_m2.push_back(tempArea);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssWidth(temp);
		ssWidth >> tempWidth;
		WidthR_m.push_back(tempWidth);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssDepth(temp);
		ssDepth >> tempDepth;
		DepthR_m.push_back(tempDepth);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssSlope(temp);
		ssSlope >> tempSlope;
		SlopeR_f.push_back(tempSlope);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssRow(temp);
		ssRow >> tempRow;
		RowRiver.push_back(static_cast<int>(tempRow));

		getline(readRiverStationSeries, temp, ',');
		istringstream sscolumn(temp);
		sscolumn >> tempColumn;
		ColRiver.push_back(static_cast<int>(tempColumn));

		getline(readRiverStationSeries, temp, ',');
		istringstream ssLongitude(temp);
		ssLongitude >> tempLongitude;
		LongitudeR_frac.push_back(tempLongitude);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssLatitude(temp);
		ssLatitude >> tempLatitude;
		LatitudeR_frac.push_back(tempLatitude);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssZRiver(temp);
		ssZRiver >> tempssZRiver;
		ZRiver_m.push_back(tempssZRiver);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssstationDistanceShading(temp);
		ssstationDistanceShading >> stationDistanceShading;
		StationDistanceShading.push_back(static_cast<int>(stationDistanceShading));

		getline(readRiverStationSeries, temp, ',');
		istringstream ssEastBankH(temp);
		ssEastBankH >> tempEastBankH;
		EastBankHR_m.push_back(tempEastBankH);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssEastTreeH(temp);
		ssEastTreeH >> tempEastTreeH;
		EastTreeHR_m.push_back(tempEastTreeH);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssEastBuildingH(temp);
		ssEastBuildingH >> tempEastBuildingH;
		EastBuildingHR_m.push_back(tempEastBuildingH);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssEastBankDist(temp);
		ssEastBankDist >> tempEastBankDist;
		EastBankDistR_m.push_back(tempEastBankDist);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssEastCanDist(temp);
		ssEastCanDist >> tempEastCanDist;
		EastCanDistR_m.push_back(tempEastCanDist);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssEastBuildingDist(temp);
		ssEastBuildingDist >> tempEastBuildingDist;
		EastBuildingDistR_m.push_back(tempEastBuildingDist);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssEastBuffer(temp);
		ssEastBuffer >> tempEastBuffer;
		EastBufferWR_m.push_back(tempEastBuffer);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssWestBankH(temp);
		ssWestBankH >> tempWestBankH;
		WestBankHR_m.push_back(tempWestBankH);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssWestTreeH(temp);
		ssWestTreeH >> tempWestTreeH;
		WestTreeHR_m.push_back(tempWestTreeH);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssWestBuildingH(temp);
		ssWestBuildingH >> tempWestBuildingH;
		WestBuildingHR_m.push_back(tempWestBuildingH);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssWestBankDist(temp);
		ssWestBankDist >> tempWestBankDist;
		WestBankDistR_m.push_back(tempWestBankDist);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssWestCanDist(temp);
		ssWestCanDist >> tempWestCanDist;
		WestCanDistR_m.push_back(tempWestCanDist);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssWestBuildingDist(temp);
		ssWestBuildingDist >> tempWestBuildingDist;
		WestBuildingDistR_m.push_back(tempWestBuildingDist);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssWestBuffer(temp);
		ssWestBuffer >> tempWestBuffer;
		WestBufferWR_m.push_back(tempWestBuffer);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssElevation(temp);
		ssElevation >> tempElevation;
		ElevationR_m.push_back(tempElevation);
		getline(readRiverStationSeries, temp, ',');
		istringstream ssStreamAzi(temp);
		ssStreamAzi >> tempStreamAzi;
		StreamAzimuthR_deg.push_back(tempStreamAzi);

		getline(readRiverStationSeries, temp, ',');
		istringstream ssstationDistanceShadingPercent(temp);
		ssstationDistanceShadingPercent >> stationDistanceShadingPercent;
		StationDistanceShadingPercent.push_back(static_cast<int>(stationDistanceShadingPercent));

		getline(readRiverStationSeries, temp, ',');
		istringstream ssShadeFactor(temp);
		ssShadeFactor >> tempShadeFactor;
		ShadeFactorR_f.push_back(tempShadeFactor);
		//Linebreak \n is read to signify end of row
		getline(readRiverStationSeries, temp, '\n');
		istringstream ssViewToSky(temp);
		ssViewToSky >> tempViewToSky;
		ViewToSkyR_f.push_back(tempViewToSky);
		if (readRiverStationSeries.eof()) {
			break;
		}
	}

	readRiverStationSeries.close();
	return true;
}

//readRiverGWTimeSeriesFile function reads time series input data for i-Tree CoolRiver model
bool Inputs::readRiverGWTimeSeriesFile(double startDate, double endDate)
{
	fstream readRiverGWTimeSeries(Directory_Input_CLArg + "RiverGWTimeSeries.csv");
	if (!readRiverGWTimeSeries)	{
		cout << "Inputs: RiverGWTimeSeries.csv could not be opened. Please check and compare file with expected input from a test case before starting new simulation." << endl;
		return false;
	}
	string temp;
	double date = -1.0;
	string HrMinSec;
	double tempGWQcms = -99999;
	double tempTCcms = -99999;
	int indexValueForGW = -99999;
	int indexValueForTC = -99999;

	string delimiter = ",";
	vector<double> headerValues;
	getline(readRiverGWTimeSeries, temp);
	int size = int(temp.size());
	size_t pos = 0;
	int counter = 0;
	string token;
	bool flag = true;
	int maxNumberOfVectors = 10;
	while ((pos = temp.find(delimiter)) != string::npos)
	{
		token = temp.substr(0, pos);
		int pos1 = int(token.find("#"));
		if (pos1 != -1) {
			string str3 = token.substr(pos1 + 1);
			int val = stoi(str3);
			headerValues.push_back(val);
			counter++;
		}
		temp.erase(0, pos + delimiter.length());
	}
	int pos1 = int(token.find("#")); // position of "live" in str
	if (pos1 != -1)	{
		string str3 = token.substr(pos1 + 1);
		int val = stoi(str3);
		headerValues.push_back(val);
		counter++;
	}
	while (readRiverGWTimeSeries.good())
	{
		getline(readRiverGWTimeSeries, temp, ',');
		istringstream ss(temp);
		ss >> date;
		if (date <= endDate && date >= startDate) {
			tempGWQcms = -99999;
			tempTCcms = -99999;
			if (readRiverGWTimeSeries.eof()) {
				break;
			}
			getline(readRiverGWTimeSeries, temp, ',');
			for (int i = 0; i < counter;) {
				getline(readRiverGWTimeSeries, temp, ',');
				istringstream ssGWQcms(temp);
				ssGWQcms >> tempGWQcms;
				indexValueForGW = int(headerValues.at(i));
				map<int, vector<double>>::iterator it = GW_Q_cms1.find(indexValueForGW);
				vector<double> tempVector;
				if (it == GW_Q_cms1.end()) {
					tempVector.push_back(tempGWQcms);
					GW_Q_cms1.insert(pair<int, vector<double>>(indexValueForGW, tempVector));
				}
				else {
					GW_Q_cms1[indexValueForGW].push_back(tempGWQcms);
				}
				if (i != counter - 2) {
					getline(readRiverGWTimeSeries, temp, ',');
				}
				else {
					//Linebreak \n is read to signify end of row
					getline(readRiverGWTimeSeries, temp, '\n');
				}
				i = i + 1;
				istringstream ssTCcms(temp);
				ssTCcms >> tempTCcms;
				indexValueForTC = int(headerValues.at(i));

				map<int, vector<double>>::iterator it1 = GW_T_C1.find(indexValueForTC);
				vector<double> tempVector1;
				if (it1 == GW_T_C1.end()) {
					tempVector1.push_back(tempTCcms);
					GW_T_C1.insert(pair<int, vector<double>>(indexValueForTC, tempVector1));
				}
				else {
					GW_T_C1[indexValueForTC].push_back(tempTCcms);
				}
				i = i + 1;
			}
		}
		else {
			getline(readRiverGWTimeSeries, temp);
		}
	}
	readRiverGWTimeSeries.close();
	return true;
}

//readRiverSWTimeSeriesFile function reads time series input data for i-Tree CoolRiver model
//Consider Refactor to replace counter not use startDate or endDate in while (readRiverSWTimeSeries.good(), as done in readRiverGWTimeSeriesFile and readRiverTimeSeriesFile
bool Inputs::readRiverSWTimeSeriesFile(double startDate, double endDate)
{
	fstream readRiverSWTimeSeries(Directory_Input_CLArg + "RiverSWTimeSeries.csv");
	if (!readRiverSWTimeSeries) {
		cout << "Inputs: RiverSWTimeSeries.csv could not be opened. Please check and compare file with expected input from a test case before starting new simulation." << endl;
		return false;
	}
	string temp;
	double date = -1.0;
	string HrMinSec;
	double tempGWQcms = -99999;
	double tempTCcms = -99999;
	int indexValueForGW = -99999;
	int indexValueForTC = -99999;

	string delimiter = ",";
	vector<double> headerValues;

	getline(readRiverSWTimeSeries, temp);
	size_t pos = 0;
	int counter = 0;
	string token;
	int maxNumberOfVectors = 10;
	
	while ((pos = temp.find(delimiter)) != string::npos)
	{
		token = temp.substr(0, pos);
		int pos1 = int(token.find("#"));
		if (pos1 != -1)	{
			string str3 = token.substr(pos1 + 1);
			int val = stoi(str3);
			if (val > maxNumberOfVectors) {
				maxNumberOfVectors = val;
			}
			headerValues.push_back(val);
			counter++;
		}
		temp.erase(0, pos + delimiter.length());
	}
	int pos1 = int(token.find("#")); // position of "live" in str
	if (pos1 != -1) {
		string str3 = token.substr(pos1 + 1);
		int val = stoi(str3);
		if (val > maxNumberOfVectors) {
			maxNumberOfVectors = val;
		}
		headerValues.push_back(val);
		counter++;
	}
	
	while (readRiverSWTimeSeries.good())
	{
		getline(readRiverSWTimeSeries, temp, ',');
		istringstream ss(temp);
		ss >> date;
	
		tempGWQcms = -99999;
		tempTCcms = -99999;
		
		if (readRiverSWTimeSeries.eof()) {
			break;
		}
		
		getline(readRiverSWTimeSeries, temp, ',');
		for (int i = 0; i < counter;) {
			getline(readRiverSWTimeSeries, temp, ',');
			istringstream ssGWQcms(temp);
			ssGWQcms >> tempGWQcms;
			indexValueForGW = int(headerValues.at(i));
			map<int, vector<double>>::iterator it = SW_Q_cms1.find(indexValueForGW);
			vector<double> tempVector;
			if (it == SW_Q_cms1.end()) {
				tempVector.push_back(tempGWQcms);
				SW_Q_cms1.insert(pair<int, vector<double>>(indexValueForGW, tempVector));
			}
			else {
				SW_Q_cms1[indexValueForGW].push_back(tempGWQcms);
			}
			if (i != counter - 2) {
				getline(readRiverSWTimeSeries, temp, ',');
			}
			else {
				getline(readRiverSWTimeSeries, temp, '\n');
			}
			i = i + 1;
			istringstream ssTCcms(temp);
			ssTCcms >> tempTCcms;
			indexValueForTC = int(headerValues.at(i));

			map<int, vector<double>>::iterator it1 = SW_T_C1.find(indexValueForTC);
			vector<double> tempVector1;
			if (it1 == SW_T_C1.end()) {
				tempVector1.push_back(tempTCcms);
				SW_T_C1.insert(pair<int, vector<double>>(indexValueForTC, tempVector1));
			}
			else {
				SW_T_C1[indexValueForTC].push_back(tempTCcms);
			}
			i = i + 1;
		}
	}
	readRiverSWTimeSeries.close();
	return true;
}

//readHecRasFile function reads HEC-RAS output as spatial series input data for i-Tree CoolRiver model
bool Inputs::readHecRasFile()
{
	fstream readHecRas(Directory_Input_CLArg + "HecRasData.csv");
	if (!readHecRas.good()) {
		cout << "Inputs: HecRasData.csv could not be opened. Please check and compare file with expected input from a test case before starting new simulation." << endl;
		return false;
	}
	//cout << "Reading HecRasData.csv file\n" << endl;

	string temp;
	double tempDummy = -1.0;

	double temp_dist = -99999;
	double temp_totalQ = -99999;
	double temp_chElev = -99999;
	double temp_waterSurElev = -99999;
	double temp_velChnl = -99999;
	double temp_xArea = -99999;
	double temp_topWidth = -99999;
	double temp_waterSurSlope = -99999;
	double temp_wettedPerimeter = -99999;

	// getting the header line and the blank line out of the way
	getline(readHecRas, temp);

	// reading in a CSV
	while (readHecRas.good())
	{
		getline(readHecRas, temp, ',');
		getline(readHecRas, temp, ',');
		istringstream temp1(temp);
		temp1 >> temp_dist;
		downStreamDist.push_back(temp_dist);
		getline(readHecRas, temp, ',');
		istringstream temp2(temp);
		temp2 >> temp_totalQ;
		totalQ.push_back(temp_totalQ);
		getline(readHecRas, temp, ',');
		istringstream temp3(temp);
		temp3 >> temp_chElev;
		chElev.push_back(temp_chElev);
		getline(readHecRas, temp, ',');
		istringstream temp4(temp);
		temp4 >> temp_waterSurElev;
		waterSurElev.push_back(temp_waterSurElev);
		getline(readHecRas, temp, ',');
		istringstream temp5(temp);
		temp5 >> temp_velChnl;
		velChnl.push_back(temp_velChnl);
		getline(readHecRas, temp, ',');
		istringstream temp6(temp);
		temp6 >> temp_xArea;
		xArea.push_back(temp_xArea);
		getline(readHecRas, temp, ',');
		istringstream temp7(temp);
		temp7 >> temp_topWidth;
		topWidth.push_back(temp_topWidth);
		getline(readHecRas, temp, ',');
		istringstream temp8(temp);
		temp8 >> temp_waterSurSlope;
		waterSurSlope.push_back(temp_waterSurSlope);
		//Linebreak \n is read to signify end of row
		getline(readHecRas, temp, '\n');
		istringstream temp9(temp);
		temp9 >> temp_wettedPerimeter;
		wettedPerimeter.push_back(temp_wettedPerimeter);
	}
	totalCount = int(downStreamDist.size());
	readHecRas.close();
}

//readDischargeData function used by Model = ECDynamic to obtain discharge data
bool Inputs::readDischargeData(string& Directory_Input_CLArg)
{
	//If Flag_Runoff_Type is OBS (observed discharge)
	if (SimulationStringParams["Flag_Runoff_Type"] == "OBS") {
		//Read observed daily discharge data obtained from USGS gages
		ifstream readOBSDischarge(Directory_Input_CLArg + "Runoff_Observed.csv");
		//if unable to read observed discharge data
		if (!readOBSDischarge) {
			//In EC dynamic model, it will calculate EC and EMC database without running Buffer simulation if not providing LC and Discharge.
			if (SimulationStringParams["Model_Selection"] != "ECDynamic") {
				cout << "Warning: Runoff_Observed.csv could not be opened." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: When the Runoff_Observed.csv file is missing there is no gage daily mean discharge data to drive the simulation." << endl;
				cout << "Correction: Create a Runoff_Observed.csv file, either manually or perhaps using the version in the director ..." << endl;
				cout << "Correction: ... \HydroPlus\TestingFilesAndScript\TestCases\StatDynBuffer\OnondagaLake\input\Runoff_Observed.csv." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}
			//if Model_Selection == ECDynamic
			else {
				cout << "Warning:Runoff_Observed.csv could not be opened." << endl;
				cout << "Explanation: When the Runoff_Observed.csv file is missing there is no gage daily mean discharge data to drive the simulation." << endl;
				cout << "Correction: Create a Runoff_Observed.csv file, either manually or perhaps using the version in the director ..." << endl;
				cout << "Correction: ... \HydroPlus\TestingFilesAndScript\TestCases\StatDynBuffer\OnondagaLake\input\Runoff_Observed.csv." << endl;
				cout << "The model will generate the local EC database without executing the entire model. " << endl;
				//false reading discharge will trigger the generation of EC database without executing the entire model; don't remove this line
				return false;
			}

		}
		else {
			string header, agency_cd, site_no, tempDischarge, tempmonth, tempday, tempyear;
			string temp_line;

			//read the fist line of the database: the header
			getline(readOBSDischarge, header);
			//Continue reading while data exist
			while (getline(readOBSDischarge, temp_line)) {
				istringstream ss(temp_line);
				getline(ss, agency_cd, ',');
				getline(ss, site_no, ',');
				getline(ss, tempmonth, ',');
				getline(ss, tempday, ',');
				getline(ss, tempyear, ',');
				getline(ss, tempDischarge, ',');

				//Remove the NA data -9999, and set up the Discharge threshold, for this case, we consider all events, th=0.
				if (int(stod(tempDischarge)) != -9999) {
					DischargeRecord.push_back(stod(tempDischarge));
					year.push_back(int(stod(tempyear)));
					month.push_back(int(stod(tempmonth)));
					day.push_back(int(stod(tempday)));
				}
			}
			return true;
		}

	}

	//If Flag_Runoff_Type is MOD (predicted discharge)
	if (SimulationStringParams["Flag_Runoff_Type"] == "MOD") {
		//Read predicted discharge botained from HydroPlus simulation
		ifstream readMODDischarge(Directory_Input_CLArg + "Runoff_Predicted.csv");
		//if unable to read predicted discharge data
		if (!readMODDischarge) {
			//In EC dynamic model, it will calculate EC and EMC database without running Buffer simulation if not providing LC and Discharge.
			if (SimulationStringParams["Model_Selection"] != "ECDynamic") {
				cout << "Warning: Runoff_Predicted.csv could not be opened." << endl;
				cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
				cout << "Explanation: When the Runoff_Predicted.csv file is missing there is no predicted daily mean discharge data to drive the simulation." << endl;
				cout << "Correction: Generate a Runoff_Predicted_for_Calibration.csv by running HydroPlus StatisticalHydro and turn Flag_CompareToObservedRunoff to 1..." << endl;
				cout << "Correction: ... then copy this file to your input folder and rename the file as Runoff_Predicted.csv." << endl;
				//Call abort function, which ends the HydroPlus.exe simulation
				abort();
			}
			//if Model_Selection == ECDynamic
			else {
				cout << "Warning:Runoff_Predicted.csv could not be opened." << endl;
				cout << "Explanation: When the Runoff_Predicted.csv file is missing there is no predicted daily mean discharge data to drive the simulation." << endl;
				cout << "Correction: Generate a Runoff_Predicted_for_Calibration.csv by running HydroPlus StatisticalHydro and turn Flag_CompareToObservedRunoff to 1..." << endl;
				cout << "Correction: ... then copy this file to your input folder and rename the file as Runoff_Predicted.csv." << endl;
				cout << "The model will generate the local EC database without executing the entire model. " << endl;
				//false reading discharge will trigger the generation of EC database without executing the entire model; don't remove this line
				return false;
			}
		}
		else {
			string header, YYYYMMDD, HHMMSS, tempDischarge;
			string temp_line;

			//read the fist line of the database: the header
			getline(readMODDischarge, header);
			//Continue reading while data exist
			while (getline(readMODDischarge, temp_line)) {
				istringstream ss(temp_line);
				getline(ss, YYYYMMDD, ',');
				getline(ss, HHMMSS, ',');
				getline(ss, tempDischarge, ',');

				//Remove the NA data -9999, and set up the Discharge threshold, for this case, we consider all events, th=0.
				if (int(stod(tempDischarge)) != -9999) {
					DischargeRecord.push_back(stod(tempDischarge));
					//read the substring YYYY as int year
					string yearString = YYYYMMDD.substr(0, 4);
					string monthString = YYYYMMDD.substr(4, 2);
					string dayString = YYYYMMDD.substr(6, 2);

					// Convert strings to integers
					year.push_back(stoi(yearString));
					month.push_back(stoi(monthString));
					day.push_back(stoi(dayString));
				}
			}
			return true;
		}
	}
}

//readPollutionData function reads the Pollutants.csv file and is used in Model = StatisticalHydro, including GI treatment in Removal_StormwaterPollutants.cpp
//Note Pollutants.csv has 1st row any text, 2nd row pollutant names, 3rd row median concentrations (mg/L), 4th row mean concentrations(mg/L)
//References:
//Smullen, J. T., Shallcross,A. L., and Cave, K. A. 1999. Updating the U.S. nationwide urban runoff quality database. Water Science Technology 39(12), 9-16.  
//Stephan, E. A., & Endreny, T. A. (2016). Weighting Nitrogen and Phosphorus Pixel Pollutant Loads to Represent Runoff and Buffering Likelihoods. JAWRA Journal of the American Water Resources Association, 52(2), 336-349. doi:doi:10.1111/1752-1688.12390
bool Inputs::readPollutionData(string& Directory_Input_CLArg)
{
	string headerline;
	string LineOfInput;
	string PollutantConcentrationMedian_mg_p_L, PollutantConcentrationMean_mg_p_L;
	string RowName_string;
	string PollutantName_string;

	//Pollutants.csv has concentration values in mg/L
	ifstream readPollutant(Directory_Input_CLArg + "Pollutants.csv");

	//If not able to open Pollutants.csv then it was not likely provided, and use default values taken from Smullen et al. (1999)
	if (!readPollutant) {
		//Command Prompt Notification to user that default pollutant event mean concentration values are being used
		cout << "Pollutant input concentrations set to default. If other values desired, then put Pollutants.csv in input folder." << endl;

		//Assign default values for pollutant event mean concentration values from Smullen et al. (1999), using a pooled USGS and NURP average
		//Note: Optional concentrations are provided in Stephan and Endreny (2016), for a variety of land cover types, some units in ug/L
		//Pollutant_Name are TSS, BOD, COD, TP, SolP, TKN, NO2_3, Cu, Pb, ZN
		Pollutant_Name = { "TSS","BOD", "COD", "TP", "SolP", "TKN", "NO2_3", "Cu", "Pb", "Zn" };
		//Pollutant_ConcentrationMedian_mg_p_L are median concentrations in units of mg/L
		Pollutant_ConcentrationMedian_mg_p_L = { 54.5, 11.5, 44.7, 0.259, 0.103, 1.47, 0.533, 0.0111, 0.0507, 0.129 };
		//Pollutant_ConcentrationMean_mg_p_L are mean concentrations in units of mg/L, as reported in the HydroPlus_TechnicalManual.pdf
		Pollutant_ConcentrationMean_mg_p_L = { 78.4, 14.1, 52.8, 0.315, 0.129, 1.73, 0.658, 0.0135, 0.0675, 0.162 };
	}

	 // Read the header lines
	getline(readPollutant, headerline);
	getline(readPollutant, headerline);

	//Read the pollutant names
	istringstream PollutantNameList_string(headerline);
	//Skip the first field
	getline(PollutantNameList_string, RowName_string, ',');
	//while PollutantNameList_string has remaining items, send them to PollutantName_string
	while (getline(PollutantNameList_string, PollutantName_string, ',')) {
		//Pollutant_Name vector stores the pollutant names (TSS BOD COD TP SolP TKN NO2_3 Cu Pb Zn)
		//Note: These pollutant names must equal the names used in HydroPlusConfig.xml for removal efficiencies, as the names are matched
		Pollutant_Name.push_back(PollutantName_string);
	}

	//getline function reads 1st data line from pollutant file, the Median concentration (mg/L)
	getline(readPollutant, LineOfInput);
	//istringstream function constructs an object PollutantConcentrationMedian_string to hold strings from LineOfInput
	istringstream PollutantConcentrationMedian_string(LineOfInput);
	//Skip the first field
	getline(PollutantConcentrationMedian_string, RowName_string, ',');
	//while PollutantConcentrationMedian_string has remaining items, send them to PollutantConcentrationMedian_mg_p_L
	while (getline(PollutantConcentrationMedian_string, PollutantConcentrationMedian_mg_p_L, ','))
	{
		//Pollutant_ConcentrationMedian_mg_p_L vector stores the median values (mg/L)
		Pollutant_ConcentrationMedian_mg_p_L.push_back(stod(PollutantConcentrationMedian_mg_p_L));
	}

	//getline function reads 2nd data line from readPollutant file, the Mean concentration (mg/L)
	getline(readPollutant, LineOfInput);
	//istringstream function constructs an object PollutantConcentrationMean_string to hold strings from LineOfInput
	istringstream PollutantConcentrationMean_string(LineOfInput);
	//Skip the first field
	getline(PollutantConcentrationMean_string, RowName_string, ',');
	//while PollutantConcentrationMean_string has remaining items, send them to PollutantConcentrationMean_mg_p_L
	while (getline(PollutantConcentrationMean_string, PollutantConcentrationMean_mg_p_L, ','))
	{
		//Pollutant_ConcentrationMean_mg_p_L vector stores the mean values (mg/L)
		Pollutant_ConcentrationMean_mg_p_L.push_back(stod(PollutantConcentrationMean_mg_p_L));
	}

	readPollutant.close();
	//return true required for bool function
	return true;
}

//RataDieNumber function converts Gregorian date YYYY MM DD into an integer number based on Rata Die System; https://en.wikipedia.org/wiki/Rata_Die
//Note: Rata Die Dates (RDD) are equivalent to Julian Dates (JD), where RDD = JD - 1,721,424.5
//Note: Leap years are incorporated into the Rata Die System: 
//Note: Leap year = yes when the YYYY number is divisible by 4, except for end-of-century years, which must be divisible by 400. 
//Note: Examples of Leap and non-Leap years: 1900 no; 2000 yes; 2020, 2024 and 2028 yes. 
int Inputs::RataDieNumber(int y, int m, int d)
{
	//Date_RataDieMethod is a number assigned to a calendar date, 
	int Date_RataDieMethod;

	//Algorithm for Month and Year adjustment for Rata Die Date and Julian Date functions
	//if m month is less than 3, January or February
	if (m < 3) {
		//y year is decreased by 1
		y = y - 1;
		//m month is increased by 12, becoming 13 or 14 month
		m = m + 12;
	}
	//Date_RataDieMethod Equation from https://stackoverflow.com/questions/14218894/number-of-days-between-two-dates-c
	//Note: Date_RataDieMethod = 365 * y + y / 4 - y / 100 + y / 400 + (153 * m - 457) / 5 + d - 306
	//Note: Method requires integer division, which C++ upholds if numerator and denominator are integers
	Date_RataDieMethod = 365 * y + y / 4 - y / 100 + y / 400 + (153 * m - 457) / 5 + d - 306;

	return Date_RataDieMethod;
}


//ExtractGregorianDateParts extracts YYYY MM DD and if available HH into an array
void Inputs::ExtractGregorianDateParts(int Date_YYYYMMDDHH, int DateParts_Gregorian[]) {

	//DateParts_Gregorian is integer array containing the Gregorian date parts, YYYY, MM, DD, and if available HH
	std::string Date_YYYYMMDD = std::to_string(Date_YYYYMMDDHH);
	//DateParts_Gregorian[0] as year YYYY from Date_YYYYMMDD, defined to contain result of atoi and substr(,).c_str() functions
	DateParts_Gregorian[0] = atoi(Date_YYYYMMDD.substr(0, 4).c_str());
	//DateParts_Gregorian[1] as month MM from Date_YYYYMMDD, defined to contain result of atoi and substr(,).c_str() functions
	DateParts_Gregorian[1] = atoi(Date_YYYYMMDD.substr(4, 2).c_str());
	//DateParts_Gregorian[2] as day DD from Date_YYYYMMDD, defined to contain result of atoi and substr(,).c_str() functions
	DateParts_Gregorian[2] = atoi(Date_YYYYMMDD.substr(6, 2).c_str());

	//if Date_YYYYMMDD length > 8 then it exceeds YYYYMMDD and contains HH, hour
	if (Date_YYYYMMDD.length() > 8) {
		//DateParts_Gregorian[3] as day HH from Date_YYYYMMDD, defined to contain result of atoi and substr(,).c_str() functions
		DateParts_Gregorian[3] = atoi(Date_YYYYMMDD.substr(8, 2).c_str());

		//If DateParts_Gregorian[3] > 23, then set to 23
		//Note: HH in the YYYYMMDDHH string should range from 0 to 23
		if (DateParts_Gregorian[3] > 23) {
			//DateParts_Gregorian[3] defined as 23, final hour of day
			DateParts_Gregorian[3] = 23;
			//Command Prompt to allert user
			cout << "Note: HydroPlusConfig.xml element ...Date_YYYYMMDD = " << Date_YYYYMMDDHH << ", > max hour, so HH=23 used in simulation." << endl;
		}
		//If DateParts_Gregorian[3] < 0, then set to 0
		//Note: HH in the YYYYMMDDHH string should range from 0 to 23
		if (DateParts_Gregorian[3] < 0) {
			//DateParts_Gregorian[3] defined as 0, final hour of day
			DateParts_Gregorian[3] = 0;
			//Command Prompt to allert user
			cout << "Note: HydroPlusConfig.xml element ...Date_YYYYMMDD = " << Date_YYYYMMDDHH << ", < min hour, so HH=00 used in simulation." << endl;
		}
	}
	//if Date_YYYYMMDD length < 8 then it does not contain hour
	else {
		//DateParts_Gregorian[3] as day HH from Date_YYYYMMDD, defined to contain -1 and indicate it has not been set
		DateParts_Gregorian[3] = -1;
	}

}

//GregorianToJulianDate function converts Gregorian date YYYY MM DD into an integer number based on Julian Date System
//Note: Julian Dates are typically float, adjusted to the prior coordinated universal time (UTC) noon
//Note: Julian Dates in HydroPlus are integer, adjusted to the prior UTC midnight
//Note: Leap years are incorporated into the Julian Date System: 
//Note: Leap year = yes when the YYYY number is divisible by 4, except for end-of-century years, which must be divisible by 400. 
//Note: Examples of Leap and non-Leap years: 1900 no; 2000 yes; 2020, 2024 and 2028 yes. 
int Inputs::GregorianToJulianDate(int DateParts_Gregorian[])
{
	//Date_JulianMethod, _01, _02 are parts of equation to obtain a number from a calendar date
	int Date_JulianMethod_01;
	int Date_JulianMethod_02;
	//Date_JulianMethod number as integer, created by changing last RHS from 1524.5 to 1524 and obtaining prior midnight
	int Date_JulianMethod;
	//Date_JulianMethod_float number as float, created by keeping last RHS as 1524.5 and obtaining prior noon
	float Date_JulianMethod_float;
	//Date_WeekDayCount number as integer for 0 to 6, indicating weekdays 0=Sunday to 6=Saturday
	int Date_WeekDayCount;

	//Gregorian date parts: Year_YYYY is year, Month_MM is month, Day_DD is day
	int Year_YYYY, Month_MM, Day_DD;
	Year_YYYY = DateParts_Gregorian[0];
	Month_MM = DateParts_Gregorian[1];
	Day_DD = DateParts_Gregorian[2];

	//Algorithm for Month and Year adjustment for Julian Date functions
	//if Month_MM month is less than 3, January or February
	if (Month_MM < 3) {
		//Year_YYYY year is decreased by 1
		Year_YYYY = Year_YYYY - 1;
		//Month_MM month is increased by 12, becoming 13 or 14 month
		Month_MM = Month_MM + 12;
	}
	//Date_JulianMethod Equation from https://quasar.as.utexas.edu/BillInfo/JulianDatesG.html
	//Note: Reda et al (2003, 2008) has similar calculation in Eq 4. 
	//Reference: Reda, I., & Andreas, A. (2003, 2008). Solar Position Algorithm for Solar Radiation Applications National Renewable Energy Laboratory, NREL/TP-560-34302. Revised 2008, 55.  
	//Note: Date_JulianMethod = 2 -(Year_YYYY/100) + (Year_YYYY/100)/4 + Day_DD + 365.25 * (Year_YYYY + 4716) + 30.6001 * (Month_MM+1) - 1524.5
	//Note: Method requires integer division, which C++ upholds if numerator and denominator are integers
	//Note: Method requires integer multiplication, which requires type casting with (int) 
	//Date_JulianMethod_01 is first terms on RHS of equation, which remain integer
	Date_JulianMethod_01 = 2 - (Year_YYYY / 100) + (Year_YYYY / 100) / 4 + Day_DD;
	//Date_JulianMethod_02 is middle terms on RHS of equation, which uses floor in multiplication to get integer products 
	Date_JulianMethod_02 = int(floor(365.25 * (Year_YYYY + 4716)) + floor(30.6001 * (Month_MM + 1)));
	//Date_JulianMethod_float is combination of 1st integer term, 2nd integer term, last RHS float term of equation
	Date_JulianMethod_float = float(Date_JulianMethod_01 + Date_JulianMethod_02 - 1524.5);
	//Date_JulianMethod is combination of 1st integer term, 2nd integer term, last RHS integer term of equation
	Date_JulianMethod = Date_JulianMethod_01 + Date_JulianMethod_02 - 1524;

	//Convert Julian Date to Day of Week Equation: Date_WeekDayCount = (Date_JulianMethod + 1) % 7
	//Note: Theory and Equation at: https://en.wikipedia.org/wiki/Julian_day
	//Note: WeekDay integers: 0=Sunday | 1=Monday | 2= Tuesday | 3=Wednesday | 4=Thursday | 5=Friday | 6=Saturday
	//Date_WeekDayCount number as integer for 0 to 6, indicating weekdays 0=Sunday to 6=Saturday
	Date_WeekDayCount = (Date_JulianMethod + 1) % 7;
	//Note, the above is not used, but archived here for future functionality

	//return Date_JulianMethod
	return Date_JulianMethod;
}

//Date_YYYYMMDD_to_JulianDate_to_JulianDay function converts to JulianDate
int Inputs::Date_YYYYMMDD_to_JulianDate_to_JulianDay(int Date_YYYYMMDD) {

	int Date_JulianMethod, Date_JulianMethod_Jan01;
	int Day_JulianDay;
	//DateParts_Gregorian[4] integer array initialized to hold parts of date for ExtractGregorianDateParts function
	int DateParts_Gregorian[4];

	//ExtractGregorianDateParts function extracts StopDate_YYYYMMDD year, month, day, and hour date parts from Gregorian date, format YYYYMMDDHH
	//Note: Even if Flag_HottestDaySimulated = 1, the output dates are from HydroPlusConfig.xml StartDate_YYYYMMDD and TimeBasedOutputPeriod0 values
	ExtractGregorianDateParts(Date_YYYYMMDD, DateParts_Gregorian);

	//Date_JulianMethod is Julian Date from GregorianToJulianDate function, which used DateParts_Gregorian[] as input
	Date_JulianMethod = GregorianToJulianDate(DateParts_Gregorian);

	//Redefine DateParts_Gregorian Month and Day to January 1st, keeping year for analysis of days completed within year
	//DateParts_Gregorian[1] is month
	DateParts_Gregorian[1] = 1;
	//DateParts_Gregorian[2] is day
	DateParts_Gregorian[2] = 1;
	//Date_JulianMethod is Julian Date from GregorianToJulianDate function, which used DateParts_Gregorian[] as input
	Date_JulianMethod_Jan01 = GregorianToJulianDate(DateParts_Gregorian);

	//Day_JulianDay is difference in days between Date_JulianMethod and Date_JulianMethod_Jan01, +1 to account for completing 1st day
	Day_JulianDay = (Date_JulianMethod - Date_JulianMethod_Jan01) + 1;

	//return Day_JulianDay
	return Day_JulianDay;
}

//Date_YYYYMMDD_to_JulianDay function calculates the sequential Julian day of a calendar date during a given year
//Note: This can handle Date_YYYYMMDD with 8 or more digits long, if it ends with HH
int Inputs::Date_YYYYMMDD_to_JulianDay(int Date_YYYYMMDD) {

	//Date_YYYYMMDD_ambiguous_str converted from int to ensure only 8 digits are taken, in case HH was sent
	string Date_YYYYMMDD_ambiguous_str = to_string(Date_YYYYMMDD);
	//Date_YYYYMMDD_str is first 8 digits of Date_YYYYMMDD_ambiguous_str using substring function
	string Date_YYYYMMDD_str = Date_YYYYMMDD_ambiguous_str.substr(0, 8);
	//Date_YYYYMMDD = stoi(Date_YYYYMMDD_str)
	Date_YYYYMMDD = stoi(Date_YYYYMMDD_str);

	//Year_YYYY = Date_YYYYMMDD / 10000, which is division by 10000 to remove the MMDD digits
	int Year_YYYY = Date_YYYYMMDD / 10000;
	//Month_MM = (Date_YYYYMMDD / 100) % 100, which is division by 100 to remove the DD digits, and modulus of 100 capturing last 2 digits
	int Month_MM = (Date_YYYYMMDD / 100) % 100;
	//Day_DD = Date_YYYYMMDD % 100, which is modulus of 100 capturing last 2 digits
	int Day_DD = Date_YYYYMMDD % 100;

	// Array for the cumulative number of days at the start of each month for a non-leap year
	vector<int> begmonth = { 0, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };

	// Determine if the year is a leap year
	// A year is a leap year if it is divisible by 4 and not divisible by 100, or if it is divisible by 400
	int leapyr = ((Year_YYYY % 4 == 0 && Year_YYYY % 100 != 0) || (Year_YYYY % 400 == 0)) ? 1 : 0;

	// Calculate the Julian day
	int Day_JulianDay = begmonth[Month_MM] + Day_DD;

	// If it's a leap year and the month is after February, add 1 day
	if (leapyr && Month_MM > 2) {
		Day_JulianDay += 1;
	}
	return Day_JulianDay;
}


void Inputs::printIncompleteECDataWarning(const string& landCoverType, const string& landCoverRange, const string& huc) {
	cout << endl;
	cout << "Warning: Incomplete Data for " << landCoverType << " Land Cover in WhiteData.csv for the specified HUC." << endl;
	cout << "Explanation: The HUC specified by " << huc << " includes " << landCoverType << " land cover types (" << landCoverRange << "), but the EC value for " << landCoverType << " is missing in WhiteData.csv." << endl;
	if (landCoverType == "Pasture_hay" || landCoverType == "Cropland") {
		string otherLandCoverType = (landCoverType == "Pasture_hay") ? "Cropland" : "Pasture_hay";
		cout << "Correction: If EC values of " << otherLandCoverType << " are found in WhiteData.csv, the EC values will be defaulted to " << otherLandCoverType << ". Otherwise, the EC values will be defaulted to 0. Please update the EC value for " << landCoverType << " land cover in WhiteData.csv for accurate simulation results." << endl;
	}
	else {
		cout << "Correction: The EC values will be defaulted to 0. Please update the EC value for " << landCoverType << " land cover in WhiteData.csv for accurate simulation results." << endl;
	}
	cout << endl;
}

bool Inputs::readExportCoefficientTable()
{
	//ExportCoefficientTable_str set to value of HydroPlusConfig.xml element WeatherStationAttributeFile
	string ExportCoefficientTable_str;
	ExportCoefficientTable_str = BufferNutrientsStringParams["ExportCoefficientFile"];

	//If ExportCoefficientTable_str is not populated, then HydroPlusConfig.xml is missing file, and look in local directory
	//Note: This option is for those our NUCFAC National runs
	if (ExportCoefficientTable_str.size() < 1) {
		ExportCoefficientTable_str = Directory_Input_CLArg + "ExportCoefficientTable.csv";
	}

	fstream readExportCoefficientTable(ExportCoefficientTable_str);
	//if readExportCoefficientTable does not exist, alert user
	if (!readExportCoefficientTable) {
		cout << "Warning: ExportCoefficientFile.csv could not be opened." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the ExportCoefficientFile.csv file is missing there is no export coefficient database to drive the simulation." << endl;
		cout << "Correction: Create a ExportCoefficientFile.csv file, either manually or perhaps using the version in the director ..." << endl;
		cout << "Correction: ... C:\iTree\HydroPlus\TestingFilesAndScript\TestCases\BufferNutrients\WhiteData.csv." << endl;
		cout << "Then, update the file path in the <ExportCoefficientFile> section of HydroPlusConfig.xml." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	// Check if HUC8 is provided
	if (BufferNutrientsStringParams["HUC8"].empty()) {
		cout << "Warning: No HUC8 provided." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HUC8 code is essential for determining the EC value for each land type within the simulated basin."  << endl;
		cout << "Correction: Please specify a valid HUC8 in the HydroPlusConfig.xml." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}

	string HUC8, header, Landuse, Parameter, SampleCount;
	string temp_line, temp_string;
	string Spercentile_5, Spercentile_10, Spercentile_25, Spercentile_50, Spercentile_75, Spercentile_95, Spercentile_100;
	double percentile_5, percentile_10, percentile_25, percentile_50, percentile_75, percentile_95, percentile_100;
	//EC vectors
	vector<double> Cropland_Sediment_vec_kg_p_ha(7, 0.0), Cropland_Total_N_vec_kg_p_ha(7, 0.0), Cropland_Total_P_vec_kg_p_ha(7, 0.0);
	vector<double> Forest_Sediment_vec_kg_p_ha(7, 0.0), Forest_Total_N_vec_kg_p_ha(7, 0.0), Forest_Total_P_vec_kg_p_ha(7, 0.0);
	vector<double> Rangeland_Sediment_vec_kg_p_ha(7, 0.0), Rangeland_Total_N_vec_kg_p_ha(7, 0.0), Rangeland_Total_P_vec_kg_p_ha(7, 0.0);
	vector<double> Pasture_hay_Sediment_vec_kg_p_ha(7, 0.0), Pasture_hay_Total_N_vec_kg_p_ha(7, 0.0), Pasture_hay_Total_P_vec_kg_p_ha(7, 0.0);
	vector<double> Urban_Sediment_vec_kg_p_ha(7, 0.0), Urban_Total_N_vec_kg_p_ha(7, 0.0), Urban_Total_P_vec_kg_p_ha(7, 0.0);
	//EMC vectors
	/*vector<double> Cropland_Sediment_vec_mg_p_L, Cropland_Total_N_vec_mg_p_L, Cropland_Total_P_vec_mg_p_L;
	vector<double> Forest_Sediment_vec_mg_p_L, Forest_Total_N_vec_mg_p_L, Forest_Total_P_vec_mg_p_L;
	vector<double> Rangeland_Sediment_vec_mg_p_L, Rangeland_Total_N_vec_mg_p_L, Rangeland_Total_P_vec_mg_p_L;
	vector<double> Pasture_hay_Sediment_vec_mg_p_L, Pasture_hay_Total_N_vec_mg_p_L, Pasture_hay_Total_P_vec_mg_p_L;
	vector<double> Urban_Sediment_vec_mg_p_L, Urban_Total_N_vec_mg_p_L, Urban_Total_P_vec_mg_p_L;*/
	//read water volumn to calculate EMC, surface runoff and water yield
	vector<double> Cropland_SURQ_mm(7, 0.0), Forest_SURQ_mm(7, 0.0), Rangeland_SURQ_mm(7, 0.0), Pasture_hay_SURQ_mm(7, 0.0), Urban_SURQ_mm(7, 0.0);
	vector<double> Cropland_WYLD_mm(7, 0.0), Forest_WYLD_mm(7, 0.0), Rangeland_WYLD_mm(7, 0.0), Pasture_hay_WYLD_mm(7, 0.0), Urban_WYLD_mm(7, 0.0);


	//read the fist line of the database: the header
	getline(readExportCoefficientTable, header);
	//For some HUCs in White's database, cropland or pasture_hay data is missing, this is the flag to track if cropland data exist 
	bool db_cropland_found = false;
	bool db_pasturehay_found = false;
	bool db_forest_found = false;
	bool db_rangeland_found = false;
	bool db_urban_found = false;
	bool HUC8_found = false;
	//If the next data exist, read a line in as the temp_line
	while (getline(readExportCoefficientTable, temp_line)) {
		//read cells within this line
		istringstream readlinedata(temp_line);
		//if (!getline(readlinedata, temp_string, ',')) break;
		getline(readlinedata, HUC8, ',');

		//cout << temp_line << endl;
		//cout << BufferNutrientsStringParams["HUC8"] << "and " << HUC8 << endl;
		
		if (HUC8 == BufferNutrientsStringParams["HUC8"] || BufferNutrientsStringParams["HUC8"].substr(1) == HUC8) {
			//found the HUC 8 in the EC database
			HUC8_found = true;
			//read each row: 1 landuse, 1 parameter & 7 values, then saving the EC values into a temp vector:ECValue
			//readExportCoefficientTable >> Landuse >> Parameter >> header >> percentile_5 >> percentile_10 >> percentile_25 >> percentile_50 >> percentile_75 >> percentile_95 >> percentile_100;
			getline(readlinedata, Landuse, ',');
			getline(readlinedata, Parameter, ',');
			getline(readlinedata, SampleCount, ',');
			getline(readlinedata, Spercentile_5, ',');
			getline(readlinedata, Spercentile_10, ',');
			getline(readlinedata, Spercentile_25, ',');
			getline(readlinedata, Spercentile_50, ',');
			getline(readlinedata, Spercentile_75, ',');
			getline(readlinedata, Spercentile_95, ',');
			getline(readlinedata, Spercentile_100, ',');
            //stod() from string to double
			percentile_5 = stod(Spercentile_5);
			percentile_10 = stod(Spercentile_10);
			percentile_25 = stod(Spercentile_25);
			percentile_50 = stod(Spercentile_50);
			percentile_75 = stod(Spercentile_75);
			percentile_95 = stod(Spercentile_95);
			percentile_100 = stod(Spercentile_100);
			//cout << Landuse << ',' << Parameter << ',' << header << ',' << percentile_5 << ',' << percentile_10 << ',' << percentile_25 << ',' << percentile_50 << ',' << percentile_75 << ',' << percentile_95 << ',' << percentile_100 << endl;
			//npos returns the index of the first occurrence of the substring in the string from given starting position. The default value of starting position is 0.
			//remove the first character to avoid the error caused by the upper and lower case. Ex. "ropland" mwans Cropland.
			//cout << Landuse << endl;
			if (Landuse.find("ropland") != string::npos) {
				db_cropland_found = true;
				if (Parameter.find("ediment") != string::npos) {
					Cropland_Sediment_vec_kg_p_ha = { percentile_5 * Ratio_Tonne_to_kg,percentile_10 * Ratio_Tonne_to_kg,percentile_25 * Ratio_Tonne_to_kg,percentile_50 * Ratio_Tonne_to_kg,percentile_75 * Ratio_Tonne_to_kg,percentile_95 * Ratio_Tonne_to_kg,percentile_100 * Ratio_Tonne_to_kg };
				}
				else if (Parameter.find("Total_N") != string::npos) {
					Cropland_Total_N_vec_kg_p_ha = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Total_P") != string::npos) {
					Cropland_Total_P_vec_kg_p_ha = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Surface Runoff") != string::npos) {
					Cropland_SURQ_mm = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Water Yield") != string::npos) {
					Cropland_WYLD_mm = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
			}
			else if (Landuse.find("orest") != string::npos) {
				db_forest_found = true;
				if (Parameter.find("ediment") != string::npos) {
					Forest_Sediment_vec_kg_p_ha = { percentile_5 * Ratio_Tonne_to_kg,percentile_10 * Ratio_Tonne_to_kg,percentile_25 * Ratio_Tonne_to_kg,percentile_50 * Ratio_Tonne_to_kg,percentile_75 * Ratio_Tonne_to_kg,percentile_95 * Ratio_Tonne_to_kg,percentile_100 * Ratio_Tonne_to_kg };
				}
				else if (Parameter.find("Total_N") != string::npos) {
					Forest_Total_N_vec_kg_p_ha = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Total_P") != string::npos) {
					Forest_Total_P_vec_kg_p_ha = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Surface Runoff") != string::npos) {
					Forest_SURQ_mm = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Water Yield") != string::npos) {
					Forest_WYLD_mm = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
			}
			else if (Landuse.find("angeland") != string::npos) {
				db_rangeland_found = true;
				if (Parameter.find("ediment") != string::npos) {
					Rangeland_Sediment_vec_kg_p_ha = { percentile_5 * Ratio_Tonne_to_kg,percentile_10 * Ratio_Tonne_to_kg,percentile_25 * Ratio_Tonne_to_kg,percentile_50 * Ratio_Tonne_to_kg,percentile_75 * Ratio_Tonne_to_kg,percentile_95 * Ratio_Tonne_to_kg,percentile_100 * Ratio_Tonne_to_kg };
				}
				else if (Parameter.find("Total_N") != string::npos) {
					Rangeland_Total_N_vec_kg_p_ha = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Total_P") != string::npos) {
					Rangeland_Total_P_vec_kg_p_ha = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Surface Runoff") != string::npos) {
					Rangeland_SURQ_mm = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Water Yield") != string::npos) {
					Rangeland_WYLD_mm = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
			}
			else if (Landuse.find("hay") != string::npos) {
				db_pasturehay_found = true;
				if (Parameter.find("ediment") != string::npos) {
					Pasture_hay_Sediment_vec_kg_p_ha = { percentile_5 * Ratio_Tonne_to_kg,percentile_10 * Ratio_Tonne_to_kg,percentile_25 * Ratio_Tonne_to_kg,percentile_50 * Ratio_Tonne_to_kg,percentile_75 * Ratio_Tonne_to_kg,percentile_95 * Ratio_Tonne_to_kg,percentile_100 * Ratio_Tonne_to_kg };
				}
				else if (Parameter.find("Total_N") != string::npos) {
					Pasture_hay_Total_N_vec_kg_p_ha = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Total_P") != string::npos) {
					Pasture_hay_Total_P_vec_kg_p_ha = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Surface Runoff") != string::npos) {
					Pasture_hay_SURQ_mm = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Water Yield") != string::npos) {
					Pasture_hay_WYLD_mm = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
			}
			else if (Landuse.find("rban") != string::npos) {
				db_urban_found = true;
				if (Parameter.find("ediment") != string::npos) {
					Urban_Sediment_vec_kg_p_ha = { percentile_5 * Ratio_Tonne_to_kg,percentile_10 * Ratio_Tonne_to_kg,percentile_25 * Ratio_Tonne_to_kg,percentile_50 * Ratio_Tonne_to_kg,percentile_75 * Ratio_Tonne_to_kg,percentile_95 * Ratio_Tonne_to_kg,percentile_100 * Ratio_Tonne_to_kg };
				}
				else if (Parameter.find("Total_N") != string::npos) {
					Urban_Total_N_vec_kg_p_ha = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Total_P") != string::npos) {
					Urban_Total_P_vec_kg_p_ha = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Surface Runoff") != string::npos) {
					Urban_SURQ_mm = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
				else if (Parameter.find("Water Yield") != string::npos) {
					Urban_WYLD_mm = { percentile_5,percentile_10,percentile_25,percentile_50,percentile_75,percentile_95,percentile_100 };
				}
			}
		}
	}

	if (!HUC8_found) {
		cout << "Warning: HUC8 value '" << BufferNutrientsStringParams["HUC8"] << "' not found in the export coefficient database." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: The HUC8 code is essential for determining the EC value for each land type within the simulated basin." << endl;
		cout << "Correction: Verify the HUC8 value and confirm it is present in the ExportCoefficientFile.csv." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	//20231117 LI for debugging
	/*cout << "LC21 + LC22 + LC23 + LC24 " << LC21 + LC22 + LC23 + LC24 << endl;
	cout << "LC41 + LC42 + LC43 " << LC41 + LC42 + LC43 << endl;
	cout << "LC71 " << LC71 << endl;
	cout << "81 " << LC81 << endl;
	cout << "82 " << LC82 << endl;*/

	if ((LC21 + LC22 + LC23 + LC24) > 0 && db_urban_found == false) {
		printIncompleteECDataWarning("Urban", "21 - 24", BufferNutrientsStringParams["HUC8"]);
	}
	if ((LC41 + LC42 + LC43) > 0 && db_forest_found == false) {
		printIncompleteECDataWarning("Forest", "41 - 43", BufferNutrientsStringParams["HUC8"]);
	}
	if ((LC71) > 0 && db_rangeland_found == false) {
		printIncompleteECDataWarning("Rangeland", "71", BufferNutrientsStringParams["HUC8"]);
	}
	if ((LC81) > 0 && db_pasturehay_found == false) {
		printIncompleteECDataWarning("Pasture_hay", "81", BufferNutrientsStringParams["HUC8"]);
	}
	if ((LC82) > 0 && db_cropland_found == false) {
		printIncompleteECDataWarning("Cropland", "82", BufferNutrientsStringParams["HUC8"]);
	}

	//if the cropland LC 82 data is missing in White's database, assign LC 81 pasture/hay to LC 82
	if (!db_cropland_found && db_pasturehay_found) {
		Cropland_Sediment_vec_kg_p_ha.assign(Pasture_hay_Sediment_vec_kg_p_ha.begin(), Pasture_hay_Sediment_vec_kg_p_ha.end());
		Cropland_Total_N_vec_kg_p_ha.assign(Pasture_hay_Total_N_vec_kg_p_ha.begin(), Pasture_hay_Total_N_vec_kg_p_ha.end());
		Cropland_Total_P_vec_kg_p_ha.assign(Pasture_hay_Total_P_vec_kg_p_ha.begin(), Pasture_hay_Total_P_vec_kg_p_ha.end());
		Cropland_SURQ_mm.assign(Pasture_hay_SURQ_mm.begin(), Pasture_hay_SURQ_mm.end());
		Cropland_WYLD_mm.assign(Pasture_hay_WYLD_mm.begin(), Pasture_hay_WYLD_mm.end());
	}
	if (!db_pasturehay_found && db_cropland_found) {
		Pasture_hay_Sediment_vec_kg_p_ha.assign(Cropland_Sediment_vec_kg_p_ha.begin(), Cropland_Sediment_vec_kg_p_ha.end());
		Pasture_hay_Total_N_vec_kg_p_ha.assign(Cropland_Total_N_vec_kg_p_ha.begin(), Cropland_Total_N_vec_kg_p_ha.end());
		Pasture_hay_Total_P_vec_kg_p_ha.assign(Cropland_Total_P_vec_kg_p_ha.begin(), Cropland_Total_P_vec_kg_p_ha.end());
		Pasture_hay_SURQ_mm.assign(Cropland_SURQ_mm.begin(), Cropland_SURQ_mm.end());
		Pasture_hay_WYLD_mm.assign(Cropland_WYLD_mm.begin(), Cropland_WYLD_mm.end());
	}


	readExportCoefficientTable.close();

	//in kg/ha
	EC_vec_map = {
		{"Water_Wetlands_TSS",{0,0,0,0,0,0,0}},
		{"Water_Wetlands_TN",{0,0,0,0,0,0,0}},
		{"Water_Wetlands_TP",{0,0,0,0,0,0,0}},

		{"Cropland_TSS", Cropland_Sediment_vec_kg_p_ha},
		{"Cropland_TN", Cropland_Total_N_vec_kg_p_ha},
		{"Cropland_TP", Cropland_Total_P_vec_kg_p_ha},

		{"Forest_TSS", Forest_Sediment_vec_kg_p_ha},
		{"Forest_TN", Forest_Total_N_vec_kg_p_ha},
		{"Forest_TP", Forest_Total_P_vec_kg_p_ha},

		{"Rangeland_TSS", Rangeland_Sediment_vec_kg_p_ha},
		{"Rangeland_TN", Rangeland_Total_N_vec_kg_p_ha},
		{"Rangeland_TP", Rangeland_Total_P_vec_kg_p_ha},

		{"Pasture_hay_TSS", Pasture_hay_Sediment_vec_kg_p_ha},
		{"Pasture_hay_TN", Pasture_hay_Total_N_vec_kg_p_ha},
		{"Pasture_hay_TP", Pasture_hay_Total_P_vec_kg_p_ha},

		{"Urban_TSS", Urban_Sediment_vec_kg_p_ha},
		{"Urban_TN", Urban_Total_N_vec_kg_p_ha},
		{"Urban_TP", Urban_Total_P_vec_kg_p_ha}
	};
	//print out map for debugging
	/*for (auto i = EC_vec_map.begin(); i != EC_vec_map.end(); i++) {
		cout << i->first <<": ";
		vector<double> temp = i->second;
		for (auto j : temp) {
			cout << j << " ";
		}
		cout << endl;
	}*/

	//in mm
	water_volume_vec_map = {
		{"Cropland_SurfaceRunoff", Cropland_SURQ_mm},
		{"Cropland_WaterYield", Cropland_WYLD_mm},

		{"Forest_SurfaceRunoff", Forest_SURQ_mm},
		{"Forest_WaterYield", Forest_WYLD_mm},

		{"Rangeland_SurfaceRunoff", Rangeland_SURQ_mm},
		{"Rangeland_WaterYield", Rangeland_WYLD_mm},

		{"Pasture_hay_SurfaceRunoff", Pasture_hay_SURQ_mm},
		{"Pasture_hay_WaterYield", Pasture_hay_WYLD_mm},

		{"Urban_SurfaceRunoff", Urban_SURQ_mm},
		{"Urban_WaterYield", Urban_WYLD_mm}
	};


	string ECfilename = "ExportCoefficient_Database.csv";

	// write out a EC database for this study area
	ofstream ECoutfile(SimulationStringParams["OutputFolder_Path"] + ECfilename);
	if (!ECoutfile.good())	{
		cout << "Could not create " << ECfilename << " in: " << SimulationStringParams["OutputFolder_Path"] << endl;
		return false;
	}
	else {
		cout << "Creating " << ECfilename <<  " in: " << SimulationStringParams["OutputFolder_Path"] << endl;
	}

	//vectors combination of outout row name
	vector<string> pollutant_name = { "TP","TN", "TSS" };
	vector<string> volume_name = { "SURQ", "WYLD" };
	vector<string> NLCDClass = { "NLCD_Class11", "NLCD_Class21","NLCD_Class22", "NLCD_Class23", "NLCD_Class24", "NLCD_Class41","NLCD_Class42", "NLCD_Class43", "NLCD_Class71","NLCD_Class81","NLCD_Class82","NLCD_Class90", "NLCD_Class95" };

	//vectors combination of input map key
	vector<string> LC_name = { "Urban","Forest", "Rangeland", "Pasture_hay", "Cropland", "Water_Wetlands"};
	//vector<string> pollutant_EMCvec_name_2nd = { "_Total_P_vec_mg_p_L","_Total_N_vec_mg_p_L", "_Sediment_vec_mg_p_L" };//need to match the sequence of pollutant_name
	vector<string> volumetype_name = {"SurfaceRunoff", "WaterYield" };//need to match the sequence of volume_name

	string rowname, vec_name;
	double Manningn = 0.0, Hydraulicradius = 0.0;
	//header
	ECoutfile << "Percentile," << "5," << "10," << "25," << "50," << "75," << "90," << "95," << "n," << "R_m" << endl;
	// Loop through pollutants
	for (const string& pollutant : pollutant_name) {
		// Loop through land use types
		for (const string& landuseclass : NLCDClass) {
			rowname = pollutant + "_" + landuseclass + "_kg_p_ha";

			int landuseclassCode = stoi(landuseclass.substr(landuseclass.size() - 2));
			double Manningn = getManningn(landuseclassCode);
			double Hydraulicradius = getHydraulicradius(landuseclassCode);

			if (landuseclass == "NLCD_Class21" || landuseclass == "NLCD_Class22" || landuseclass == "NLCD_Class23" || landuseclass == "NLCD_Class24") { //21 - 24 Urban
				vec_name = LC_name[0] + "_" + pollutant;
				Manningn = getManningn(21);
				Hydraulicradius = getHydraulicradius(21);
			}
			else if (landuseclass == "NLCD_Class41" || landuseclass == "NLCD_Class42" || landuseclass == "NLCD_Class43") { // 41- 43 Forest
				vec_name = LC_name[1] + "_" + pollutant;
				Manningn = getManningn(41);
				Hydraulicradius = getHydraulicradius(41);
			}
			else if (landuseclass == "NLCD_Class71") {// 71 Rangeland
				vec_name = LC_name[2] + "_" + pollutant;
				Manningn = getManningn(71);
				Hydraulicradius = getHydraulicradius(71);
			}
			else if (landuseclass == "NLCD_Class81") {// 81 Pasture_hay
				vec_name = LC_name[3] + "_" + pollutant;
				Manningn = getManningn(81);
				Hydraulicradius = getHydraulicradius(81);
			}
			else if (landuseclass == "NLCD_Class82") {//82 Cropland
				vec_name = LC_name[4]  + "_" + pollutant;
				Manningn = getManningn(82);
				Hydraulicradius = getHydraulicradius(82);
			}
			else if (landuseclass == "NLCD_Class11" || landuseclass == "NLCD_Class90" || landuseclass == "NLCD_Class95") {//11 Water, 90&95 wetlands
				vec_name = LC_name[5] + "_" + pollutant;
				if (landuseclass == "NLCD_Class11") {
					Manningn = getManningn(11);
					Hydraulicradius = getHydraulicradius(11);
				}
				else if (landuseclass == "NLCD_Class90" || landuseclass == "NLCD_Class95") {
					Manningn = getManningn(90);
					Hydraulicradius = getHydraulicradius(90);
				} 
			}
			
			//cout << rowname << ", read map[" << vec_name << "]" << endl;
			ECoutfile << rowname;
			for (int i = 0; i < 7; i++) {
				ECoutfile << "," << EC_vec_map[vec_name][i];
			}
			ECoutfile << "," << Manningn << "," << Hydraulicradius<< endl;
		}
	}
	
	//water volume loop, loop through SURQ and WYLD
	for (int volume_type = 0; volume_type < 2; volume_type++) {
		// Loop through land use types
		for (const string& landuseclass : NLCDClass) {
			//don't write EMC for water land type (water&wetlands)
			if (landuseclass != "NLCD_Class11" && landuseclass != "NLCD_Class90" && landuseclass != "NLCD_Class95") {
				rowname = volume_name[volume_type] + "_" + landuseclass + "_mm";

				if (landuseclass == "NLCD_Class21" || landuseclass == "NLCD_Class22" || landuseclass == "NLCD_Class23" || landuseclass == "NLCD_Class24") { //21 - 24 Urban
					vec_name = LC_name[0] + "_" + volumetype_name[volume_type];
				}
				else if (landuseclass == "NLCD_Class41" || landuseclass == "NLCD_Class42" || landuseclass == "NLCD_Class43") { // 41- 43 Forest
					vec_name = LC_name[1] + "_" + volumetype_name[volume_type];
				}
				else if (landuseclass == "NLCD_Class71") {// 71 Rangeland
					vec_name = LC_name[2] + "_" + volumetype_name[volume_type];
				}
				else if (landuseclass == "NLCD_Class81") {// 81 Pasture_hay
					vec_name = LC_name[3] + "_" + volumetype_name[volume_type];
				}
				else if (landuseclass == "NLCD_Class82") {//82 Cropland
					vec_name = LC_name[4] + "_" + volumetype_name[volume_type];
				}
				
				ECoutfile << rowname;
				for (int i = 0; i < 7; i++) {
					ECoutfile << "," << water_volume_vec_map[vec_name][i];
				}
				ECoutfile << endl;
			
			}
			
		}
	}
	ECoutfile.close();
	
	cout << "Created " << ECfilename << endl;



	// write out an EMC database for this study area
	double flow_75th_L, flow_50th_L, flow_25th_L;
	string flowmapkey;
	string ECmapkey, EMCmapkey;
	vector<double> EC_vec_temp;
	double min_C_mg_p_L, max_C_mg_p_L, other_C_mg_p_L;

	//EMC map created by the loops is like this:
	/*EMC_vec_map = {
		{"Cropland_Sediment_vec_mg_p_L", Cropland_Sediment_vec_mg_p_L},
		{"Cropland_Total_N_vec_mg_p_L", Cropland_Total_N_vec_mg_p_L},
		{"Cropland_Total_P_vec_mg_p_L", Cropland_Total_P_vec_mg_p_L},

		{"Forest_Sediment_vec_mg_p_L", Forest_Sediment_vec_mg_p_L},
		{"Forest_Total_N_vec_mg_p_L", Forest_Total_N_vec_mg_p_L},
		{"Forest_Total_P_vec_mg_p_L", Forest_Total_P_vec_mg_p_L},

		{"Rangeland_Sediment_vec_mg_p_L", Rangeland_Sediment_vec_mg_p_L},
		{"Rangeland_Total_N_vec_mg_p_L", Rangeland_Total_N_vec_mg_p_L},
		{"Rangeland_Total_P_vec_mg_p_L", Rangeland_Total_P_vec_mg_p_L},

		{"Pasture_hay_Sediment_vec_mg_p_L", Pasture_hay_Sediment_vec_mg_p_L},
		{"Pasture_hay_Total_N_vec_mg_p_L", Pasture_hay_Total_N_vec_mg_p_L},
		{"Pasture_hay_Total_P_vec_mg_p_L", Pasture_hay_Total_P_vec_mg_p_L},

		{"Urban_Sediment_vec_mg_p_L", Urban_Sediment_vec_mg_p_L},
		{"Urban_Total_N_vec_mg_p_L", Urban_Total_N_vec_mg_p_L},
		{"Urban_Total_P_vec_mg_p_L", Urban_Total_P_vec_mg_p_L}
	};*/

	//check the water volume type for EMC calculation
	string WaterVolume_Type = BufferNutrientsStringParams["WaterVolume_Type"];

	for (string LC : LC_name) {// for 5 type of land cover
		//don't create EMC for waters (water&wetlands)
		if (LC != "Water_Wetlands") {
			if (WaterVolume_Type == "1") {//option 1 is Water Yield
				flowmapkey = LC + "_" + "WaterYield";
			}
			if (WaterVolume_Type == "2") { //option 2 is surface runoff
					flowmapkey = LC + "_" + "SurfaceRunoff";
			}
			
			flow_75th_L = water_volume_vec_map[flowmapkey][4] * Ratio_mmHa_to_L;
			flow_50th_L = water_volume_vec_map[flowmapkey][3] * Ratio_mmHa_to_L;
			flow_25th_L = water_volume_vec_map[flowmapkey][2] * Ratio_mmHa_to_L;

			for (const string& pollutant : pollutant_name) {//loop through each pollutant
				//e.g. Urban_TP
				ECmapkey = LC + "_" + pollutant;
				EMCmapkey = LC + "_" + pollutant;

				EC_vec_temp = EC_vec_map[ECmapkey];

				for (auto it = EC_vec_temp.begin(); it != EC_vec_temp.end(); it++) {//for percentiles, 7 times

					if (it == EC_vec_temp.begin()) {//min
						min_C_mg_p_L = *it * Ratio_kg_to_mg / flow_75th_L;
						EMC_vec_map[EMCmapkey].push_back(min_C_mg_p_L);
					}
					else if (it == EC_vec_temp.end() - 1) {//max
						max_C_mg_p_L = *it * Ratio_kg_to_mg / flow_25th_L;
						EMC_vec_map[EMCmapkey].push_back(max_C_mg_p_L);
					}
					else {//others
						other_C_mg_p_L = *it * Ratio_kg_to_mg / flow_50th_L;
						EMC_vec_map[EMCmapkey].push_back(other_C_mg_p_L);
					}
				}
			}
		}
		
	}

	/*for (auto i = EMC_vec_map.begin(); i != EMC_vec_map.end(); i++) {
		vector<double> temp = i->second;
		for (auto j : temp) {
			cout << j << " ";
		}
		cout << endl;
	}*/

	string EMCfilename;
	if (WaterVolume_Type == "1") {//option 1 is Water Yield
		EMCfilename = "EventMeanConcentration_Database_WaterYield.csv";
	}
	if (WaterVolume_Type == "2") { //option 2 is surface runoff
		EMCfilename = "EventMeanConcentration_Database_SurfaceRunoff.csv";
	}

	ofstream EMCoutfile(SimulationStringParams["OutputFolder_Path"] + EMCfilename);
	if (!EMCoutfile.good()) {
		cout << "Could not create " << EMCfilename << " in " << SimulationStringParams["OutputFolder_Path"] << endl;
		return false;
	}
	else {
		cout << "Creating " << EMCfilename <<" in: " << SimulationStringParams["OutputFolder_Path"] << endl;
	}

	//write the header line
	EMCoutfile << "Percentile," << "Min_5th," << "Low_10th," << "25th," << "Med_50th," << "75th," << "High_90th," << "Max_95th" << endl;
	// Loop through pollutants
	for (const string& pollutant : pollutant_name) {
		// Loop through land use types
		for (const string& landuseclass : NLCDClass) {
			//don't write EMC for water land type (water&wetlands)
			if (landuseclass != "NLCD_Class11" && landuseclass != "NLCD_Class90" && landuseclass != "NLCD_Class95") {
				rowname = pollutant + "_" + landuseclass + "_mg_p_L";

				if (landuseclass == "NLCD_Class21" || landuseclass == "NLCD_Class22" || landuseclass == "NLCD_Class23" || landuseclass == "NLCD_Class24") { //21 - 24 Urban
					vec_name = LC_name[0] + "_" + pollutant;
				}
				else if (landuseclass == "NLCD_Class41" || landuseclass == "NLCD_Class42" || landuseclass == "NLCD_Class43") { // 41- 43 Forest
					vec_name = LC_name[1] + "_" + pollutant;
				}
				else if (landuseclass == "NLCD_Class71") {// 71 Rangeland
					vec_name = LC_name[2] + "_" + pollutant;
				}
				else if (landuseclass == "NLCD_Class81") {// 81 Pasture_hay
					vec_name = LC_name[3] + "_" + pollutant;
				}
				else if (landuseclass == "NLCD_Class82") {//82 Cropland
					vec_name = LC_name[4] + "_" + pollutant;
				}

				EMCoutfile << rowname;
				for (int i = 0; i < 7; i++) {
					EMCoutfile << "," << EMC_vec_map[vec_name][i];
				}
				EMCoutfile << endl;
			}
		}
	}
	EMCoutfile.close();

	cout << "Created " << EMCfilename << endl;

	return true;
}

//Inputs::readMultipleWeatherStations_List_CSV function reads from WeatherStationAttributeTable.csv the weather station ID, location, and characteristics
//Note: WeatherStationAttributeTable.csv Header is: OBJECTID,USAF,WBAN,Station_Easting_m,Station_Northing_m,Elev_m,Slope_deg,Aspect_deg,Station_TI_option,NLCD_Class,Station_TC_per,Station_IC_per,Station_SVeg_per,Station_Soil_per,Station_Water_per, Station_Path_folder
//Note: Flag_MultipleStation = 1 computes distance between stations and map pixels that use the same projection, e.g., Albers_Conical_Equal_Area, Lambert_Azimuthal_Equal_Area, or UTM
//Note: Consider refactor to clearly name the input variables, e.g., changing Station_TC_per to TreeCanopy_Cover_percent; not all variables (e.g., Station_SVeg_per (%)) are used in function
bool Inputs::readMultipleWeatherStations_List_CSV()
{
	//WeatherStationAttributeFile_str set to value of HydroPlusConfig.xml element WeatherStationAttributeFile
	string WeatherStationAttributeFile_str;
	WeatherStationAttributeFile_str = SimulationStringParams["WeatherStationAttributeFile"];
	//cout << "Input File Size " << WeatherStationAttributeFile_str.size() << endl;
	//If WeatherStationAttributeFile_str is not populated, then HydroPlusConfig.xml is missing file, and look in local directory
	//Note: This option is for those our NUCFAC National runs
	if (WeatherStationAttributeFile_str.size() < 1) {
		WeatherStationAttributeFile_str = "WeatherStationAttributeTable.csv";
	}
	//fstream C++ function used to assign WeatherStation as the WeatherStationAttributeFile_str file
	fstream WeatherStation(WeatherStationAttributeFile_str);
	//If WeatherStation does not exist, then alert user
	if (!WeatherStation) {
		cout << endl;
		cout << "Warning: This program cannot open the required file " << WeatherStationAttributeFile_str << "." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: This file is required when HydroPlusConfig.xml element Flag_MultipleStation = 1. " << endl;
		cout << "Correction: If WeatherStationAttributeTable.csv exists, then make sure it is not opened and is located in ..." << endl;
		cout << "... the directory where your Command Prompt is calling the HydroPlus.exe model." << endl;
		cout << "Correction: If WeatherStationAttributeTable.csv does not exist, then 1) create it with the following header: " << endl;
		cout << "... OBJECTID,USAF,WBAN,Station_Easting_m,Station_Northing_m,Station_Elev_m,Station_Slope_deg,Station_Aspect_deg,Station_TI_option,NLCD_Class,Station_TC_per,Station_IC_per,Station_SVeg_per,Station_Soil_per,Station_Water_per, Station_Path_folder." << endl;
		cout << "... and 2) provide a row of data after the header for each weather station, given values for each header variable." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
		cout << "WeatherStationAttributeTable.csv could not be opened." << endl;
		return false;
	}

	//Variables in file are: OBJECTID,USAF,WBAN,Station_Easting_m,Station_Northing_m,Station_Elev_m,Station_Slope_deg,Station_Aspect_deg,Station_TI_option,NLCD_Class,Station_TC_per,Station_IC_per,Station_SVeg_per,Station_Soil_per,Station_Water_per, Station_Path_folder
	string Station_OBJECTID, Station_USAF, Station_WBAN, Station_Elevation_m_str, Station_Slope_deg_str, Station_Aspect_deg_str,
		Station_TI_option, Station_NLCD_Class, Station_TC_per_str, Station_IC_per_str, Station_SVeg_per_str, Station_Soil_per_str, Station_Water_per_str, Station_Path_folder, Station_Easting_m_str, Station_Northing_m_str;
	double Station_Elevation_m, Station_Slope_deg, Station_Aspect_deg, Station_TC_per, Station_IC_per, Station_SVeg_per, Station_Soil_per, Station_Water_per, Station_Easting_m, Station_Northing_m;
	string header, temp_line, temp_string;

	getline(WeatherStation, header);
	//While the next data exist, read a line in as the temp_line
	while (getline(WeatherStation, temp_line)) {
		//read cells within this line
		istringstream readlinedata(temp_line);

		getline(readlinedata, Station_OBJECTID, ',');
		getline(readlinedata, Station_USAF, ',');
		getline(readlinedata, Station_WBAN, ',');
		getline(readlinedata, Station_Easting_m_str, ',');
		getline(readlinedata, Station_Northing_m_str, ',');
		getline(readlinedata, Station_Elevation_m_str, ',');
		getline(readlinedata, Station_Slope_deg_str, ',');
		getline(readlinedata, Station_Aspect_deg_str, ',');
		getline(readlinedata, Station_TI_option, ',');
		getline(readlinedata, Station_NLCD_Class, ',');
		getline(readlinedata, Station_TC_per_str, ',');
		getline(readlinedata, Station_IC_per_str, ',');
		getline(readlinedata, Station_SVeg_per_str, ',');
		getline(readlinedata, Station_Soil_per_str, ',');
		getline(readlinedata, Station_Water_per_str, ',');
		getline(readlinedata, Station_Path_folder, ',');

		//Station_Path_folder is updated by EnsureTrailingBackslash function to ensure it has no parenthesis and ends with backslash
		Station_Path_folder = EnsureTrailingBackslash(Station_Path_folder);

		//stod() from string to double
		Station_Easting_m = stod(Station_Easting_m_str);
		Station_Northing_m = stod(Station_Northing_m_str);
		Station_Elevation_m = stod(Station_Elevation_m_str);
		Station_Slope_deg = stod(Station_Slope_deg_str);
		Station_Aspect_deg = stod(Station_Aspect_deg_str);
		Station_TC_per = stod(Station_TC_per_str);
		Station_IC_per = stod(Station_IC_per_str);
		Station_SVeg_per = stod(Station_SVeg_per_str);
		Station_Soil_per = stod(Station_Soil_per_str);
		Station_Water_per = stod(Station_Water_per_str);

		Station_OBJECTID_vec.push_back(Station_OBJECTID);
		Station_USAF_vec.push_back(Station_USAF);
		Station_WBAN_vec.push_back(Station_WBAN);
		Station_Easting_m_vec.push_back(Station_Easting_m);
		Station_Northing_m_vec.push_back(Station_Northing_m);
		Station_Elev_m_vec.push_back(Station_Elevation_m);
		Station_Slope_deg_vec.push_back(Station_Slope_deg);
		Station_Aspect_deg_vec.push_back(Station_Aspect_deg);
		Station_TI_option_vec.push_back(Station_TI_option);
		Station_NLCD_Class_vec.push_back(Station_NLCD_Class);
		Station_TC_per_vec.push_back(Station_TC_per);
		Station_IC_per_vec.push_back(Station_IC_per);
		Station_SVeg_per_vec.push_back(Station_SVeg_per);
		Station_Soil_per_vec.push_back(Station_Soil_per);
		Station_Water_per_vec.push_back(Station_Water_per);
		Station_Path_folder_vec.push_back(Station_Path_folder);
	}
}

//readWaterFeaturesMap function reads water features such as rivers and lakes, typically from NHD (National Hydrography Data)
void Inputs::readWaterFeaturesMap() {
	ifstream readWaterFeaturesMap(Directory_Input_CLArg + "nhdwaters10.asc");
	if (!readWaterFeaturesMap) {
		cout << "Warning: nhdwaters10.asc could not be opened." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the nhdwaters10.asc is missing there is no tree cover data to drive the simulation." << endl;
		cout << "Correction: Create a nhdwaters10.asc file, either using DEM generated water layer by turning the Flag_FlowDirection_Source flag to DEM." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	string buffer;
	readWaterFeaturesMap >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;
	double tempdata = 0.0;
	int count = 0;

	//while loop to read water as tempdata (fraction) from readTreeCover while count < product of nrows and ncols
	while (readWaterFeaturesMap >> tempdata && count < nrows * ncols) {
		//if tempdata not equal to NODATA then 
		if (tempdata != NODATA) {
			//non-water area
			if (tempdata == 0) { 
				//Waters10.push_back(0);
				Waters01.push_back(1); 
				Waters0.push_back(foldersize);
			}
			//water area
			if (tempdata == 1) {
				//Waters10.push_back(1);
				Waters01.push_back(0);
				Waters0.push_back(0);
			}
		}
		//else tempdata equals NODATA 
		else {
			//TreeCoverData (%) is defined as NODATA_code
			//Waters10.push_back(NODATA);
			Waters01.push_back(NODATA);
			Waters0.push_back(NODATA);
		}
		++count;
	}
	readWaterFeaturesMap.close();
	cout << "Finish reading nhdwaters10.asc." << endl;
}

//readNHDFlowDirectionMap function reads NHD (National Hydrography Data) Flow Direction map, which forces flow to planimetric hydrographic features 
void Inputs::readNHDFlowDirectionMap() {
	ifstream readNHDFlowDirectionMap(Directory_Input_CLArg + "nhdfdr.asc");
	if (!readNHDFlowDirectionMap) {
		cout << "Warning: nhdfdr.asc could not be opened." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		cout << "Explanation: When the nhdfdr.asc is missing there is no tree cover data to drive the simulation." << endl;
		cout << "Correction: Create a nhdfdr.asc file, either using DEM generated water layer by turning the Flag_FlowDirection_Source flag to DEM." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	string buffer;
	readNHDFlowDirectionMap >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;
	double tempdata = 0.0, code_temp;
	int count = 0;

	//while loop to read NHD flow direction as tempdata (fraction) from readTreeCover while count < product of nrows and ncols
	while (readNHDFlowDirectionMap >> tempdata && count < nrows * ncols) {
		//if tempdata not equal to NODATA then 
		if (tempdata != NODATA) {
			//convert flow direction code from ESRI format to Whitebox format
			code_temp = flowdircode_esri_to_whitebox(tempdata);
			//push back the code (whitebox) to the fdr vector 
			NHDfdr.push_back(code_temp);
			//push back the code (whitebox) to the reveresed fdr vector 
			NHDfdr_reverse.push_back(D8_flowdir_reverse(code_temp));
		}
		//else tempdata equals NODATA 
		else {
			//TreeCoverData (%) is defined as NODATA_code
			NHDfdr.push_back(NODATA);
			NHDfdr_reverse.push_back(NODATA);
		}
		++count;
	}
	readNHDFlowDirectionMap.close();
	cout << "Finish reading nhdfdr.asc." << endl;
}

//D8_flowdir_reverse function will reverse the NHD Flow Direction map from its D8 (1 of 8 directions), to provide estimates of dispersal area
double Inputs::D8_flowdir_reverse(double flowdir) {
	vector <double> dir;
	vector <double> dir_r;

	dir = { 1, 2, 4, 8, 16, 32, 64, 128 };
	dir_r = { 16, 32, 64, 128, 1, 2, 4, 8 };

	auto it_dir_r = dir_r.begin();
	for (auto it_dir = dir.begin(); it_dir != dir.end(); it_dir++) {
		if (flowdir == *it_dir) { 
			return *it_dir_r; 
			break;
		}
		else {
			it_dir_r++;
		}
	}
}

//flowdircode_esri_to_whitebox function converts flow direction from ESRI ArcGIS to Whitebox (e.g., HydroPlus)
//Note: HydroPlus uses the Whitebox flow directions nomenclature
double Inputs::flowdircode_esri_to_whitebox(double flowdir) {
	vector <double> dir_esri;//read from NHD fdr
	vector <double> dir_wb;//fdr in HydroPlus

	//dir_wb (HP) is flow direction vector that corresponds with dir_esri, each rotating clockwise, with 1st value any side of pixel
	//cte 2024 to SE=4, S=8, SW=16, W=32, NW=64, N=128, NE=1
	//cte 202411 Warning, dir_wb vector 1st value 2 is East in map, conflicting with inflowDirValues vector 1st value 16 is Northeast
	//Note: 
	dir_wb = { 2, 4, 8, 16, 32, 64, 128, 1 };
	//dir_esri is flow direction vector starting with East (E=1), rotating clockwise to SE=2, S=4, SW=8, W=16, NW=32, N=64, NE=128
	dir_esri = { 1, 2, 4, 8, 16, 32, 64, 128 };

	//Note: HydroPlus WhiteBox flowDir is 1 for NE corner, 2 for E, 4 for SE, 8 for S, 16 for SW, 32 for W, 64 for NW, 128 for N
	//Whitebox FDR		Whitebox -FDR	Pairs of x,y 
	//64	128	1		4	8	16		-1,-1	0,-1	1,-1
	//32		2		2		32		-1,0			1,0
	//16	8	4		1	128	64		-1,1	0,1		1,1

	//Note: ArcGIS ESRI flowDir is 128 for NE corner, 1 for E, 2 for SE, 4 for S, 8 for SW, 16 for W, 32 for NW, 64 for N
	//ESRI FDR			ESIR -FDR		Pairs of x,y 
	//32	64	128		2	4	8		-1,-1	0,-1	1,-1
	//16		1		1		16		-1,0			1,0
	//8		4	2		128	64	32		-1,1	0,1		1,1

	auto it_dir_wb = dir_wb.begin();
	for (auto it_dir_esri = dir_esri.begin(); it_dir_esri != dir_esri.end(); it_dir_esri++) {
		if (flowdir == *it_dir_esri) {
			return *it_dir_wb;
			break;
		}
		else { 
			it_dir_wb++;
		}
	}
}


//flowdircode_whitebox_to_esri function converts flow direction from Whitebox (e.g., HydroPlus) to ESRI ArcGIS
//Note: HydroPlus uses the Whitebox flow directions nomenclature
double Inputs::flowdircode_whitebox_to_esri(double flowdir) {
	vector <double> dir_esri;//read from NHD fdr
	vector <double> dir_wb;//fdr in HydroPlus

	//dir_wb (HP) is flow direction vector starting with East (E=2), rotating clockwise to SE=4, S=8, SW=16, W=32, NW=64, N=128, NE=1
	dir_wb = { 2, 4, 8, 16, 32, 64, 128, 1 };
	//dir_esri is flow direction vector starting with East (E=1), rotating clockwise to SE=2, S=4, SW=8, W=16, NW=32, N=64, NE=128
	dir_esri = { 1, 2, 4, 8, 16, 32, 64, 128 };
	
	//Note: HydroPlus WhiteBox flowDir is 1 for NE corner, 2 for E, 4 for SE, 8 for S, 16 for SW, 32 for W, 64 for NW, 128 for N
	//Whitebox FDR		Whitebox -FDR	Pairs of x,y 
	//64	128	1		4	8	16		-1,-1	0,-1	1,-1
	//32		2		2		32		-1,0			1,0
	//16	8	4		1	128	64		-1,1	0,1		1,1

	//Note: ArcGIS ESRI flowDir is 128 for NE corner, 1 for E, 2 for SE, 4 for S, 8 for SW, 16 for W, 32 for NW, 64 for N
	//ESRI FDR			ESIR -FDR		Pairs of x,y 
	//32	64	128		2	4	8		-1,-1	0,-1	1,-1
	//16		1		1		16		-1,0			1,0
	//8		4	2		128	64	32		-1,1	0,1		1,1

	auto it_dir_esri = dir_esri.begin();
	for (auto it_dir_wb = dir_wb.begin(); it_dir_wb != dir_wb.end(); it_dir_wb++) {
		if (flowdir == *it_dir_wb) {
			return *it_dir_esri;
			break;
		}
		else {
			it_dir_esri++;
		}
	}
}


// getManningn function obtains Manning roughness n values based on NLCD Class inputs and from Table 8.1 in Wurbs and James (2001) ISBN 0-13-081293-5
double Inputs::getManningn(double nlcd_categories) {
	vector <double> nlcd;
	vector <double> Manningn;
	//nlcd is the class of nlcd pixel
	nlcd = { 11, 21, 22, 23, 24, 31, 41, 42, 43, 52, 71, 81, 82, 90, 95 };
	// Lists of Manning roughness values, n, for each NLCD LC type, from Table 8.1 in Wurbs and James (2001) ISBN 0-13-081293-5
	Manningn = { 0.01, 0.011, 0.011, 0.011, 0.011, 0.011, 0.5, 0.5, 0.5, 0.24, 0.24, 0.13, 0.13, 0.095, 0.095 };

	auto it_Manningn = Manningn.begin();
	for (auto it_nlcd = nlcd.begin(); it_nlcd != nlcd.end(); it_nlcd++) {
		if (nlcd_categories == *it_nlcd) return *it_Manningn;
		else it_Manningn++;
	}
}

//getHydraulicradius function gets flow path hydraulic radius values, R (ft), based on NLCD Class and from Table 8.1 in Wurbs and James (2001) ISBN 0-13-081293-5
double Inputs::getHydraulicradius(double nlcd_categories) {
	vector <double> nlcd;
	vector <double> Hydraulicradius;
	//nlcd is the class of nlcd pixel
	nlcd = { 11, 21, 22, 23, 24, 31, 41, 42, 43, 52, 71, 81, 82, 90, 95 };
	//Lists of Hydraulic radius values, R (ft), for each NLCD LC type, from Table 8.1 in Wurbs and James (2001) ISBN 0-13-081293-5
	//Hydraulicradius = { 0.06, 0.06, 0.06, 0.06, 0.06, 0.06, 0.25, 0.25, 0.25, 0.15, 0.15, 0.04, 0.04, 1.0, 1.0 };
	//change from ft to meter
	Hydraulicradius = { 0.018288, 0.018288, 0.018288, 0.018288, 0.018288, 0.018288, 0.0762, 0.0762, 0.0762, 0.04572, 0.04572, 0.012192, 0.012192, 0.3048, 0.3048 };
	
	
	auto it_Hydraulicradius = Hydraulicradius.begin();
	for (auto it_nlcd = nlcd.begin(); it_nlcd != nlcd.end(); it_nlcd++) {
		if (nlcd_categories == *it_nlcd) return *it_Hydraulicradius;
		else it_Hydraulicradius++;
	}
}

//get_P_release_mean function obtains Phosphorus percent released from NLCD Class based on estimates of their filtering potential
double Inputs::get_P_release_mean(double nlcd_categories) {
	vector <double> NLCD_Class;
	vector <double> TotalPhosphorus_AverageReleased_percent;
	//NLCD_Class is NLCD Class for the pixel, using the Anderson Level 2 land cover classification described by the MRLC legend
	//Note: https://www.mrlc.gov/data/legends/national-land-cover-database-class-legend-and-description
	NLCD_Class = { 11, 21, 22, 23, 24, 31, 41, 42, 43, 52, 71, 81, 82, 90, 95 };
	//TotalPhosphorus_AverageReleased_percent (%) are based on values of Endreny and Wood (2003)
	//Note: Vector of values correspond with the NLCD_Class vector, e.g., NLCD_Class = 21 has TotalPhosphorus_AverageReleased_percent = 95
	//TotalPhosphorus_AverageReleased_percent = { 99, 100, 100, 100, 100, 100, 70, 70, 70, 36.27, 100, 100, 100, 51.63, 51.63 };
	//cte 2025 Li Zhang - why are these values different from the values above, attributed to Endreny adn Wood (2003)
	TotalPhosphorus_AverageReleased_percent = { 95, 70, 80, 95, 99, 50, 30, 25, 28, 45, 35, 85, 95, 35, 35 };
	//trapping
	// water 5 
	// Open Space 30 Low Intensity	20 	Medium Intensity 5 High Intensity 1 	
	// Barren Land 50 	
	// Deciduous Forest 70 	Evergreen Forest 75 	Mixed Forest 72 
	//Shrub	55				Grassland 65
	// 	Pasture/Hay 15 		Cultivated Crops 5 	
	// Woody Wetlands 65 	Emergent Herbaceous Wetlands 65		  

	auto it_P_release_mean = TotalPhosphorus_AverageReleased_percent.begin();
	for (auto it_nlcd = NLCD_Class.begin(); it_nlcd != NLCD_Class.end(); it_nlcd++) {
		if (nlcd_categories == *it_nlcd) return *it_P_release_mean;
		else it_P_release_mean++;
	}
}


// readSoilDepthMap_cm function reads soil depth from SSURGO maps in units of cm
void Inputs::readSoilDepthMap_cm() {
	string file_name = "s_depth.asc";;
	double tempdata = 0.0;
	int total_count = 0;
	double sum = 0.0;
	int count = 0;
	//min initialized to positive infinity
	double min = DBL_MAX;  

	ifstream readSoilDepthMap_cm(Directory_Input_CLArg + file_name);
	if (!readSoilDepthMap_cm) {
		cout << "Warning: s_depth.asc could not be opened." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	string buffer;
	readSoilDepthMap_cm >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;

	//while loop to read NHD flow direction as tempdata (fraction) from readTreeCover while count < product of nrows and ncols
	while (readSoilDepthMap_cm >> tempdata && total_count < nrows * ncols) {
		//if tempdata not equal to NODATA then 
		if (tempdata != NODATA) {
			s_depth.push_back(tempdata);
			// Update min to find min value
			if (tempdata < min && tempdata > 0) {
				min = tempdata;
			}
			sum += tempdata;
			++count;
		}
		//else tempdata equals NODATA 
		else {
			//soil depth (cm) is defined as NODATA_code
			s_depth.push_back(NODATA);
		}
		++total_count;
	}
	s_depth_avg = sum / count;
	
	soildepth_min = min;
	readSoilDepthMap_cm.close();
	cout << "Finish reading soil depth from prior SSURGO analysis." << "The mean soil depth s_depth_avg is: " << s_depth_avg << " cm, the minimum soil depth is: " << soildepth_min << " cm." << endl;
}


//readHydraulicConductivityMap_umps function reads hydraulic conductivity (um/s) from SSURGO maps
void Inputs::readHydraulicConductivityMap_umps() {
	string file_name = "s_ksat.asc";
	double tempdata = 0.0;
	int count = 0;
	//min initialized to positive infinity
	double min = DBL_MAX;

	ifstream readHydraulicConductivityMap_umps(Directory_Input_CLArg + file_name);
	if (!readHydraulicConductivityMap_umps) {
		cout << "Warning: s_ksat.asc could not be opened." << endl;
		cout << "Aborting: This warning triggers the HydroPlus simulation to abort." << endl;
		//Call abort function, which ends the HydroPlus.exe simulation
		abort();
	}
	string buffer;
	readHydraulicConductivityMap_umps >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;


	//while loop to read NHD flow direction as tempdata (fraction) from readTreeCover while count < product of nrows and ncols
	while (readHydraulicConductivityMap_umps >> tempdata && count < nrows * ncols) {
		//if tempdata not equal to NODATA then 
		if (tempdata != NODATA) {
			s_Ksat.push_back(tempdata);
			// Update min to find min value
			if (tempdata < min && tempdata > 0) {
				min = tempdata;
			}
		}
		//else tempdata equals NODATA 
		else {
			//TreeCoverData (%) is defined as NODATA_code
			s_Ksat.push_back(NODATA);
		}
		++count;
	}
	conductivity_min = min;
	readHydraulicConductivityMap_umps.close();
	cout << "Finish reading conductivity from prior SSURGO analysis, the minimum conductivity is: " << conductivity_min << " um/s." << endl;
}

//calculateSoilTransmissivityChangeUnits function converts transmissivity to m^2/day from SSURGO units of (cm*um)/s
void Inputs::calculateSoilTransmissivityChangeUnits() {
	double s_trans_temp = 0;
	double sum = 0.0;
	int count = 0;
	//ratio_m2_p_day_cm_um_p_s_to is the ratio of m^2/day to (cm*um)/s, where (cm*um)/s was used by Gridded SSURGO Soil Data for product of sil depth and conductivity
	double ratio_m2_p_day_cm_um_p_s_to = 0.000864;
	double ratio_m_to_cm = 0.01;
	double ratio_sec_to_day = 86400.0;
	double ratio_m_to_um = 0.000001;

	//Depth_SoilTransmissivityLayer_cm (cm) is read from s_depth (cm) as units from SSURGO
	auto Depth_SoilTransmissivityLayer_cm = s_depth.begin();

	//For HydraulicConductivity_SoilLayer_um_p_s (um/s) in vector of s_Ksat (um/s) as units from SSURGO
	for (double HydraulicConductivity_SoilLayer_um_p_s : s_Ksat) {
		//If HydraulicConductivity_SoilLayer_um_p_s (um/s) is not NODATA then
		if (HydraulicConductivity_SoilLayer_um_p_s != NODATA) {
			//set the conductivity = 0 to be the minimum non-zero HydraulicConductivity_SoilLayer_um_p_s to avoid log(0) in buffer calculation
			if (HydraulicConductivity_SoilLayer_um_p_s <= 0) HydraulicConductivity_SoilLayer_um_p_s = conductivity_min;
			//set the soildepth = 0 to be the minimum non-zero HydraulicConductivity_SoilLayer_um_p_s to avoid log(0) in buffer calculation
			if (*Depth_SoilTransmissivityLayer_cm <= 0) *Depth_SoilTransmissivityLayer_cm = soildepth_min;

			//s_trans_temp (m2/day) is product of Depth_SoilTransmissivityLayer_cm (cm), ratio_m_to_cm, HydraulicConductivity_SoilLayer_um_p_s (um/s), ratio_sec_to_day, and ratio_m_to_um
			//Note: Product ratio_m_to_cm, ratio_sec_to_day, ratio_m_to_um = 0.000864 = (1/100)*((60*60*24)/1)*(1/10^6) = m/cm * s/day * m/um
			s_trans_temp = *Depth_SoilTransmissivityLayer_cm * ratio_m_to_cm * HydraulicConductivity_SoilLayer_um_p_s * ratio_sec_to_day * ratio_m_to_um;

			//s_trans (m/day) vector is appended with s_trans_temp (m/day)
			s_trans.push_back(s_trans_temp);

			//sum (m/day) is increased by s_trans_temp (m/day)
			sum += s_trans_temp;
			//count is increased
			++count;
		}
		//Else If HydraulicConductivity_SoilLayer_um_p_s (um/s) is NODATA then
		else {
			//s_trans (um/s) is set to NODATA
			s_trans.push_back(NODATA);
		}
		//Depth_SoilTransmissivityLayer_cm (cm) is increased
		++Depth_SoilTransmissivityLayer_cm;
	}

	//s_trans_avg (m/day) is quotient of sum and count
	s_trans_avg = sum / count;
	//s_trans_avg = calculateMean(s_trans);
	cout << "The mean transmissivity is: " << s_trans_avg << " m^2/day." << endl;
	cout << "Finish calculating transmissivity from prior SSURGO analysis." << endl;
}

// readWatertableMap_cm function reads watertable depth from SSURGO maps in units of cm
bool Inputs::readWatertableMap_cm() {
	string file_name = "s_wtable.asc";;
	double tempdata = 0.0;
	int total_count = 0;

	ifstream readWatertable_cm(Directory_Input_CLArg + file_name);
	if (!readWatertable_cm) {
		return false;
	}
	string buffer;
	readWatertable_cm >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;

	//while loop to read NHD flow direction as tempdata (fraction) from readTreeCover while count < product of nrows and ncols
	while (readWatertable_cm >> tempdata && total_count < nrows * ncols) {
		//if tempdata not equal to NODATA then 
		if (tempdata != NODATA) {
			s_wtable.push_back(tempdata);
		}
		//else tempdata equals NODATA 
		else {
			//watertable depth (cm) is defined as NODATA_code
			s_wtable.push_back(NODATA);
		}
		++total_count;
	}
	
	readWatertable_cm.close();
	cout << "Finish reading watertable(cm) from prior SSURGO analysis." << endl;
	return true;
}


//calculateMean function calculates mean values
double Inputs::calculateMean(const vector<double>& data) {
	//sum initialized to zero
	double sum = 0.0;
	//count initialized to zero
	int count = 0;

	//For loop of value through all data
	for (double value : data) {
		//if value not NODATA then
		if (value != NODATA) {
			//sum increased by value
			sum += value;
			//count increased
			++count;
		}
	}

	//If count equals 0 then return 0
	if (count == 0) {
		cout << "No valid data to calculate mean." << endl;
		//return 0
		return 0.0; 
	}

	//return the quotient of sum and count, which is a simple mean
	return sum / count;
}

//void Inputs::writeAsciiMap function will write an ASCII Map for a given vector 
void Inputs::writeAsciiMap(const string& MapData_Name_str, const vector<double>& MapData_vec) {
	
	//outfile defined using ofstream function directed to OutputFolder_Path + MapData_Name_str + .asc
	ofstream outfile(SimulationStringParams["OutputFolder_Path"] + "z_map_" + MapData_Name_str + ".asc");

	//outfile assigned header width boundary of 14 and 15, variable ncols defined within Inputs.cpp
	outfile << left << setw(14) << "ncols" << setw(15) << ncols << endl;
	outfile << left << setw(14) << "nrows" << setw(15) << nrows << endl;
	outfile << left << setw(14) << "xllcorner" << setw(15) << setprecision(15) << xllcorner << endl;
	outfile << left << setw(14) << "yllcorner" << setw(15) << setprecision(15) << yllcorner << endl;
	outfile << left << setw(14) << "cellsize" << setw(15) << setprecision(15) << cellsize << endl;
	outfile << left << setw(14) << "NODATA_value" << setw(15) << NODATA << endl;

	//For loop of MapPixel_ID in MapData_vec.size
	//MapPixel_ID is map vector location, i.e., DataDrawer_ID, computed as a function of col_max and pixel row, column ID
	//Note: Conversion Eq: MapPixel_ID = row * col_max + col, with MapPixel_ID starting at 0, row & col starting at 1
	//Note: Conversion Eq: row = [int(MapPixel_ID/col_max)+1]; col = [mod(MapPixel_ID,col_max)+1]
	for (int MapPixel_ID = 0; MapPixel_ID < MapData_vec.size(); ++MapPixel_ID) {
		//row = (MapPixel_ID / ncols) + 1, extracted from vector
		int row = (MapPixel_ID / ncols) + 1;
		//col = (MapPixel_ID % ncols) + 1, extracted from vector
		int col = (MapPixel_ID % ncols) + 1;
		//newLine = (col != ncols) ? false : true, determined for each pixel
		bool newLine = (col != ncols) ? false : true;

		//if (NLCDLandCoverData[MapPixel_ID] != NODATA) then 
		if (NLCDLandCoverData[MapPixel_ID] != NODATA) {
			//outfile writes MapData_vec[MapPixel_ID]
			outfile << fixed << setprecision(5) << MapData_vec[MapPixel_ID];
		}
		else {
			//outfile writes NODATA
			outfile << fixed << setprecision(0) << NODATA;

		}
		//If newLine is true, then write return with endl;
		if (newLine) {
			outfile << endl;
		}
		//Else write space for next data point
		else {
			outfile << " ";
		}
	}
	//outfile close function
	outfile.close();
}

//Inputs::ProcessTemperatureExecutionParams function reads HydroPlusConfig.xml inputs for Spatial Buffer with GI
void Inputs::ProcessSpatialBufferGIExecutionParams() {
	for (auto i : BufferSpatialWeightingStringParams) {
		/*if (i.first.find("GI_type") != string::npos) {
			GI_type = i.second;
		}*/
		if (i.first.find("GI_ulcorner") != string::npos) {
			GI_ulcorner_rows.push_back(atoi(i.second.substr(0, i.second.find(",")).c_str()) - 1);//user provided/ArcGIS row col start from 1
			GI_ulcorner_cols.push_back(atoi(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str()) - 1);//user provided/ArcGIS row col start from 1

		}
		if (i.first.find("GI_lrcorner") != string::npos) {
			GI_lrcorner_rows.push_back(atoi(i.second.substr(0, i.second.find(",")).c_str()) - 1);//user provided/ArcGIS row col start from 1
			GI_lrcorner_cols.push_back(atoi(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str()) - 1);//user provided/ArcGIS row col start from 1
		}
	}
}

//calculate the contributing area for GIs
//need to be called after getting TerrainProcessor::FlowAccumulation_Map
//cab't be called within Buffer since the FlowAccumulation_Map has been replaced

double Inputs::BufferGIContributingAreaCalc(int gi_index) {
	
	double total_contributing_area = 0;
	//if Flag_FlowDirection_Source is NHD, replace DEM fdr with NHD fdr and redo the flow accumulation to get the contributing cell list
	if (Inputs::BufferSpatialWeightingStringParams["Flag_FlowDirection_Source"] == "NHD") {
		cout << "Generating water layer from nhdwaters10.asc. " << endl;
		//Call ReceivingWater_ZeroValue_CreateMap function sending input, saving waters01 and waters0 layer to buffer
		Buffer::ReceivingWater_ZeroValue_CreateMap_from_NHD();
		cout << "Updating flow direction from nhdfdr.asc. " << endl;
		//Algorithm to replace DEM derived flow direction (FDR) with NHD FDR values, for the following flow accum	
		Buffer::ReplaceFlowDirectionOrganizerWithVector(Inputs::NHDfdr);
		// Re-check conditions for saving contributing/dispersal area pixels
		TerrainProcessor::setContributingDispersalFlags();
		//start flow accumulation process to get the contributing pixels
		//accumulate the non-water area
		TerrainProcessor::FlowAccum(Buffer::waters01, TerrainProcessor::DX_vec);
		//mark ContributingArea been saved and avoid running it again in the future accumulation
		TerrainProcessor::saveContributingDispersalArea = false;
	}
	
	//start creating contributing area 01 map, to avoid accumulating one cell several times
	Buffer::ContributingAreaPixels_map_writeout.clear();
	Buffer::DispersalAreaPixels_map_writeout.clear();
	Buffer::ContributingAreaPixels_map_writeout.resize(nrows * ncols, NODATA);
	Buffer::DispersalAreaPixels_map_writeout.resize(nrows * ncols, NODATA);

	int GI_ulcorner_row = Inputs::GI_ulcorner_rows[gi_index];
	int GI_ulcorner_col = Inputs::GI_ulcorner_cols[gi_index];
	int GI_lrcorner_row = Inputs::GI_lrcorner_rows[gi_index];
	int GI_lrcorner_col = Inputs::GI_lrcorner_cols[gi_index];
	
	// Iterate over each pixel in the block
	for (int gi_row = GI_ulcorner_row; gi_row <= GI_lrcorner_row; ++gi_row) {
		for (int gi_col = GI_ulcorner_col; gi_col <= GI_lrcorner_col; ++gi_col) {
			// Check if the specified row and column are within the map boundaries
			if (gi_row >= 0 && gi_row < nrows && gi_col >= 0 && gi_col < ncols) {
				//create the maps, for  multiple gi pixels, it's cumulative
				Buffer::Contributing_dispersal_CreateMap(gi_row, gi_col);
			}
		}
	}
	//finish creating contributing area 01 map

	//else {//won't work when using NHD flow direction since buffer has not been run yet. 
	for (int pixelIndex = 0; pixelIndex < Buffer::ContributingAreaPixels_map_writeout.size(); pixelIndex++) {
		//double contributingPixels = Buffer::ContributingAreaPixels_map_writeout[pixelIndex];
		if (Buffer::ContributingAreaPixels_map_writeout[pixelIndex] >= 1) {
			total_contributing_area += cellsize * cellsize;
		}
	}
	return total_contributing_area;
}

//calculate the area for GIs
//need to be called after getting TerrainProcessor::FlowAccumulation_Map
double Inputs::BufferGIAreaCalc(int gi_index) {
	// Check if the GI index is within bounds for all vectors
	if (gi_index >= GI_ulcorner_rows.size() ||
		gi_index >= GI_ulcorner_cols.size() ||
		gi_index >= GI_lrcorner_rows.size() ||
		gi_index >= GI_lrcorner_cols.size()) {

		cout << "Warning: Mismatched GI vector lengths detected." << endl;
		cout << "Aborting: Simulation is aborted due to mismatched vector lengths." << endl;
		cout << "Explanation: The number of upper-left and lower-right corner coordinates (GI_ulcorner and GI_lrcorner) must match the number of GI folders." << endl;
		cout << "Correction: Check the GI DataFolder and ensure that the locations specified in BufferSpatialWeightingStringParams (pairs of GI_ulcorner and GI_lrcorner) have the same number of elements as the GI folders." << endl;
		return 0.0;
	}

	// Retrieve the coordinates for the current GI
	int GI_ulcorner_row = GI_ulcorner_rows[gi_index];
	int GI_ulcorner_col = GI_ulcorner_cols[gi_index];
	int GI_lrcorner_row = GI_lrcorner_rows[gi_index];
	int GI_lrcorner_col = GI_lrcorner_cols[gi_index];

	// Calculate the total area for the GI
	double gi_area = ((GI_lrcorner_row - GI_ulcorner_row) + 1) * ((GI_lrcorner_col - GI_ulcorner_col) + 1) * cellsize * cellsize;

	return gi_area;
}

//MapAllignment_check function will test if maps used in Buffer Model are alligned, i.e., spatial congruent for stacked calculation
//Note: Algorithm determines if two 1D vectors representing maps are not aligned
bool Inputs::MapAllignment_check(const vector<double>& map1, const vector<double>& map2, const string& map1Name, const string& map2Name) {
	bool findmismatch = false;
	double NODATA_code = LocationParams["NODATA"];
	//cte 2024 Refactor
	cout << "MapAllignment_check map1Name = " << map1Name << " map2Name = " << map2Name << endl;

	// Check if the vectors have the same size
	if (map1.size() != map2.size()) {
		cout << "Vectors " << map1Name << " and " << map2Name << " are not aligned due to size mismatch." << endl;
		findmismatch = true;
		return false; // Vectors are not aligned due to size mismatch
	}
	// Compare corresponding elements of the vectors
	for (size_t i = 0; i < map1.size(); ++i) {
		// Check if both elements are -9999 (nodata) or if they are different
		if ((map1[i] == NODATA && map2[i] != NODATA) || (map1[i] != NODATA_code && map2[i] == NODATA_code)) {
			//cout << "Warning: Nodata cells do not match in Pixel " << i << " of " << map1Name << " and " << map2Name
				//<< ", " << map1Name << " is " << map1[i] << ", " << map2Name << " is " << map2[i] << endl;
			findmismatch = true;
		}
	}

	if (findmismatch) {
		cout << "Warning: Nodata cells do not match in NODADA Pixels. " << endl;
		return false;
	}
	else {
		// Maps are aligned, and nodata cells match
		return true;
	}
}

//Projection_Albers_to_LatitudeLongitude creates vector pair of double latitude and longitude from easting and northing for WKID 102008
//Note: Source: Snyder, J.P., (1987). Map projections: A working manual. USGS Professional Paper 1395 and EPSG documentation
//Note: Reverses spherical earth algorithm for Latitude Longitude to Albers Equal Area (AEA) projection formula
pair<double, double> Projection_Albers_to_LatitudeLongitude(double mapEasting_m, double mapNorthing_m, const ProjectionParams_Albers& ProjectionParams_Albers) {

	//ScaleFactor_StandardParallels = (sin(ProjectionParams_Albers.phi1) + sin(ProjectionParams_Albers.phi2)) / 2.0
	double ScaleFactor_StandardParallels = (sin(ProjectionParams_Albers.phi1) + sin(ProjectionParams_Albers.phi2)) / 2.0;
	//Coef_1_ScaleFactor = pow(cos(ProjectionParams_Albers.phi1), 2) + 2 * ScaleFactor_StandardParallels * sin(ProjectionParams_Albers.phi1)
	double Coef_1_ScaleFactor = pow(cos(ProjectionParams_Albers.phi1), 2) + 2 * ScaleFactor_StandardParallels * sin(ProjectionParams_Albers.phi1);
	//Distance_to_ReferenceParallel_m = Earth_Radius_m * sqrt(Coef_1_ScaleFactor - 2 * ScaleFactor_StandardParallels * sin(ProjectionParams_Albers.phi0)) / ScaleFactor_StandardParallels
	double Distance_to_ReferenceParallel_m = Earth_Radius_m * sqrt(Coef_1_ScaleFactor - 2 * ScaleFactor_StandardParallels * sin(ProjectionParams_Albers.phi0)) / ScaleFactor_StandardParallels;

	//Distance_to_MapPixel_m = sqrt(mapEasting_m * mapEasting_m + pow(Distance_to_ReferenceParallel_m - mapNorthing_m, 2))
	double Distance_to_MapPixel_m = sqrt(mapEasting_m * mapEasting_m + pow(Distance_to_ReferenceParallel_m - mapNorthing_m, 2));
	//Angle_from_ReferenceParallel_rad = atan2(mapEasting_m, Distance_to_ReferenceParallel_m - mapNorthing_m)
	double Angle_from_ReferenceParallel_rad = atan2(mapEasting_m, Distance_to_ReferenceParallel_m - mapNorthing_m);

	//Latitude_MapPixel_rad = asin((Coef_1_ScaleFactor - (Distance_to_MapPixel_m * ScaleFactor_StandardParallels / Earth_Radius_m)) / (2 * ScaleFactor_StandardParallels))
	double Latitude_MapPixel_rad = asin((Coef_1_ScaleFactor - (Distance_to_MapPixel_m * ScaleFactor_StandardParallels / Earth_Radius_m)) / (2 * ScaleFactor_StandardParallels));
	//Longitude_MapPixel_rad = ProjectionParams_Albers.lambda0 + Angle_from_ReferenceParallel_rad / ScaleFactor_StandardParallels
	double Longitude_MapPixel_rad = ProjectionParams_Albers.lambda0 + Angle_from_ReferenceParallel_rad / ScaleFactor_StandardParallels;

	//Latitude_MapPixel_dd = Latitude_MapPixel_rad * 180.0 / M_PI, convert radian to decimal degree
	double Latitude_MapPixel_dd = Latitude_MapPixel_rad * 180.0 / M_PI;
	//Longitude_MapPixel_dd = Longitude_MapPixel_rad * 180.0 / M_PI, convert radian to decimal degree
	double Longitude_MapPixel_dd = Longitude_MapPixel_rad * 180.0 / M_PI;

	//return Latitude_MapPixel_dd and Longitude_MapPixel_dd 
	return { Latitude_MapPixel_dd, Longitude_MapPixel_dd };
}

//Projection_Lambert_to_LatitudeLongitude creates vector pair of double latitude and longitude from easting and northing for WKID 3034
//Note: Source: Snyder, J.P., (1987). Map projections: A working manual. USGS Professional Paper 1395 and EPSG documentation
//Note: Reverses spherical earth algorithm for Latitude Longitude to Lambert Conformal Conic (LCC) projection formula
pair<double, double> Projection_Lambert_to_LatitudeLongitude(double mapEasting_m, double mapNorthing_m, const ProjectionParams_Lambert& ProjectionParams_Lambert) {
	// Define constants
	const double ScaleFactor_StandardParallels = (log(cos(ProjectionParams_Lambert.phi1) / cos(ProjectionParams_Lambert.phi2))) /
		(log(tan(M_PI / 4 + ProjectionParams_Lambert.phi2 / 2) / tan(M_PI / 4 + ProjectionParams_Lambert.phi1 / 2)));
	const double Coef_1_ScaleFactor = cos(ProjectionParams_Lambert.phi1) * pow(tan(M_PI / 4 + ProjectionParams_Lambert.phi1 / 2), ScaleFactor_StandardParallels) / ScaleFactor_StandardParallels;
	const double rho0 = Earth_Radius_m * Coef_1_ScaleFactor / pow(tan(M_PI / 4 + ProjectionParams_Lambert.phi0 / 2), ScaleFactor_StandardParallels);

	// Calculate Distance_to_CentralParallel_m and Angle_from_CentralMeridian_rad
	const double Delta_Easting_m = mapEasting_m - ProjectionParams_Lambert.falseEasting;
	const double Delta_Northing_m = rho0 - (mapNorthing_m - ProjectionParams_Lambert.falseNorthing);
	const double Distance_to_CentralParallel_m = sqrt(Delta_Easting_m * Delta_Easting_m + Delta_Northing_m * Delta_Northing_m);
	const double Angle_from_CentralMeridian_rad = atan2(Delta_Easting_m, Delta_Northing_m);

	// Calculate Latitude and Longitude
	const double Latitude_MapPixel_rad = 2 * atan(pow(Earth_Radius_m * Coef_1_ScaleFactor / Distance_to_CentralParallel_m, 1 / ScaleFactor_StandardParallels)) - M_PI / 2;
	const double Longitude_MapPixel_rad = ProjectionParams_Lambert.lambda0 + Angle_from_CentralMeridian_rad / ScaleFactor_StandardParallels;

	// Convert to decimal degrees
	double Latitude_MapPixel_dd = Latitude_MapPixel_rad * 180.0 / M_PI;
	double Longitude_MapPixel_dd = Longitude_MapPixel_rad * 180.0 / M_PI;

	// Return the pair of latitude and longitude
	return { Latitude_MapPixel_dd, Longitude_MapPixel_dd };
}

//Projection_UTM_to_LatitudeLongitude creates vector pair of double latitude and longitude from easting and northing for WKID 32600 or 32700
//Note: Source: Snyder, J.P., (1987). Map projections: A working manual. USGS Professional Paper 1395 and EPSG documentation
//Note: Reverses spherical earth algorithm for Latitude Longitude to Universal Transverse Mercator (UTM) projection formula
pair<double, double> Projection_UTM_to_LatitudeLongitude(double mapEasting_m, double mapNorthing_m, const ProjectionParams_UTM& projectionParams) {
	const double Earth_Eccentricity_squared = pow(Earth_Eccentricity, 2);
	// Second eccentricity squared
	const double Coeff_1_Eccentricity = Earth_Eccentricity_squared / (1 - Earth_Eccentricity_squared);
	const double ScaleFactor_UTM = projectionParams.scaleFactor;

	// Adjust easting and northing by false origins
	double Delta_Easting_m = mapEasting_m - projectionParams.falseEasting;
	double Delta_Northing_m = mapNorthing_m;

	// Adjust for southern hemisphere
	if (Delta_Northing_m < 0) {
		Delta_Northing_m += projectionParams.falseNorthing;
	}

	// Calculate the meridional arc
	double Meridional_Arc_m = Delta_Northing_m / ScaleFactor_UTM;

	// Calculate the footpoint latitude
	double Latitude_Footprint_m = Meridional_Arc_m / (Earth_Radius_m * (1 - Earth_Eccentricity_squared / 4 - 3 * Earth_Eccentricity_squared * Earth_Eccentricity_squared / 64 - 5 * Earth_Eccentricity_squared * Earth_Eccentricity_squared * Earth_Eccentricity_squared / 256));
	double Coeff_2_Eccentricity = (1 - sqrt(1 - Earth_Eccentricity_squared)) / (1 + sqrt(1 - Earth_Eccentricity_squared));

	double Latitude_CentralMeridian_rad = Latitude_Footprint_m
		+ (3 * Coeff_2_Eccentricity / 2 - 27 * Coeff_2_Eccentricity * Coeff_2_Eccentricity * Coeff_2_Eccentricity / 32) * sin(2 * Latitude_Footprint_m)
		+ (21 * Coeff_2_Eccentricity * Coeff_2_Eccentricity / 16 - 55 * Coeff_2_Eccentricity * Coeff_2_Eccentricity * Coeff_2_Eccentricity * Coeff_2_Eccentricity / 32) * sin(4 * Latitude_Footprint_m)
		+ (151 * Coeff_2_Eccentricity * Coeff_2_Eccentricity * Coeff_2_Eccentricity / 96) * sin(6 * Latitude_Footprint_m)
		+ (1097 * Coeff_2_Eccentricity * Coeff_2_Eccentricity * Coeff_2_Eccentricity * Coeff_2_Eccentricity / 512) * sin(8 * Latitude_Footprint_m);

	// Calculate latitude and longitude
	double Coeff_3_Eccentricity = Coeff_1_Eccentricity * pow(cos(Latitude_CentralMeridian_rad), 2);
	double Tangent_Latitude_squared = pow(tan(Latitude_CentralMeridian_rad), 2);
	double Radius_PrimeVerticalCurvature = Earth_Radius_m / sqrt(1 - Earth_Eccentricity_squared * pow(sin(Latitude_CentralMeridian_rad), 2));
	double Radius_MeridianCurvature = Earth_Radius_m * (1 - Earth_Eccentricity_squared) / pow(1 - Earth_Eccentricity_squared * pow(sin(Latitude_CentralMeridian_rad), 2), 1.5);
	double Delta_Easting_Normalized_m = Delta_Easting_m / (Radius_PrimeVerticalCurvature * ScaleFactor_UTM);

	double Latitude_MapPixel_rad = Latitude_CentralMeridian_rad
		- (Radius_PrimeVerticalCurvature * tan(Latitude_CentralMeridian_rad) / Radius_MeridianCurvature)
		* (Delta_Easting_Normalized_m * Delta_Easting_Normalized_m / 2
			- (5 + 3 * Tangent_Latitude_squared + 10 * Coeff_3_Eccentricity - 4 * Coeff_3_Eccentricity * Coeff_3_Eccentricity - 9 * Coeff_1_Eccentricity) * pow(Delta_Easting_Normalized_m, 4) / 24
			+ (61 + 90 * Tangent_Latitude_squared + 298 * Coeff_3_Eccentricity + 45 * Tangent_Latitude_squared * Tangent_Latitude_squared - 252 * Coeff_1_Eccentricity - 3 * Coeff_3_Eccentricity * Coeff_3_Eccentricity) * pow(Delta_Easting_Normalized_m, 6) / 720);

	double Longitude_MapPixel_rad = projectionParams.centralMeridian
		+ (Delta_Easting_Normalized_m
			- (1 + 2 * Tangent_Latitude_squared + Coeff_3_Eccentricity) * pow(Delta_Easting_Normalized_m, 3) / 6
			+ (5 - 2 * Coeff_3_Eccentricity + 28 * Tangent_Latitude_squared - 3 * Coeff_3_Eccentricity * Coeff_3_Eccentricity + 8 * Coeff_1_Eccentricity + 24 * Tangent_Latitude_squared * Tangent_Latitude_squared) * pow(Delta_Easting_Normalized_m, 5) / 120)
		/ cos(Latitude_CentralMeridian_rad);

	// Convert radians to degrees
	double Latitude_MapPixel_dd = Latitude_MapPixel_rad * 180.0 / M_PI;
	double Longitude_MapPixel_dd = Longitude_MapPixel_rad * 180.0 / M_PI;

	return { Latitude_MapPixel_dd, Longitude_MapPixel_dd };
}

//ProjectionParams_UTM initializeUTMParams function convertes MapProjection_WKID into params values
ProjectionParams_UTM initializeUTMParams(int MapProjection_WKID) {
	ProjectionParams_UTM params;
	// Extract the last two digits for the zone
	params.zone = MapProjection_WKID % 100;
	// Central meridian in radians
	params.centralMeridian = (params.zone * 6 - 183) * M_PI / 180.0;
	// UTM always uses 500,000m false easting
	params.falseEasting = 500000.0;
	// 0 for northern hemisphere, 10,000,000 for southern
	params.falseNorthing = (MapProjection_WKID / 100 == 326) ? 0.0 : 10000000.0;
	// UTM scale factor
	params.scaleFactor = 0.9996;

	return params;
}

//LatitudeLongitude_Projection_Manager function will call Projection_Albers_to_LatitudeLongitude function with pointer to address of Latitude_Longitude_vectorPair_dd
void Inputs::LatitudeLongitude_Projection_Manager(vector<vector<pair<double, double>>>& Latitude_Longitude_vectorPair_dd) {
	//MapMetadata struct initializes instance mapMetadata
	MapMetadata mapMetadata;
	//mapMetadata.lowerLeftX_m = input->xllcorner
	mapMetadata.lowerLeftX_m = xllcorner;
	//mapMetadata.lowerLeftY_m = input->yllcorner
	mapMetadata.lowerLeftY_m = yllcorner;
	//mapMetadata.cellSize_m = input->cellsize
	mapMetadata.cellSize_m = cellsize;
	//mapMetadata.ncols = input->ncols
	mapMetadata.ncols = ncols;
	//mapMetadata.nrows = input->nrows
	mapMetadata.nrows = nrows;

	//cte 2025 Refactor to put this as HydroPlusConfig input
	
	int MapProjection_WKID = LocationParams["MapProjection_WKID"];

	// Initialize projection parameters and function pointer
	function<pair<double, double>(double, double)> projection_to_LatitudeLongitude_Function;
	// Albers Equal Area
	if (MapProjection_WKID == 102008) { 
		ProjectionParams_Albers projectionParams;
		projection_to_LatitudeLongitude_Function = [&projectionParams](double easting, double northing) {
			return Projection_Albers_to_LatitudeLongitude(easting, northing, projectionParams);
		};
	}
	// Universal Transverse Mercator
	else if ((MapProjection_WKID >= 32601 && MapProjection_WKID <= 32660) ||
		(MapProjection_WKID >= 32701 && MapProjection_WKID <= 32760)) { 
		ProjectionParams_UTM projectionParams = initializeUTMParams(MapProjection_WKID);
		projection_to_LatitudeLongitude_Function = [&projectionParams](double easting, double northing) {
			return Projection_UTM_to_LatitudeLongitude(easting, northing, projectionParams);
		};
	}
	// Lambert Conformal Conic
	else if (MapProjection_WKID == 3034) { 
		ProjectionParams_Lambert projectionParams;
		projection_to_LatitudeLongitude_Function = [&projectionParams](double easting, double northing) {
			return Projection_Lambert_to_LatitudeLongitude(easting, northing, projectionParams);
		};
	}
	//cte 2024 allowing for default call of Albers
	else {
		ProjectionParams_Albers projectionParams;
		projection_to_LatitudeLongitude_Function = [&projectionParams](double easting, double northing) {
			return Projection_Albers_to_LatitudeLongitude(easting, northing, projectionParams);
			};
		cerr << "Unsupported WKID: " << MapProjection_WKID << ". Using Albers WKID 102008" << endl;
		cerr << "HydroPlusConfig.xml TemperatureLocationParams MapProjection_WKID parameter" << endl;
		//throw invalid_argument("Unsupported WKID");
	}	//Latitude_Longitude_vectorPair_dd.resize to contain mapMetadata.nrows and mapMetadata.ncols

	//Latitude_Longitude_vectorPair_dd resize as the latitude-longitude vector
	Latitude_Longitude_vectorPair_dd.resize(mapMetadata.nrows, vector<pair<double, double>>(mapMetadata.ncols));

	//For loop row to mapMetadata.nrows
	for (int row = 0; row < mapMetadata.nrows; ++row) {
		//For loop col to mapMetadata.nrows
		for (int col = 0; col < mapMetadata.ncols; ++col) {
			//mapEasting_m = mapMetadata.lowerleftX + col * mapMetadata.cellsize
			double mapEasting_m = mapMetadata.lowerLeftX_m + col * mapMetadata.cellSize_m;
			//mapNorthing_m = mapMetadata.lowerLeftY_m + (mapMetadata.nrows - row - 1) * mapMetadata.cellSize_m
			double mapNorthing_m = mapMetadata.lowerLeftY_m + (mapMetadata.nrows - row - 1) * mapMetadata.cellSize_m;
			//Latitude_Longitude_vectorPair_dd[row][col] = projection_to_LatitudeLongitude_Function(mapEasting_m, mapNorthing_m)
			//Note: projection_to_LatitudeLongitude_Function has pointer to projectionParams and Projection_Albers_to_LatitudeLongitude or 
			Latitude_Longitude_vectorPair_dd[row][col] = projection_to_LatitudeLongitude_Function(mapEasting_m, mapNorthing_m);
			if (row == 120 && col == 120) {
				//cte 2024
				cout << "MapProjection_WKID= " << MapProjection_WKID << endl;
				cout << "Latitude_dd= " << Latitude_Longitude_vectorPair_dd[row][col].first << " Longitude_dd= " << Latitude_Longitude_vectorPair_dd[row][col].second << endl;
			}
		}
	}
}
//EnsureTrailingBackslash function to ensure it has no parenthesis and ends with backslash
string EnsureTrailingBackslash(const string& directoryPath) {
	//cleanedPath defined as path to remove any quotation marks
	string cleanedPath = directoryPath;
	//cleanedPath uses .erase with remove(cleanedPath.begin() and cleanedPath.end(), cleanedPath.end() to emove quotation marks
	cleanedPath.erase(remove(cleanedPath.begin(), cleanedPath.end(), '\"'), cleanedPath.end());

	//If not cleanedPath.empty and not cleanedPath.back = backslash then add a trailing backslash
	//Note: backslash \ is an escape character, so 2 backslashes are needed to reference only 1 backslash, e.g., "\\" = "\"
	if (!cleanedPath.empty() && cleanedPath.back() != '\\') {
		return cleanedPath + "\\";
	}
	return cleanedPath;
}
//bool EnsureFileExists function to check if a file exists using <fstream>
bool EnsureFileExists(const string& filePath) {
	ifstream file(filePath);
	return file.good();
}
